SystemOrganization addCategory: #'XML-Parser'!
SystemOrganization addCategory: 'XML-Parser-DOM'!
SystemOrganization addCategory: 'XML-Parser-DTD'!
SystemOrganization addCategory: 'XML-Parser-Exceptions'!
SystemOrganization addCategory: 'XML-Parser-File'!
SystemOrganization addCategory: 'XML-Parser-HTTP'!
SystemOrganization addCategory: 'XML-Parser-Namespaces'!
SystemOrganization addCategory: 'XML-Parser-Streams'!
SystemOrganization addCategory: 'XML-Parser-URI'!
SystemOrganization addCategory: 'XML-Parser-Util'!
SystemOrganization addCategory: 'XML-Parser-Validation'!
SystemOrganization addCategory: 'XML-Parser-Validation-DFA'!

!Character class methodsFor: '*XML-Parser' stamp: 'monty 4/7/2015 09:25'!
xmlParserSeparators
	^ String
		with: 32 asCharacter "Character space asciiValue"
		with: 10 asCharacter "Character lf asciiValue"
		with: 9 asCharacter "Character tab asciiValue"
		with: 13 asCharacter "Character cr asciiValue"! !

!Character class methodsFor: '*XML-Parser' stamp: 'monty 4/7/2015 09:25'!
xmlParserWhitespace
	^ self xmlParserSeparators copyWith:
		12 asCharacter "Character newPage asciiValue"! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 4/6/2015 10:26'!
xmlParserIsSeparator
	"Overridden in Gemstone version"
	value = 32 "space"
		ifTrue: [^ true].
	value = 10 "lf"
		ifTrue: [^ true].
	value = 9 "tab"
		ifTrue: [^ true].
	value = 13 "cr"
		ifTrue: [^ true].
	"no form feeds"
	^ false.! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 4/6/2015 10:26'!
xmlParserIsWhitespace
	"Overridden in Gemstone version"
	value = 32 "space"
		ifTrue: [^ true].
	value = 10 "lf"
		ifTrue: [^ true].
	value = 9 "tab"
		ifTrue: [^ true].
	value = 13 "cr"
		ifTrue: [^ true].
	value = 12 "form feed"
		ifTrue: [^ true].
	^ false.! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 12/14/2014 12:25'!
xmlParserSplit: aCollection
	"faster, portable version"
	| parts |

	parts := OrderedCollection new.
	self xmlParserSplit: aCollection do: [:each | parts addLast: each].
	^ parts.! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 12/14/2014 12:25'!
xmlParserSplit: aCollection do: aBlock
	"faster, portable version"
	| start |

	start := 1.
	1 to: aCollection size do: [:end |
		((aCollection at: end) == self)
			ifTrue: [
				aBlock value: (aCollection copyFrom: start to: end - 1).
				start := end + 1]].
	start > aCollection size
		ifTrue: [aBlock value: '']
		ifFalse: [aBlock value: (aCollection copyFrom: start to: aCollection size)].! !

Error subclass: #XMLException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLException commentStamp: 'monty 12/23/2014 16:36' prior: 0!
A generic exception class that supports message formatting.!

!XMLException class methodsFor: 'signalling' stamp: 'monty 12/22/2014 17:06'!
formatSignal: aString with: aFirstValue
	^ self
		formatSignal: aString
		withAll: (Array with: aFirstValue)! !

!XMLException class methodsFor: 'signalling' stamp: 'monty 12/22/2014 17:06'!
formatSignal: aString with: aFirstValue with: aSecondValue
	^ self
		formatSignal: aString
		withAll: (Array with: aFirstValue with: aSecondValue)! !

!XMLException class methodsFor: 'signalling' stamp: 'monty 12/22/2014 17:06'!
formatSignal: aString with: aFirstValue with: aSecondValue with: aThirdValue
	^ self
		formatSignal: aString
		withAll: (Array with: aFirstValue with: aSecondValue with: aThirdValue)! !

!XMLException class methodsFor: 'signalling' stamp: 'monty 12/22/2014 17:06'!
formatSignal: aString with: aFirstValue with: aSecondValue with: aThirdValue with: aFourthValue
	^ self
		formatSignal: aString
		withAll:
			(Array
				with: aFirstValue
				with: aSecondValue
				with: aThirdValue
				with: aFourthValue)! !

!XMLException class methodsFor: 'signalling' stamp: 'monty 12/22/2014 17:06'!
formatSignal: aString withAll: anArray
	^ self signal: (aString format: anArray)! !

!XMLException class methodsFor: 'signalling' stamp: 'monty 12/22/2014 17:06'!
signalMessageFrom: anException
	^ self signal: anException messageText! !

!XMLException methodsFor: 'private' stamp: 'monty 1/6/2015 09:25'!
append: anObject withDescription: aDescription
	self messageText:
		('{1} ({2} {3})' format:
			(Array
				with: (self messageText ifNil: [''])
				with: aDescription
				with: anObject))! !

XMLException subclass: #XMLParserException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLParserException commentStamp: 'monty 1/23/2014 17:59' prior: 0!
A base class for XML exceptions to make catching XML errors easier.!

XMLParserException subclass: #XMLDOMException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLDOMException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked when a DOM tree is manipulated wrong.!

XMLParserException subclass: #XMLEncodingException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLEncodingException commentStamp: 'monty 11/11/2014 04:38' prior: 0!
Invoked for encoding errors.!

XMLParserException subclass: #XMLFileException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLFileException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked for file IO errors.!

XMLParserException subclass: #XMLHTTPException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLHTTPException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked for HTTP errors.!

XMLParserException subclass: #XMLLimitException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLLimitException commentStamp: 'monty 1/25/2014 00:19' prior: 0!
Invoked when a security limit on input is exceeded. Use the messages in XMLParserLimits and DTDExternalEntityLimits to configure these limits or the removeLimits message to remove them completely.!

XMLParserException subclass: #XMLNamespaceException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLNamespaceException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when namespaces are used improperly and namespace checking is on.!

!XMLParserException methodsFor: 'appending' stamp: 'monty 1/6/2015 09:19'!
appendLineNumber: aLineNumber
	self
		append: aLineNumber
		withDescription: 'line'! !

!XMLParserException methodsFor: 'testing' stamp: 'monty 11/26/2014 06:03'!
shouldPassOnWhenCaught
	^ true! !

XMLParserException subclass: #XMLParsingInterruptException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLParsingInterruptException commentStamp: 'monty 11/26/2014 05:54' prior: 0!
Used to interrupt parsing from a SAXHandler. Will be silently caught.!

!XMLParsingInterruptException methodsFor: 'testing' stamp: 'monty 11/26/2014 06:03'!
shouldPassOnWhenCaught
	^ false! !

XMLParserException subclass: #XMLValidationException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLValidationException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when invalid XML is detected and validation is on.!

XMLParserException subclass: #XMLWellFormednessException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLWellFormednessException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when XML markup is not well-formed.!

!UndefinedObject methodsFor: '*XML-Parser' stamp: 'monty 4/5/2015 02:36'!
xmlParserIsSeparator
	"hack optimization to avoid testing for nil before sending
	isSeparator in XMLParser"
	^ false! !

!String methodsFor: '*XML-Parser' stamp: 'monty 12/27/2014 02:57'!
asXMLEscapedString
	| stream |

	stream := (self class new: self size) writeStream.
	"to:do: for performance"
	1 to: self size do: [:i | | nextChar |
		(nextChar := self at: i) == $<
			ifTrue: [stream nextPutAll: '&lt;']
			ifFalse: [
				nextChar == $> "needed to escape ]]> in PCDATA"
					ifTrue: [stream nextPutAll: '&gt;']
					ifFalse: [
						nextChar == $&
							ifTrue: [stream nextPutAll: '&amp;']
							ifFalse: [stream nextPut: nextChar]]]].
	^ stream contents.! !

!String methodsFor: '*XML-Parser' stamp: 'JAAyer 4/14/2010 21:26'!
asXMLString
	^ XMLString string: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 1/10/2014 00:56'!
asXMLURI
	^ XMLURI fromString: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 3/22/2015 05:41'!
expandWithNamespaceURI: aURI
	aURI size > 0
		ifTrue: [^ 'Q{', aURI, '}', self]
		ifFalse: [^ self]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/22/2014 20:01'!
hasPrefixBeforeLocalName
	^ (self
		indexOf: $:
		startingAt: 2) > 0! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 19:47'!
isNamespaceAttributeName
	^ (self beginsWith: 'xmlns')
		and: [self size = 5
			or: [(self at: 6) == $:]]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 12/27/2014 03:00'!
isQualifiedName: aQualifiedName orLocalName: aLocalName
	"optimized"
	^ self = aQualifiedName
		or: [(aQualifiedName size > aLocalName size)
			and: [self = aLocalName]]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/2/2014 17:45'!
isReservedAttributeName
	^ self size > 4
		and: [self beginsWith: 'xml:']! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 20:23'!
localNameAfterPrefix
	^ self localNameAfterPrefix: self prefixBeforeLocalName! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 19:41'!
localNameAfterPrefix: aPrefix
	"optimized"
	| prefixSize |

	(prefixSize := aPrefix size) > 0
		ifTrue: [^ self copyFrom: prefixSize + 2 to: self size]
		ifFalse: [^ self]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 10/18/2014 05:47'!
parseXML
	^ XMLDOMParser parse: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 20:23'!
prefixBeforeLocalName
	"Split the name into namespace prefix
	(Optimized code)"
	| i |

	i := self size.
	[i > 1]
		whileTrue: [
			((self at: i) == $:)
				ifTrue: [^ (self copyFrom: 1 to: i - 1)].
			i := i - 1].
	^ ''.! !

!String methodsFor: '*XML-Parser' stamp: 'monty 12/27/2014 02:59'!
prefixBeforeLocalName: aLocalName
	"optimized"
	| prefixSize |

	(prefixSize := self size - aLocalName size - 1) > 0
		ifTrue: [^ self copyFrom: 1 to: prefixSize]
		ifFalse: [^ '']! !

!String methodsFor: '*XML-Parser' stamp: 'monty 12/27/2014 02:59'!
qualifyWithPrefix: aPrefix
	"optimized"
	aPrefix size > 0
		ifTrue: [^ (aPrefix copyWith: $:), self]
		ifFalse: [^ self]! !

!PositionableStream methodsFor: '*XML-Parser' stamp: 'monty 12/14/2014 12:29'!
xmlParserAtBeginning
	"overridden in Gemstone version"
	^ position = 0! !

!Collection methodsFor: '*XML-Parser' stamp: 'monty 12/11/2014 05:17'!
includesQualifiedName: aQualifiedName orLocalName: aLocalName
	"optimized"
	1 to: self size do: [:i |
		((self at: i)
			isQualifiedName: aQualifiedName
			orLocalName: aLocalName)
			ifTrue: [^ true]].
	^ false.! !

Collection subclass: #XMLNFAStateSet
	instanceVariableNames: 'set isAccepting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAStateSet commentStamp: 'monty 1/24/2014 22:44' prior: 0!
This class is a set of NFA states. It wraps an IdentitySet and keeps track of if any of its states is accepting.!

!XMLNFAStateSet class methodsFor: 'instance creation' stamp: 'monty 12/11/2014 05:39'!
new
	^ self new: 5! !

!XMLNFAStateSet class methodsFor: 'instance creation' stamp: 'monty 12/11/2014 05:39'!
new: aCapacity
	^ self basicNew initialize: aCapacity! !

!XMLNFAStateSet methodsFor: 'comparing' stamp: 'monty 1/9/2014 22:12'!
= anObject
	^ self == anObject
		or: [self class == anObject class
			and: [set = anObject set]]! !

!XMLNFAStateSet methodsFor: 'adding' stamp: 'monty 12/28/2013 18:52'!
add: aState
	isAccepting
		ifFalse: [isAccepting := aState isAccepting].

	^ set add: aState.! !

!XMLNFAStateSet methodsFor: 'enumerating' stamp: 'monty 12/28/2013 18:47'!
do: aBlock
	set do: aBlock! !

!XMLNFAStateSet methodsFor: 'comparing' stamp: 'monty 12/28/2013 18:50'!
hash
	^ set hash! !

!XMLNFAStateSet methodsFor: 'testing' stamp: 'monty 12/28/2013 18:48'!
includes: aState
	^ set includes: aState! !

!XMLNFAStateSet methodsFor: 'initialization' stamp: 'monty 12/11/2014 05:39'!
initialize: aCapacity
	set := IdentitySet new: aCapacity.
	isAccepting := false.! !

!XMLNFAStateSet methodsFor: 'testing' stamp: 'monty 12/28/2013 18:53'!
isAccepting
	^ isAccepting! !

!XMLNFAStateSet methodsFor: 'removing' stamp: 'monty 12/28/2013 18:46'!
remove: aState ifAbsent: anExceptionBlock
	set
		remove: aState
		ifAbsent: [^ anExceptionBlock value].

	isAccepting
		ifTrue: [isAccepting := set anySatisfy: [:each | each isAccepting]].

	^ aState.! !

!XMLNFAStateSet methodsFor: 'accessing' stamp: 'monty 12/28/2013 18:50'!
set
	^ set! !

!XMLNFAStateSet methodsFor: 'accessing' stamp: 'monty 12/28/2013 19:01'!
size
	^ set size! !

Collection subclass: #XMLOrderedList
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLOrderedList commentStamp: '<historical>' prior: 0!
This class functions as an OrderedCollection that will not raise exceptions when attempts are made to access absent objects. This means messages like #at:, #after:, and #first will return nil when the requested object cannot be retrieved. Messages that modify the collection based on the presence or absence of other objects such as #add:after: or #at:put: will still raise exceptions as usual.

!

XMLOrderedList subclass: #XMLObservableList
	instanceVariableNames: 'observer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLObservableList commentStamp: '<historical>' prior: 0!
This class is an is an ordered list that sends notifications to observers when objects are added to it or removed from it. Subclasses should override the #beforeAdding:, #afterAdding: and #afterRemoving: messages to send custom notifications.!

XMLObservableList subclass: #XMLAttributeList
	instanceVariableNames: 'attributesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttributeList commentStamp: 'monty 10/28/2014 18:54' prior: 0!
This class represents an ordered list of attribute nodes that also keeps a dictionary mapping attribute names to nodes for faster lookups. You usually shouldn't have to use this class directly.!

!XMLAttributeList methodsFor: 'adding' stamp: 'monty 10/28/2014 14:38'!
addName: aName value: aValue
	^ self addLast:
		(self attributeClass
			name: aName
			value: aValue)! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 2/3/2015 23:40'!
afterAdding: anAttribute at: anIndex
	self attributesByName
		at: anAttribute name
		ifPresent: [:oldAttribute |
			self
				replacedAttribute: oldAttribute
				withAttributeAt: anIndex];
		at: anAttribute name
		put: anAttribute.
	observer ifNotNil: [observer addedAttribute: anAttribute].! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 2/3/2015 23:40'!
afterAddingLast: anAttribute
	self attributesByName
		at: anAttribute name
		ifPresent: [:oldAttribute |
			self
				replacedAttribute: oldAttribute
				withAttributeAt: self size];
		at: anAttribute name
		put: anAttribute.
	observer ifNotNil: [observer addedAttribute: anAttribute].! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 18:37'!
afterRemoving: anAttribute
	self attributesByName removeKey: anAttribute name.
	observer ifNotNil: [observer removedAttribute: anAttribute].! !

!XMLAttributeList methodsFor: 'converting' stamp: 'monty 11/20/2014 19:49'!
asDictionary
	| dictionary |

	dictionary := OrderPreservingStringDictionary new: self size.
	self do: [:each | dictionary at: each name put: each value].
	^ dictionary.! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 11/6/2014 15:43'!
attributeAssociations
	^ self collect: [:each | each name -> each value]! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 14:15'!
attributeClass
	^ XMLAttribute
! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 14:14'!
attributeNames
	^ self collect: [:each | each name]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'monty 10/28/2014 14:15'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self do: [:each | aTwoArgumentBlock value: each name value: each value]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'monty 10/28/2014 14:15'!
attributeNamesDo: aBlock
	self do: [:each | aBlock value: each name]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 14:14'!
attributeNodeAt: aName
	^ self attributeNodeAt: aName ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 18:35'!
attributeNodeAt: aName ifAbsent: aBlock
	^ self attributesByName at: aName ifAbsent: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:36'!
attributeNodeAt: aLocalName namespaceURI: aURI
	^ self
		attributeNodeAt: aLocalName
		namespaceURI: aURI
		ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:36'!
attributeNodeAt: aLocalName namespaceURI: aURI ifAbsent: aBlock
	^ self
		detect: [:each |
			each name = aLocalName
				and: [each namespaceURI = aURI]]
		ifNone: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 15:00'!
attributeValueAt: aName
	^ self attributeValueAt: aName ifAbsent: ['']! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 18:35'!
attributeValueAt: aName ifAbsent: aBlock
	^ (self attributesByName
		at: aName
		ifAbsent: [^ aBlock value]) value! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 14:14'!
attributeValueAt: aName ifAbsentPut: aBlock
	^ self
		attributeValueAt: aName
		ifAbsent: [self attributeValueAt: aName put: aBlock value]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 18:35'!
attributeValueAt: aName put: aValue
	(self includesAttribute: aName)
		ifTrue: [(self attributeNodeAt: aName) value: aValue]
		ifFalse: [self addName: aName value: aValue].
	^ aValue.! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 12/11/2014 07:25'!
attributesByName
	^ attributesByName ifNil: [attributesByName := Dictionary new: 5]! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 1/21/2015 11:39'!
beforeAdding: anAttribute
	observer ifNotNil: [observer addingAttribute: anAttribute]! !

!XMLAttributeList methodsFor: 'testing' stamp: 'monty 10/28/2014 18:35'!
includesAttribute: aName
	^ self attributesByName includesKey: aName! !

!XMLAttributeList methodsFor: 'removing' stamp: 'monty 10/28/2014 14:42'!
removeAttributeNodeAt: aName
	^ self remove: (self attributeNodeAt: aName).! !

!XMLAttributeList methodsFor: 'removing' stamp: 'monty 11/23/2014 00:53'!
removeAttributeNodeAt: aName ifAbsent: aBlock
	^ self remove:
		(self
			attributeNodeAt: aName
			ifAbsent: [^ aBlock value]).! !

!XMLAttributeList methodsFor: 'notifying' stamp: 'monty 10/28/2014 18:35'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self attributesByName
		removeKey: anOldName;
		at: aNewName ifPresent: [:attribute | self remove: attribute];
		at: aNewName put: anAttribute! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 2/9/2015 01:31'!
replacedAttribute: anAttribute withAttributeAt: anIndex
	| i |

	(i :=
		self
			indexOf: anAttribute
			otherThan: anIndex) > 0
		ifTrue: [self removeAt: i]! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 18:35'!
setFromDictionary: anOrderPreservingDictionary
	| association |

	"hack to reuse a dictionary for speed"
	attributesByName := anOrderPreservingDictionary dictionary.

	"add the XMLAttributes in order directly to the collection
	to avoid sending notifications and also replace the value in
	the attributesByName association with the XMLAttribute"
	anOrderPreservingDictionary keysDo: [:each |
		(association := attributesByName associationAt: each ifAbsent: nil)
			value:
				(collection addLast: 
					(self attributeClass new
						setName: each
						value: association value
						element: observer))].! !

!XMLAttributeList methodsFor: 'sorting' stamp: 'monty 12/2/2014 23:05'!
sort
	self sort: [:a :b | a sortKey <= b sortKey]! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 1/6/2015 20:11'!
writeAttributesOn: aTagWriter
	self do: [:each |
		aTagWriter
			writeAttributeName: each name
			value: each value]! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 1/6/2015 20:11'!
writeSortedAttributesOn: aTagWriter
	self sorted do: [:each |
		aTagWriter
			writeAttributeName: each name
			value: each value]! !

XMLObservableList subclass: #XMLInternalSubsetList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLInternalSubsetList commentStamp: 'JAAyer 7/27/2010 12:45' prior: 0!
This class represents the list of nodes that comprise a document's internal DTD subset.!

!XMLInternalSubsetList methodsFor: 'private' stamp: 'monty 1/21/2015 11:55'!
afterAdding: aNode at: anIndex
	observer ifNotNil: [observer addedNode: aNode]! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'monty 10/18/2014 00:54'!
afterRemoving: aNode
	observer ifNotNil: [observer removedNode: aNode]! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'monty 10/29/2014 09:00'!
beforeAdding: aNode
	observer ifNotNil: [observer addingNode: aNode]! !

!XMLInternalSubsetList methodsFor: 'sorting' stamp: 'monty 1/18/2015 09:31'!
sort
	self sort: [:a :b | a sortKey <= b sortKey]! !

XMLObservableList subclass: #XMLNodeList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeList commentStamp: '<historical>' prior: 0!
This class represents an ordered list of child nodes for some kind of node with elements, probably either an XML document or element object. Non-modifying accessing messages like #first and #last will never raise exceptions and return nil instead.!

XMLNodeList subclass: #XMLCachingNodeList
	instanceVariableNames: 'cachedElements cachedElementsByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLCachingNodeList commentStamp: '<historical>' prior: 0!
This is a node list that caches child elements by name, keeping both an ordered list of elements and a dictionary that allows for faster retrieval of elements by qualified or local name.

You can inject it into the DOM parser like this:
parser := XMLDOMParser on: xmlSource.
parser nodeListClass: XMLCachingNodeList.!

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 02:54'!
afterAdding: aNode at: anIndex
	aNode isElement
		ifTrue: [
			anIndex = self size
				ifTrue: [self cacheElementAddedLast: aNode]
				ifFalse: [
					self
						cacheElement: aNode
						addedAt: anIndex]].
	super
		afterAdding: aNode
		at: anIndex.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 02:06'!
afterAddingLast: aNode
	aNode isElement
		ifTrue: [self cacheElementAddedLast: aNode].
	super afterAddingLast: aNode.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 02:07'!
afterRemoving: aNode
	aNode isElement
		ifTrue: [self uncacheElement: aNode].
	super afterRemoving: aNode.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:00'!
cacheElement: anElement addedAt: anIndex
	(self findElementAfterIndex: anIndex)
		ifNil: [self cachedElements addLast: anElement]
		ifNotNil: [:elementAfter |
			self cachedElements
				add: anElement
				before: elementAfter].
	self
		cachedElementsByNameAt: anElement name
		add: anElement
		beforeNextAfterIndex: anIndex.
	anElement hasPrefix
		ifTrue: [
			self
				cachedElementsByNameAt: anElement localName
				add: anElement
				beforeNextAfterIndex: anIndex].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:00'!
cacheElementAddedLast: anElement
	self cachedElements addLast: anElement.
	self
		cachedElementsByNameAt: anElement name
		addLast: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				cachedElementsByNameAt: anElement localName
				addLast: anElement].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 1/16/2015 13:46'!
cachedElements
	^ cachedElements ifNil: [cachedElements := OrderedCollection new]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 1/16/2015 13:50'!
cachedElementsByName
	^ cachedElementsByName ifNil: [cachedElementsByName := Dictionary new]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:06'!
cachedElementsByNameAt: aName add: anElement beforeNextAfterIndex: anIndex
	| elementAfter elementsWithName |

	(elementAfter :=
		self
			findElementAfterIndex: anIndex
			named: aName)
		ifNil: [
			^ self
				cachedElementsByNameAt: aName
				addLast: anElement].

	(elementsWithName := self cachedElementsByName at: aName) isCollection
		ifTrue: [elementsWithName add: anElement before: elementAfter]
		ifFalse: [
			self cachedElementsByName
				at: aName
				put: (OrderedCollection with: anElement with: elementAfter)].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:08'!
cachedElementsByNameAt: aName addLast: anElement
	| elementsWithName |

	elementsWithName :=
		self cachedElementsByName
			at: aName
			ifAbsent: [
				self cachedElementsByName at: aName put: anElement.
				^ self].

	 elementsWithName isCollection
		ifTrue: [elementsWithName addLast: anElement]
		ifFalse: [
			self cachedElementsByName
				at: aName
				put: (OrderedCollection with: elementsWithName with: anElement)].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:01'!
cachedElementsByNameAt: aName remove: anElement
	| elementsWithName |

	(elementsWithName := self cachedElementsByName at: aName) isCollection
		ifTrue: [
			elementsWithName remove: anElement.
			elementsWithName isEmpty
				ifTrue: [self cachedElementsByName removeKey: aName]]
		ifFalse: [self cachedElementsByName removeKey: aName]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 1/17/2015 08:29'!
copyClass
	"use superclass for copies for performance"
	^ XMLNodeList! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 1/17/2015 08:18'!
elementAt: aName ifAbsent: aBlock
	| elementsWithName |

	(elementsWithName :=
		self cachedElementsByName
			at: aName
			ifAbsent: [^ aBlock value]) isCollection
		ifTrue: [^ elementsWithName first]
		ifFalse: [^ elementsWithName]! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 1/16/2015 13:43'!
elements
	^ self copyEmpty
		addAll: self cachedElements;
		yourself! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 1/16/2015 13:53'!
elementsAt: aName
	| elementsWithName copiedElements |

	copiedElements := self copyEmpty.
	(elementsWithName :=
		self cachedElementsByName
			at: aName
			ifAbsent: [^ copiedElements]) isCollection
		ifTrue: [copiedElements addAll: elementsWithName]
		ifFalse: [copiedElements addLast: elementsWithName].
	^ copiedElements.! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 1/17/2015 08:16'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self cachedElements
		detect: aBlock
		ifNone: anExceptionBlock! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 1/16/2015 13:44'!
elementsDo: aBlock
	self cachedElements do: aBlock! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 1/17/2015 08:24'!
elementsSelect: aBlock
	| selectedElements |

	selectedElements := self copyEmpty.
	self cachedElements do: [:each |
		(aBlock value: each)
			ifTrue: [selectedElements addLast: each]].
	^ selectedElements.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 15:34'!
findElementAfterIndex: anIndex
	anIndex + 1 to: self size do: [:i | | node |
		node := self at: i.
		node isElement
			ifTrue: [^ node]].
	^ nil.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 00:30'!
findElementAfterIndex: anIndex named: aName
	anIndex + 1 to: self size do: [:i | | node |
		node := self at: i.
		(node isElement
			and: [node isNamed: aName])
			ifTrue: [^ node]].
	^ nil.! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 1/16/2015 13:59'!
firstElement
	^ self cachedElements
		at: 1
		ifAbsent: [self defaultValue]! !

!XMLCachingNodeList methodsFor: 'testing' stamp: 'monty 1/16/2015 13:47'!
hasElements
	^ cachedElements notNil and: [cachedElements notEmpty]! !

!XMLCachingNodeList methodsFor: 'testing' stamp: 'monty 1/16/2015 13:48'!
includesElement: aName
	^ self cachedElementsByName includesKey: aName! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 1/16/2015 14:00'!
lastElement
	^ self cachedElements
		at: self cachedElements size
		ifAbsent: [self defaultValue]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 02:58'!
rebuildCachedElements
	self cachedElements removeAll.
	self cachedElementsByName removeAll.
	self do: [:each |
		each isElement
			ifTrue: [self cacheElementAddedLast: each]].! !

!XMLCachingNodeList methodsFor: 'notifying' stamp: 'monty 2/9/2015 06:00'!
renamedElement: anElement from: anOldName to: aNewName
	| oldPrefix index |

	self
		cachedElementsByNameAt: anOldName
		remove: anElement.
	(oldPrefix := anOldName prefixBeforeLocalName)
		ifNotEmpty: [
			self
				cachedElementsByNameAt: (anOldName localNameAfterPrefix: oldPrefix)
				remove: anElement].
	index := self indexOf: anElement.
	self
		cachedElementsByNameAt: anElement name
		add: anElement
		beforeNextAfterIndex: index.
	anElement hasPrefix
		ifTrue: [
			self
				cachedElementsByNameAt: anElement localName
				add: anElement
				beforeNextAfterIndex: index].! !

!XMLCachingNodeList methodsFor: 'sorting' stamp: 'monty 2/2/2015 01:32'!
sort: aSortBlock
	super sort: aSortBlock.

	self cachedElements sort: aSortBlock.
	self cachedElementsByName valuesDo: [:each |
		each isCollection
			ifTrue: [each sort: aSortBlock]].! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 2/2/2015 02:58'!
swap: aFirstIndex with: aSecondIndex
	super swap: aFirstIndex with: aSecondIndex.

	((self at: aFirstIndex) isElement
		or: [(self at: aSecondIndex) isElement])
		ifTrue: [self rebuildCachedElements].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:00'!
uncacheElement: anElement
	self cachedElements remove: anElement.
	self
		cachedElementsByNameAt: anElement name
		remove: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				cachedElementsByNameAt: anElement localName
				remove: anElement].! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 1/21/2015 11:54'!
afterAdding: aNode at: anIndex
	observer ifNotNil: [
		aNode isElement
			ifTrue: [observer addedElement: aNode]
			ifFalse: [observer addedNode: aNode]]! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 1/21/2015 12:27'!
afterAddingLast: aNode
	observer ifNotNil: [
		aNode isElement
			ifTrue: [observer addedElement: aNode]
			ifFalse: [observer addedNode: aNode]]! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 10/28/2014 19:05'!
afterRemoving: aNode
	observer ifNotNil: [
		aNode isElement
			ifTrue: [observer removedElement: aNode]
			ifFalse: [observer removedNode: aNode]]! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 10/28/2014 13:48'!
beforeAdding: aNode
	observer ifNotNil: [observer addingNode: aNode]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/25/2014 21:08'!
contentNodes
	^ self select: [:each | each isContentNode]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:08'!
contentNodesDo: aBlock
	self do: [:each | each isContentNode ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/25/2014 22:02'!
contentStringAt: aName
	^ (self elementAt: aName ifAbsent: [^ '']) contentString! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:27'!
elementAfter: anElement
	| last |

	self elementsDo: [:each |
		last = anElement
			ifTrue: [^ each].
		last := each].
	^ self defaultValue.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName
	^ self elementAt: aName ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:13'!
elementAt: aName ifAbsent: aBlock
	^ self
		detect: [:each | each isElement and: [each isNamed: aName]]
		ifNone: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
elementAt: aName namespaceURI: aURI
	^ self
		elementAt: aName
		namespaceURI: aURI
		ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
elementAt: aName namespaceURI: aURI ifAbsent: aBlock
	^ (self elementsAt: aName)
		detect: [:each | each namespaceURI = aURI]
		ifNone: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:27'!
elementBefore: anElement
	| last |

	last := self defaultValue.
	self elementsDo: [:each |
		each = anElement
			ifTrue: [^ last].
		last := each].
	^ self defaultValue.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/26/2014 08:19'!
elementNames
	^ self elementsCollect: [:each | each name]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:07'!
elements
	^ self select: [:each | each isElement]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:12'!
elementsAt: aName
	^ self elementsSelect: [:each | each isNamed: aName]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:58'!
elementsAt: aName do: aBlock
	(self elementsAt: aName) do: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
elementsAt: aName namespaceURI: aURI
	^ (self elementsAt: aName) select: [:each | each namespaceURI = aURI]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:34'!
elementsCollect: aBlock
	^ self elements collect: aBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock
	^ self elementsDetect: aBlock ifNone: [self defaultValue]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:10'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self
		detect: [:each | each isElement and: [aBlock value: each]]
		ifNone: anExceptionBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:11'!
elementsDo: aBlock
	self do: [:each | each isElement ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:08'!
elementsSelect: aBlock
	^ self select: [:each | each isElement and: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:23'!
firstElement
	^ self
		detect: [:each | each isElement]
		ifNone: [self defaultValue]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 10/25/2014 21:01'!
hasContentNodes
	^ self anySatisfy: [:each | each isContentNode]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:03'!
hasElements
	^ self anySatisfy: [:each | each isElement]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 10/25/2014 21:01'!
hasStringNodes
	^ self anySatisfy: [:each | each isStringNode]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 10/25/2014 21:04'!
includesElement: aName
	self elementAt: aName ifAbsent: [^ false].
	^ true.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:23'!
lastElement
	self reverseDo: [:each |
		each isElement
			ifTrue: [^ each]].
	^ self defaultValue.! !

!XMLNodeList methodsFor: 'notifying' stamp: 'monty 10/18/2014 01:06'!
renamedElement: anElement from: anOldName to: aNewName! !

!XMLNodeList methodsFor: 'sorting' stamp: 'monty 12/3/2014 00:45'!
sort
	self sort: [:a :b | a sortKey <= b sortKey]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 20:59'!
stringNodes
	^ self select: [:each | each isStringNode]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 20:57'!
stringNodesDo: aBlock
	self do: [:each | each isStringNode ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:00'!
strings
	^ self
		select: [:each | each isStringNode]
		thenCollect: [:each | each string]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 20:58'!
stringsDo: aBlock
	self stringNodesDo: [:each | aBlock value: each string]! !

!XMLObservableList class methodsFor: 'instance creation' stamp: 'monty 10/18/2014 01:02'!
observer: anObject
	^ self new observer: anObject! !

!XMLObservableList class methodsFor: 'as yet unclassified' stamp: 'monty 1/17/2015 11:52'!
ofSize: aSize
	self shouldNotImplement! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 1/21/2015 14:53'!
add: anObject after: aPreviousObject
	^ self
		add: anObject
		afterIndex:
			(collection
				indexOf: aPreviousObject
				ifAbsent: [-1])! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 1/21/2015 11:51'!
add: anObject afterIndex: anIndex
	self beforeAdding: anObject.
	collection
		add: anObject
		afterIndex: anIndex.
	self
		afterAdding: anObject
		at: anIndex + 1.
	^ anObject.! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 1/21/2015 14:53'!
add: anObject before: aNextObject
	^ self
		add: anObject
		beforeIndex:
			(collection
				indexOf: aNextObject
				ifAbsent: [-1])! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 2/2/2015 01:29'!
add: anObject beforeIndex: anIndex
	self beforeAdding: anObject.
	collection
		add: anObject
		beforeIndex: anIndex.
	self
		afterAdding: anObject
		at: anIndex.
	^ anObject.! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 1/21/2015 11:44'!
addFirst: anObject
	self beforeAdding: anObject.
	collection addFirst: anObject.
	self afterAdding: anObject at: 1.
	^ anObject.! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 1/21/2015 12:26'!
addLast: anObject
	self beforeAdding: anObject.
	collection addLast: anObject.
	self afterAddingLast: anObject.
	^ anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 1/21/2015 11:46'!
afterAdding: anObject at: anIndex
	observer ifNotNil: [observer added: anObject to: self]! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 1/21/2015 12:26'!
afterAddingLast: anObject
	self
		afterAdding: anObject
		at: self size! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 10/18/2014 00:51'!
afterRemoving: anObject
	observer ifNotNil: [observer removed: anObject from: self]! !

!XMLObservableList methodsFor: 'accessing' stamp: 'monty 1/21/2015 12:35'!
at: anIndex put: anObject
	| oldObject |

	"gemstone behavior"
	anIndex = (self size + 1)
		ifTrue: [^ self addLast: anObject].

	self beforeAdding: anObject.

	oldObject := collection at: anIndex.
	collection at: anIndex put: nil.
	oldObject
		ifNotNil: [self afterRemoving: oldObject].
	
	collection at: anIndex put: anObject.
	self
		afterAdding: anObject
		at: anIndex.
	^ anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 10/18/2014 00:51'!
beforeAdding: anObject
	observer ifNotNil: [observer adding: anObject to: self]! !

!XMLObservableList methodsFor: 'copying' stamp: 'monty 1/16/2015 13:09'!
deepCopyWithObserver: anObserver
	| newList |

	newList := self copyEmpty observer: anObserver.
	self do: [:each | newList addLast: each copy].
	^ newList.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 2/9/2015 01:31'!
indexOf: anObject otherThan: anIndex
	"used by subclasses to remove duplicates"
	1 to: anIndex - 1 do: [:i |
		(collection at: i) = anObject
			ifTrue: [^ i]].
	anIndex + 1 to: collection size do: [:i |
		(collection at: i) = anObject
			ifTrue: [^ i]].
	^ 0.! !

!XMLObservableList methodsFor: 'observing' stamp: 'monty 10/18/2014 00:50'!
observer
	^ observer! !

!XMLObservableList methodsFor: 'observing' stamp: 'monty 10/18/2014 00:50'!
observer: anObserver
	observer := anObserver! !

!XMLObservableList methodsFor: 'removing' stamp: 'monty 1/21/2015 10:00'!
remove: anObject ifAbsent: aBlock
	collection remove: anObject ifAbsent: [^ aBlock value].
	self afterRemoving: anObject.
	^ anObject.! !

!XMLObservableList methodsFor: 'removing' stamp: 'monty 1/21/2015 10:00'!
removeAt: anIndex
	| removed |

	removed := collection removeAt: anIndex.
	self afterRemoving: removed.
	^ removed.! !

!XMLObservableList methodsFor: 'removing' stamp: 'monty 1/21/2015 09:55'!
reset
	| removed |

	removed := collection copy.
	collection reset.
	removed do: [:each | self afterRemoving: each].! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/17/2015 08:28'!
collection: aCollection
	^ self basicNew setCollection: aCollection! !

!XMLOrderedList class methodsFor: 'private' stamp: 'monty 1/29/2014 01:12'!
collectionClass
	^ OrderedCollection! !

!XMLOrderedList class methodsFor: 'private' stamp: 'monty 10/25/2014 20:34'!
inspectorClass
	^ self collectionClass inspectorClass! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/28/2014 05:54'!
new
	^ self new: 5! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/16/2015 07:12'!
new: aCapacity
	^ self basicNew setCollection: (self collectionClass new: aCapacity)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/16/2015 07:12'!
new: aSize withAll: anObject
	| newList |

	"reimplement for portability"
	newList := self new: aSize.
	1 to: aSize do: [:i | newList addLast: anObject].
	^ newList.! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/16/2015 07:11'!
newFrom: aCollection
	^ (self new: aCollection size)
		addAll: aCollection;
		yourself! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:43'!
ofSize: aSize
	^ self new: aSize withAll: nil! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/23/2014 10:54'!
, aCollection 
	^ self
		copyReplaceFrom: self size + 1
		to: self size
		with: aCollection! !

!XMLOrderedList methodsFor: 'comparing' stamp: 'monty 10/22/2014 02:44'!
= aCollection
	^ self == aCollection
		or: [self species == aCollection species
			and: [collection = aCollection collection]]! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:01'!
add: anObject
	^ self addLast: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:08'!
add: anObject after: aPriorObject
	^ collection add: anObject after: aPriorObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject afterIndex: anIndex
	^ collection add: anObject afterIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject before: aSubsequentObject
	^ collection add: anObject before: aSubsequentObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:10'!
add: anObject beforeIndex: anIndex
	^ collection add: anObject beforeIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:02'!
addAll: aCollection
	^ self addAllLast: aCollection! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:10'!
addAllFirst: aCollection
	aCollection reverseDo: [:each | self addFirst: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:11'!
addAllLast: aCollection
	aCollection do: [:each | self addLast: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:11'!
addFirst: anObject
	^ collection addFirst: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:12'!
addLast: anObject
	^ collection addLast: anObject! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:10'!
after: anObject
	^ self after: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:17'!
after: anObject ifAbsent: aBlock
	^ collection after: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:56'!
allButFirst
	^ self allButFirst: 1! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 18:05'!
allButFirst: anInteger
	^ self copyFrom: anInteger + 1 to: self size! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 12/14/2014 13:41'!
allButFirstDo: aBlock
	"reimplemented for portability"
	2 to: self size do: [:i | aBlock value: (self at: i)]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:56'!
allButLast
	^ self allButLast: 1! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 18:02'!
allButLast: anInteger
	^ self copyFrom: 1 to: self size - anInteger! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 12/14/2014 13:42'!
allButLastDo: aBlock
	"reimplemented for portability"
	1 to: self size - 1 do: [:i | aBlock value: (self at: i)]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
anyOne
	^ self first! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 12/13/2014 20:03'!
asArray
	^ Array withAll: collection! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 12/13/2014 20:03'!
asOrderedCollection
	^ OrderedCollection withAll: collection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 1/16/2015 13:06'!
at: anIndex
	"optimized"
	(anIndex > 0 and: [anIndex <= collection size])
		ifTrue: [^ collection at: anIndex]
		ifFalse: [^ self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:17'!
at: anIndex ifAbsent: aBlock
	^ collection at: anIndex ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 1/18/2015 18:06'!
at: anIndex put: aValue
	"treats index = size + 1 as addLast: gemstone compatibility"
	anIndex = (collection size + 1)
		ifTrue: [^ collection addLast: aValue]
		ifFalse: [^ collection at: anIndex put: aValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/5/2010 00:55'!
atLast: anIndexFromEnd
	^ self atLast: anIndexFromEnd ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:18'!
atLast: anIndexFromEnd ifAbsent: aBlock
	^ collection atLast: anIndexFromEnd ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:11'!
before: anObject
	^ self before: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:17'!
before: anObject ifAbsent: aBlock
	 ^ collection before: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'testing' stamp: 'monty 10/22/2014 16:11'!
beginsWith: aCollection
	^ collection beginsWith: aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 13:14'!
capacity
	^ collection capacity! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 1/17/2015 04:10'!
collect: aBlock
	| collected |

	collected := self species new: self size.
	self do: [:each | collected addLast: (aBlock value: each)].
	^ collected.! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 1/17/2015 04:11'!
collect: aCollectBlock thenSelect: aSelectBlock
	| collected |

	collected := self species new.
	self do: [:each | | collectedObject |
		(aSelectBlock value: (collectedObject := aCollectBlock value: each))
			ifTrue: [collected addLast: collectedObject]].
	^ collected.! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAA 3/25/2011 15:35'!
collection
	^ collection! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/17/2015 12:35'!
copy
	^ self copyEmpty
		addAll: self;
		yourself! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 06:16'!
copyAfter: anObject
	| copy |

	copy := self copyEmpty.
	(self
		indexOf: anObject
		ifAbsent: [^ copy]) + 1
			to: self size
			do: [:i | copy addLast: (self at: i)].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 06:16'!
copyAfterLast: anObject
	| copy |

	copy := self copyEmpty.
	(self
		lastIndexOf: anObject
		ifAbsent: [^ copy]) + 1
			to: self size
			do: [:i | copy addLast: (self at: i)].
	^ copy.! !

!XMLOrderedList methodsFor: 'private' stamp: 'monty 1/17/2015 08:26'!
copyClass
	^ self class! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/17/2015 08:28'!
copyEmpty
	^ self copyClass collection: collection copyEmpty! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 13:04'!
copyFrom: aStart to: anEnd
	| copy |

	copy := self copyEmpty.
	(aStart max: 1)
		to: (anEnd min: self size)
		do: [:i | copy addLast: (self at: i)].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/17/2015 15:28'!
copyReplaceAll: aSearchCollection with: aReplacementCollection
	| copy index size searchCollectionSize |

	"this implementation always returns a copy, even if it doesn't replace"
	(searchCollectionSize := aSearchCollection size) > 0
		ifFalse: [^ self copy].

	copy := self copyEmpty.
	index := 1.
	size := self size.
	[index <= size]
		whileTrue: [
			(self at: index) = (aSearchCollection at: 1)
				ifTrue: [| searchCollectionIndex |
					searchCollectionIndex := 2.
					[searchCollectionIndex <= searchCollectionSize
						and: [
							(self at: index + searchCollectionIndex - 1) =
								(aSearchCollection at: searchCollectionIndex)]]
						whileTrue: [searchCollectionIndex := searchCollectionIndex + 1].
					searchCollectionIndex > searchCollectionSize
						ifTrue: [
							copy addAll: aReplacementCollection.
							index := index + searchCollectionSize]
						ifFalse: [
							copy addLast: (self at: index).
							index := index + 1]]
				ifFalse: [ 
					copy addLast: (self at: index).
					index := index + 1]].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 13:04'!
copyReplaceFrom: aStart to: aStop with: aReplacementCollection
	| copy start stop |

	(self size < 1
		or: [self size < aStart])
		ifTrue: [
			^ self copy
				addAll: aReplacementCollection;
				yourself].
	copy := self copyEmpty.
	start := aStart max: 1.
	stop := aStop min: self size.
	1 to: self size do: [:i |
		(i = start)
			ifTrue: [
				copy addAll: aReplacementCollection.
				start > stop
					ifTrue: [copy addLast: (self at: i)]]
			ifFalse: [
				(i > start and: [i <= stop])
					ifFalse: [copy addLast: (self at: i)]]].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 06:21'!
copyUpTo: anObject
	| copy |

	copy := self copyEmpty.
	1
		to:
			(self
				indexOf: anObject
				ifAbsent: [self size + 1]) - 1
		do: [:i | copy addLast: (self at: i)].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 06:21'!
copyUpToLast: anObject
	| copy |

	copy := self copyEmpty.
	1
		to:
			(self
				lastIndexOf: anObject
				ifAbsent: [collection size + 1]) - 1
		do: [:i | copy addLast: (self at: i)].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 13:29'!
copyWith: anObject
	^ self copy
		addLast: anObject;
		yourself! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/17/2015 12:35'!
copyWithFirst: anObject
	^ self copyEmpty
		addLast: anObject;
		addAll: self;
		yourself! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 13:30'!
copyWithout: anObject
	^ self copy
		remove: anObject ifAbsent: [nil];
		yourself! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 13:30'!
copyWithoutAll: aCollection
	| copy |

	copy := self copy.
	aCollection do: [:each | copy remove: each ifAbsent: [nil]].
	^ copy.! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 3/12/2010 06:48'!
defaultValue
	^ nil! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:21'!
detect: aBlock
	^ self detect: aBlock ifNone: [self defaultValue]! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:21'!
detect: aBlock ifNone: anExceptionBlock
	^ collection detect: aBlock ifNone: anExceptionBlock! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 3/12/2015 20:20'!
do: aBlock
	collection do: aBlock! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/22/2014 16:21'!
doWithIndex: aBlock
	^ self withIndexDo: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
eighth
	^ self at: 8! !

!XMLOrderedList methodsFor: 'testing' stamp: 'monty 10/22/2014 16:11'!
endsWith: aCollection
	^ collection endsWith: aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
fifth
	^ self at: 5! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:56'!
first
	^ self at: 1! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:54'!
first: anInteger
	^ self copyFrom: 1 to: anInteger! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
fourth
	^ self at: 4! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/17/2015 08:28'!
grownBy: aSize
	^ self copyClass collection: (collection grownBy: aSize)! !

!XMLOrderedList methodsFor: 'comparing' stamp: 'monty 10/22/2014 02:44'!
hash
	^ collection hash! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:59'!
identityIndexOf: anObject
	^ self identityIndexOf: anObject ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:09'!
identityIndexOf: anObject ifAbsent: aBlock
	^ collection identityIndexOf: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:59'!
indexOf: anObject
	^ self indexOf: anObject ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:07'!
indexOf: anObject ifAbsent: aBlock
	^ collection indexOf: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:08'!
indexOf: anObject startingAt: anIndex 
	^ self indexOf: anObject startingAt: anIndex ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 05:45'!
indexOf: anObject startingAt: anIndex ifAbsent: aBlock
	^ collection
		indexOf: anObject
		startingAt: (anIndex max: 1)
		ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'testing' stamp: 'monty 10/22/2014 16:11'!
isSequenceable
	^ true! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:00'!
last
	^ self at: self size! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:55'!
last: anInteger
	^ self copyFrom: self size - anInteger + 1 to: self size! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:06'!
lastIndexOf: anObject
	^ self lastIndexOf: anObject ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 07:32'!
lastIndexOf: anObject ifAbsent: aBlock
	^ collection lastIndexOf: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
ninth
	^ self at: 9! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/22/2014 16:23'!
pairsDo: aBlock
	collection pairsDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 10/22/2014 16:10'!
readStream
	^ ReadStream on: self! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 1/16/2015 07:23'!
reject: aBlock
	| selected |

	selected := self copyEmpty.
	self do: [:each |
		(aBlock value: each)
			ifFalse: [selected addLast: each]].
	^ selected.! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 1/17/2015 04:11'!
reject: aRejectBlock thenCollect: aCollectBlock
	| collected |

	collected := self species new.
	self do: [:each |
		(aRejectBlock value: each)
			ifFalse: [collected addLast: (aCollectBlock value: each)]].
	^ collected.! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:33'!
remove: anObject ifAbsent: aBlock
	^ collection remove: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 16:00'!
removeAll
	[self isEmpty] whileFalse: [self removeLast]! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 15:46'!
removeAt: anIndex
	^ collection removeAt: anIndex! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:25'!
removeFirst
	^ self removeAt: 1! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:32'!
removeFirst: aNumber
	| removed |

	removed := self copyFrom: 1 to: aNumber.
	aNumber timesRepeat: [self removeFirst].
	^ removed.! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:24'!
removeLast
	^ self removeAt: self size! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:33'!
removeLast: aNumber
	| removed |

	removed := self copyFrom: self size - (aNumber - 1) to: self size.
	aNumber timesRepeat: [self removeLast].
	^ removed.! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:02'!
replaceFrom: aStart to: aStop with: aReplacementCollection
	^ self
		replaceFrom: aStart
		to: aStop
		with: aReplacementCollection
		startingAt: 1! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:04'!
replaceFrom: aStart to: aStop with: aReplacementCollection startingAt: aReplacementStart
	| offset |

	offset := 0.
	[aStart + offset <= aStop]
		whileTrue: [
			self
				at: aStart + offset
				put: (aReplacementCollection at: aReplacementStart + offset).
			offset := offset + 1].! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 10/19/2014 06:31'!
reset
	collection reset! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 10/22/2014 16:10'!
reverse
	^ self reversed! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 13:35'!
reverseDo: aBlock
	collection reverseDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 1/17/2015 12:33'!
reversed
	| reversed |

	reversed := self copyEmpty.
	self reverseDo: [:each | reversed addLast: each].
	^ reversed.! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
second
	^ self at: 2! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 1/16/2015 07:24'!
select: aBlock
	| selected |

	selected := self copyEmpty.
	self do: [:each |
		(aBlock value: each)
			ifTrue: [selected addLast: each]].
	^ selected.! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 1/17/2015 04:11'!
select: aSelectBlock thenCollect: aCollectBlock
	| collected |

	collected := self species new.
	self do: [:each |
		(aSelectBlock value: each)
			ifTrue: [collected addLast: (aCollectBlock value: each)]].
	^ collected.! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 8/4/2010 13:38'!
setCollection: aCollection
	collection := aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
seventh
	^ self at: 7! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
sixth
	^ self at: 6! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:49'!
size
	^ collection size! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 10/22/2014 16:18'!
sort
	self sort: [:a :b | a <= b]! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 2/2/2015 01:33'!
sort: aSortBlock
	collection sort: aSortBlock! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 1/18/2015 09:31'!
sorted
	^ self copy sort! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 10/19/2014 08:07'!
sorted: aSortBlock
	^ self copy sort: aSortBlock! !

!XMLOrderedList methodsFor: 'private' stamp: 'monty 10/28/2014 15:07'!
species
	^ XMLOrderedList! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 2/1/2015 23:47'!
swap: aFirstIndex with: aSecondIndex
	| temp |

	temp := collection at: aFirstIndex.
	collection
		at: aFirstIndex
		put: (collection at: aSecondIndex).
	collection
		at: aSecondIndex
		put: temp.! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
third
	^ self at: 3! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 1/17/2015 04:10'!
with: aCollection collect: aBlock
	| collected |

	collected := self species new: self size.
	1 to: (self size min: aCollection size) do: [:i |
		collected addLast:
			(aBlock
				value: (self at: i)
				value: (aCollection at: i))].
	^ collected.! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/23/2014 04:54'!
with: aCollection do: aBlock
	1 to: (self size min: aCollection size) do: [:i |
		(aBlock
			value: (self at: i)
			value: (aCollection at: i))]! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 1/17/2015 04:10'!
withIndexCollect: aTwoArgumentBlock
	| collected |

	collected := self species new: self size.
	1 to: self size do: [:i |
		collected addLast:
			(aTwoArgumentBlock
				value: (self at: i)
				value: i)].
	^ collected.! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/22/2014 16:47'!
withIndexDo: aBlock
	collection withIndexDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 10/22/2014 16:10'!
writeStream
	^ WriteStream on: self! !

!SequenceableCollection methodsFor: '*XML-Parser' stamp: 'monty 10/19/2014 06:23'!
lastOrNil
	"for speed to avoid at:ifAbsent: closure"
	| lastOrNilOffset |

	(lastOrNilOffset := self size) > 0
		ifTrue: [^ self at: lastOrNilOffset]
		ifFalse: [^ nil]! !

!Stream methodsFor: '*XML-Parser' stamp: 'monty 12/11/2014 06:32'!
parseXML
	^ XMLDOMParser parse: self! !

Object subclass: #DTDAttributeDefinitionParser
	instanceVariableNames: 'element attribute type defaultValueValidator'
	classVariableNames: 'DefaultValueValidators NonEnumerationValidators'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDAttributeDefinitionParser commentStamp: '<historical>' prior: 0!
This class parses an attribute definition in an ATTLIST declaration and creates a validator for it.!

!DTDAttributeDefinitionParser class methodsFor: 'instance creation' stamp: 'monty 12/10/2014 07:14'!
element: anElement attribute: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	^ self new
		setElement: anElement
		attribute: anAttribute
		type: aType
		defaultValueValidator:
			((DefaultValueValidators
				at: aDefaultPragma
				ifAbsent: [XMLDefaultValueAttributeValidator])
					attribute: anAttribute
					defaultValue: aDefaultValue).! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 1/12/2015 00:30'!
initialize
	"self initialize"

	self
		initializeDefaultValueValidators;
		initializeNonEnumerationValidators! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 12/31/2013 02:53'!
initializeDefaultValueValidators
	(DefaultValueValidators := Dictionary new)
		at: 'FIXED' put: XMLFixedValueAttributeValidator;
		at: 'IMPLIED' put: XMLImpliedValueAttributeValidator;
		at: 'REQUIRED' put: XMLRequiredValueAttributeValidator! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 12/11/2014 10:37'!
initializeNonEnumerationValidators
	(NonEnumerationValidators := Dictionary new)
		at: 'CDATA' put: XMLCDataAttributeValidator;
		at: 'ID' put: XMLIDAttributeValidator;
		at: 'IDREF' put: XMLIDRefAttributeValidator;
		at: 'IDREFS' put: XMLIDRefsAttributeValidator;
		at: 'ENTITY' put: XMLEntityAttributeValidator;
		at: 'ENTITIES' put: XMLEntitiesAttributeValidator;
		at: 'NMTOKEN' put: XMLNmtokenAttributeValidator;
		at: 'NMTOKENS' put: XMLNmtokensAttributeValidator! !

!DTDAttributeDefinitionParser methodsFor: 'parsing' stamp: 'monty 12/11/2014 09:40'!
parse
	| attributeValidator |

	attributeValidator :=
		self validatorClassForType
			element: element
			attribute: attribute
			defaultValueValidator: defaultValueValidator.
	attributeValidator isEnumerationValidator
		ifTrue: [attributeValidator setAllowedValuesFromString: type].

	^ attributeValidator.! !

!DTDAttributeDefinitionParser methodsFor: 'initialization' stamp: 'monty 12/10/2014 07:14'!
setElement: anElement attribute: anAttribute type: aType defaultValueValidator: aDefaultValueValidator
	element := anElement.
	attribute := anAttribute.
	type := aType.
	defaultValueValidator := aDefaultValueValidator.! !

!DTDAttributeDefinitionParser methodsFor: 'private ' stamp: 'monty 12/11/2014 04:57'!
validatorClassForType
	^ NonEnumerationValidators
		at: type
		ifAbsent: [
			type first == $N
				ifTrue: [XMLNotationEnumerationAttributeValidator]
				ifFalse: [XMLEnumerationAttributeValidator]]! !

Object subclass: #DTDContentModelParser
	instanceVariableNames: 'element contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDContentModelParser commentStamp: '<historical>' prior: 0!
This is a base class for DTD content model parsers that turn element declarations into validators.!

!DTDContentModelParser class methodsFor: 'instance creation' stamp: 'monty 12/21/2013 03:14'!
element: aName contentModel: aContentModel
	^ self new
		setElement: aName
		contentModel: aContentModel! !

!DTDContentModelParser class methodsFor: 'parsing' stamp: 'monty 12/21/2013 20:00'!
parserForContentModel: aContentModel
	"the tokenizer removes all spaces from the content model"
	^ (aContentModel notEmpty and: [aContentModel first == $(])
		ifTrue: [
			(aContentModel size > 1 and: [aContentModel second == $#])
				ifTrue: [DTDMixedContentModelParser]
				ifFalse: [DTDDFAContentModelParser]]
		ifFalse: [DTDLiteralContentModelParser]! !

!DTDContentModelParser methodsFor: 'parsing' stamp: 'monty 12/21/2013 19:57'!
parse
	self subclassResponsibility! !

!DTDContentModelParser methodsFor: 'initialization' stamp: 'monty 12/21/2013 03:13'!
setElement: aName contentModel: aContentModel
	element := aName.
	contentModel := aContentModel.! !

DTDContentModelParser subclass: #DTDDFAContentModelParser
	instanceVariableNames: 'operatorStack nfaBuilder'
	classVariableNames: 'OperatorApplicators SpecialCharacterParsers'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDFAContentModelParser commentStamp: 'monty 12/29/2013 04:00' prior: 0!
This class parses a content model using postfix conversion and an NFABuilder to build a NFA before converting it to a DFA.!

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 1/12/2015 00:31'!
initialize
	"self initialize"

	self
		initializeSpecialCharacterParsers;
		initializeOperatorApplicators! !

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 12/8/2014 15:48'!
initializeOperatorApplicators
	"use Array for faster lookup"
	(OperatorApplicators := Array new: 256)
		at: $, asciiValue put: #applyConcatenation;
		at: $| asciiValue put: #applyBranch;
		at: $? asciiValue put: #applyOptional;
		at: $* asciiValue put: #applyStar;
		at: $+ asciiValue put: #applyPlus! !

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 12/8/2014 15:48'!
initializeSpecialCharacterParsers
	"use Array for faster lookup"
	(SpecialCharacterParsers := Array new: 256)
		at: $( asciiValue put: #parseParen:;
		at: $) asciiValue put: #parseParen:;
		at: $, asciiValue put: #parseBinaryOperator:;
		at: $| asciiValue put: #parseBinaryOperator:;
		at: $? asciiValue put: #parseUnaryOperator:;
		at: $* asciiValue put: #parseUnaryOperator:;
		at: $+ asciiValue put: #parseUnaryOperator:! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/28/2013 19:46'!
applyBinaryOperatorOnStack
	nfaBuilder totalFragments > 1
		ifFalse: [self errorMissingOperand].

	self applyOperator: operatorStack removeLast.! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/30/2013 22:29'!
applyOperator: aCharacter
	nfaBuilder perform: (OperatorApplicators at: aCharacter asciiValue)! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/18/2013 22:44'!
errorMismatchedParens
	XMLValidationException signal: 'Mismatched parentheses in content model'! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/18/2013 22:44'!
errorMissingOperand
	XMLValidationException signal: 'Missing element name in content model'! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 11/2/2014 09:17'!
hasOperatorOnStack
	^ (operatorStack isEmpty or: [operatorStack last == $(]) not! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/18/2013 22:44'!
isBinaryOperator: aFirstOperator higherPrecedenceThan: aSecondOperator
	^ aFirstOperator == $, and: [aSecondOperator == $|]! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 1/16/2014 05:09'!
isSpecialCharacter: aCharacter
	^ (self parserForSpecialCharacter: aCharacter) notNil! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/9/2014 06:35'!
parse
	"the tokenizer removes all spaces from the content model"
	| nameStart |

	operatorStack := OrderedCollection new.
	nfaBuilder := XMLNFABuilder new.
	1 to: contentModel size do: [:i |
		(self parserForSpecialCharacter: (contentModel at: i))
				ifNil: [
					nameStart
						ifNil: [nameStart := i]]
				ifNotNil: [:selector |
					nameStart
						ifNotNil: [
							nfaBuilder pushMatcher:
								(contentModel copyFrom: nameStart to: i - 1).
							nameStart := nil].
					self
						perform: selector
						with: (contentModel at: i)]].
	self popRemainingOperators.

	^ XMLDFAElementValidator
		element: element
		dfa: nfaBuilder firstStateOfAcceptingNFA asDFAState.! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/29/2013 02:23'!
parseBinaryOperator: aCharacter
	self hasOperatorOnStack
		ifTrue: [
			(self
				isBinaryOperator: aCharacter
				higherPrecedenceThan: operatorStack last)
					ifFalse: [self applyBinaryOperatorOnStack]].

	operatorStack addLast: aCharacter.! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/29/2013 02:24'!
parseParen: aCharacter
	aCharacter == $)
		ifTrue: [self popParenthesizedOperators]
		ifFalse: [operatorStack addLast: $(].! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/29/2013 02:23'!
parseUnaryOperator: aCharacter
	nfaBuilder hasFragments
		ifFalse: [self errorMissingOperand].

	self applyOperator: aCharacter.! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/8/2014 15:43'!
parserForSpecialCharacter: aCharacter
	| asciiValue |

	((asciiValue := aCharacter asciiValue) > 0
		and: [asciiValue <= SpecialCharacterParsers size])
			ifTrue: [^ SpecialCharacterParsers at: asciiValue]
			ifFalse: [^ nil]! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/28/2013 23:02'!
popParenthesizedOperators
	[operatorStack notEmpty]
		whileTrue: [
			operatorStack last == $(
				ifTrue: [
					operatorStack removeLast.
					^ self]
				ifFalse: [self applyBinaryOperatorOnStack]].

	self errorMismatchedParens.! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/18/2013 22:44'!
popRemainingOperators
	[operatorStack notEmpty]
		whileTrue: [
			operatorStack last == $(
				ifTrue: [self errorMismatchedParens].
			self applyBinaryOperatorOnStack].

	nfaBuilder totalFragments = 1
		ifFalse: [self errorMissingOperand].! !

DTDContentModelParser subclass: #DTDLiteralContentModelParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDLiteralContentModelParser commentStamp: 'monty 2/2/2014 06:12' prior: 0!
Parser for ANY and EMPTY elements.!

!DTDLiteralContentModelParser methodsFor: 'parsing' stamp: 'monty 12/10/2014 07:20'!
parse
	contentModel = 'ANY'
		ifTrue: [^ XMLAnyElementValidator element: element]
		ifFalse: [^ XMLEmptyElementValidator element: element]! !

DTDContentModelParser subclass: #DTDMixedContentModelParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDMixedContentModelParser commentStamp: 'monty 2/2/2014 06:13' prior: 0!
This class parses mixed content declarations like "(#PCDATA|one|two)" using a DTDMixedContentListParser.!

!DTDMixedContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 00:44'!
parse
	^ XMLMixedContentElementValidator
		element: element
		allowedElements: self parseAllowedElements! !

!DTDMixedContentModelParser methodsFor: 'private' stamp: 'monty 12/11/2014 04:50'!
parseAllowedElements
	^ (DTDEnumerationListParser on: contentModel) parse
		remove: '#PCDATA';
		yourself! !

Object subclass: #DTDDoctypeDefinition
	instanceVariableNames: 'root elementValidators attributeValidators generalEntities parameterEntities unparsedEntities notations elementIDValidator unparsedEntityValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDoctypeDefinition commentStamp: 'monty 4/6/2015 08:06' prior: 0!
This class represents the parsed internal and external subset of a DTD taken together.!

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 3/22/2015 16:23'!
addExternalGeneralEntity: aName resolver: aResolver uri: aSystemURI from: aDocumentOrEntityURI
	self generalEntities
		at: aName
		ifAbsentPut: [
			aResolver
				resolveExternalGeneralEntity: aName
				uri: aSystemURI
				from: aDocumentOrEntityURI]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 3/22/2015 16:22'!
addExternalParameterEntity: aName resolver: aResolver uri: aSystemURI from: aDocumentOrEntityURI
	self parameterEntities
		at: aName
		ifAbsentPut: [
			aResolver
				resolveExternalParameterEntity: aName
				uri: aSystemURI
				from: aDocumentOrEntityURI]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 3/22/2015 16:22'!
addGeneralEntity: aName replacement: aReplacement from: aDocumentOrEntityURI
	self generalEntities
		at: aName
		ifAbsentPut: [
			DTDGeneralEntity 
				name: aName
				replacement: aReplacement
				uri: aDocumentOrEntityURI]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 11/14/2013 23:59'!
addNotation: aName
	self notations add: aName! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 3/22/2015 16:23'!
addParameterEntity: aName replacement: aReplacement from: aDocumentOrEntityURI
	self parameterEntities
		at: aName
		ifAbsentPut: [
			DTDParameterEntity
				name: aName
				replacement: aReplacement
				uri: aDocumentOrEntityURI]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:49'!
addUnparsedEntity: aName ndata: aNotation
	self unparsedEntities
		at: aName
		ifAbsentPut: [aNotation]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/1/2015 20:54'!
addValidatorForAttributeIn: anElement named: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	| attributeValidatorsForElement |

	attributeValidatorsForElement :=
		self attributeValidators
			at: anElement
			ifAbsentPut: [Dictionary new].
	(attributeValidatorsForElement includesKey: aName)
		ifTrue: [^ self].

	attributeValidatorsForElement
		at: aName
		put: 
			((DTDAttributeDefinitionParser
				element:  anElement
				attribute: aName
				type: aType
				defaultPragma: aDefaultPragma
				defaultValue: aDefaultValue) parse
					initializeWithValidatorsFrom: self).! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 12/21/2013 03:14'!
addValidatorForElement: anElement contentModel: aContentModel
	self elementValidators
		at: anElement
		ifAbsentPut: [
			((DTDContentModelParser parserForContentModel: aContentModel)
				element: anElement
				contentModel: aContentModel) parse]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/19/2013 01:01'!
attributeValidators
	^ attributeValidators ifNil: [attributeValidators := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 4/5/2015 21:15'!
clearIDsAndIDReferences
	self hasElementIDValidator
		ifTrue: [self elementIDValidator clearIDsAndIDReferences]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 3/21/2015 23:03'!
elementIDValidator
	^ elementIDValidator ifNil: [elementIDValidator := XMLElementIDValidator new]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/18/2013 19:26'!
elementValidators
	^ elementValidators ifNil: [elementValidators := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 20:51'!
generalEntities
	^ generalEntities ifNil: [generalEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/3/2015 03:11'!
generalEntityAt: aName
	^ self
		generalEntityAt: aName
		ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/3/2015 03:11'!
generalEntityAt: aName ifAbsent: aBlock
	^ self generalEntities
		at: aName
		ifAbsent: aBlock! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/18/2013 19:45'!
hasAttributeValidators
	^ attributeValidators notNil and: [attributeValidators notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/21/2015 23:03'!
hasElementIDValidator
	^ elementIDValidator notNil! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/18/2013 19:45'!
hasElementValidators
	^ elementValidators notNil and: [elementValidators notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/8/2015 14:22'!
hasEntities
	^ self hasGeneralEntities
		or: [self hasParameterEntities
			or: [self hasUnparsedEntities]]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/8/2015 14:12'!
hasGeneralEntities
	^ generalEntities notNil and: [generalEntities notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/8/2015 14:12'!
hasNotations
	^ notations notNil and: [notations notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/8/2015 14:13'!
hasParameterEntities
	^ parameterEntities notNil and: [parameterEntities notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/8/2015 14:22'!
hasUnparsedEntities
	^ unparsedEntities notNil and: [unparsedEntities notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/8/2015 14:20'!
hasValidators
	^ (attributeValidators notNil
		and: [attributeValidators notEmpty])
			or: [elementValidators notNil
				and: [elementValidators notEmpty]]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/15/2013 00:00'!
includesNotation: aName
	^ self notations includes: aName! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 23:59'!
notations
	^ notations ifNil: [notations := Set new]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 20:51'!
parameterEntities
	^ parameterEntities ifNil: [parameterEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/3/2015 03:11'!
parameterEntityAt: aName
	^ self
		parameterEntityAt: aName
		ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/3/2015 03:11'!
parameterEntityAt: aName ifAbsent: aBlock
	^ self parameterEntities
		at: aName
		ifAbsent: aBlock! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:56'!
root
	^ root ifNil: [root := '']! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 3/8/2015 09:42'!
root: aRootElement
	root := aRootElement! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 23:50'!
unparsedEntities
	^ unparsedEntities ifNil: [unparsedEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/1/2015 20:48'!
unparsedEntityValidator
	^ unparsedEntityValidator
		ifNil: [
			unparsedEntityValidator :=
				XMLUnparsedEntityValidator
					unparsedEntities: self unparsedEntities
					notations: self notations]! !

Object subclass: #DTDEntityConfiguration
	instanceVariableNames: 'replacesContentEntityReferences resolvesExternalEntities preservesUndeclaredEntityReferences externalEntityResolver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDEntityConfiguration commentStamp: 'monty 1/3/2015 04:39' prior: 0!
This class contains configurations for entity replacement/retrieval. It is separate from XMLConfiguration so it is only lazy initialized when needed and saves memory.!

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:28'!
externalEntityResolver
	^ externalEntityResolver ifNil: [externalEntityResolver := DTDCachingExternalEntityResolver new]! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:22'!
externalEntityResolver: anExternalEntityResolver
	externalEntityResolver := anExternalEntityResolver! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 04:17'!
preservesUndeclaredEntityReferences
	^ preservesUndeclaredEntityReferences ifNil: [preservesUndeclaredEntityReferences := false]! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 04:13'!
preservesUndeclaredEntityReferences: aBoolean
	preservesUndeclaredEntityReferences := aBoolean! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 09:37'!
replacesContentEntityReferences
	^ replacesContentEntityReferences ifNil: [replacesContentEntityReferences := true]! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 09:37'!
replacesContentEntityReferences: aBoolean
	replacesContentEntityReferences := aBoolean! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:22'!
resolvesExternalEntities
	^ resolvesExternalEntities ifNil: [resolvesExternalEntities := false]! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:22'!
resolvesExternalEntities: aBoolean
	resolvesExternalEntities := aBoolean! !

Object subclass: #DTDEnumerationListParser
	instanceVariableNames: 'listString listValues startIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDEnumerationListParser commentStamp: 'monty 12/8/2014 15:21' prior: 0!
This is a parser of lists in the form of "(one|two|three)"
(Assumes spaces have been removed)!

!DTDEnumerationListParser class methodsFor: 'instance creation' stamp: 'monty 10/30/2014 12:58'!
on: aString
	^ self new on: aString! !

!DTDEnumerationListParser methodsFor: 'private' stamp: 'monty 12/11/2014 05:50'!
addListValue: aListValue
	| oldSize |

	"checking the size avoids includes: hash lookup"
	oldSize := listValues size.
	listValues add: aListValue.
	(listValues size > oldSize)
		ifFalse: [self errorDuplicateListValue: aListValue].! !

!DTDEnumerationListParser methodsFor: 'private' stamp: 'monty 10/31/2014 21:20'!
errorDuplicateListValue: aValue
	XMLValidationException
		formatSignal: 'Duplicate value "{1}" in list'
		with: aValue! !

!DTDEnumerationListParser methodsFor: 'initialization' stamp: 'monty 12/11/2014 04:51'!
on: aString
	listString := aString.
	listValues := Set new.
	startIndex := 2.! !

!DTDEnumerationListParser methodsFor: 'parsing' stamp: 'monty 12/11/2014 05:50'!
parse
	"the tokenizer removes all spaces from lists"
	| valueStartIndex nextChar |

	valueStartIndex := startIndex.
	startIndex to: listString size do: [:i | 
		((nextChar := listString at: i) == $)
			or: [nextChar == $|])
			ifTrue: [
				self addListValue:
					(listString
						copyFrom: valueStartIndex
						to: i - 1).
				nextChar == $)
					ifTrue: [^ listValues].
				valueStartIndex := i + 1]].
	^ listValues.! !

DTDEnumerationListParser subclass: #DTDNotationEnumerationListParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDNotationEnumerationListParser commentStamp: 'monty 1/25/2014 03:36' prior: 0!
This class parses notation attribute value lists.!

!DTDNotationEnumerationListParser methodsFor: 'initialization' stamp: 'monty 12/11/2014 04:55'!
on: aString
	super on: aString.
	startIndex := 11. "'NOTATION (' size + 1"! !

Object subclass: #DTDExternalEntityLimits
	instanceVariableNames: 'externalSubsetReadLimit maxExternalEntities maxExternalEntitySize resolvesLocalFromNonLocal'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntityLimits commentStamp: 'monty 1/25/2014 04:01' prior: 0!
This class places security limits on the number and size of external parsed entities. You can set these using the configuration messages in DTDExternalEntityResolver or remove them using removeLimits.!

!DTDExternalEntityLimits class methodsFor: 'accessing' stamp: 'monty 1/27/2015 22:48'!
defaultExternalSubsetReadLimit
	^ XMLParserLimits defaultReadLimit
		ifNotNil: [:limit | (limit / 4) asInteger]! !

!DTDExternalEntityLimits class methodsFor: 'accessing' stamp: 'monty 12/9/2014 07:57'!
defaultMaxExternalEntitySize
	^ 2500000! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/27/2015 22:46'!
externalSubsetReadLimit
	^ externalSubsetReadLimit! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/27/2015 22:46'!
externalSubsetReadLimit: aLimit
	externalSubsetReadLimit := aLimit! !

!DTDExternalEntityLimits methodsFor: 'initialization' stamp: 'monty 1/27/2015 22:46'!
initialize
	super initialize.

	externalSubsetReadLimit := self class defaultExternalSubsetReadLimit.
	maxExternalEntities := 16.
	maxExternalEntitySize := self class defaultMaxExternalEntitySize.
	resolvesLocalFromNonLocal := false.! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntities
	^ maxExternalEntities! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntities: aLimit
	maxExternalEntities := aLimit! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntitySize
	^ maxExternalEntitySize! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntitySize: aLimit
	maxExternalEntitySize := aLimit! !

!DTDExternalEntityLimits methodsFor: 'removing' stamp: 'monty 1/27/2015 22:47'!
removeAll
	externalSubsetReadLimit := nil.
	maxExternalEntities := nil.
	maxExternalEntitySize := nil.
	resolvesLocalFromNonLocal := nil.! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/24/2014 06:47'!
resolvesLocalFromNonLocal
	^ resolvesLocalFromNonLocal! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/24/2014 06:47'!
resolvesLocalFromNonLocal: aBoolean
	resolvesLocalFromNonLocal := aBoolean! !

Object subclass: #DTDExternalEntityResolver
	instanceVariableNames: 'externalEntityLimits totalResolvedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntityResolver commentStamp: 'monty 1/23/2014 05:48' prior: 0!
This class resolves external entity references using XMLURI, XMLHTTPRequest, and XMLFileHandle.!

DTDExternalEntityResolver subclass: #DTDCachingExternalEntityResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

DTDCachingExternalEntityResolver class
	instanceVariableNames: 'entityCache'!

!DTDCachingExternalEntityResolver commentStamp: 'monty 12/18/2014 19:25' prior: 0!
This resolver can cache entities so they don't have to be retrieved again (like over HTTP). (The default resolver)!

DTDCachingExternalEntityResolver class
	instanceVariableNames: 'entityCache'!

!DTDCachingExternalEntityResolver class methodsFor: 'clearing' stamp: 'monty 11/15/2014 20:59'!
clearEntityCache
	"self clearEntityCache"
	self entityCache clear! !

!DTDCachingExternalEntityResolver class methodsFor: 'private' stamp: 'monty 11/15/2014 20:56'!
entityCache
	^ entityCache ifNil: [entityCache := XMLKeyValueCache maxSize: 64]! !

!DTDCachingExternalEntityResolver class methodsFor: 'private' stamp: 'monty 3/21/2015 22:36'!
entityCacheAt: aURI ifAbsentPut: aBlock
	^ self entityCache
		at: aURI
		ifAbsentPut: aBlock! !

!DTDCachingExternalEntityResolver class methodsFor: 'private' stamp: 'monty 3/21/2015 22:36'!
entityCacheAt: aURI put: anEntity
	^ self entityCache
		at: aURI
		put: anEntity.! !

!DTDCachingExternalEntityResolver class methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:57'!
maxCachedEntities
	^ self entityCache maxSize! !

!DTDCachingExternalEntityResolver class methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:56'!
maxCachedEntities: anInteger
	self entityCache maxSize: anInteger! !

!DTDCachingExternalEntityResolver methodsFor: 'clearing' stamp: 'monty 1/13/2014 03:45'!
clearEntityCache
	self class clearEntityCache! !

!DTDCachingExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 1/13/2014 17:52'!
maxCachedEntities
	^ self class maxCachedEntities! !

!DTDCachingExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 1/13/2014 17:53'!
maxCachedEntities: anInteger
	self class maxCachedEntities: anInteger! !

!DTDCachingExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 3/21/2015 22:37'!
resolveEntityURI: aURI
	| uri |

	^ (uri := aURI asXMLURI) isLocalURI
		ifTrue: [super resolveEntityURI: uri]
		ifFalse: [
			(self class
				entityCacheAt: uri
				ifAbsentPut: [(super resolveEntityURI: uri) contents])
					readStream]! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 10/31/2014 21:03'!
checkExternalEntityLimit
	(self maxExternalEntities notNil
		and: [self totalResolvedEntities + 1 > self maxExternalEntities])
		ifTrue: [self erorrExternalEntityLimitExceeded]! !

!DTDExternalEntityResolver methodsFor: 'combining' stamp: 'monty 3/22/2015 03:50'!
combineURI: aBaseURI with: aRelativeURI
	| baseURI combinedURI |

	baseURI := XMLURI fromString: aBaseURI.
	combinedURI := baseURI / aRelativeURI.
	(combinedURI isLocalURI
		and: [baseURI isLocalURI not
			and: [self resolvesLocalFromNonLocal notNil
				and: [self resolvesLocalFromNonLocal not]]])
		ifTrue: [self errorLocalFromNonLocal].

	^ combinedURI.! !

!DTDExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 1/22/2014 18:55'!
defaultDocumentURI
	^ XMLFileHandle currentWorkingDirectory, '/default'! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 11/6/2014 12:21'!
erorrExternalEntityLimitExceeded
	XMLLimitException signal: 'External entity limit exceeded; use #maxExternalEntities: to change'! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 10/31/2014 21:33'!
errorLocalFromNonLocal
	XMLLimitException signal:
		'Local URI referenced from non-local URI; use #resolvesLocalFromNonLocal: to change'! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:15'!
externalEntityLimits
	^ externalEntityLimits ifNil: [externalEntityLimits := DTDExternalEntityLimits new]! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/27/2015 22:53'!
externalSubsetReadLimit
	^ self externalEntityLimits externalSubsetReadLimit! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 3/17/2015 17:37'!
externalSubsetReadLimit: aLimit
	"change the external subset read limit or nil to disable"

	self externalEntityLimits externalSubsetReadLimit: aLimit! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 1/23/2014 05:45'!
incrementEntityTotal
	totalResolvedEntities := (totalResolvedEntities ifNil: [0]) + 1! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:17'!
maxExternalEntities
	^ self externalEntityLimits maxExternalEntities! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:54'!
maxExternalEntities: aLimit
	self externalEntityLimits maxExternalEntities: aLimit! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:20'!
maxExternalEntitySize
	^ self externalEntityLimits maxExternalEntitySize! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:54'!
maxExternalEntitySize: aLimit
	self externalEntityLimits maxExternalEntitySize: aLimit! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:16'!
removeLimits
	self externalEntityLimits removeAll! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 3/21/2015 22:36'!
resolveEntityURI: aURI
	| readStream |

	self checkExternalEntityLimit.
	readStream := aURI asXMLURI getUpToLimit: self maxExternalEntitySize.
	self incrementEntityTotal.

	^ readStream.! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 3/22/2015 03:51'!
resolveExternalEntity: aName uri: anEntityURI from: aSourceURI class: aClass
	| combinedURI replacement replacementStream |

	combinedURI := self combineURI: aSourceURI with: anEntityURI.

	replacementStream := self resolveEntityURI: combinedURI.
	replacement  := replacementStream contents.
	replacementStream close.

	^ aClass
		name: aName
		replacement: replacement
		uri: combinedURI asString.! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 3/22/2015 03:10'!
resolveExternalGeneralEntity: aName uri: anEntityURI from: aSourceURI
	^ self
		resolveExternalEntity: aName
		uri: anEntityURI
		from: aSourceURI
		class: DTDExternalGeneralEntity! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 3/22/2015 03:10'!
resolveExternalParameterEntity: aName uri: anEntityURI from: aSourceURI
	^ self
		resolveExternalEntity: aName
		uri: anEntityURI
		from: aSourceURI
		class: DTDExternalParameterEntity! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/24/2014 06:50'!
resolvesLocalFromNonLocal
	^ self externalEntityLimits resolvesLocalFromNonLocal! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/24/2014 06:50'!
resolvesLocalFromNonLocal: aBoolean
	self externalEntityLimits resolvesLocalFromNonLocal: aBoolean! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 1/16/2014 06:40'!
totalResolvedEntities
	^ totalResolvedEntities ifNil: [totalResolvedEntities := 0]! !

Object subclass: #DTDParsedEntity
	instanceVariableNames: 'name replacement uri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDParsedEntity commentStamp: '<historical>' prior: 0!
This is a base class for internal and external parsed entities with replacements that can be inserted into an XML document.!

DTDParsedEntity subclass: #DTDGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDGeneralEntity commentStamp: 'monty 1/5/2014 00:54' prior: 0!
This class is for general parsed entities with replacements.
!

DTDGeneralEntity subclass: #DTDExternalGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalGeneralEntity commentStamp: 'monty 10/7/2014 02:19' prior: 0!
A general entity with an external replacement.!

!DTDExternalGeneralEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:57'!
isExternal
	^ true! !

!DTDGeneralEntity methodsFor: 'replacing' stamp: 'monty 11/16/2014 21:00'!
constructReplacementOn: aTokenizer
	aTokenizer constructReplacementForGeneralEntity: self! !

!DTDGeneralEntity methodsFor: 'accessing' stamp: 'monty 1/2/2015 19:59'!
referencePrefix
	^ '&'! !

DTDParsedEntity subclass: #DTDParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDParameterEntity commentStamp: 'monty 11/20/2014 04:05' prior: 0!
This class is for parameter entities with replacements.!

DTDParameterEntity subclass: #DTDExternalParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalParameterEntity commentStamp: 'monty 10/7/2014 02:19' prior: 0!
A parameter entity with an external replacement.!

!DTDExternalParameterEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:58'!
isExternal
	^ true! !

!DTDParameterEntity methodsFor: 'replacing' stamp: 'monty 11/16/2014 21:00'!
constructReplacementOn: aTokenizer
	aTokenizer constructReplacementForParameterEntity: self! !

!DTDParameterEntity methodsFor: 'accessing' stamp: 'monty 1/2/2015 19:59'!
referencePrefix
	^ '%'! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 11/16/2013 06:02'!
name: aString replacement: aValueString
	^ self new
		setName: aString
		replacement: aValueString
		uri: nil! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:36'!
name: aString replacement: aValueString uri: aURI
	^ self new
		setName: aString
		replacement: aValueString
		uri: aURI! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:37'!
named: aString
	^ self
		name: aString
		replacement: ''! !

!DTDParsedEntity methodsFor: 'converting' stamp: 'monty 1/2/2015 19:51'!
asReference
	^ self printString! !

!DTDParsedEntity methodsFor: 'replacing' stamp: 'monty 11/16/2014 21:00'!
constructReplacementOn: aTokenizer
	self subclassResponsibility! !

!DTDParsedEntity methodsFor: 'testing' stamp: 'monty 4/5/2015 07:55'!
hasReplacement
	^ self replacement notEmpty! !

!DTDParsedEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:56'!
isExternal
	^ false! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/13/2013 23:45'!
name
	^ name ifNil: [name := '']! !

!DTDParsedEntity methodsFor: 'printing' stamp: 'monty 1/4/2015 00:41'!
printOn: aStream
	aStream
		nextPutAll: self referencePrefix;
		nextPutAll: self name;
		nextPut: $;! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 1/2/2015 19:55'!
referencePrefix
	self subclassResponsibility! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/13/2013 19:15'!
replacement
	^ replacement ifNil: [replacement := '']! !

!DTDParsedEntity methodsFor: 'initialization' stamp: 'monty 3/21/2015 22:36'!
setName: aName replacement: aReplacement uri: aURI
	name := aName.
	replacement := aReplacement.
	uri := aURI.! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/16/2013 02:55'!
uri
	^ uri ifNil: [uri := '']! !

Object subclass: #SAXHandler
	instanceVariableNames: 'input documentURI configuration isInContent isInCDataSection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXHandler commentStamp: 'monty 4/6/2015 07:14' prior: 0!
This class is an XML parser that uses event handling. The acronym "SAX" refers to Java's "Simple API for XML," on which SAXHandler's API is based. To use this class, create a subclass and override handlers under the handling "content" and "lexical" categories as needed. By default, namespace support and validation are enabled (but not external entity resolution).

To instantiate a SAXHandler, send any of the #on: or #parse- messages to a subclass.

The class-side #parse- messages take some input source and parse it immediately, returning the result:
	SAXHandlerSubclass parse: xml.
	SAXHandlerSubclass parseURL: aUrl. 
	SAXHandlerSubclass parse: xml usingNamespaces: false

The #on: messages create new parsers on the given input string, stream, URL, or file and return it; the instance can then be further configured and ultimately sent #parseDocument to parse the input:
	(SAXHandlerSubclass on: xml)
		isValidating: true;
		resolvesExternalEntities: true;
		parseDocument.

#interruptParsing can be sent from within a handler to stop parsing before end, and there is also #parseDocumentWhile: and #parseDocumentUntil: 
	(SAXHandlerSubclass on: xml)
		parseDocumentWhile: [self shouldKeepParsing].

To parse incrementally, send #parser to an instance to get the underlying XMLParser object and send it #parseToken repeatedly:
	(handler := SAXHandlerSubclass on: xml)
		isValidating: false; 
		preservesUndeclaredEntityReferences: true.
	parser := handler parser.

	"Only parse the first 10 tokens:"
	10 timesRepeat: [parser parseToken].

There are security limits on input you can remove with #removeLimits or change with messages like #documentReadLimit:
	(SAXHandlerSubclass on: xml)
		removeLimits;
		documentReadLimit: newReadLimit;
		maxEntityReplacementDepth: newMaxEntityDepth;
		parseDocument.
		
#optimizeForLargeDocuments  should be used when parsing large documents if you don't care for validating or namespaces.!

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 4/7/2015 06:07'!
on: aStringOrStream
	"Returns new parser on the argument string or stream but doesn't parse"

	^ self new on: aStringOrStream! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 4/7/2015 06:07'!
on: aStringOrStream documentURI: aURIString
	"Returns new parser with associated base URI but doesn't parse"

	^ (self on: aStringOrStream) documentURI: aURIString! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 4/7/2015 05:35'!
onFileNamed: aFilePathString
	"Returns new parser on the specified file but doesn't parse"

	^ self
		on: (XMLFileHandle path: aFilePathString) readStream
		documentURI: aFilePathString! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 4/7/2015 06:08'!
onURL: aURLString
	"Fetches URL and returns new parser on the response but doesn't parse"

	^ self
		onURL: aURLString
		upToLimit: XMLParserLimits defaultDocumentReadLimit! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 4/7/2015 06:08'!
onURL: aURLString upToLimit: aLimit
	"Fetches URL and returns new parser on the response up to
	the input limit (use nil to disable) but doesn't parse"

	^ (self
		on: (aURLString asXMLURI getUpToLimit: aLimit)
		documentURI: aURLString)
			documentReadLimit: aLimit! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 4/7/2015 05:35'!
parse: aStringOrStream
	"Parse document in argument string or stream"

	^ (self on: aStringOrStream) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 4/7/2015 05:35'!
parse: aStringOrStream documentURI: aURI
	"Parse document in argument string or stream with associated base URI"

	^ (self
		on: aStringOrStream
		documentURI: aURI) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 4/7/2015 06:02'!
parse: aStringOrStream usingNamespaces: aBoolean
	"Parses string or stream possibly with namespace checking (default enabled)"

	^ (self on: aStringOrStream)
		usesNamespaces: aBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 4/7/2015 06:02'!
parse: aStringOrStream usingNamespaces: aNamespaceBoolean validation: aValidationBoolean
	"Parses string or stream possibly with namespace checking
	or validation (both defualt enabled)"

	^ (self on: aStringOrStream)
		usesNamespaces: aNamespaceBoolean;
		isValidating: aValidationBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 4/7/2015 06:02'!
parse: aStringOrStream usingNamespaces: aNamespaceBoolean validation: aValidationBoolean externalEntities: anEntityResolutionBoolean
	"Parses string or stream possibly with namespace checking
	or validation (both defualt enabled)
	or external entity resolution (default disabled)"

	^ (self on: aStringOrStream)
		usesNamespaces: aNamespaceBoolean;
		isValidating: aValidationBoolean;
		resolvesExternalEntities: anEntityResolutionBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'deprecated' stamp: 'monty 10/27/2014 12:13'!
parseDocumentFrom: aStringOrStream
	self deprecated: 'use #parse: instead'.

	^ self parse: aStringOrStream.! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 4/7/2015 05:36'!
parseFileNamed: aFilePathString
	"Parses document in specified file"

	^ (self onFileNamed: aFilePathString) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 4/7/2015 05:36'!
parseURL: aURLString
	"Fetches URL and parses response"

	^ (self onURL: aURLString) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 4/7/2015 06:10'!
parseURL: aURLString upToLimit: aLimit
	"Fetches URL upto specified input limit (nil to disable) and parses response"

	^ (self
		onURL: aURLString
		upToLimit: aLimit) parseDocument! !

!SAXHandler methodsFor: 'handling - declarations' stamp: 'monty 12/30/2013 23:57'!
attributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue! !

!SAXHandler methodsFor: 'handling - content' stamp: 'mir 1/8/2002 18:27'!
characters: aString
	"This call corresponds to the Java SAX call
	characters(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'JAAyer 11/10/2010 19:23'!
comment: aCommentString
	"This call corresponds to the Java SAX ext call
	comment(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 10/27/2014 09:14'!
configuration
	^ configuration! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 12/3/2014 11:59'!
configuration: aConfiguration
	(configuration := aConfiguration) parserHandlerClass: self class! !

!SAXHandler methodsFor: 'private' stamp: 'JAAyer 11/23/2010 15:35'!
configurationClass
	^ XMLConfiguration! !

!SAXHandler methodsFor: 'private' stamp: 'monty 12/27/2013 03:36'!
currentScope: aNamespaceScope! !

!SAXHandler methodsFor: 'private' stamp: 'monty 1/9/2014 05:28'!
defaultDocumentURI
	^ self externalEntityResolver defaultDocumentURI! !

!SAXHandler methodsFor: 'private' stamp: 'monty 3/8/2015 09:29'!
doctypeDefinition: aDoctypeDefinition! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 3/17/2015 17:32'!
documentReadLimit
	^ self configuration documentReadLimit! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 3/17/2015 17:32'!
documentReadLimit: aLimit
	"change the document read limit or nil to disable"

	self configuration documentReadLimit: aLimit! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 3/21/2015 23:25'!
documentURI
	^ documentURI ifNil: [documentURI := self defaultDocumentURI]! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 4/7/2015 05:04'!
documentURI: aURI
	documentURI := aURI! !

!SAXHandler methodsFor: 'private' stamp: 'monty 2/9/2014 02:43'!
driverClass
	^ SAXParserDriver! !

!SAXHandler methodsFor: 'handling - declarations' stamp: 'monty 11/13/2013 23:54'!
elementDeclaration: aName contentModel: aContentModel
! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'JAAyer 2/21/2010 15:13'!
endCData
	"This call corresponds to the Java SAX ext call
	endCData()."! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/27/2014 14:38'!
endCDataSection
	isInCDataSection := false! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'monty 11/12/2013 18:01'!
endDTD
! !

!SAXHandler methodsFor: 'handling - content' stamp: 'JAAyer 11/27/2010 06:50'!
endDocument
	"This call corresponds to the Java SAX call
	endDocument()."! !

!SAXHandler methodsFor: 'handling - content' stamp: 'JAAyer 11/10/2010 19:23'!
endElement: aQualifiedName
! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/21/2015 22:36'!
endElement: aQualifiedName prefix: aPrefix uri: aURI
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: aQualifiedName! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/21/2015 22:36'!
endElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: aQualifiedName prefix: aPrefix uri:  aURI! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'JAAyer 11/10/2010 19:23'!
endEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	endEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'handling - content' stamp: 'JAA 3/22/2011 11:32'!
endPrefixMapping: aPrefix
	"This call corresonds to the Java SAX call
	endPrefixMapping(java.lang.String prefix)."! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 12/8/2013 02:33'!
externalEntityResolver
	^ self configuration externalEntityResolver! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 12/29/2013 01:55'!
externalEntityResolver: anExternalEntityResolver
	"This can be used to set a different DTDExternalEntityResolver"

	self configuration externalEntityResolver: anExternalEntityResolver! !

!SAXHandler methodsFor: 'handling - declarations' stamp: 'monty 3/21/2015 22:36'!
generalEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotation! !

!SAXHandler methodsFor: 'handling - declarations' stamp: 'monty 11/14/2013 04:02'!
generalEntityDeclaration: aName replacement: aReplacement! !

!SAXHandler methodsFor: 'handling - content' stamp: 'mir 8/11/2000 16:25'!
ignorableWhitespace: aString
	"This call corresonds to the Java SAX call
	ignorableWhitespace(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/22/2014 03:19'!
input
	^ input! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/3/2014 12:17'!
interruptParsing
	"Can be sent within a handler to stop parsing;
	will be silently caught within parseDocument or other parsing methods"

	XMLParsingInterruptException signal! !

!SAXHandler methodsFor: 'testing' stamp: 'monty 10/27/2014 09:15'!
isInCDataSection
	^ isInCDataSection! !

!SAXHandler methodsFor: 'testing' stamp: 'monty 3/24/2015 20:25'!
isInContent
	^ isInContent! !

!SAXHandler methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:20'!
isValidating
	^ self configuration isValidating! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 12/29/2013 03:00'!
isValidating: aBoolean
	"If true and document contstraints are present (in a DTD),
	the document will be checked against them (default).

	Enabling disables preservesUndeclaredEntityReferences.
	Disabling disables requiresDocumentConstraints."

	self configuration isValidating: aBoolean! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 3/17/2015 17:33'!
maxEntityReplacementDepth
	^ self configuration maxEntityReplacementDepth! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 3/17/2015 17:33'!
maxEntityReplacementDepth: aLimit
	"change the max replacement depth or nil to disable"

	self configuration maxEntityReplacementDepth: aLimit! !

!SAXHandler methodsFor: 'handling - declarations' stamp: 'monty 3/21/2015 22:36'!
notationDeclaration: aName publicID: aPublicID systemID: aSystemID
! !

!SAXHandler methodsFor: 'initialization' stamp: 'monty 4/7/2015 05:51'!
on: aStringOrStream
	input := aStringOrStream.

	"a single instance can parse multiple documents with on:,
	so all these values must be reset"
	documentURI := nil.
	isInContent := false.
	isInCDataSection := false.

	"but the configurations can be shared, so it is only
	default initialized once"
	configuration 
		ifNil: [self configuration: self configurationClass new].! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 4/4/2015 19:09'!
optimizeForLargeDocuments
	"use this for faster parsing/less memory use with large documents"
	self
		isValidating: false;
		usesNamespaces: false;
		documentReadLimit: nil! !

!SAXHandler methodsFor: 'handling - declarations' stamp: 'monty 3/21/2015 22:36'!
parameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID! !

!SAXHandler methodsFor: 'handling - declarations' stamp: 'monty 11/14/2013 04:05'!
parameterEntityDeclaration: aName replacement: aReplacement! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/3/2014 12:00'!
parseDocument
	"parser the entire input document"

	self parser parseDocument.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/3/2014 12:27'!
parseDocumentUntil: aBlock
	"parse the input document until aBlock is true"

	self parser parseDocumentUntil: aBlock.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/3/2014 12:01'!
parseDocumentWhile: aBlock
	"parse the input document while aBlock is true"

	self parser parseDocumentWhile: aBlock.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/3/2014 12:19'!
parser
	"returns a new XMLParser object for this handler"

	^ self parserClass
		driver: (self driverClass saxHandler: self)
		on: self input! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/3/2014 00:18'!
parserClass
	^ XMLParser! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/4/2014 14:07'!
parsingResult
	"value returned by parseDocument and other parsing methods"

	^ self! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 1/6/2015 04:12'!
preservesUndeclaredEntityReferences
	^ self configuration preservesUndeclaredEntityReferences! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 1/6/2015 22:31'!
preservesUndeclaredEntityReferences: aBoolean
	"If true, references to undeclared entities in content are
	preserved (not default).
	Disables validation"

	self configuration preservesUndeclaredEntityReferences: aBoolean! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/21/2015 22:51'!
processingInstruction: aTargetString data: aDataString
	"This call corresonds to the Java SAX call
	processingInstruction(java.lang.String target, java.lang.String data)."! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 3/17/2015 17:38'!
removeLimits
	"remove all input security limits (unsafe)"

	self configuration removeLimits! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 1/3/2015 23:55'!
replacesContentEntityReferences
	^ self configuration replacesContentEntityReferences! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 1/6/2015 22:31'!
replacesContentEntityReferences: aBoolean
	"Replace general parsed entity and character references
	in content (default)"

	self configuration replacesContentEntityReferences: aBoolean! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 12/21/2013 20:07'!
requiresDocumentConstraints
	^ self configuration requiresDocumentConstraints! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 12/29/2013 03:23'!
requiresDocumentConstraints: aBoolean
	"If true, constraints on elements and attributes (in a DTD)
	will be required (not default).
	Enables validation."

	self configuration requiresDocumentConstraints: aBoolean! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 11/16/2013 07:33'!
resolvesExternalEntities
	^ self configuration resolvesExternalEntities! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 1/24/2014 23:52'!
resolvesExternalEntities: aBoolean
	"When true, external DTDs and parsed entities will be
	fetched (not default)."

	self configuration resolvesExternalEntities: aBoolean! !

!SAXHandler methodsFor: 'handling - content' stamp: 'JAAyer 11/10/2010 19:22'!
skippedEntity: anEntityName
	"This call corresonds to the Java SAX call
	skippedEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'mir 8/11/2000 18:53'!
startCData
	"This call corresponds to the Java SAX ext call
	startCData()."! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/27/2014 14:38'!
startCDataSection
	isInCDataSection := true! !

!SAXHandler methodsFor: 'private' stamp: 'monty 3/24/2015 20:25'!
startContent
	isInContent := true! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'monty 3/21/2015 22:36'!
startDTD: aRootElementName publicID: aPublicID systemID: aSystemID
	"This call corresponds to the Java SAX ext call
	startDTD(java.lang.String name, java.lang.String publicId, java.lang.String systemId)."! !

!SAXHandler methodsFor: 'handling - content' stamp: 'mir 8/11/2000 16:45'!
startDocument
	"This call corresonds to the Java SAX call
	startDocument()."! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 11/14/2013 00:13'!
startElement: aQualifiedName attributes: aDictionary
! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/21/2015 22:36'!
startElement: aQualifiedName prefix: aPrefix uri: aURI attributes: aDictionary
	self startElement: aQualifiedName attributes: aDictionary! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/21/2015 22:36'!
startElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName attributes: aDictionary
	"This call corresonds to the Java SAX call
	startElement(java.lang.String namespaceURI, java.lang.String localName,
		java.lang.String qName, Attributes atts)."

	self
		startElement: aQualifiedName
		prefix: aPrefix
		uri: aURI
		attributes: aDictionary! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'JAAyer 11/10/2010 19:22'!
startEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	startEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/21/2015 22:36'!
startPrefixMapping: aPrefix uri: aURI
	"This call corresonds to the Java SAX call
	startPrefixMapping(java.lang.String prefix, java.lang.String uri)."! !

!SAXHandler methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:20'!
usesNamespaces
	^ self configuration usesNamespaces! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 12/29/2013 03:03'!
usesNamespaces: aBoolean
	"Namespace checking when true (default)"

	self configuration usesNamespaces: aBoolean! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 11/14/2013 00:14'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
! !

SAXHandler subclass: #XMLDOMParser
	instanceVariableNames: 'document nodeStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMParser commentStamp: 'monty 11/6/2014 12:11' prior: 0!
This class is an XML parser that produces a tree of nodes representing the parsed document's structure. The root of this tree will typically be a document node, and it will have a single child element node as its root. The root element can have any number of additional elements as its children, as well as non-element child nodes. Those elements can themselves have an arbitrary number of children, and so on. To see what type of node objects a node tree created by this parser will contain and for how to manipulate them browse the XML-Parser-DOM category.

By default, CDATA sections are treated as regular string nodes and comments are ignored entirely. To suppress this behavior, enable #preservesCDataNodes: or #preservesCommentNodes: before parsing.

Lastly. you can control what node classes the DOM parser uses to create node objects by injecting a custom node factory with #nodeFactory: prior to parsing. See XMLNodeFactory for more information.!

!XMLDOMParser methodsFor: 'handling - declarations' stamp: 'monty 12/31/2013 00:02'!
attributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self topNode addNode:
		(self nodeFactory attributeDeclarationClass
			element: anElement
			attribute: aName
			type: aType
			defaultPragma: aDefaultPragma
			defaultValue: aDefaultValue)! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 11/1/2014 23:03'!
characters: aString
	(self isInCDataSection and: [self preservesCDataNodes])
		ifTrue: [self topNode addNode: (self nodeFactory newCData: aString)]
		ifFalse: [self topNode addContent: (self nodeFactory newString: aString)]
! !

!XMLDOMParser methodsFor: 'handling - lexical' stamp: 'monty 11/1/2014 23:02'!
comment: aComment
	self preservesCommentNodes
		ifTrue: [self topNode addNode: (self nodeFactory newComment: aComment)]! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/23/2010 14:37'!
configurationClass
	^ XMLDOMConfiguration! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 12/27/2013 03:43'!
currentScope: aNamespaceScope
	self topNode setNamespaceScope: aNamespaceScope! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 3/8/2015 10:12'!
doctypeDefinition: aDoctypeDefinition
	self document doctypeDefinition: aDoctypeDefinition! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 3/9/2010 01:38'!
document
	^ document! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'monty 11/6/2014 14:07'!
document: aDocument
	document := aDocument! !

!XMLDOMParser methodsFor: 'handling - declarations' stamp: 'monty 12/21/2013 03:14'!
elementDeclaration: aName contentModel: aContentModel
	self topNode addNode:
		(self nodeFactory elementDeclarationClass
			element: aName
			contentModel: aContentModel)! !

!XMLDOMParser methodsFor: 'handling - lexical' stamp: 'monty 11/12/2013 23:00'!
endDTD
	self popNode! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 3/21/2015 22:36'!
endElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName
	self popNode! !

!XMLDOMParser methodsFor: 'handling - declarations' stamp: 'monty 3/21/2015 22:36'!
generalEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotation
	self topNode addNode:
		(self nodeFactory generalEntityDeclarationClass
			name: aName
			publicID: aPublicID
			systemID: aSystemID
			ndata: aNotation)! !

!XMLDOMParser methodsFor: 'handling - declarations' stamp: 'monty 12/21/2013 03:02'!
generalEntityDeclaration: aName replacement: aReplacement
	self topNode addNode:
		(self nodeFactory generalEntityDeclarationClass
			name: aName
			replacement: aReplacement)! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 1/3/2015 00:05'!
ignorableWhitespace: aString
	self preservesIgnorableWhitespace
		ifTrue: [self characters: aString]! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory
	^ self configuration nodeFactory! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory: aNodeFactory
	self configuration nodeFactory: aNodeFactory! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 12/9/2013 01:37'!
nodeListClass
	^ self configuration nodeListClass! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 12/9/2013 01:37'!
nodeListClass: aClass
	self configuration nodeListClass: aClass! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 10/27/2014 09:12'!
nodeStack
	^ nodeStack! !

!XMLDOMParser methodsFor: 'handling - declarations' stamp: 'monty 3/21/2015 22:36'!
notationDeclaration: aName publicID: aPublicID systemID: aSystemID
	self topNode addNode:
		(self nodeFactory notationDeclarationClass
			name: aName
			publicID: aPublicID
			systemID: aSystemID)! !

!XMLDOMParser methodsFor: 'initialization' stamp: 'monty 4/7/2015 05:08'!
on: aStringOrStream
	super on: aStringOrStream.

	nodeStack := OrderedCollection new: 10.! !

!XMLDOMParser methodsFor: 'handling - declarations' stamp: 'monty 3/21/2015 22:36'!
parameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID
	self topNode addNode:
		(self nodeFactory parameterEntityDeclarationClass
			name: aName
			publicID: aPublicID
			systemID: aSystemID)! !

!XMLDOMParser methodsFor: 'handling - declarations' stamp: 'monty 12/21/2013 03:02'!
parameterEntityDeclaration: aName replacement: aReplacement
	self topNode addNode:
		(self nodeFactory parameterEntityDeclarationClass
			name: aName
			replacement: aReplacement)! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'monty 11/15/2014 16:44'!
parser
	| parser |

	parser := super parser.
	self topNode
		ifNotNil: [:root |
			parser initialContext: root innerXMLContext.

			self nodeStack do: [:each |
				each isElement
					ifTrue: [
						self
							startInnerXMLElement: each
							forParser: parser]]].

	^ parser.! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'monty 12/19/2013 00:38'!
parsingResult
	^ self document! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/16/2010 09:31'!
popNode
	^ self nodeStack removeLast! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCDataNodes
	^ self configuration preservesCDataNodes! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:17'!
preservesCDataNodes: aBoolean
	self configuration preservesCDataNodes: aBoolean! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCommentNodes
	^ self configuration preservesCommentNodes! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCommentNodes: aBoolean
	self configuration preservesCommentNodes: aBoolean! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 1/3/2015 00:05'!
preservesIgnorableWhitespace
	^ self configuration preservesIgnorableWhitespace! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 1/3/2015 00:05'!
preservesIgnorableWhitespace: aBoolean
	self configuration preservesIgnorableWhitespace: aBoolean! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 3/21/2015 22:51'!
processingInstruction: aTargetString data: aDataString
	self topNode addNode:
		(self nodeFactory newPI
			target: aTargetString;
			data: aDataString)! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 2/4/2014 01:33'!
pushElement: anElement
	self topNode addNode: anElement.
	^ self pushNode: anElement.! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 11/14/2014 21:11'!
pushInnerXMLRootNode: aRootNode
	self
		configuration: aRootNode configuration;
		document: aRootNode document;
		pushNode: aRootNode.! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/28/2010 13:25'!
pushNode: aNode
	^ self nodeStack addLast: aNode
! !

!XMLDOMParser methodsFor: 'handling - lexical' stamp: 'monty 3/21/2015 22:36'!
startDTD: aRootElementName publicID: aPublicID systemID: aSystemID
	| doctypeDeclaration |

	doctypeDeclaration := 
		self nodeFactory doctypeDeclarationClass
			root: aRootElementName
			publicID: aPublicID
			systemID: aSystemID.
	self pushNode: doctypeDeclaration.
	self document doctypeDeclaration: doctypeDeclaration.! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 11/6/2014 14:07'!
startDocument
	self topNode ifNotNil: [^ self].

	self
		document: self nodeFactory newDocument;
		pushNode: self document.

	self document configuration: self configuration.! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 3/21/2015 22:36'!
startElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName attributes: aDictionary
	self pushElement:
		((self nodeFactory
			classForElement: aQualifiedName
			prefix: aPrefix
			uri: aURI
			localName: aLocalName) new
				setQualifiedName: aQualifiedName
				localName: aLocalName
				attributes: aDictionary)! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 11/15/2014 04:07'!
startInnerXMLElement: anElement forParser: aParser
	aParser driver
		setRootElement: anElement name
		currentScope: anElement namespaceScope! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 10/7/2014 03:06'!
topNode
	^ self nodeStack lastOrNil! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 11/19/2013 05:37'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	self document ifNotNil: [
		self document
			version: aVersionFloat;
			encoding: anEncoding;
			isStandalone: aBoolean]! !

Object subclass: #XMLAbstractFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Util'!

XMLAbstractFactory class
	instanceVariableNames: 'implementation'!

!XMLAbstractFactory commentStamp: 'monty 1/6/2015 23:11' prior: 0!
An abstract class for dynamically choosing supported concrete implementations for different platforms.!

XMLAbstractFactory class
	instanceVariableNames: 'implementation'!

!XMLAbstractFactory class methodsFor: 'private' stamp: 'monty 1/27/2015 22:58'!
clearImplementations
	"self clearImplementations"

	self allSubclassesDo: [:each | each implementation: nil]! !

!XMLAbstractFactory class methodsFor: 'private' stamp: 'monty 11/12/2014 18:22'!
errorUnsupported
	self subclassResponsibility! !

!XMLAbstractFactory class methodsFor: 'private' stamp: 'monty 11/18/2014 07:03'!
implementation
	^ implementation
		ifNil: [
			self isSupported
				ifTrue: [implementation := self]
				ifFalse: [implementation := self selectImplementation]]! !

!XMLAbstractFactory class methodsFor: 'private' stamp: 'monty 12/18/2014 19:07'!
implementation: aClass
	implementation := aClass! !

!XMLAbstractFactory class methodsFor: 'testing' stamp: 'monty 11/12/2014 18:21'!
isSupported
	^ false! !

!XMLAbstractFactory class methodsFor: 'instance creation' stamp: 'monty 11/12/2014 18:24'!
new
	^ self implementation basicNew initialize! !

!XMLAbstractFactory class methodsFor: 'private' stamp: 'monty 11/12/2014 18:22'!
preferredImplementation
	self subclassResponsibility! !

!XMLAbstractFactory class methodsFor: 'private' stamp: 'monty 11/12/2014 18:25'!
selectImplementation
	self preferredImplementation isSupported
		ifTrue: [^ self preferredImplementation].

	self allSubclassesDo: [:each |
		each isSupported
			ifTrue: [^ each]].

	self errorUnsupported.! !

XMLAbstractFactory subclass: #XMLFileHandle
	instanceVariableNames: 'path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileHandle commentStamp: 'monty 11/20/2014 02:27' prior: 0!
This is an abstract class for file handles to read/write/delete files. Subclasses need to implement the abstract methods.!

XMLFileHandle subclass: #XMLFileDirectoryFileHandle
	instanceVariableNames: 'fileDirectory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileDirectoryFileHandle commentStamp: 'monty 10/24/2014 16:06' prior: 0!
This class uses FileDirectory/DirectoryEntry to implement file IO.!

!XMLFileDirectoryFileHandle class methodsFor: 'private' stamp: 'monty 10/22/2014 04:43'!
currentWorkingDirectoryPathString
	^ (Smalltalk at: #FileDirectory) default pathName! !

!XMLFileDirectoryFileHandle class methodsFor: 'testing' stamp: 'monty 10/25/2014 20:40'!
isSupported
	^ Smalltalk includesKey: #FileDirectory! !

!XMLFileDirectoryFileHandle methodsFor: 'private' stamp: 'monty 12/18/2014 18:43'!
deleteFileOrDo: aBlock
	[self fileDirectory
		deleteFileNamed: path
		ifAbsent: [^ aBlock value]]
		on: Error
		do: aBlock! !

!XMLFileDirectoryFileHandle methodsFor: 'private' stamp: 'monty 12/16/2014 15:54'!
directoryEntry
	^ self fileDirectory directoryEntryFor: path! !

!XMLFileDirectoryFileHandle methodsFor: 'private' stamp: 'monty 12/16/2014 15:58'!
fileDirectory
	^ fileDirectory ifNil: [fileDirectory := (Smalltalk at: #FileDirectory) default]! !

!XMLFileDirectoryFileHandle methodsFor: 'accessing' stamp: 'monty 12/18/2014 18:16'!
fileSize
	^ (self directoryEntry ifNil: [^ 0]) fileSize! !

!XMLFileDirectoryFileHandle methodsFor: 'testing' stamp: 'monty 10/22/2014 05:27'!
isFile
	^ self directoryEntry notNil
		and: [self directoryEntry isDirectory not]! !

!XMLFileHandle class methodsFor: 'accessing' stamp: 'monty 1/23/2014 01:34'!
currentWorkingDirectory
	^ self implementation currentWorkingDirectoryPathString! !

!XMLFileHandle class methodsFor: 'private' stamp: 'monty 1/22/2014 18:50'!
currentWorkingDirectoryPathString
	self subclassResponsibility! !

!XMLFileHandle class methodsFor: 'private' stamp: 'monty 10/31/2014 20:54'!
errorUnsupported
	XMLFileException signal: 'File IO unsupported'.! !

!XMLFileHandle class methodsFor: 'instance creation' stamp: 'monty 10/25/2014 17:49'!
path: aSystemPath
	^ self new path: aSystemPath! !

!XMLFileHandle class methodsFor: 'private' stamp: 'monty 10/25/2014 18:56'!
preferredImplementation
	^ XMLFileSystemFileHandle! !

!XMLFileHandle methodsFor: 'deleting' stamp: 'monty 11/17/2014 23:00'!
delete
	self deleteFileOrDo: [:error | XMLFileException signalMessageFrom: error]! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 1/21/2014 22:32'!
deleteFileOrDo: aBlock
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 11/6/2014 12:16'!
errorFileSize: aFileSize exceedsMaxSize: aMaxSize
	XMLLimitException
		formatSignal: 'File of size {1} exceeds max size limit of {2}: {3}'
		with: aFileSize
		with: aMaxSize
		with: path! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 10/31/2014 20:49'!
errorNotAFile
	XMLFileException
		formatSignal: 'Not a file: {1}'
		with: path! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 1/22/2014 19:07'!
fileSize
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'requesting' stamp: 'monty 1/13/2014 18:14'!
get
	^ self readStream! !

!XMLFileHandle methodsFor: 'requesting' stamp: 'monty 1/13/2014 18:14'!
getUpToLimit: aMaxSize
	^ self readStreamUpToLimit: aMaxSize! !

!XMLFileHandle methodsFor: 'testing' stamp: 'monty 1/9/2014 01:58'!
isFile
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 1/9/2014 04:34'!
path
	^ path! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 1/9/2014 04:23'!
path: aPath
	path := aPath! !

!XMLFileHandle methodsFor: 'reading' stamp: 'monty 1/13/2014 18:13'!
readStream
	^ self readStreamUpToLimit: nil! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 11/17/2014 00:40'!
readStreamOnFileOrDo: aBlock
	^ [XMLFileReadStreamFactory on: path]
		on: Error
		do: aBlock! !

!XMLFileHandle methodsFor: 'reading' stamp: 'monty 11/17/2014 23:00'!
readStreamUpToLimit: aMaxSize
	| fileSize |

	self isFile
		ifFalse: [self errorNotAFile].
	
	(fileSize := self fileSize) > 0
		ifFalse: [ ^ '' readStream].
	(aMaxSize notNil and: [fileSize > aMaxSize])
		ifTrue: [self errorFileSize: fileSize exceedsMaxSize: aMaxSize].

	^ self readStreamOnFileOrDo: [:error | XMLFileException signalMessageFrom: error]! !

!XMLFileHandle methodsFor: 'writing' stamp: 'monty 11/17/2014 23:00'!
writeStream
	^ self writeStreamOnFileOrDo: [:error | XMLFileException signalMessageFrom: error]! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 11/17/2014 00:49'!
writeStreamOnFileOrDo: aBlock
	^ [XMLFileWriteStreamFactory on: path]
		on: Error
		do: aBlock! !

XMLFileHandle subclass: #XMLFileSystemFileHandle
	instanceVariableNames: 'fileReference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileSystemFileHandle commentStamp: 'monty 1/9/2014 23:29' prior: 0!
This class uses the Pharo FileSystem API to implement file IO.!

!XMLFileSystemFileHandle class methodsFor: 'private' stamp: 'monty 4/6/2015 10:49'!
currentWorkingDirectoryPathString
	^ String streamContents: [:stream |
		"the class-side workingDirectory isn't used for backwards compatibility"
		(Smalltalk at: #FileSystem) disk workingDirectory
			path segments do: [:each |
				stream
					nextPut: $/;
					nextPutAll: each]].! !

!XMLFileSystemFileHandle class methodsFor: 'testing' stamp: 'monty 10/25/2014 20:40'!
isSupported
	^ Smalltalk includesKey: #FileSystem! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 1/22/2014 19:10'!
deleteFileOrDo: aBlock
	[self fileReference delete]
		on: self fileSystemError 
		do: aBlock! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 1/13/2014 04:37'!
fileReference
	^ fileReference ifNil: [fileReference := path asFileReference]! !

!XMLFileSystemFileHandle methodsFor: 'accessing' stamp: 'monty 1/22/2014 19:07'!
fileSize
	^ self fileReference size! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 1/22/2014 19:09'!
fileSystemError
	^ Smalltalk at: #FileSystemError! !

!XMLFileSystemFileHandle methodsFor: 'testing' stamp: 'monty 1/13/2014 04:37'!
isFile
	^ self fileReference isFile! !

XMLAbstractFactory subclass: #XMLFileReadStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileReadStreamFactory commentStamp: 'monty 11/20/2014 02:27' prior: 0!
An abstract factory for file read streams used by XMLFileHandle.!

!XMLFileReadStreamFactory class methodsFor: 'private' stamp: 'monty 11/17/2014 00:44'!
errorUnsupported
	XMLFileException signal: 'File reading unsupported'! !

!XMLFileReadStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/17/2014 00:38'!
fileReadStreamOn: aPath
	self subclassResponsibility! !

!XMLFileReadStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/17/2014 00:38'!
on: aPath
	^ self implementation fileReadStreamOn: aPath! !

!XMLFileReadStreamFactory class methodsFor: 'private' stamp: 'monty 11/20/2014 02:32'!
preferredImplementation
	^ XMLStandardFileStreamReadStreamFactory! !

XMLFileReadStreamFactory subclass: #XMLGsFileReadStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLGsFileReadStreamFactory commentStamp: 'monty 12/14/2014 14:34' prior: 0!
A GsFile read stream factory. This stream is preferred for Gemstone because it does no encoding conversion so XMLParser can do its own.!

!XMLGsFileReadStreamFactory class methodsFor: 'instance creation' stamp: 'monty 12/18/2014 18:16'!
fileReadStreamOn: aPath
	^ (Smalltalk at: #GsFile) openReadOnServer: aPath! !

!XMLGsFileReadStreamFactory class methodsFor: 'testing' stamp: 'monty 12/14/2014 14:33'!
isSupported
	^ (Smalltalk includesKey: #GsFile)! !

XMLFileReadStreamFactory subclass: #XMLStandardFileStreamReadStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLStandardFileStreamReadStreamFactory commentStamp: 'monty 11/20/2014 02:28' prior: 0!
A StandardFileStream read stream factory. This stream is preferred for Pharo/Squeak because it does no encoding conversion so XMLParser can do its own.!

!XMLStandardFileStreamReadStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/20/2014 03:19'!
fileReadStreamOn: aPath
	^ (Smalltalk at: #StandardFileStream) readOnlyFileNamed: aPath! !

!XMLStandardFileStreamReadStreamFactory class methodsFor: 'testing' stamp: 'monty 11/17/2014 00:36'!
isSupported
	^ (Smalltalk includesKey: #StandardFileStream)! !

XMLAbstractFactory subclass: #XMLFileWriteStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileWriteStreamFactory commentStamp: 'monty 11/20/2014 02:28' prior: 0!
An abstract factory for file write streams used by XMLFileHandle.!

XMLFileWriteStreamFactory subclass: #XMLFileStreamWriteStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileStreamWriteStreamFactory commentStamp: 'monty 11/20/2014 02:28' prior: 0!
A FileStream write stream factory, default for Pharo/Squeak.!

!XMLFileStreamWriteStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/17/2014 00:46'!
fileWriteStreamOn: aPath
	^ (Smalltalk at: #FileStream) fileNamed: aPath! !

!XMLFileStreamWriteStreamFactory class methodsFor: 'testing' stamp: 'monty 11/17/2014 00:46'!
isSupported
	^ (Smalltalk includesKey: #FileStream)! !

!XMLFileWriteStreamFactory class methodsFor: 'private' stamp: 'monty 11/17/2014 00:44'!
errorUnsupported
	XMLFileException signal: 'File writing unsupported'! !

!XMLFileWriteStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/17/2014 00:43'!
fileWriteStreamOn: aPath
	self subclassResponsiblity! !

!XMLFileWriteStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/17/2014 00:43'!
on: aPath
	^ self implementation fileWriteStreamOn: aPath! !

!XMLFileWriteStreamFactory class methodsFor: 'private' stamp: 'monty 11/17/2014 00:44'!
preferredImplementation
	^ XMLFileStreamWriteStreamFactory! !

XMLAbstractFactory subclass: #XMLHTTPRequest
	instanceVariableNames: 'url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTTP'!

!XMLHTTPRequest commentStamp: 'monty 11/20/2014 02:29' prior: 0!
A base class for HTTP requests. Subclasses need to implement the abstract requesting method to return a ReadStream on the respone or raise an exception if it is too large or an HTTP error happens.!

!XMLHTTPRequest class methodsFor: 'private' stamp: 'monty 10/31/2014 20:54'!
errorUnsupported
	XMLHTTPException signal: 'HTTP unsupported'.! !

!XMLHTTPRequest class methodsFor: 'private' stamp: 'monty 10/25/2014 18:56'!
preferredImplementation
	^ XMLZincHTTPRequest! !

!XMLHTTPRequest class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
url: aURL
	^ self new url: aURL! !

!XMLHTTPRequest methodsFor: 'private' stamp: 'monty 10/26/2014 09:14'!
defaultTimeout
	^ 30! !

!XMLHTTPRequest methodsFor: 'requesting' stamp: 'monty 1/9/2014 05:08'!
get
	^ self getUpToLimit: nil! !

!XMLHTTPRequest methodsFor: 'private' stamp: 'monty 1/13/2014 19:42'!
getResponseUpToLimit: aMaxSize withTimeout: aTimeout onError: aBlock
	self subclassResponsibility! !

!XMLHTTPRequest methodsFor: 'requesting' stamp: 'monty 12/16/2014 20:26'!
getUpToLimit: aMaxSize
	^ self
		getResponseUpToLimit: aMaxSize
		withTimeout: self defaultTimeout
		onError: [:error |
			(self isMaxSizeException: error)
				ifTrue: [XMLLimitException signal: 'HTTP reponse limit exceeded']
				ifFalse: [XMLHTTPException signalMessageFrom: error]]! !

!XMLHTTPRequest methodsFor: 'private' stamp: 'monty 12/16/2014 20:26'!
isMaxSizeException: anException
	self subclassResponsibility! !

!XMLHTTPRequest methodsFor: 'accessing' stamp: 'monty 1/10/2014 00:10'!
url
	^ url! !

!XMLHTTPRequest methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
url: aURL
	url := aURL! !

XMLHTTPRequest subclass: #XMLZincHTTPRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTTP'!

!XMLZincHTTPRequest commentStamp: '<historical>' prior: 0!
This client uses Zinc-HTTP, but does not refer to Zinc classes directly to avoid compile errors on platforms without Zinc.!

!XMLZincHTTPRequest class methodsFor: 'testing' stamp: 'monty 10/25/2014 17:43'!
isSupported
	^ Smalltalk includesKey: #ZnClient! !

!XMLZincHTTPRequest methodsFor: 'private' stamp: 'monty 12/16/2014 20:27'!
getResponseUpToLimit: aMaxSize withTimeout: aTimeout onError: aBlock
	^ self
		setZincMaxEntitySize: aMaxSize
		during: [
			((Smalltalk at: #ZnClient) new
				timeout: aTimeout;
				url: url;
				get) readStream]
		onError: aBlock! !

!XMLZincHTTPRequest methodsFor: 'private' stamp: 'monty 12/16/2014 20:26'!
isMaxSizeException: anException
	^ anException isKindOf: (Smalltalk at: #ZnEntityTooLarge)! !

!XMLZincHTTPRequest methodsFor: 'private' stamp: 'monty 12/16/2014 20:35'!
setZincMaxEntitySize: aMaxSize during: aDuringBlock onError: anErrorBlock
	^ [(Smalltalk at: #ZnMaximumEntitySize)
		value: aMaxSize
		during: [
			"value:during: doesn't return during block on gemstone"
			^ aDuringBlock value]]
		on: Error
		do: anErrorBlock! !

Object subclass: #XMLAttributeDefaultValidator
	instanceVariableNames: 'defaultValue attribute'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeDefaultValidator commentStamp: '<historical>' prior: 0!
This is a base class for attribute default value validators that implement the constraints (like #REQUIRED) from the default declaration.!

!XMLAttributeDefaultValidator class methodsFor: 'instance creation' stamp: 'monty 1/21/2014 05:35'!
attribute: anAttribute
	^ self attribute: anAttribute defaultValue: ''! !

!XMLAttributeDefaultValidator class methodsFor: 'instance creation' stamp: 'monty 12/10/2013 06:16'!
attribute: anAttribute defaultValue: aDefaultValue
	^ self new setAttribute: anAttribute defaultValue: aDefaultValue! !

!XMLAttributeDefaultValidator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:48'!
attribute
	^ attribute! !

!XMLAttributeDefaultValidator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:48'!
defaultValue
	^ defaultValue! !

!XMLAttributeDefaultValidator methodsFor: 'initialization' stamp: 'monty 12/10/2013 06:17'!
setAttribute: anAttribute defaultValue: aDefaultValue
	attribute := anAttribute.
	defaultValue := aDefaultValue.! !

!XMLAttributeDefaultValidator methodsFor: 'validating' stamp: 'monty 12/28/2013 20:02'!
validateAttributeDefaultIn: anAttributeDictionary
	self subclassResponsibility! !

XMLAttributeDefaultValidator subclass: #XMLDefaultValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDefaultValueAttributeValidator commentStamp: 'monty 2/2/2014 04:47' prior: 0!
This class will put a default value for an attribute if it has none.!

!XMLDefaultValueAttributeValidator methodsFor: 'validating' stamp: 'monty 10/30/2014 12:48'!
validateAttributeDefaultIn: anAttributeDictionary
	((anAttributeDictionary at: self attribute ifAbsent: ['']) isEmpty 
		and: [self defaultValue notEmpty])
		ifTrue: [anAttributeDictionary at: self attribute put: self defaultValue]! !

XMLDefaultValueAttributeValidator subclass: #XMLFixedValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLFixedValueAttributeValidator commentStamp: 'monty 2/2/2014 08:40' prior: 0!
This class fixes the value of an attribute to a default.!

!XMLFixedValueAttributeValidator methodsFor: 'private' stamp: 'monty 11/4/2014 15:09'!
errorNonDefaultValue
	XMLValidationException
		formatSignal: '#FIXED attribute "{1}" can only have "{2}" as value'
		with: self attribute
		with: self defaultValue! !

!XMLFixedValueAttributeValidator methodsFor: 'validating' stamp: 'monty 11/4/2014 15:09'!
validateAttributeDefaultIn: anAttributeDictionary
	(anAttributeDictionary
		at: self attribute
		ifAbsentPut: [self defaultValue]) = self defaultValue
		ifFalse: [self errorNonDefaultValue]! !

XMLAttributeDefaultValidator subclass: #XMLImpliedValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLImpliedValueAttributeValidator commentStamp: 'monty 2/2/2014 06:07' prior: 0!
Attributes declared #IMPLIED are optional.
!

!XMLImpliedValueAttributeValidator methodsFor: 'validating' stamp: 'monty 12/27/2013 15:36'!
validateAttributeDefaultIn: anAttributeDictionary! !

XMLAttributeDefaultValidator subclass: #XMLRequiredValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLRequiredValueAttributeValidator commentStamp: 'monty 2/2/2014 16:37' prior: 0!
Attributes declared #REQUIRED require values.!

!XMLRequiredValueAttributeValidator methodsFor: 'private' stamp: 'monty 10/31/2014 17:11'!
errorRequiredAttributeMissing
	XMLValidationException
		formatSignal: 'Required attribute "{1}" is missing'
		with: self attribute! !

!XMLRequiredValueAttributeValidator methodsFor: 'validating' stamp: 'monty 10/31/2014 17:10'!
validateAttributeDefaultIn: anAttributeDictionary
	(anAttributeDictionary includesKey: self attribute)
		ifFalse: [self errorRequiredAttributeMissing]! !

Object subclass: #XMLAttributeValidator
	instanceVariableNames: 'element attribute defaultValueValidator'
	classVariableNames: 'NormalizedSpace'
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeValidator commentStamp: '<historical>' prior: 0!
This is a base class for attribute validators.!

!XMLAttributeValidator class methodsFor: 'instance creation' stamp: 'monty 12/10/2013 06:15'!
element: anElement attribute: anAttribute defaultValueValidator: aDefaultValueValidator
	^ self new
		setElement: anElement
		attribute: anAttribute
		defaultValueValidator: aDefaultValueValidator! !

!XMLAttributeValidator class methodsFor: 'class initialization' stamp: 'monty 12/10/2014 11:36'!
initialize
	"self initialize"

	NormalizedSpace := Character space! !

!XMLAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/10/2013 06:17'!
attribute
	^ attribute! !

!XMLAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/10/2013 06:17'!
element
	^ element! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 3/22/2015 03:50'!
errorInvalidID: anID
	XMLValidationException
		formatSignal: 'Invalid ID "{1}" in attribute "{2}"'
		with: anID
		with: self attribute! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 12/23/2014 21:14'!
errorInvalidXMLIDAttributeValidator
	XMLValidationException signal: 'Attribute "xml:id" must have ID type'! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 3/22/2015 05:53'!
errorInvalidXMLSpaceAttributeValidator
	XMLValidationException signal:
		'Attribute "xml:space" must be declared as enumeration type
		with only "default" and "preserver" as values'! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 12/27/2014 03:03'!
furtherNormalizeAttributeValueIn: anAttributeDictionary
	"The tokenizer normalizes all whitespace in attribute values to spaces
	and this step normalizes again by removing leading and trailing spaces
	and turing multiple spaces to single spaces."
	| attributeAssociation semiNormalizedValue writeStream isInWhitespace |

	attributeAssociation :=
		anAttributeDictionary
			associationAt: self attribute
			ifAbsent: [^ self].
	semiNormalizedValue := attributeAssociation value.
	writeStream := (String new: semiNormalizedValue size) writeStream.
	isInWhitespace := false.
	"to:do: for speed"
	1 to: semiNormalizedValue size do: [:i | | nextChar |
		(nextChar := semiNormalizedValue at: i) == NormalizedSpace
			ifTrue: [
				isInWhitespace
					ifFalse: [isInWhitespace := true]]
			ifFalse: [
				isInWhitespace
					ifTrue: [
						writeStream xmlParserAtBeginning
							ifFalse: [writeStream nextPut: NormalizedSpace].
						isInWhitespace := false].
				writeStream nextPut: nextChar]].
	attributeAssociation value: writeStream contents.! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:57'!
hasIDRefs
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 1/4/2014 05:02'!
hasNotationValues
	^ false! !

!XMLAttributeValidator methodsFor: 'initialization' stamp: 'monty 1/1/2015 20:54'!
initializeWithValidatorsFrom: aValidatorContainer
	self validateReservedAttributeValidator! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2014 09:40'!
isEnumerationValidator
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/10/2013 06:06'!
isIDValidator
	^ false! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 12/14/2014 12:25'!
listValuesIn: aSpaceSeparatedList do: aBlock
	NormalizedSpace
		xmlParserSplit: aSpaceSeparatedList
		do: aBlock! !

!XMLAttributeValidator methodsFor: 'initialization' stamp: 'monty 12/10/2013 06:15'!
setElement: anElement attribute: anAttribute defaultValueValidator: aDefaultValueValidator
	element := anElement.
	attribute := anAttribute.
	defaultValueValidator := aDefaultValueValidator.! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2013 02:14'!
shouldFurtherNormalizeAttributeValue
	^ true! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 2/10/2014 19:58'!
validateAttributeDefaultIn: anAttributeDictionary
	defaultValueValidator validateAttributeDefaultIn: anAttributeDictionary.

	self shouldFurtherNormalizeAttributeValue
		ifTrue: [self furtherNormalizeAttributeValueIn: anAttributeDictionary].! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 12/28/2013 20:03'!
validateAttributeValue: aValue
	self subclassResponsibility! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 11/19/2014 18:35'!
validateAttributes: anAttributeDictionary
	self
		validateAttributeDefaultIn: anAttributeDictionary;
		validateAttributeValue:
			(anAttributeDictionary
				at: self attribute
				ifAbsent: [^ self])! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 12/23/2014 20:57'!
validateReservedAttributeValidator
	(self attribute at: 1) == $x
		ifTrue: [
			(self attribute = 'xml:id')
				ifTrue: [^ self validateXMLIDAttributeValidator].
			(self attribute = 'xml:space')
				ifTrue: [^ self validateXMLSpaceAttributeValidator]]! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 03:58'!
validateXMLIDAttributeValidator
	self errorInvalidXMLIDAttributeValidator! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 03:55'!
validateXMLSpaceAttributeValidator
	self errorInvalidXMLSpaceAttributeValidator! !

XMLAttributeValidator subclass: #XMLCDataAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLCDataAttributeValidator commentStamp: 'monty 2/2/2014 08:12' prior: 0!
CDATA attribute values are left alone (whitespace is not normalized further).!

!XMLCDataAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2013 02:14'!
shouldFurtherNormalizeAttributeValue
	^ false! !

!XMLCDataAttributeValidator methodsFor: 'validating' stamp: 'monty 12/28/2013 20:04'!
validateAttributeValue: aValue
	! !

XMLAttributeValidator subclass: #XMLEntityAttributeValidator
	instanceVariableNames: 'unparsedEntityValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEntityAttributeValidator commentStamp: 'monty 2/2/2014 16:44' prior: 0!
This class validates attributes that contain unparsed entity names (that must be declared).!

XMLEntityAttributeValidator subclass: #XMLEntitiesAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEntitiesAttributeValidator commentStamp: 'monty 2/2/2014 16:44' prior: 0!
This class validates attributes with lists of unparsed entity name values separated by spaces.!

!XMLEntitiesAttributeValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:30'!
validateAttributeValue: aValue
	self
		listValuesIn: aValue
		do: [:each |
			self unparsedEntityValidator
				validateEntityReference: each
				inAttribute: self attribute]! !

!XMLEntityAttributeValidator methodsFor: 'initialization' stamp: 'monty 1/1/2015 20:55'!
initializeWithValidatorsFrom: aValidatorContainer
	unparsedEntityValidator := aValidatorContainer unparsedEntityValidator.
	self validateReservedAttributeValidator.! !

!XMLEntityAttributeValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 22:00'!
isEntityValidator
	^ true! !

!XMLEntityAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 20:35'!
unparsedEntityValidator
	^ unparsedEntityValidator! !

!XMLEntityAttributeValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:30'!
validateAttributeValue: aValue
	self unparsedEntityValidator
		validateEntityReference: aValue
		inAttribute: self attribute! !

XMLAttributeValidator subclass: #XMLEnumerationAttributeValidator
	instanceVariableNames: 'allowedValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEnumerationAttributeValidator commentStamp: 'monty 2/2/2014 08:13' prior: 0!
This class validates attributes that can have any value taken from a list like "(one|two|thee)".!

!XMLEnumerationAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/9/2013 02:17'!
allowedValues
	^ allowedValues ifNil: [allowedValues := Set new]! !

!XMLEnumerationAttributeValidator methodsFor: 'private ' stamp: 'monty 12/11/2014 05:53'!
enumerationListParserClass
	^ DTDEnumerationListParser! !

!XMLEnumerationAttributeValidator methodsFor: 'private ' stamp: 'monty 10/31/2014 17:30'!
errorInvalidValue: aValue
	XMLValidationException
		formatSignal: 'Invalid value "{1}" for attribute "{2}"'
		with: aValue
		with: self attribute! !

!XMLEnumerationAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2014 09:40'!
isEnumerationValidator
	^ true! !

!XMLEnumerationAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/11/2014 05:53'!
setAllowedValuesFromString: aString
	allowedValues := (self enumerationListParserClass on: aString) parse! !

!XMLEnumerationAttributeValidator methodsFor: 'validating' stamp: 'monty 10/31/2014 17:30'!
validateAttributeValue: aValue
	(self allowedValues includes: aValue)
		ifFalse: [self errorInvalidValue: aValue]! !

!XMLEnumerationAttributeValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 05:56'!
validateXMLSpaceAttributeValidator
	(self allowedValues size <= 2
		and: [self allowedValues allSatisfy: [:each |
			(each = 'preserve') or: [each = 'default']]])
		ifFalse: [self errorInvalidXMLSpaceAttributeValidator]! !

XMLEnumerationAttributeValidator subclass: #XMLNotationEnumerationAttributeValidator
	instanceVariableNames: 'unparsedEntityValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNotationEnumerationAttributeValidator commentStamp: 'monty 2/2/2014 08:10' prior: 0!
This class validates attributes that can have NOTATION names from a list as values.!

!XMLNotationEnumerationAttributeValidator methodsFor: 'private ' stamp: 'monty 12/11/2014 05:53'!
enumerationListParserClass
	^ DTDNotationEnumerationListParser! !

!XMLNotationEnumerationAttributeValidator methodsFor: 'testing' stamp: 'monty 1/4/2014 04:58'!
hasNotationValues
	^ true! !

!XMLNotationEnumerationAttributeValidator methodsFor: 'initialization' stamp: 'monty 1/1/2015 23:26'!
initializeWithValidatorsFrom: aValidatorContainer
	(unparsedEntityValidator := aValidatorContainer unparsedEntityValidator)
		validateNotationAttribute: self attribute
		forElement: self element.
	self validateReservedAttributeValidator.! !

!XMLNotationEnumerationAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 20:50'!
unparsedEntityValidator
	^ unparsedEntityValidator! !

!XMLNotationEnumerationAttributeValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:31'!
validateAttributeValue: aValue
	super validateAttributeValue: aValue.
	self unparsedEntityValidator
		validateNotationReference: aValue
		inAttribute: self attribute.! !

!XMLNotationEnumerationAttributeValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 03:56'!
validateXMLSpaceAttributeValidator
	self errorInvalidXMLSpaceAttributeValidator! !

XMLAttributeValidator subclass: #XMLIDAttributeValidator
	instanceVariableNames: 'elementIdValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDAttributeValidator commentStamp: 'monty 2/2/2014 05:57' prior: 0!
This class validates ID attributes that must be unique in each element of the type.!

!XMLIDAttributeValidator class methodsFor: 'private' stamp: 'monty 12/23/2014 18:32'!
classForAttribute: anAttribute
	anAttribute = 'xml:id'
		ifTrue: [^ XMLXMLIDIDAttributeValidator]
		ifFalse: [^ XMLIDAttributeValidator]! !

!XMLIDAttributeValidator class methodsFor: 'instance creation' stamp: 'monty 12/23/2014 18:33'!
element: anElement attribute: anAttribute defaultValueValidator: aDefaultValueValidator
	^ (self classForAttribute: anAttribute) new
		setElement: anElement
		attribute: anAttribute
		defaultValueValidator: aDefaultValueValidator! !

!XMLIDAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 19:49'!
elementIDValidator
	^ elementIdValidator! !

!XMLIDAttributeValidator methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
elementIDValidator: anElementIDValidator
	elementIdValidator := anElementIDValidator! !

!XMLIDAttributeValidator methodsFor: 'initialization' stamp: 'monty 1/1/2015 23:27'!
initializeWithValidatorsFrom: aValidatorContainer
	(elementIdValidator := aValidatorContainer elementIDValidator)
		validateIDAttribute: self attribute
		forElement: self element.
	self validateReservedAttributeValidator.! !

!XMLIDAttributeValidator methodsFor: 'testing' stamp: 'monty 12/10/2013 06:07'!
isIDValidator
	^ true! !

!XMLIDAttributeValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:32'!
validateAttributeValue: aValue
	self elementIDValidator
		validateID: aValue
		inAttribute: self attribute! !

XMLIDAttributeValidator subclass: #XMLXMLIDIDAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLXMLIDIDAttributeValidator commentStamp: 'monty 12/23/2014 18:33' prior: 0!
A validator of xml:id attributes.!

!XMLXMLIDIDAttributeValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:33'!
validateAttributeValue: aValue
	self elementIDValidator validateXMLID: aValue! !

!XMLXMLIDIDAttributeValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 03:58'!
validateXMLIDAttributeValidator
! !

XMLAttributeValidator subclass: #XMLIDRefAttributeValidator
	instanceVariableNames: 'elementIdValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDRefAttributeValidator commentStamp: 'monty 2/2/2014 16:44' prior: 0!
This class validates attributes that refer to an ID of another element with a ID attribute.!

!XMLIDRefAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 19:51'!
elementIDValidator
	^ elementIdValidator! !

!XMLIDRefAttributeValidator methodsFor: 'initialization' stamp: 'monty 1/1/2015 20:55'!
initializeWithValidatorsFrom: aValidatorContainer
	elementIdValidator := aValidatorContainer elementIDValidator.
	self validateReservedAttributeValidator.! !

!XMLIDRefAttributeValidator methodsFor: 'validating' stamp: 'monty 1/3/2015 23:24'!
validateAttributeValue: aValue
	self elementIDValidator
		validateIDReference: aValue
		inAttribute: self attribute! !

XMLIDRefAttributeValidator subclass: #XMLIDRefsAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDRefsAttributeValidator commentStamp: 'monty 2/2/2014 08:13' prior: 0!
This class validates attributes with lists of ID ref values separated by spaces.!

!XMLIDRefsAttributeValidator methodsFor: 'validating' stamp: 'monty 1/3/2015 23:24'!
validateAttributeValue: aValue
	self
		listValuesIn: aValue
		do: [:each |
			self elementIDValidator
				validateIDReference: each
				inAttribute: self attribute]! !

XMLAttributeValidator subclass: #XMLNmtokenAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNmtokenAttributeValidator commentStamp: 'monty 12/11/2014 04:41' prior: 0!
This class validates Nmtoken name attributes.!

!XMLNmtokenAttributeValidator methodsFor: 'private' stamp: 'monty 12/11/2014 10:03'!
errorInvalidNmtoken: aName
	XMLValidationException
		formatSignal: 'Invalid Nmtoken name "{1}" in attribute "{2}"'
		with: aName
		with: self attribute! !

!XMLNmtokenAttributeValidator methodsFor: 'validating' stamp: 'monty 12/11/2014 04:37'!
validateAttributeValue: aValue
	self validateNmtoken: aValue! !

!XMLNmtokenAttributeValidator methodsFor: 'validating' stamp: 'monty 12/11/2014 04:37'!
validateNmtoken: aName
	(XMLWellFormedParserTokenizer isNmtoken: aName)
		ifFalse: [self errorInvalidNmtoken: aName]! !

XMLNmtokenAttributeValidator subclass: #XMLNmtokensAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNmtokensAttributeValidator commentStamp: 'monty 12/11/2014 04:41' prior: 0!
This class validates attributes with lists of Nmtoken name values separated by spaces.!

!XMLNmtokensAttributeValidator methodsFor: 'validating' stamp: 'monty 12/11/2014 04:37'!
validateAttributeValue: aValue
	self
		listValuesIn: aValue
		do: [:each | self validateNmtoken: each]! !

Object subclass: #XMLConfiguration
	instanceVariableNames: 'isValidating usesNamespaces requiresDocumentConstraints parserHandlerClass parserLimits entityConfiguration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLConfiguration commentStamp: 'monty 3/17/2015 17:45' prior: 0!
This class stores the shared settings and components for a parser. The configuration values are stored in separate inst vars instead of using a dictionary for speed, and some sub-configurations are lazy initialized only if needed.!

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 16:52'!
defaultParserHandlerClass
	^ SAXHandler! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 17:31'!
documentReadLimit
	^ self parserLimits documentReadLimit! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 17:31'!
documentReadLimit: aLimit
	self parserLimits documentReadLimit: aLimit! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:31'!
entityConfiguration
	^ entityConfiguration ifNil: [entityConfiguration := DTDEntityConfiguration new]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 16:54'!
entityConfiguration: anEntityConfiguration
	entityConfiguration := anEntityConfiguration! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:24'!
externalEntityResolver
	^ self entityConfiguration externalEntityResolver! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:23'!
externalEntityResolver: anExternalEntityResolver
	self entityConfiguration externalEntityResolver: anExternalEntityResolver! !

!XMLConfiguration methodsFor: 'initialization' stamp: 'monty 3/12/2015 13:50'!
initialize
	super initialize.

	"always used, so explicitly initialized"
	isValidating := true.
	usesNamespaces := true.
	requiresDocumentConstraints := false.
	parserLimits := XMLParserLimits new.! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:39'!
isValidating
	^ isValidating! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/29/2013 02:42'!
isValidating: aBoolean
	(isValidating := aBoolean)
		ifTrue: [self preservesUndeclaredEntityReferences: false]
		ifFalse: [self requiresDocumentConstraints: false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 17:34'!
maxEntityReplacementDepth
	^ self parserLimits maxEntityReplacementDepth! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 17:34'!
maxEntityReplacementDepth: aLimit
	self parserLimits maxEntityReplacementDepth: aLimit! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 16:54'!
parserHandlerClass
	^ parserHandlerClass ifNil: [parserHandlerClass := self defaultParserHandlerClass]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:10'!
parserHandlerClass: aClass
	parserHandlerClass := aClass! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:39'!
parserLimits
	^ parserLimits! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 16:53'!
parserLimits: aParserLimits
	parserLimits := aParserLimits! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 04:12'!
preservesUndeclaredEntityReferences
	^ self entityConfiguration preservesUndeclaredEntityReferences! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 06:12'!
preservesUndeclaredEntityReferences: aBoolean
	self entityConfiguration preservesUndeclaredEntityReferences: aBoolean.
	aBoolean
		ifTrue: [self isValidating: false].! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 17:39'!
removeLimits
	self parserLimits removeAll.
	self externalEntityResolver removeLimits.! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 23:55'!
replacesContentEntityReferences
	^ self entityConfiguration replacesContentEntityReferences! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 23:55'!
replacesContentEntityReferences: aBoolean
	self entityConfiguration replacesContentEntityReferences: aBoolean! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:39'!
requiresDocumentConstraints
	^ requiresDocumentConstraints! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/25/2014 21:13'!
requiresDocumentConstraints: aBoolean
	(requiresDocumentConstraints := aBoolean)
		ifTrue: [self isValidating: true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:27'!
resolvesExternalEntities
	^ self entityConfiguration resolvesExternalEntities! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:27'!
resolvesExternalEntities: aBoolean
	self entityConfiguration resolvesExternalEntities: aBoolean! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:39'!
usesNamespaces
	^ usesNamespaces! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:06'!
usesNamespaces: aBoolean
	usesNamespaces := aBoolean! !

XMLConfiguration subclass: #XMLDOMConfiguration
	instanceVariableNames: 'preservesCDataNodes preservesCommentNodes preservesIgnorableWhitespace nodeFactory nodeListClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMConfiguration commentStamp: '<historical>' prior: 0!
This class stores additional settings and components relevant to the DOM parser and DOM nodes.!

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 16:54'!
defaultParserHandlerClass
	^ XMLDOMParser! !

!XMLDOMConfiguration methodsFor: 'initialization' stamp: 'monty 3/12/2015 13:50'!
initialize
	super initialize.

	"always used, so explicitly initialized"
	preservesCDataNodes := false.
	preservesCommentNodes := false.
	preservesIgnorableWhitespace := false.
	nodeFactory := XMLNodeFactory new.
	nodeListClass := XMLNodeList.! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:50'!
nodeFactory
	^ nodeFactory! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:10'!
nodeFactory: aFactory
	nodeFactory := aFactory! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:45'!
nodeListClass
	^ nodeListClass! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:22'!
nodeListClass: aClass
	nodeListClass := aClass! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:45'!
preservesCDataNodes
	^ preservesCDataNodes! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCDataNodes: aBoolean
	preservesCDataNodes := aBoolean! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:45'!
preservesCommentNodes
	^ preservesCommentNodes! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCommentNodes: aBoolean
	preservesCommentNodes := aBoolean! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:45'!
preservesIgnorableWhitespace
	^ preservesIgnorableWhitespace! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:04'!
preservesIgnorableWhitespace: aBoolean
	preservesIgnorableWhitespace := aBoolean! !

Object subclass: #XMLDFAState
	instanceVariableNames: 'nfaStates transitions dfaStateCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLDFAState commentStamp: 'monty 12/28/2013 22:34' prior: 0!
This class is a DFA state made by grouping one or more NFA states.!

!XMLDFAState class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 23:18'!
nfaStates: anNFAStateSet
	^ self
		nfaStates: anNFAStateSet
		dfaStateCache: nil! !

!XMLDFAState class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 23:18'!
nfaStates: anNFAStateSet dfaStateCache: aDFAStateCache
	^ self new
		setNFAStates: anNFAStateSet
		dfaStateCache: aDFAStateCache! !

!XMLDFAState methodsFor: 'enumerating' stamp: 'monty 12/28/2013 00:21'!
detectNextStateMatching: aMatcher
	^ self detectNextStateMatching: aMatcher ifNone: [nil]! !

!XMLDFAState methodsFor: 'enumerating' stamp: 'monty 12/11/2014 05:07'!
detectNextStateMatching: aMatcher ifNone: aBlock
	^ (self transitions
		at: aMatcher
		ifAbsentPut: [self dfaStateMatching: aMatcher])
			ifNil: [aBlock value]! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/11/2014 05:07'!
dfaStateMatching: aMatcher
	| matchingNfaStates |

	matchingNfaStates := XMLNFAStateSet new.
	nfaStates do: [:state |
		(state matches: aMatcher)
			ifTrue: [state nextState addTo: matchingNfaStates]].
	matchingNfaStates isEmpty
		ifTrue: [^ nil].
	^ dfaStateCache dfaStateFor: matchingNfaStates.! !

!XMLDFAState methodsFor: 'testing' stamp: 'monty 12/28/2013 18:54'!
isAccepting
	^ nfaStates isAccepting! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/29/2013 01:10'!
nfaStates
	^ nfaStates! !

!XMLDFAState methodsFor: 'initialization' stamp: 'monty 3/21/2015 23:17'!
setNFAStates: anNFAStateSet dfaStateCache: aDFAStateCache
	nfaStates := anNFAStateSet.
	dfaStateCache := (aDFAStateCache ifNil: [XMLDFAStateCache firstState: self]).! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/28/2013 01:09'!
transitions
	^ transitions ifNil: [transitions := Dictionary new]! !

Object subclass: #XMLDFAStateCache
	instanceVariableNames: 'firstState cachedStates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLDFAStateCache commentStamp: 'monty 12/27/2014 03:19' prior: 0!
This class uses a dictionary with NFAStateSet keys to cache DFA states. (Does not use the regular XML key/value  cache for performance)!

!XMLDFAStateCache class methodsFor: 'instance creation' stamp: 'monty 12/29/2013 01:02'!
firstState: aFirstState
	^ self new firstState: aFirstState! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 03:29'!
clearCache
	cachedStates
		removeAll;
		at: firstState nfaStates
		put: firstState! !

!XMLDFAStateCache methodsFor: 'accessing' stamp: 'monty 12/27/2014 03:12'!
dfaStateFor: aStateSet
	^ cachedStates
		at: aStateSet
		ifAbsent: [
			cachedStates size >= self maxCachedStates
				ifTrue: [self clearCache].

			cachedStates
				at: aStateSet
				put: (self newDFAStateFor: aStateSet)]! !

!XMLDFAStateCache methodsFor: 'accessing' stamp: 'monty 11/19/2014 19:28'!
firstState: aFirstState
	firstState := aFirstState.

	(cachedStates := Dictionary new: 5)
		at: firstState nfaStates
		put: firstState.! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 01:07'!
maxCachedStates
	^ 32! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 03:29'!
newDFAStateFor: aStateSet
	^ XMLDFAState
		nfaStates: aStateSet
		dfaStateCache: self.! !

Object subclass: #XMLDocumentValidator
	instanceVariableNames: 'elementValidatorStack defaultXMLIDValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDocumentValidator commentStamp: '<historical>' prior: 0!
This is a base class for XML document validators.!

XMLDocumentValidator subclass: #DTDDocumentValidator
	instanceVariableNames: 'doctypeDefinition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDocumentValidator commentStamp: '<historical>' prior: 0!
This class validates documents using a DTD.!

!DTDDocumentValidator class methodsFor: 'instance creation' stamp: 'monty 3/2/2015 11:13'!
doctypeDefinition: aDoctypeDefinition
	^ self new doctypeDefinition: aDoctypeDefinition! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:14'!
attributeValidatorsFor: anElement
	^ self doctypeDefinition attributeValidators
		at: anElement
		ifAbsent: [nil]! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:15'!
doctypeDefinition
	^ doctypeDefinition! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:15'!
doctypeDefinition: aDoctypeDefinition
	doctypeDefinition := aDoctypeDefinition! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:14'!
elementIDValidator
	self doctypeDefinition elementIDValidator! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:14'!
elementValidatorFor: aName
	^ (self doctypeDefinition elementValidators
		at: aName
		ifAbsent: [nil]) copy! !

!DTDDocumentValidator methodsFor: 'private' stamp: 'monty 10/31/2014 17:14'!
errorInvalidRoot: anElementName
	XMLValidationException
		formatSignal: 'Element <{1}> cannot be root'
		with: anElementName! !

!DTDDocumentValidator methodsFor: 'testing' stamp: 'monty 3/2/2015 11:14'!
hasAttributeValidators
	^ self doctypeDefinition hasAttributeValidators! !

!DTDDocumentValidator methodsFor: 'testing' stamp: 'monty 3/2/2015 11:14'!
hasElementValidators
	^ self doctypeDefinition hasElementValidators! !

!DTDDocumentValidator methodsFor: 'validating' stamp: 'monty 10/30/2014 19:33'!
unvalidatedAttributes: anAttributeCollection
	XMLValidationException
		formatSignal: 'Undeclared attribute "{1}"'
		with: anAttributeCollection anyOne! !

!DTDDocumentValidator methodsFor: 'validating' stamp: 'monty 10/30/2014 19:33'!
unvalidatedElement: aName
	XMLValidationException
		formatSignal: 'Undeclared element <{1}>'
		with: aName! !

!DTDDocumentValidator methodsFor: 'validating' stamp: 'monty 3/2/2015 11:14'!
validateIDReferences
	self doctypeDefinition hasElementIDValidator
		ifTrue: [self doctypeDefinition elementIDValidator validateIDReferences]! !

!DTDDocumentValidator methodsFor: 'validating' stamp: 'monty 3/2/2015 11:14'!
validateRootElement: anElementName
	(self doctypeDefinition root notEmpty
		and: [self doctypeDefinition root ~= anElementName])
		ifTrue: [self errorInvalidRoot: anElementName]! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/19/2013 01:04'!
attributeValidatorsFor: anElement
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 12/11/2014 08:30'!
currentElementValidator
	^ self elementValidatorStack lastOrNil! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 20:07'!
defaultElementValidatorFor: aName
	^ XMLAnyElementValidator element: aName! !

!XMLDocumentValidator methodsFor: 'private' stamp: 'monty 3/22/2015 04:03'!
defaultXMLIDValidator
	"used to validate xml:id attributes in the absence of DTD or other validators"
	^ defaultXMLIDValidator
		ifNil: [
			defaultXMLIDValidator :=
				XMLXMLIDIDAttributeValidator
					element: ''
					attribute: 'xml:id'
					defaultValueValidator:
						(XMLImpliedValueAttributeValidator attribute: 'xml:id').
			defaultXMLIDValidator
				elementIDValidator: XMLElementIDValidator new;
				yourself]! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:57'!
elementValidatorFor: aName
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:46'!
elementValidatorStack
	^ elementValidatorStack ifNil: [elementValidatorStack := OrderedCollection new]! !

!XMLDocumentValidator methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorMissingContentFor: aName
	XMLValidationException
		formatSignal: 'Element <{1}> terminated prematurely'
		with: aName! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 03:31'!
hasAttributeValidators
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 03:32'!
hasElementValidators
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/2/2014 22:18'!
hasValidators
	^ self hasElementValidators or: [self hasAttributeValidators]! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 12/21/2013 20:29'!
unvalidatedAttributes: anAttributeCollection! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 12/21/2013 20:29'!
unvalidatedElement: aName! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 12/11/2014 05:41'!
validateAttributes: anAttributeDictionary with: aValidatorDictionary
	| unvalidatedAttributes |

	unvalidatedAttributes := Set new: anAttributeDictionary size.
	anAttributeDictionary keysDo: [:each | unvalidatedAttributes add: each].
	aValidatorDictionary
		ifNotNil: [
			aValidatorDictionary keysAndValuesDo: [:attribute :validator |
				validator validateAttributes: anAttributeDictionary.
				unvalidatedAttributes
					remove: attribute
					ifAbsent: nil]].

	unvalidatedAttributes isEmpty
		ifFalse: [self unvalidatedAttributes: unvalidatedAttributes].! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:28'!
validateEndDocument
	self validateIDReferences! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 12/7/2013 19:45'!
validateEndTag: aName
	self hasElementValidators
		ifTrue: [
			(self currentElementValidator element = aName)
				ifTrue: [
					self currentElementValidator atEnd
						ifFalse: [self errorMissingContentFor: aName]].
			self elementValidatorStack removeLast]! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:24'!
validateIDReferences
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 11/18/2013 03:44'!
validatePCData: aString
	self hasElementValidators
		ifTrue: [self currentElementValidator validatePCData: aString]! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 11/2/2014 20:22'!
validateReservedAttributes: aDictionary for: aStartTag! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 12/21/2013 20:08'!
validateRootElement: anElementName! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 12/11/2014 08:31'!
validateStartTag: aName
	self currentElementValidator
		ifNotNil: [:currentValidator | currentValidator validateStartTag: aName].

	self elementValidatorStack addLast:
		((self elementValidatorFor: aName)
			ifNil: [
				self
					unvalidatedElement: aName;
					defaultElementValidatorFor: aName]).! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 12/29/2013 05:05'!
validateStartTag: aName attributes: anAttributeDictionary
	(self hasElementValidators or: [self hasAttributeValidators])
		ifFalse: [^ self].

	self validateStartTag: aName.
	self
		validateAttributes: anAttributeDictionary
		with: (self attributeValidatorsFor: aName).! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 23:23'!
validateXMLIDAttributeIn: anAttributeDictionary
	self hasAttributeValidators
		ifFalse: [self defaultXMLIDValidator validateAttributes: anAttributeDictionary]! !

Object subclass: #XMLElementIDValidator
	instanceVariableNames: 'elementsAndIdAttributes idReferences ids'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLElementIDValidator commentStamp: 'monty 1/3/2015 04:41' prior: 0!
This class is shared by attribute validators to validate element IDs and ID references, including xml:id IDs.!

!XMLElementIDValidator methodsFor: 'accessing' stamp: 'monty 4/6/2015 10:27'!
clearIDsAndIDReferences
	"Gemstone's Set does not understand removeAll, so nil reassignment
	is used instead (will be lazy initialized by accessors) "
	ids := nil.
	idReferences := nil.! !

!XMLElementIDValidator methodsFor: 'accessing' stamp: 'monty 3/2/2015 12:53'!
elementsAndIDAttributes
	^ elementsAndIdAttributes ifNil: [elementsAndIdAttributes := Dictionary new]! !

!XMLElementIDValidator methodsFor: 'private' stamp: 'monty 3/22/2015 03:50'!
errorDuplicateID: anID inAttribute: anAttribute
	XMLValidationException
		formatSignal: 'Duplicate element ID "{1}" used in attribute "{2}"'
		with: anID
		with: anAttribute! !

!XMLElementIDValidator methodsFor: 'private' stamp: 'monty 3/22/2015 03:50'!
errorInvalidID: anID inAttribute: anAttribute
	XMLValidationException
		formatSignal: 'Invalid element ID value "{1}" in attribute "{2}"'
		with: anID
		with: anAttribute! !

!XMLElementIDValidator methodsFor: 'private' stamp: 'monty 1/1/2015 19:00'!
errorMultipleIDAttributesForElement: anElement
	XMLValidationException
		formatSignal: 'Element <{1}> has multiple ID attributes'
		with: anElement! !

!XMLElementIDValidator methodsFor: 'private' stamp: 'monty 3/22/2015 03:50'!
errorUnknownIDReference: anID
	XMLValidationException
		formatSignal: 'Reference to unknown element ID "{1}"'
		with: anID! !

!XMLElementIDValidator methodsFor: 'accessing' stamp: 'monty 1/3/2015 23:25'!
idReferences
	^ idReferences ifNil: [idReferences := Set new]! !

!XMLElementIDValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 18:54'!
ids
	^ ids ifNil: [ids := Set new]! !

!XMLElementIDValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 03:50'!
validateID: anID inAttribute: anAttribute
	(XMLWellFormedParserTokenizer isName: anID)
		ifFalse: [
			self
				errorInvalidID: anID
				inAttribute: anAttribute].
	self
		validateUniquenessOfID: anID
		inAttribute: anAttribute.! !

!XMLElementIDValidator methodsFor: 'validating' stamp: 'monty 3/2/2015 12:53'!
validateIDAttribute: anAttribute forElement: anElement
	| oldSize |

	"checking size after is faster than includes test"
	oldSize := self elementsAndIDAttributes size.
	(self elementsAndIDAttributes
		at: anElement
		put: anAttribute;
		size) > oldSize
		ifFalse: [self errorMultipleIDAttributesForElement: anElement].! !

!XMLElementIDValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 03:50'!
validateIDReference: anID inAttribute: anAttribute
	(XMLWellFormedParserTokenizer isName: anID)
		ifFalse: [
			self
				errorInvalidID: anID
				inAttribute: anAttribute].
	self idReferences add: anID.! !

!XMLElementIDValidator methodsFor: 'validating' stamp: 'monty 1/3/2015 23:24'!
validateIDReferences
	self idReferences do: [:each |
		(self ids includes: each)
			ifFalse: [self errorUnknownIDReference: each]]! !

!XMLElementIDValidator methodsFor: 'private' stamp: 'monty 3/22/2015 03:50'!
validateUniquenessOfID: anID inAttribute: anAttribute
	| oldSize |

	"checking size after is faster than includes test"
	oldSize := self ids size.
	(self ids
		add: anID;
		size) > oldSize
		ifFalse: [
			self
				errorDuplicateID: anID
				inAttribute: anAttribute].! !

!XMLElementIDValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 03:50'!
validateXMLID: anID
	(XMLWellFormedParserTokenizer isNCName: anID)
		ifFalse: [
			self
				errorInvalidID: anID
				inAttribute: 'xml:id'].
	self
		validateUniquenessOfID: anID
		inAttribute: 'xml:id'.! !

Object subclass: #XMLElementNester
	instanceVariableNames: 'openElements rootElement hasExternalRootElement scopedAttributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLElementNester commentStamp: 'monty 12/27/2014 03:06' prior: 0!
This is a base class for element nesters that are used by parser drivers to keep track of which elements are open while parsing.!

!XMLElementNester methodsFor: 'private' stamp: 'monty 1/25/2014 21:08'!
copiedScopedAttributes
	| copiedAttributes |

	copiedAttributes :=
		self hasScopedAttributes
			ifTrue: [self scopedAttributes removeLast copy]
			ifFalse: [Dictionary new].
	self scopedAttributes addLast: copiedAttributes.

	^ copiedAttributes.! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 10/7/2014 03:16'!
currentElement
	^ self openElements lastOrNil! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 10/27/2014 09:09'!
currentScopedAttributes
	^ self scopedAttributes lastOrNil! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 10/22/2014 13:56'!
enclosedElementsAsString: anElement
	^ String streamContents: [:stream |
		self
			printFrom: (self openElements lastIndexOf: anElement) + 1
			on: stream]! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 1/20/2014 19:59'!
endElement: anElementName
	self openElements removeLast.
	self hasScopedAttributes
		ifTrue: [scopedAttributes removeLast].! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 2/2/2014 02:45'!
endNesting! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 11/3/2014 13:20'!
hasExternalRootElement
	^ hasExternalRootElement! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 12/6/2013 17:29'!
hasOpenElement: anElementName
	^ self openElements includes: anElementName! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 11/3/2014 13:18'!
hasOpenElements
	^ self openElements size > 0! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 1/20/2014 19:59'!
hasScopedAttributes
	^ scopedAttributes notNil and: [scopedAttributes notEmpty]! !

!XMLElementNester methodsFor: 'initialization' stamp: 'monty 11/3/2014 13:20'!
initialize
	super initialize.

	openElements := OrderedCollection new: 10.
	hasExternalRootElement := false.! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 2/10/2014 02:12'!
isCurrentElementRoot
	^ self openElements size = 1! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 11/3/2014 13:19'!
isInElement
	^ self openElements size > 0 or: [self hasExternalRootElement]! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 10/27/2014 09:07'!
openElements
	^ openElements! !

!XMLElementNester methodsFor: 'enumerating' stamp: 'monty 12/11/2014 05:29'!
openElementsReverseDo: aBlock
	"reimplemented so it is always safe to remove at the end while scanning backwards"
	| i |

	i := self openElements size.
	[i > 0]
		whileTrue: [
			aBlock value: (self openElements at: i).
			i := i - 1].! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 10/22/2014 13:52'!
printFrom: aStart on: aStream
	self
		printFrom: aStart
		to: aStart + self printLimit - 1
		on: aStream! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 10/23/2014 12:04'!
printFrom: aStart to: aStop on: aStream
	aStart to: aStop do: [:i |
		i > self openElements size
			ifTrue: [^ self].
		i > aStart
			ifTrue: [aStream nextPutAll: ', '].

		aStream
			nextPut: $<;
			nextPutAll: (self openElements at: i);
			nextPut: $>].

	(aStop < self openElements size)
		ifTrue: [aStream nextPutAll: '...'].! !

!XMLElementNester methodsFor: 'private' stamp: 'monty 10/22/2014 13:52'!
printLimit
	^ 5! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 10/22/2014 13:52'!
printOn: aStream
	self
		printFrom: 1
		on: aStream! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 1/20/2014 20:00'!
scopedAttributeAt: aName
	^ self
		scopedAttributeAt: aName
		ifAbsent: ['']! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 1/21/2014 02:46'!
scopedAttributeAt: aName ifAbsent: aBlock
	^ self currentScopedAttributes
		ifNil: [aBlock value]
		ifNotNil: [:attributes |
			attributes
				at: aName
				ifAbsent: aBlock]! !

!XMLElementNester methodsFor: 'private' stamp: 'monty 1/20/2014 19:33'!
scopedAttributes
	^ scopedAttributes ifNil: [scopedAttributes := OrderedCollection new]! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 11/3/2014 13:20'!
setExternalRootElement: anElement
	rootElement := anElement.
	hasExternalRootElement := true.! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 1/5/2015 07:16'!
setScopedAttributesNamed: aNameCollection from: aDictionary
	| attributes |

	aNameCollection do: [:each |
		aDictionary
			at: each
			ifPresent: [:value |
				"used to copy only once if needed"
				(attributes ifNil: [attributes := self copiedScopedAttributes])
					at: each
					put: value]]! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 1/20/2014 19:59'!
startElement: anElementName
	self openElements addLast: anElementName.
	self hasScopedAttributes
		ifTrue: [scopedAttributes addLast: self currentScopedAttributes].! !

XMLElementNester subclass: #XMLWellFormedElementNester
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLWellFormedElementNester commentStamp: '<historical>' prior: 0!
The class keeps track of elements as they start and end and ensures that they nest properly.!

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 12/11/2013 21:43'!
endElement: anElementName
	(self currentElement = anElementName)
		ifFalse: [
			^ (self hasOpenElement: anElementName)
				ifTrue: [self errorImproperlyNestedElements: anElementName]
				ifFalse: [self errorUnmatchedEndTag: anElementName]].

	super endElement: anElementName.! !

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 11/3/2014 13:25'!
endNesting
	self hasOpenElements
		ifTrue: [self errorUnterminatedElements]! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorImproperlyNestedElements: anElementName
	XMLWellFormednessException
		formatSignal: '<{1}> element ended before its child element(s) {2}'
		with: anElementName
		with: (self enclosedElementsAsString: anElementName)! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 11/3/2014 13:25'!
errorMultipleRoots
	XMLWellFormednessException signal: 'Document cannot have multiple root elements'! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorUnmatchedEndTag: anElementName
	XMLWellFormednessException
		formatSignal: 'End </{1}> tag with no corresponding start <{1}> tag'
		with: anElementName! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 11/3/2014 13:26'!
errorUnterminatedElements
	XMLWellFormednessException signal: 'Unterminated elements: ', self printString! !

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 11/3/2014 13:29'!
startElement: anElementName
	rootElement
		ifNil: [rootElement := anElementName]
		ifNotNil: [
			(self openElements isEmpty
				and: [self hasExternalRootElement not])
				ifTrue: [self errorMultipleRoots]].

	super startElement: anElementName.! !

Object subclass: #XMLElementValidator
	instanceVariableNames: 'element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLElementValidator commentStamp: '<historical>' prior: 0!
This is a base class for element content validators.!

XMLElementValidator subclass: #XMLAnyElementValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAnyElementValidator commentStamp: '<historical>' prior: 0!
This class validates elements that can contain any content.!

!XMLAnyElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:42'!
atEnd
	^ true! !

XMLElementValidator subclass: #XMLDFAElementValidator
	instanceVariableNames: 'dfa currentState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDFAElementValidator commentStamp: '<historical>' prior: 0!
This class can use a DFA to validate element content.
!

!XMLDFAElementValidator class methodsFor: 'instance creation' stamp: 'monty 3/22/2015 04:05'!
element: aName dfa: aDFA
	^ self new
		setElement: aName
		dfa: aDFA! !

!XMLDFAElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 05:43'!
atEnd
	^ currentState isAccepting! !

!XMLDFAElementValidator methodsFor: 'copying' stamp: 'monty 11/18/2013 05:41'!
postCopy
	currentState := dfa! !

!XMLDFAElementValidator methodsFor: 'accessing' stamp: 'monty 3/22/2015 04:04'!
setElement: aName dfa: aDFA
	element := aName.
	dfa := aDFA.
	currentState := dfa.! !

!XMLDFAElementValidator methodsFor: 'validating' stamp: 'monty 12/28/2013 00:03'!
validatePCData: aString
	self errorUnexpectedPCData: aString! !

!XMLDFAElementValidator methodsFor: 'validating' stamp: 'monty 12/28/2013 00:21'!
validateStartTag: aName
	currentState :=
		currentState
			detectNextStateMatching: aName
			ifNone: [self errorUnexpectedElement: aName]! !

!XMLElementValidator class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:48'!
element: aName
	^ self new element: aName! !

!XMLElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:41'!
atEnd
	self subclassResponsibility! !

!XMLElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:48'!
element
	^ element! !

!XMLElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:48'!
element: aName
	element := aName! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorUnexpectedElement: aName
	XMLValidationException
		formatSignal: 'Unexpected <{1}> element in <{2}> element'
		with: aName
		with: self element! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorUnexpectedPCData: aString
	XMLValidationException
		formatSignal: 'Unexpected PCDATA in <{1}> element'
		with: self element! !

!XMLElementValidator methodsFor: 'validating' stamp: 'monty 11/18/2013 05:24'!
validatePCData: aString! !

!XMLElementValidator methodsFor: 'validating' stamp: 'monty 11/18/2013 05:24'!
validateStartTag: aName! !

XMLElementValidator subclass: #XMLEmptyElementValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEmptyElementValidator commentStamp: '<historical>' prior: 0!
This class validates empty elements.
!

!XMLEmptyElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:41'!
atEnd
	^ true! !

!XMLEmptyElementValidator methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorMustBeEmpty
	XMLValidationException
		formatSignal: 'Element <{1}> must be empty'
		with: self element! !

!XMLEmptyElementValidator methodsFor: 'validating' stamp: 'monty 11/18/2013 04:32'!
validatePCData: aString
	self errorMustBeEmpty! !

!XMLEmptyElementValidator methodsFor: 'validating' stamp: 'monty 11/18/2013 04:34'!
validateStartTag: aName
	self errorMustBeEmpty! !

XMLElementValidator subclass: #XMLMixedContentElementValidator
	instanceVariableNames: 'allowedElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLMixedContentElementValidator commentStamp: '<historical>' prior: 0!
This class validates elements that can contain a mix of PCDATA and certain elements.!

!XMLMixedContentElementValidator class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:50'!
element: aName allowedElements: aNameCollection
	^ self new
		setElement: aName
		allowedElements: aNameCollection! !

!XMLMixedContentElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:51'!
allowedElements
	^ allowedElements! !

!XMLMixedContentElementValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 01:14'!
atEnd
	^ true! !

!XMLMixedContentElementValidator methodsFor: 'initialization' stamp: 'monty 12/7/2013 19:50'!
setElement: aName allowedElements: aNameCollection
	element := aName.
	allowedElements := aNameCollection.! !

!XMLMixedContentElementValidator methodsFor: 'validating' stamp: 'monty 12/7/2013 19:51'!
validateStartTag: aName
	(self allowedElements includes: aName)
		ifFalse: [self errorUnexpectedElement: aName]
	! !

Object subclass: #XMLIntegerReader
	instanceVariableNames: 'stream'
	classVariableNames: 'CharacterDigitValues'
	poolDictionaries: ''
	category: 'XML-Parser-Util'!

!XMLIntegerReader commentStamp: 'monty 1/21/2014 01:36' prior: 0!
This class reads hex or decimal integers from a minimal stream supporting next, peek, and atEnd.!

!XMLIntegerReader class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 07:04'!
initialize
	"self initialize"

	"array instead of dictionary for speed"
	CharacterDigitValues := Array new: 256 withAll: -1.
	($0 to: $9) do: [:each |
		CharacterDigitValues
			at: each asciiValue
			put: each asciiValue - $0 asciiValue].
	($A to: $F) do: [:each |
		CharacterDigitValues
			at: each asciiValue
			put: each asciiValue - $A asciiValue + 10].
	($a to: $f) do: [:each |
		CharacterDigitValues
			at: each asciiValue
			put: each asciiValue - $a asciiValue + 10].! !

!XMLIntegerReader class methodsFor: 'private' stamp: 'monty 10/24/2014 20:28'!
maxBase
	^ 16! !

!XMLIntegerReader class methodsFor: 'private' stamp: 'monty 10/24/2014 20:15'!
maxIntegerValue
	"SmallInteger maxVal"
	^ 1073741823! !

!XMLIntegerReader class methodsFor: 'instance creation' stamp: 'monty 11/16/2014 04:55'!
on: aStream
	^ self new on: aStream! !

!XMLIntegerReader class methodsFor: 'instance creation' stamp: 'monty 10/27/2014 08:36'!
readFrom: aStream
	^ (self on: aStream) nextInteger! !

!XMLIntegerReader class methodsFor: 'reading' stamp: 'monty 10/27/2014 08:37'!
readFrom: aStream withBase: aBase
	^ (self on: aStream) nextIntegerWithBase: aBase! !

!XMLIntegerReader class methodsFor: 'reading' stamp: 'monty 10/27/2014 08:43'!
readFrom: aStream withBase: aFirstBase startingWithAny: aCharacterCollection orBase: aSecondBase
	^ (self on: aStream)
		nextIntegerWithBase: aFirstBase
		startingWithAny: aCharacterCollection
		orBase: aSecondBase! !

!XMLIntegerReader class methodsFor: 'reading' stamp: 'monty 1/21/2014 17:42'!
readFrom: aStream withBase: aBase upTo: anInteger
	^ (self on: aStream)
		nextIntegerWithBase: aBase
		upTo: anInteger! !

!XMLIntegerReader methodsFor: 'testing' stamp: 'monty 10/27/2014 08:39'!
atEnd
	^ stream atEnd! !

!XMLIntegerReader methodsFor: 'accessing' stamp: 'monty 10/27/2014 08:39'!
next
	^ stream next! !

!XMLIntegerReader methodsFor: 'private' stamp: 'monty 10/27/2014 08:40'!
nextDigitWithBase: aBase
	| digit |
	
	digit :=
		CharacterDigitValues
			at: self peek asciiValue
			ifAbsent: [^ nil].
	(digit >= 0 and: [digit < aBase])
		ifFalse: [^ nil].
	self next.
	^ digit.! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 10/27/2014 08:37'!
nextInteger
	^ self nextIntegerWithBase: 10! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 10/27/2014 08:35'!
nextIntegerWithBase: aBase
	^ self nextIntegerWithBase: aBase upTo: nil! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 10/27/2014 08:44'!
nextIntegerWithBase: aFirstBase startingWithAny: aCharacterCollection orBase: aSecondBase
	(aCharacterCollection includes: self peek)
		ifTrue: [
			self next.
			^ self nextIntegerWithBase: aFirstBase]
		ifFalse: [^ self nextIntegerWithBase: aSecondBase]! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 11/14/2014 20:15'!
nextIntegerWithBase: aBase upTo: aReadLimit
	| integer maxValue maxBaseValue digitValue readLimit |

	(aBase > 1 and: [aBase <= self class maxBase])
		ifFalse: [Error signal: 'Unsupported base ', aBase printString].

	maxValue := self class maxIntegerValue.
	maxBaseValue := (maxValue / aBase) floor.
	readLimit := aReadLimit.
	[self atEnd
		or: [(readLimit notNil and: [readLimit <= 0])
			or: [(digitValue := self nextDigitWithBase: aBase) isNil]]]
		whileFalse: [
			integer ifNil: [integer := 0].
	
			integer > maxBaseValue
				ifTrue: [^ maxValue].
			integer := integer * aBase.

			integer > (maxValue - digitValue)
				ifTrue: [^ maxValue].
			integer := integer + digitValue.
			readLimit ifNotNil: [readLimit := readLimit - 1]].
	^ integer.! !

!XMLIntegerReader methodsFor: 'initialization' stamp: 'monty 11/16/2014 04:54'!
on: aStream
	stream := aStream! !

!XMLIntegerReader methodsFor: 'accessing' stamp: 'monty 10/27/2014 08:39'!
peek
	^ stream peek! !

Object subclass: #XMLKeyValueCache
	instanceVariableNames: 'collection maxSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Util'!

!XMLKeyValueCache commentStamp: 'monty 11/11/2014 23:48' prior: 0!
A simple key-value cache that empties when full.
!

!XMLKeyValueCache class methodsFor: 'instance creation' stamp: 'monty 11/15/2014 20:57'!
collection: aCollection maxSize: anInteger
	^ self basicNew initialize
		setCollection: aCollection
		maxSize: anInteger! !

!XMLKeyValueCache class methodsFor: 'private' stamp: 'monty 11/20/2014 02:25'!
defaultMaxSize
	^ 16! !

!XMLKeyValueCache class methodsFor: 'instance creation' stamp: 'monty 11/15/2014 20:56'!
maxSize: anInteger
	^ self
		collection: (Dictionary new: anInteger)
		maxSize: anInteger! !

!XMLKeyValueCache class methodsFor: 'instance creation' stamp: 'monty 11/15/2014 20:56'!
new
	^ self maxSize: self defaultMaxSize! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 21:01'!
at: aKey
	^ self collection at: aKey! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 21:01'!
at: aKey ifAbsent: aBlock
	^ self collection at: aKey ifAbsent: aBlock! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 21:02'!
at: aKey ifAbsentPut: aBlock
	^ self collection
		at: aKey
		ifAbsent: [
			self collection size >= self maxSize
				ifTrue: [self clear].

			self collection
				at: aKey
				put: aBlock value]! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/16/2014 00:04'!
at: aKey put: aValue
	(self collection includesKey: aKey)
		ifFalse: [
			self collection size >= self maxSize
				ifTrue: [self clear]].
	^ self collection at: aKey put: aValue.! !

!XMLKeyValueCache methodsFor: 'clearing' stamp: 'monty 11/15/2014 20:59'!
clear
	self collection removeAll! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/11/2014 23:45'!
collection
	^ collection! !

!XMLKeyValueCache methodsFor: 'testing' stamp: 'monty 11/15/2014 23:59'!
includesKey: aKey
	^ self collection includesKey: aKey! !

!XMLKeyValueCache methodsFor: 'testing' stamp: 'monty 11/12/2014 04:03'!
isEmpty
	^ self collection isEmpty! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:57'!
maxSize
	^ maxSize! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:59'!
maxSize: anInteger
	anInteger < maxSize
		ifTrue: [self clear].

	maxSize := anInteger.! !

!XMLKeyValueCache methodsFor: 'copying' stamp: 'monty 11/16/2014 07:27'!
postCopy
	collection := collection copy! !

!XMLKeyValueCache methodsFor: 'initialization' stamp: 'monty 11/15/2014 20:57'!
setCollection: aCollection maxSize: anInteger
	collection := aCollection.
	maxSize := anInteger.! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:48'!
size
	^ self collection size! !

Object subclass: #XMLNFABuilder
	instanceVariableNames: 'fragmentStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFABuilder commentStamp: 'monty 1/7/2014 19:39' prior: 0!
This class builds an NFA using a stack of NFAFragments.!

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 01:26'!
applyBranch
	| branchState leftFragment  rightFragment |

	leftFragment := self popFragment.
	rightFragment := self popFragment.
	branchState :=
		XMLNFABranchingState
			leftBranch: leftFragment firstState
			rightBranch: rightFragment firstState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchState
			terminalStates: (leftFragment combinedTerminalStatesFrom: rightFragment)).! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 10/24/2014 12:44'!
applyConcatenation
	| firstFragment secondFragment |

	secondFragment := self popFragment.
	firstFragment := self popFragment.

	firstFragment connectToFragment: secondFragment.

	self pushFragment: firstFragment.! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 10/24/2014 13:07'!
applyOptional
	| optionalFragment branchingState |

	optionalFragment := self popFragment.
	branchingState :=
		XMLNFABranchingState branch: optionalFragment firstState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchingState
			terminalStates: (optionalFragment combinedTerminalStatesWith: branchingState))! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 01:19'!
applyPlus
	| plusFragment branchState |

	plusFragment := self topFragment.
	branchState := XMLNFABranchingState branch: plusFragment firstState.
	plusFragment
		connectToState: branchState;
		terminalState: branchState.! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 23:55'!
applyStar
	| branchState starFragment |

	starFragment := self popFragment.
	branchState := XMLNFABranchingState branch: starFragment firstState.
	starFragment connectToState: branchState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchState
			terminalState: branchState)! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 22:51'!
firstStateOfAcceptingNFA
	| firstFragment |

	[self totalFragments > 1]
		whileTrue: [self applyConcatenation].

	firstFragment := self popFragment.
	firstFragment connectToState: XMLNFAAcceptingState new.

	^ firstFragment firstState.! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 14:16'!
fragmentStack
	^ fragmentStack ifNil: [fragmentStack := OrderedCollection new]! !

!XMLNFABuilder methodsFor: 'testing' stamp: 'monty 12/6/2013 06:40'!
hasFragments
	^ self fragmentStack notEmpty! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 08:01'!
popFragment
	^ self fragmentStack removeLast! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 08:01'!
pushFragment: aFragment
	self fragmentStack addLast: aFragment! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 21:41'!
pushMatcher: aMatcher
	| firstState |

	firstState := XMLNFAMatchingState matcher: aMatcher.
	self pushFragment:
		(XMLNFAFragment
			firstState: firstState
			terminalState: firstState).! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 11/17/2013 12:01'!
topFragment
	^ self fragmentStack last! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 15:04'!
totalFragments
	^ self fragmentStack size! !

Object subclass: #XMLNFAFragment
	instanceVariableNames: 'firstState terminalStates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAFragment commentStamp: 'monty 12/28/2013 21:51' prior: 0!
This is a fragment used to build an NFA by the NFABuilder.!

!XMLNFAFragment class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:50'!
firstState: aFirstState terminalState: aTerminalState
	^ self new
		firstState: aFirstState;
		terminalState: aTerminalState! !

!XMLNFAFragment class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:50'!
firstState: aFirstState terminalStates: aTerminalStateCollection
	^ self new
		firstState: aFirstState;
		terminalStates: aTerminalStateCollection! !

!XMLNFAFragment methodsFor: 'converting' stamp: 'monty 10/24/2014 13:21'!
combinedTerminalStatesFrom: aFragment
	^ terminalStates isCollection
		ifTrue: [aFragment terminalStatesAddedTo: terminalStates]
		ifFalse: [
			aFragment terminalStates isCollection
				ifTrue: [self terminalStatesAddedTo: aFragment terminalStates]
				ifFalse: [
					self
						terminalStateCollectionWith: terminalStates
						and: aFragment terminalStates]]! !

!XMLNFAFragment methodsFor: 'converting' stamp: 'monty 10/24/2014 13:07'!
combinedTerminalStatesWith: aState
	^ terminalStates isCollection
		ifTrue: [
			terminalStates
				addLast: aState;
				yourself]
		ifFalse: [self terminalStateCollectionWith: terminalStates and: aState]! !

!XMLNFAFragment methodsFor: 'connecting' stamp: 'monty 10/24/2014 12:45'!
connectToFragment: aFragment
	self connectToState: aFragment firstState.
	terminalStates := aFragment terminalStates.! !

!XMLNFAFragment methodsFor: 'connecting' stamp: 'monty 10/24/2014 13:06'!
connectToState: aState
	terminalStates isCollection
		ifTrue: [terminalStates do: [:each | each connectToState: aState]]
		ifFalse: [terminalStates connectToState: aState]! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 11/17/2013 10:48'!
firstState
	^ firstState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 11/17/2013 11:51'!
firstState: aState
	firstState := aState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 10/24/2014 12:35'!
terminalState: aTerminalState
	terminalStates := aTerminalState! !

!XMLNFAFragment methodsFor: 'private' stamp: 'monty 10/24/2014 12:52'!
terminalStateCollectionWith: aFirstState and: aSecondState
	^ (OrderedCollection new: 4)
		addLast: aFirstState;
		addLast: aSecondState;
		yourself! !

!XMLNFAFragment methodsFor: 'private' stamp: 'monty 10/24/2014 12:40'!
terminalStates
	^ terminalStates! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 12/27/2013 23:50'!
terminalStates: aTerminalStateCollection
	terminalStates := aTerminalStateCollection! !

!XMLNFAFragment methodsFor: 'private' stamp: 'monty 10/24/2014 13:21'!
terminalStatesAddedTo: aCollection
	terminalStates isCollection
		ifTrue: [aCollection addAll: terminalStates]
		ifFalse: [aCollection addLast: terminalStates].
	^ aCollection.! !

Object subclass: #XMLNFAState
	instanceVariableNames: 'lastAddedTo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAState commentStamp: 'monty 12/28/2013 22:34' prior: 0!
This is a base class for NFA states.!

XMLNFAState subclass: #XMLNFAAcceptingState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAAcceptingState commentStamp: 'monty 10/7/2014 01:27' prior: 0!
An NFA in this state is considered to accept the input.!

!XMLNFAAcceptingState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:23'!
isAccepting
	^ true! !

XMLNFAState subclass: #XMLNFABranchingState
	instanceVariableNames: 'leftBranch rightBranch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFABranchingState commentStamp: 'monty 10/7/2014 01:27' prior: 0!
This state can branch off into two different states without matching against anything.!

!XMLNFABranchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:43'!
branch: aLeftBranch
	^ self new
		setLeftBranch: aLeftBranch
		rightBranch: nil! !

!XMLNFABranchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 22:04'!
leftBranch: aLeftBranch rightBranch: aRightBranch
	^ self new
		setLeftBranch: aLeftBranch
		rightBranch: aRightBranch! !

!XMLNFABranchingState methodsFor: 'adding' stamp: 'monty 12/29/2013 01:10'!
addTo: aStateSet
	lastAddedTo == aStateSet
		ifFalse: [
			"This avoids getting caught in a loop"
			lastAddedTo := aStateSet.

			leftBranch addTo: aStateSet.
			rightBranch addTo: aStateSet]! !

!XMLNFABranchingState methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:45'!
connectToState: aState
	leftBranch ifNil: [leftBranch := aState].
	rightBranch ifNil: [rightBranch := aState].! !

!XMLNFABranchingState methodsFor: 'initialization' stamp: 'monty 12/28/2013 00:45'!
setLeftBranch: aLeftBranch rightBranch: aRightBranch
	leftBranch := aLeftBranch.
	rightBranch := aRightBranch.! !

XMLNFAState subclass: #XMLNFAMatchingState
	instanceVariableNames: 'matcher nextState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAMatchingState commentStamp: 'monty 10/9/2014 20:58' prior: 0!
This state can pass to a next state if the input matches against an object.!

!XMLNFAMatchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 21:15'!
matcher: aMatcher
	^ self new matcher: aMatcher! !

!XMLNFAMatchingState methodsFor: 'connecting' stamp: 'monty 12/28/2013 21:52'!
connectToState: aState
	nextState := aState! !

!XMLNFAMatchingState methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:16'!
matcher: aMatcher
	matcher := aMatcher! !

!XMLNFAMatchingState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:15'!
matches: anObject
	^ matcher = anObject! !

!XMLNFAMatchingState methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:13'!
nextState
	^ nextState! !

!XMLNFAState methodsFor: 'adding' stamp: 'monty 12/29/2013 01:10'!
addTo: aStateSet
	lastAddedTo == aStateSet
		ifFalse: [
			"This avoids getting caught in a loop"
			lastAddedTo := aStateSet.

			aStateSet add: self]! !

!XMLNFAState methodsFor: 'converting' stamp: 'monty 12/28/2013 22:33'!
asDFAState
	| nfaStates |

	nfaStates := XMLNFAStateSet new.
	self addTo: nfaStates.

	^ XMLDFAState nfaStates: nfaStates.! !

!XMLNFAState methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:45'!
connectToState: aState! !

!XMLNFAState methodsFor: 'testing' stamp: 'monty 1/7/2014 19:37'!
isAccepting
	^ false! !

!XMLNFAState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:10'!
matches: anObject
	^ false! !

Object subclass: #XMLNamespaceScope
	instanceVariableNames: 'prefixMappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScope commentStamp: 'monty 12/29/2014 01:12' prior: 0!
A namespace scope is a possibly empty set of mappings from namespace prefixes to URIs and an optional default namespace URI to be used for unprefixed elements. Scopes can be created from other scopes by copying. (Uses copy-on-write to reduce memory use.)!

!XMLNamespaceScope class methodsFor: 'accessing' stamp: 'monty 10/8/2014 23:27'!
xmlNamespaceURI
	^ 'http://www.w3.org/XML/1998/namespace'! !

!XMLNamespaceScope class methodsFor: 'accessing' stamp: 'monty 10/8/2014 23:27'!
xmlnsNamespaceURI
	^ 'http://www.w3.org/2000/xmlns/'! !

!XMLNamespaceScope methodsFor: 'comparing' stamp: 'monty 10/12/2014 06:32'!
= anObject
	self == anObject
		ifTrue: [^ true].
	self class == anObject class
		ifFalse: [^ false].

	^ self hasNamespaces
		ifTrue: [anObject hasNamespaces and: [prefixMappings = anObject prefixMappings]]
		ifFalse: [anObject hasNamespaces not].! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 12/29/2014 01:15'!
copiedPrefixMappings
	"replaces current mappings with copy for copy-on-write"
	prefixMappings
		ifNil: [^ self prefixMappings]
		ifNotNil: [^ prefixMappings := prefixMappings copy]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 10/10/2014 05:47'!
defaultNamespace
	^ prefixMappings
		ifNil: ['']
		ifNotNil: [prefixMappings at: '' ifAbsent: ['']]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
defaultNamespace: aURI
	self
		validatePrefix: ''
		beforeMappingTo: aURI.

	"copy-on-write"
	self copiedPrefixMappings
		at: ''
		put: aURI.! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:38'!
errorAttribute: anAttribute aliasesNamespace: aNamespace
	XMLNamespaceException
		formatSignal: 'Attribute "{1}" has alias for namespace "{2}"'
		with: anAttribute
		with: aNamespace! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 3/24/2015 19:59'!
errorCannotAliasXMLOrXMLNSWithPrefix: aPrefix
	aPrefix
		ifEmpty: [
			XMLNamespaceException signal:
				'Cannot use "xml" or  "xmlns" namespace URI as default namespace']
		ifNotEmpty: [
			XMLNamespaceException
				formatSignal:
					'Cannot alias the "xml" or "xmlns" namespace URIs with prefix "{1}"'
				with: aPrefix]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 11/6/2014 12:18'!
errorCannotMapPrefixToEmptyURI
	XMLNamespaceException signal: 'Cannot map namespace prefixes to empty URIs'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 3/24/2015 19:59'!
errorCannotMapXMLNSPrefix
	XMLNamespaceException signal:
		'Cannot map "xmlns" prefix to any namespace, even the current "xmlns" namespace'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 3/24/2015 20:00'!
errorCannotRemapXMLPrefix
	XMLNamespaceException signal: 'Cannot map the "xml" prefix to a new namespace URI'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 3/24/2015 20:00'!
errorCannotUseXMLNSPrefix	
	XMLNamespaceException signal: 'Cannot use reserved "xmlns" prefix for elements or attributes'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:42'!
errorInvalidPrefix: aPrefix	
	XMLNamespaceException
		formatSignal: 'Prefix "{1}" contains ":" character'
		with: aPrefix! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:40'!
errorInvalidUnprefixedName: aName
	XMLNamespaceException
		formatSignal: 'Name "{1}" contains ":" character'
		with: aName! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:50'!
errorUnmappedPrefix: aPrefix
	XMLNamespaceException
		formatSignal: 'Unmapped namespace prefix "{1}" in name'
		with: aPrefix! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 10/12/2014 03:21'!
hasDefaultNamespace
	^ self includesPrefix: ''! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 10/10/2014 05:51'!
hasNamespaces
	^ prefixMappings notNil and: [prefixMappings notEmpty]! !

!XMLNamespaceScope methodsFor: 'comparing' stamp: 'monty 12/28/2014 23:33'!
hash
	| hash |

	hash := self class hash.
	prefixMappings
		ifNotNil: [hash := hash bitXor: prefixMappings hash].
	^ hash.! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 11/17/2014 16:45'!
includesPrefix: aPrefix
	self prefixMappingAt: aPrefix ifAbsent: [^ false].
	^ true.! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 3/21/2015 22:37'!
includesPrefix: aPrefix mappedTo: aURI
	^ (self prefixMappingAt: aPrefix ifAbsent: [^ false]) = aURI! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 12/29/2014 01:12'!
inheritMappingsFrom: anEnclosingScope
	anEnclosingScope hasNamespaces
		ifFalse: [^ self].
	self hasNamespaces
		ifTrue: [| copiedPrefixMappings |
			anEnclosingScope prefixesAndURIsDo: [:prefix :uri |
				(self includesPrefix: prefix)
					ifFalse: [
						(copiedPrefixMappings "used so we only copy once"
							ifNil: [copiedPrefixMappings := self copiedPrefixMappings])
							at: prefix
							put: uri]]]
		ifFalse: [
			"can reuse because of copy-on-write"
			prefixMappings := anEnclosingScope prefixMappings].! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
mapPrefix: aPrefix to: aURI
	self
		validatePrefix: aPrefix
		beforeMappingTo: aURI.

	"copy-on-write"
	self copiedPrefixMappings
		at: aPrefix
		put: aURI.! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 1/5/2015 21:04'!
mapPrefixesAndURIs: anAssociationCollection
	| copiedPrefixMappings |

	anAssociationCollection associationsDo: [:each |
		self
			validatePrefix: each key
			beforeMappingTo: each value.
		(copiedPrefixMappings "used so we only copy once"
			ifNil: [copiedPrefixMappings := self copiedPrefixMappings])
			at: each key
			put: each value]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 3/12/2015 14:15'!
prefixMappingAt: aPrefix
	^ self
		prefixMappingAt: aPrefix
		ifAbsent: ['']! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 11/17/2014 16:45'!
prefixMappingAt: aPrefix ifAbsent: aBlock
	^ prefixMappings
		ifNil: [aBlock value]
		ifNotNil: [prefixMappings at: aPrefix ifAbsent: aBlock]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:46'!
prefixMappings
	^ prefixMappings ifNil: [prefixMappings := OrderPreservingStringDictionary new: 5]! !

!XMLNamespaceScope methodsFor: 'enumerating' stamp: 'monty 11/23/2014 23:20'!
prefixesAndURIsDo: aTwoArgumentBlock
	prefixMappings
		ifNotNil: [prefixMappings keysAndValuesDo: aTwoArgumentBlock]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 3/21/2015 23:01'!
prefixesOfNamespaceURI: aURI
	| prefixes |

	prefixes := Set new.
	aURI = self class xmlNamespaceURI
		ifTrue: [prefixes add: 'xml']
		ifFalse: [
			aURI = self class xmlnsNamespaceURI
				ifTrue: [prefixes add: 'xmlns']
				ifFalse: [
					self prefixesAndURIsDo: [:prefix :uri |
						uri = aURI
							ifTrue: [prefixes add: prefix]]]].
	^ prefixes.! !

!XMLNamespaceScope methodsFor: 'printing' stamp: 'monty 10/14/2014 05:42'!
printOn: aStream
	self prefixesAndURIsDo: [:prefix :uri |
		aStream nextPutAll: 'xmlns'.
		prefix ifNotEmpty: [
			aStream
				nextPut: $:;
				nextPutAll: prefix].
		aStream
			nextPut: $=;
			nextPut: $";
			nextPutAll: uri;
			nextPut: $";
			space]! !

!XMLNamespaceScope methodsFor: 'resolving' stamp: 'monty 3/12/2015 18:13'!
resolvePrefix: aPrefix
	"resolve namespace from prefix"
	^ self
		resolvePrefix: aPrefix
		ifUnresolvable: ['']! !

!XMLNamespaceScope methodsFor: 'resolving' stamp: 'monty 3/12/2015 14:36'!
resolvePrefix: aPrefix ifUnresolvable: aBlock
	"resolve namespace from prefix"
	^ self
		prefixMappingAt: aPrefix
		ifAbsent: [
			aPrefix = 'xml'
				ifTrue: [self class xmlNamespaceURI]
				ifFalse: [
					aPrefix = 'xmlns'
						ifTrue: [self class xmlnsNamespaceURI]
						ifFalse: [aBlock value]]]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 12/26/2014 23:54'!
size
	^ prefixMappings
		ifNil: [0]
		ifNotNil: [prefixMappings size]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 12/28/2014 23:39'!
unmapPrefix: aPrefix
	"copy-on-write"
	^ self copiedPrefixMappings 
		removeKey: aPrefix
		ifAbsent: [
			XMLNamespaceException
				formatSignal: 'Prefix "{1}" was not mapped and cannot be unmapped'
				with: aPrefix]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 3/24/2015 20:00'!
validateAndResolvePrefix: aPrefix
	^ self
		prefixMappingAt: aPrefix
		ifAbsent: [
			aPrefix = 'xml'
				ifTrue: [self class xmlNamespaceURI]
				ifFalse: [					
					(aPrefix = 'xmlns'
						ifTrue: [self errorCannotUseXMLNSPrefix]
						ifFalse: [self errorUnmappedPrefix: aPrefix])]]! !

!XMLNamespaceScope methodsFor: 'validating' stamp: 'monty 12/11/2014 05:24'!
validateAndResolvePrefix: aPrefix ofElement: aName
	aPrefix isEmpty "avoid ifEmpty: closure overhead"
		ifTrue: [
			self validateUnprefixedName: aName.
			^ self defaultNamespace]
		ifFalse: [^ self validateAndResolvePrefix: aPrefix]! !

!XMLNamespaceScope methodsFor: 'validating' stamp: 'monty 11/16/2014 20:23'!
validateAttributePrefixes: aDictionary
	"performance critical method"
	| namespacedAttributes |

	aDictionary keysDo: [:each | | prefix |
		(prefix := each prefixBeforeLocalName) isEmpty "avoid ifNotEmpty: closure overhead"
			ifTrue: [self validateUnprefixedName: each]
			ifFalse: [ | namespace oldSize |
				"not initialized unless needed"
				namespacedAttributes
					ifNil: [namespacedAttributes := Set new: aDictionary size].
				namespace := self validateAndResolvePrefix: prefix.

				"checking size after is faster than includes:"
				oldSize := namespacedAttributes size.
				namespacedAttributes add: namespace, (each localNameAfterPrefix: prefix).
				namespacedAttributes size > oldSize
					ifFalse: [self errorAttribute: each aliasesNamespace: namespace]]]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 3/24/2015 20:00'!
validatePrefix: aPrefix beforeMappingTo: aURI
	aPrefix = 'xml'
		ifTrue: [
			aURI = self class xmlNamespaceURI
				ifFalse: [self errorCannotRemapXMLPrefix]]
		ifFalse: [
			aPrefix isEmpty "avoid ifEmpty: overhead"
				ifFalse: [
					aPrefix = 'xmlns'
						ifTrue: [self errorCannotMapXMLNSPrefix].
					(aPrefix indexOf: $:) > 0 
						ifTrue: [self errorInvalidPrefix: aPrefix].
					aURI isEmpty
						ifTrue: [self errorCannotMapPrefixToEmptyURI]].
			(aURI = self class xmlNamespaceURI
				or: [aURI = self class xmlnsNamespaceURI])
				ifTrue: [self errorCannotAliasXMLOrXMLNSWithPrefix: aPrefix]]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:40'!
validateUnprefixedName: aName
	"catch names like ':name' with an empty prefix"
	(aName size > 0
		and: [(aName at: 1) == $:])
		ifTrue: [self errorInvalidUnprefixedName: aName]! !

Object subclass: #XMLNamespaceScopeAllocator
	instanceVariableNames: 'scopes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScopeAllocator commentStamp: '<historical>' prior: 0!
This class handles the allocation of namespace scopes using a stack. Sending #enterScope to an instance will cause it to push a new scope onto the stack that inherits all mappings from the previous, enclosing scope and makes it the current scope. Sending #exitScop causes that scope to be removed and the parent scope to again be the current scope.!

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:26'!
currentScope
	^ self scopes lastOrNil ifNil: [self emptyScope]! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:27'!
currentScope: aScope
	self scopes addLast: aScope! !

!XMLNamespaceScopeAllocator methodsFor: 'private' stamp: 'JAAyer 3/13/2010 13:44'!
emptyScope
	^ XMLNamespaceScope new! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'monty 10/30/2014 12:27'!
enterScope
	^ self scopes addLast: self currentScope copy! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'monty 10/30/2014 12:27'!
exitScope
	^ self scopes removeLast! !

!XMLNamespaceScopeAllocator methodsFor: 'initialization' stamp: 'monty 10/27/2014 09:21'!
initialize
	super initialize.
	scopes := OrderedCollection new: 10.! !

!XMLNamespaceScopeAllocator methodsFor: 'private' stamp: 'monty 10/30/2014 12:26'!
scopes
	^ scopes! !

Object subclass: #XMLNestedStreamReader
	instanceVariableNames: 'stream onClose nestedStreams peekChar normalizedLineEndingChar currentLineNumber readLimit isEscapingQuotes'
	classVariableNames: 'CarriageReturn LineFeed'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLNestedStreamReader commentStamp: '<historical>' prior: 0!
This class represents a stream reader that implements pushBack: using nested streams. This enables subsitution/replacement to be performed without modifying the underlying collections streamed-over or having to copy them. It also performs line-ending normalization, transforming CR and CRLF sequences into a single LF character.

(The code in this class is optimized; refactor with care.)!

!XMLNestedStreamReader class methodsFor: 'class initialization' stamp: 'monty 11/28/2014 01:34'!
initialize
	"self initialize"

	CarriageReturn := Character cr.
	LineFeed := Character lf.! !

!XMLNestedStreamReader class methodsFor: 'instance creation' stamp: 'monty 11/12/2014 21:36'!
on: aStream
	^ self
		on: aStream
		readLimit: nil! !

!XMLNestedStreamReader class methodsFor: 'instance creation' stamp: 'monty 12/8/2014 08:20'!
on: aStream readLimit: aReadLimit
	^ self new
		setStream: aStream
		readLimit: aReadLimit! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 11/28/2014 00:37'!
atEnd
	peekChar == nil
		ifFalse: [^ false].
	stream atEnd
		ifFalse: [^ false].

	nestedStreams ifNotNil: [
		nestedStreams do: [:each |
			each key atEnd
				ifFalse: [^ false]]].
	^ true.! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 2/6/2014 01:01'!
atQuote
	^ self peek == $"
		or: [peekChar == $']! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 3/1/2015 17:15'!
basicNext
	"Returns next character in the stream after performing line-ending normalization."
	| nextChar |

	stream atEnd
		ifTrue: [
			self popNestedStreamWhileAtEnd.
			stream atEnd
				ifTrue: [^ nil]].
	(nextChar := stream next) == CarriageReturn
		ifTrue: [
			self popNestedStreamWhileAtEnd.
			stream atEnd
				ifFalse: [
					stream peek == LineFeed
						ifTrue: [stream next]].
			nextChar := normalizedLineEndingChar]
		ifFalse: [
			nextChar == LineFeed
				ifTrue: [nextChar := normalizedLineEndingChar]].
	isEscapingQuotes
		ifTrue: [^ self escapeNextQuoteCharacter: nextChar]
		ifFalse: [^ nextChar].! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 11/13/2014 05:11'!
closeCurrentStream
	(stream respondsTo: #close)
		ifTrue: [stream close].
	onClose ifNotNil: [onClose value].! !

!XMLNestedStreamReader methodsFor: 'file open/close' stamp: 'monty 11/28/2014 00:39'!
closeStreams
	nestedStreams ifNotNil: [
		[nestedStreams notEmpty]
			whileTrue: [self popNestedStream]].
	self closeCurrentStream.! !

!XMLNestedStreamReader methodsFor: 'encodings' stamp: 'monty 11/18/2014 19:18'!
convertFromEncoding: anEncodingName
	| streamDecoder |

	"encoding declarations for the default encoding are ignored on ReadStreams,
	which are assumed to be on strings that don't need decoding"
	streamDecoder := XMLStreamDecoder newForEncoding: anEncodingName.
	(streamDecoder == nil
		or: [(stream isKindOf: ReadStream)
			and: [streamDecoder isDefaultStreamDecoder]])
		ifFalse: [self decodeStreamWithDecoder: streamDecoder].! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 7/14/2010 00:39'!
currentLineNumber
	^ currentLineNumber! !

!XMLNestedStreamReader methodsFor: 'encodings' stamp: 'monty 11/20/2014 00:17'!
decodeStreamWithDecoder: aStreamDecoder
	(stream respondsTo: #streamDecoder:)
		ifTrue: [stream streamDecoder: aStreamDecoder]
		ifFalse: [
			stream :=
				XMLDecodingReadStreamAdapter
					on: stream
					streamDecoder: aStreamDecoder]! !

!XMLNestedStreamReader methodsFor: 'encodings' stamp: 'monty 11/28/2014 10:17'!
detectCurrentEncoding
	(XMLStreamEncodingDetector for: self) detectCurrentEncoding! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 10/31/2014 21:21'!
errorNestingLimitExceeded
	XMLLimitException signal: 'Cannot further nest input streams'! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 4/4/2015 19:06'!
errorReadLimitExceeded
	XMLLimitException signal:
		'Security read limit on input exceeded; ',
		'use #documentReadLimit: (or #externalSubsetReadLimit:) ',
		'to change or disable, or #defaultReadLimit: to ',
		'persistently change or disable'! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 3/1/2015 17:13'!
escapeNextQuoteCharacter: aCharacter
	aCharacter ==  $"
		ifTrue: [
			self pushBack: '#34;'.
			^ $&].
	aCharacter == $'
		ifTrue: [
			self pushBack: '#39;'.
			^ $&].
	^ aCharacter.! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 3/1/2015 17:12'!
isEscapingQuotes
	^ isEscapingQuotes! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 3/1/2015 17:11'!
isEscapingQuotes: aBoolean
	isEscapingQuotes := aBoolean! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 11/18/2014 05:43'!
maxNestedStreams
	^ 32! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 10/31/2014 15:03'!
next
	| nextChar |

	peekChar
		ifNil: [nextChar := self basicNext]
		ifNotNil: [
			nextChar := peekChar.
			peekChar := nil].

	nextChar == normalizedLineEndingChar
		ifTrue: [currentLineNumber := currentLineNumber + 1].
	readLimit ifNotNil: [
		(readLimit := readLimit - 1) >= 0
			ifFalse: [self errorReadLimitExceeded]].

	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 10/26/2014 14:18'!
nextInteger
	^ XMLIntegerReader
		readFrom: self
		withBase: 16
		startingWithAny: 'x'
		orBase: 10! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 4/3/2015 04:44'!
nextMatchAll: aString
	| nextChar oldCurrentLineNumber oldReadLimit |

	"inlined peek"
	(peekChar ifNil: [peekChar := self basicNext]) == (aString at: 1)
		ifFalse: [^ false].

	oldCurrentLineNumber := currentLineNumber.
	oldReadLimit := readLimit.
	self next.
	"we unrolled the first iteration with the above inlined peek"
	2 to: aString size do: [:i |
		(nextChar := self peek) == (aString at: i)
			ifTrue: [self next]
			ifFalse: [
				self pushBack: (aString copyFrom: 1 to: i -1).
				currentLineNumber := oldCurrentLineNumber.
				readLimit := oldReadLimit.
				^ false]].
	^ true.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 1/16/2014 09:08'!
normalizedLineEndingChar
	^ normalizedLineEndingChar! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 12/18/2013 17:25'!
normalizedLineEndingChar: aCharacter
	normalizedLineEndingChar := aCharacter! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 2/2/2014 02:43'!
peek
	^ peekChar ifNil: [peekChar := self basicNext]! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 11/28/2014 00:37'!
popNestedStream
	| nextStreamAndCallback |

	self closeCurrentStream.
	nextStreamAndCallback := nestedStreams removeLast.
	stream := nextStreamAndCallback key.
	onClose := nextStreamAndCallback value.! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 11/28/2014 01:35'!
popNestedStreamWhileAtEnd
	nestedStreams ifNotNil: [
		[stream atEnd
			and: [nestedStreams notEmpty]]
			whileTrue: [self popNestedStream]]! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 12/13/2014 20:15'!
pushBack: aString
	aString size > 1
		ifTrue: [^ self pushBackStream: aString readStream].
	aString size = 1
		ifTrue: [self pushBackCharacter: (aString at: 1)]! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 12/8/2014 06:20'!
pushBackCharacter: aCharacter
	peekChar
		ifNil: [peekChar := aCharacter]
		ifNotNil: [
			self
				pushBackStream: (XMLSingleCharacterReadStream on: aCharacter)
				onClose: nil]! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 12/8/2014 06:20'!
pushBackStream: aStream
	self
		pushBackStream: aStream
		onClose: nil! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 12/8/2014 06:20'!
pushBackStream: aStream onClose: aBlock
	nestedStreams
		ifNil: [(nestedStreams := OrderedCollection new: self maxNestedStreams)].
	(nestedStreams size >= self maxNestedStreams)
		ifTrue: [self errorNestingLimitExceeded].

	nestedStreams addLast: stream -> onClose.
	peekChar
		ifNotNil: [
			nestedStreams addLast: (XMLSingleCharacterReadStream on: peekChar) -> nil.
			peekChar := nil].

	self stream: aStream.
	onClose := aBlock.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 4/3/2015 02:01'!
readLimit
	^ readLimit! !

!XMLNestedStreamReader methodsFor: 'initialization' stamp: 'monty 3/1/2015 17:12'!
setStream: aStream readLimit: aReadLimit
	currentLineNumber := 1.
	normalizedLineEndingChar := LineFeed.
	readLimit := aReadLimit.
	isEscapingQuotes := false.
	self stream: aStream.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'monty 4/5/2015 06:37'!
skipSeparators
	"nil understands xmlParserIsSeparator so the result of the inlined peek
	does not need to be tested for nil first"
	[(peekChar ifNil: [peekChar := self basicNext]) "inlined peek"
		xmlParserIsSeparator]
		whileTrue: [
			"inlined next"
			peekChar == normalizedLineEndingChar
				ifTrue: [currentLineNumber := currentLineNumber + 1].
			readLimit ifNotNil: [
				(readLimit := readLimit - 1) >= 0
					ifFalse: [self errorReadLimitExceeded]].
			peekChar := nil].! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 1/4/2014 08:09'!
skipTo: aDelimiter
	[self atEnd] whileFalse: [
		self next == aDelimiter
			ifTrue: [^ true]].
	^ false.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 08:01'!
stream
	^ stream! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 12/11/2014 09:45'!
stream: aStream
	stream := 
		(aStream atEnd
			or: [aStream peek isCharacter])
			ifTrue: [aStream]
			ifFalse: [XMLBinaryReadStreamAdapter on: aStream]! !

Object subclass: #XMLNode
	instanceVariableNames: 'parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNode commentStamp: '<historical>' prior: 0!
This is a base class for XML nodes. It contains testing messages and methods to set and access the parent node and sibling nodes. It also provides a number messages to control printing.!

XMLNode subclass: #XMLAttribute
	instanceVariableNames: 'name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttribute commentStamp: 'monty 2/8/2014 02:27' prior: 0!
The class represents an attribute node, to be stored in an attribute node list. You generally won't need to interact with these objects directly; the various #attribute- messages element nodes respond to will normally suffice. The name of an attribute node can be namespaced using prefixes; however, the prefix must be mapped to a namespace URI in the containing element node. Unprefixed attributes have no namespace, not even a default namespace URI if one is in scope, per the XML namespace standard.!

!XMLAttribute class methodsFor: 'instance creation' stamp: 'monty 10/18/2014 04:03'!
name: aName value: aValue
	^ self new	
		setName: aName
		value: aValue
		element: nil! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 10/29/2014 07:42'!
element
	"reuse instance variable to save memory"
	^ parent! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 10/29/2014 07:42'!
element: anElement
	"reuse instance variable to save memory"
	parent := anElement! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 19:44'!
expandedName
	^ self localName expandWithNamespaceURI: self namespaceURI! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 12/23/2014 16:06'!
hasLanguage: aLanguage
	| language xmlLang |

	language := aLanguage asLowercase.
	^ (xmlLang := self value asLowercase) = language
		or: [(xmlLang copyUpTo: $-) = language].! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 2/13/2015 22:20'!
hasNamespaceURI
	^ self namespaceURI notEmpty! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 11/22/2014 19:58'!
hasPrefix
	^ self name hasPrefixBeforeLocalName! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 1/29/2015 00:30'!
isAttribute
	^ true! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 12/28/2014 08:02'!
isInLanguage: aLanguage
	self name = 'xml:lang'
		ifTrue: [^ self hasLanguage: aLanguage]
		ifFalse: [^ super isInLanguage: aLanguage]! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 11/16/2014 17:28'!
isNamed: aName
	^ aName
		isQualifiedName: self name
		orLocalName: self localName! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 11/16/2014 18:10'!
isNamedAny: aNameCollection
	^ aNameCollection
		includesQualifiedName: self name
		orLocalName: self localName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 20:21'!
localName
	^ self name localNameAfterPrefix! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 17:36'!
name
	^ name ifNil: [name := '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 18:01'!
name: aName
	self renameWith: [name := aName]! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 1/29/2015 00:52'!
namespaceURI
	| prefix |

	"attributes must have prefixes to be namespaced"
	(self hasParent
		and: [(prefix := self prefix) notEmpty])
		ifTrue: [^ self parent resolvePrefix: prefix]
		ifFalse: [^ '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/26/2014 22:19'!
prefix
	^ self name prefixBeforeLocalName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 19:50'!
prefix: aPrefix
	self renameWith: [name := self localName qualifyWithPrefix:  aPrefix]! !

!XMLAttribute methodsFor: 'private' stamp: 'monty 1/29/2015 00:52'!
renameWith: aBlock
	| oldName |

	oldName := self name.
	aBlock value.
	(self name ~= oldName
		and: [self hasParent])
			ifTrue: [
				self parent
					renamedAttribute: self
					from: oldName
					to: self name].! !

!XMLAttribute methodsFor: 'initialization' stamp: 'monty 11/16/2014 17:35'!
setName: aName value: aValue element: anElement
	name := aName.
	value := aValue.
	parent := anElement.! !

!XMLAttribute methodsFor: 'private' stamp: 'monty 12/2/2014 22:56'!
sortKey
	^ self name! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
value
	^ value ifNil: [value := '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 08:18'!
value: aString
	value := aString! !

!XMLAttribute methodsFor: 'printing' stamp: 'monty 2/9/2014 04:31'!
writeXMLOn: aWriter
	aWriter
		writeName: self name
		escapedContentValue: self value! !

XMLNode subclass: #XMLComment
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLComment commentStamp: '<historical>' prior: 0!
This class represents comments preserved as nodes. Comments are ignored during parsing by default. If you want them preserved as comment nodes, use #preservesCommentNodes: with the DOM parser before parsing.!

!XMLComment class methodsFor: 'instance creation' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	^ self new string: aString! !

!XMLComment methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ true! !

!XMLComment methodsFor: 'private' stamp: 'monty 12/3/2014 01:08'!
sortKey
	^ self string! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string
	^ string ifNil: [string := '']! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	string := aString! !

!XMLComment methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter comment: self string! !

XMLNode subclass: #XMLDeclarationNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDeclarationNode commentStamp: 'monty 10/26/2014 06:12' prior: 0!
This is a generic class to represent markup declarations in the internal subset. Each subclass represents a specific kind of declaration.!

XMLDeclarationNode subclass: #XMLAttributeDeclaration
	instanceVariableNames: 'element attribute type defaultPragma defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttributeDeclaration commentStamp: 'monty 2/2/2014 16:47' prior: 0!
This is a class for <!!ATTLIST> declarations with definitions for single attributes. (An ATTLIST with multiple attribute defs is the same as a series of separate ATTLISTs for each def)!

!XMLAttributeDeclaration class methodsFor: 'instance creation' stamp: 'monty 10/26/2014 06:11'!
element: anElementName attribute: anAttributeName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	^ self new
		element: anElementName;
		attribute: anAttributeName;
		type: aType;
		defaultPragma: aDefaultPragma;
		defaultValue: aDefaultValue! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:53'!
attribute
	^ attribute ifNil: [attribute := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:07'!
attribute: anAttributeName
	attribute := anAttributeName! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:14'!
defaultPragma
	^ defaultPragma ifNil: [defaultPragma := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:10'!
defaultPragma: aDefaultPragma
	defaultPragma := aDefaultPragma! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:11'!
defaultValue
	^ defaultValue ifNil: [defaultValue := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:10'!
defaultValue: aDefaultValue
	defaultValue := aDefaultValue! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:21'!
element
	^ element ifNil: [element := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:10'!
element: anElementName
	element := anElementName! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:42'!
type
	^ type ifNil: [type := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:09'!
type: aType 
	type := aType! !

!XMLAttributeDeclaration methodsFor: 'printing' stamp: 'monty 12/31/2013 00:16'!
writeXMLOn: aWriter
	aWriter attributeDeclaration
		element: self element;
		attribute: self attribute;
		type: self type;
		defaultPragma: self defaultPragma;
		defaultValue: self defaultValue! !

!XMLDeclarationNode methodsFor: 'testing' stamp: 'monty 12/22/2013 06:10'!
isDeclaration
	^ true! !

XMLDeclarationNode subclass: #XMLElementDeclaration
	instanceVariableNames: 'element contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLElementDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is a class for <!!ELEMENT> declarations.!

!XMLElementDeclaration class methodsFor: 'instance creation' stamp: 'monty 10/26/2014 06:04'!
element: aName contentModel: aContentModel
	^ self new
		element: aName;
		contentModel: aContentModel! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
contentModel
	^ contentModel ifNil: [contentModel := '']! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:03'!
contentModel: aContentModel
	contentModel := aContentModel! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:21'!
element
	^ element ifNil: [element := '']! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:03'!
element: aName
	element := aName! !

!XMLElementDeclaration methodsFor: 'printing' stamp: 'monty 12/21/2013 03:25'!
writeXMLOn: aWriter
	aWriter elementDeclaration
		element: self element;
		contentModel: self contentModel! !

XMLDeclarationNode subclass: #XMLEntityDeclaration
	instanceVariableNames: 'name replacement publicID systemID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLEntityDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is an abstract class for general and parameter <!!ENTITY> declarations.!

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
name: aName publicID: aPublicID systemID: aSystemID
	^ self new
		name: aName;
		publicID: aPublicID;
		systemID: aSystemID! !

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'monty 10/26/2014 06:06'!
name: aName replacement: aReplacement
	^ self new
		name: aName;
		replacement: aReplacement! !

!XMLEntityDeclaration methodsFor: 'testing' stamp: 'monty 11/30/2013 21:53'!
isParameter
	^ false! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:22'!
name
	^ name ifNil: [name := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:01'!
name: aName
	name := aName! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
publicID
	^ publicID ifNil: [publicID := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
publicID: aPublicID
	publicID := aPublicID! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
replacement
	^ replacement ifNil: [replacement := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:05'!
replacement: aReplacement
	replacement := aReplacement! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
systemID
	^ systemID ifNil: [systemID := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
systemID: aSystemID
	systemID := aSystemID! !

XMLEntityDeclaration subclass: #XMLGeneralEntityDeclaration
	instanceVariableNames: 'ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLGeneralEntityDeclaration commentStamp: 'monty 2/2/2014 16:47' prior: 0!
This is a class for internal/external parsed/unparsed general <!!ENTITY> declarations.!

!XMLGeneralEntityDeclaration class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
name: aName publicID: aPublicID systemID: aSystemID ndata: aNotationName
	^ self new
		name: aName;
		publicID: aPublicID;
		systemID: aSystemID;
		ndata: aNotationName! !

!XMLGeneralEntityDeclaration methodsFor: 'accessing' stamp: 'monty 11/23/2014 18:54'!
ndata
	^ ndata ifNil: [ndata := '']! !

!XMLGeneralEntityDeclaration methodsFor: 'accessing' stamp: 'monty 11/23/2014 18:54'!
ndata: anNdata
	ndata := anNdata! !

!XMLGeneralEntityDeclaration methodsFor: 'printing' stamp: 'monty 11/23/2014 18:55'!
writeXMLOn: aWriter
	aWriter entityDeclaration
		name: self name;
		isParameter: false;
		replacement: self replacement;
		publicID: self publicID;
		systemID: self systemID;
		ndata: self ndata! !

XMLEntityDeclaration subclass: #XMLParameterEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLParameterEntityDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is class for internal/external parameter <!!ENTITY> declarations.!

!XMLParameterEntityDeclaration methodsFor: 'testing' stamp: 'monty 11/13/2013 19:34'!
isParameter
	^ true! !

!XMLParameterEntityDeclaration methodsFor: 'printing' stamp: 'monty 11/23/2014 18:55'!
writeXMLOn: aWriter
	aWriter entityDeclaration
		name: self name;
		isParameter: true;
		replacement: self replacement;
		publicID: self publicID;
		systemID: self systemID! !

XMLDeclarationNode subclass: #XMLNotationDeclaration
	instanceVariableNames: 'name publicID systemID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNotationDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is class for <!!NOTATION> declarations.
!

!XMLNotationDeclaration class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
name: aName publicID: aPublicID systemID: aSystemID
	^ self new
		name: aName;
		publicID: aPublicID;
		systemID: aSystemID! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:22'!
name
	^ name ifNil: [name := '']! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 10:35'!
name: aName
	name := aName! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
publicID
	^ publicID ifNil: [publicID := '']! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
publicID: aPublicID
	publicID := aPublicID! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
systemID
	^ systemID ifNil: [systemID := '']! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
systemID: aSystemID
	systemID := aSystemID! !

!XMLNotationDeclaration methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter notationDeclaration
		name: self name;
		publicID: self publicID;
		systemID: self systemID! !

!XMLNode methodsFor: 'private' stamp: 'monty 2/11/2014 00:13'!
acceptsContent
	^ false! !

!XMLNode methodsFor: 'adding' stamp: 'JAAyer 7/20/2010 10:15'!
addContent: aStringOrStringNode
	XMLDOMException signal: 'Node cannot contain content'! !

!XMLNode methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:59'!
allNodesDo: aBlock
	aBlock value: self! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 3/8/2015 09:17'!
ancestorElements
	| ancestorElements |

	ancestorElements := self newListForSelect.
	self ancestorElementsDo: [:each | ancestorElements addLast: each].
	^ ancestorElements.! !

!XMLNode methodsFor: 'enumerating' stamp: 'monty 3/8/2015 09:19'!
ancestorElementsDo: aBlock
	| ancestor |

	ancestor := self parent.
	[ancestor notNil
		and: [ancestor isElement]]
		whileTrue: [
			aBlock value: ancestor.
			ancestor := ancestor parent].! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 3/8/2015 09:17'!
ancestorNodes
	| ancestors |

	ancestors := self newListForSelect.
	self ancestorNodesDo: [:each | ancestors addLast: each].
	^ ancestors.! !

!XMLNode methodsFor: 'enumerating' stamp: 'monty 3/8/2015 13:21'!
ancestorNodesDetect: aBlock ifNone: anExceptionBlock
	self ancestorNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNode methodsFor: 'enumerating' stamp: 'monty 3/8/2015 09:18'!
ancestorNodesDo: aBlock
	| ancestor |

	ancestor := self parent.
	[ancestor notNil]
		whileTrue: [
			aBlock value: ancestor.
			ancestor := ancestor parent].! !

!XMLNode methodsFor: 'converting' stamp: 'monty 4/6/2015 08:05'!
asString
	"for compatibility with GS where Object>>asString isn't defined"
	^ self printString! !

!XMLNode methodsFor: 'testing' stamp: 'monty 11/23/2014 20:53'!
canHaveChildren
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 12/6/2014 12:58'!
canonicallyEquals: aNode
	^ self canonicallyPrinted = aNode canonicallyPrinted! !

!XMLNode methodsFor: 'printing' stamp: 'monty 2/2/2015 16:15'!
canonicallyPrintOn: aStream
 	"Prints on aStream as canonical XML"
	self
		withNewXMLWriterOn: aStream
		do: [:writer |
			writer
				isCanonical: true;
				writeWith: [self writeXMLOn: writer]]
		whenAbsent: [self printOn: aStream]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 11/19/2014 23:49'!
canonicallyPrinted
 	"Prints as canonical XML"
	^ String streamContents: [:stream | self canonicallyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 4/1/2015 19:09'!
configuration
	^ self parent ifNotNil: [:node | node configuration]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 11/20/2014 02:36'!
contentString
 	"This returns the combined content string of all descendants with spaces inserted where necessary."
	^ String streamContents: [:stream | self printContentOn: stream]! !

!XMLNode methodsFor: 'enumerating' stamp: 'monty 11/6/2014 21:30'!
descendentNodesDo: aBlock! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 10/30/2014 22:39'!
document
	^ self parent ifNotNil: [:node | node document]! !

!XMLNode methodsFor: 'private' stamp: 'monty 10/31/2014 20:56'!
errorXMLWritingUnsupported
	XMLDOMException signal: 'The XMLWriter package is required for writng DOM objects'! !

!XMLNode methodsFor: 'private' stamp: 'monty 1/7/2015 02:16'!
escapesContentEntitiesWhenWriting
	self hasParent
		ifTrue: [^ self parent escapesContentEntitiesWhenWriting]
		ifFalse: [^ true] "default to escaping"! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:49'!
hasChildren
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 10/29/2014 09:00'!
hasParent
	^ parent notNil! !

!XMLNode methodsFor: 'testing' stamp: 'monty 1/29/2015 00:30'!
isAttribute
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/30/2010 19:19'!
isContentNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 12/22/2013 06:10'!
isDeclaration
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:50'!
isDocument
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 11/5/2014 11:36'!
isElementNamed: aName
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 11/5/2014 11:36'!
isElementNamedAny: aNameCollection
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 12/23/2014 15:51'!
isInLanguage: aLanguage
	^ self hasParent
		and: [self parent isInLanguage: aLanguage]! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:33'!
isStringNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 11/5/2014 12:09'!
isStringNode: aStringOrStringNode
	^ false! !

!XMLNode methodsFor: 'private' stamp: 'monty 3/8/2015 09:20'!
newListForCollect
	"collect can return non-nodes, so a basic ordered list is used"
	^ XMLOrderedList new: 10! !

!XMLNode methodsFor: 'private' stamp: 'monty 3/8/2015 09:20'!
newListForSelect
	"select returns only nodes, so a node list is used"
	^ XMLNodeList new: 10! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 10/29/2014 12:54'!
nextNode
	^ self parent ifNotNil: [:node | node nodeAfter: self]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent
	^ parent! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent: aParentNode
	parent := aParentNode! !

!XMLNode methodsFor: 'copying' stamp: 'monty 10/29/2014 07:40'!
postCopy
	parent := nil! !

!XMLNode methodsFor: 'printing' stamp: 'monty 2/2/2015 16:17'!
prettyPrintOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [:writer |
			writer
				enablePrettyPrinting;
				writeWith: [self writeXMLOn: writer]]
		whenAbsent: [self printOn: aStream]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/6/2010 01:18'!
prettyPrinted
	^ String streamContents: [:stream | self prettyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 10/29/2014 12:55'!
previousNode
	^ self parent ifNotNil: [:node | node nodeBefore: self]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 12/8/2014 06:12'!
printContentOn: aStream
 	"Prints on aStream the content strings of all descendants with spaces inserted where necessary."
	self printRawContentOn: (XMLNodeContentWriteStreamAdapter on: aStream)! !

!XMLNode methodsFor: 'printing' stamp: 'monty 2/2/2015 16:16'!
printOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [:writer | writer writeWith: [self writeXMLOn: writer]]
		whenAbsent: [super printOn: aStream]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 11/19/2014 23:54'!
printRawContentOn: aStream 
 	"Prints on aStream the content strings of all descendants without spaces inserted."! !

!XMLNode methodsFor: 'printing' stamp: 'monty 2/2/2015 16:17'!
printWithoutSelfClosingTagsOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [:writer |
			writer
				outputsSelfClosingTags: false;
				writeWith: [self writeXMLOn: writer]]
		whenAbsent: [self printOn: aStream]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 10/12/2014 21:27'!
printedWithoutSelfClosingTags
	^ String streamContents: [:stream | self printWithoutSelfClosingTagsOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 11/19/2014 23:44'!
rawContentString
 	"This returns the combined content string of all descendants without spaces inserted."
	^ String streamContents: [:stream | self printRawContentOn: stream].! !

!XMLNode methodsFor: 'private' stamp: 'monty 12/4/2014 14:00'!
sortKey
	"used to compare nodes when sorting"

	^ self class name asString! !

!XMLNode methodsFor: 'validating' stamp: 'monty 4/6/2015 07:04'!
validate
	"revalidate against the DTD it was parsed with (if any)"

	self document ifNotNil: [:node | node validate]! !

!XMLNode methodsFor: 'validating' stamp: 'monty 4/5/2015 21:03'!
validateWith: aValidator! !

!XMLNode methodsFor: 'private' stamp: 'monty 10/31/2014 20:56'!
withNewXMLWriterOn: aStream do: aOneArgBlock
	^ self
		withNewXMLWriterOn: aStream
		do: aOneArgBlock
		whenAbsent: [self errorXMLWritingUnsupported]! !

!XMLNode methodsFor: 'private' stamp: 'monty 1/5/2015 09:02'!
withNewXMLWriterOn: aStream do: aOneArgBlock whenAbsent: aZeroArgBlock
	| writer |

	writer :=
		(Smalltalk
			at: #XMLWriter
			ifAbsent: [^ aZeroArgBlock value]) on: aStream.
	self escapesContentEntitiesWhenWriting
		ifFalse: [writer escapesContentEntityReferences: false].
	^ aOneArgBlock value: writer.! !

!XMLNode methodsFor: 'printing' stamp: 'monty 12/6/2014 13:02'!
writeXMLOn: aWriter! !

XMLNode subclass: #XMLNodeWithChildren
	instanceVariableNames: 'nodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeWithChildren commentStamp: 'monty 12/8/2014 15:16' prior: 0!
This class represents a node that can contain child nodes. You probably will only use its API through one its subclasses, specifically the document, element and doctype declaration node classes.

This class provides message to access, add and remove child nodes under the "accessing" category. The nodes themselves are stored in some type of XMLObservableList, the exact type of which varies depending on the subclass and can be accessed directly using #nodes. You can modify the #nodes list directly if you like, but adding nodes to it or removing nodes from it will have the effect of adding them to removing them from the parent node. If this is not what you want, then make sure you copy the return value of #nodes before manipulating it.

The "enumerating" category provides three different modes of enumeration: the #nodes* enumerating messages enumerate child nodes of the receiver only, while the #allNode* forms recursively enumerate all descendent nodes, including the receiver itself, using depth-first traversal, while the #descendentNode* forms  enumerate only descendent nodes and not the receiver.!

XMLNodeWithChildren subclass: #XMLDoctypeDeclaration
	instanceVariableNames: 'root publicID systemID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDoctypeDeclaration commentStamp: '<historical>' prior: 0!
This class represents the <!!DOCTYPE ...> declaration one often finds at the top of an XML document. It stores the root element name, formal public identifier (FPI) if any, system identifier (a URI) and then any nodes comprising the internal subset.!

!XMLDoctypeDeclaration class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:36'!
root: aRootElementName publicID: aPublicID systemID: aSystemID
	^ self new
		root: aRootElementName;
		publicID: aPublicID;
		systemID: aSystemID! !

!XMLDoctypeDeclaration methodsFor: 'private' stamp: 'monty 3/8/2015 09:17'!
newListForSelect
	^ XMLInternalSubsetList new: 10! !

!XMLDoctypeDeclaration methodsFor: 'private' stamp: 'monty 11/26/2013 21:51'!
nodeListClass
	^ XMLInternalSubsetList! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 20:01'!
publicID
	^ publicID ifNil: [publicID := '']! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 20:01'!
publicID: aPublicID
	publicID := aPublicID! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 10/10/2014 07:12'!
root
	^ root ifNil: [root := '']! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 11/23/2014 04:42'!
root: aRootElementName
	root := aRootElementName! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 20:01'!
systemID
	^ systemID ifNil: [systemID := '']! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 20:01'!
systemID: aSystemID
	systemID := aSystemID! !

!XMLDoctypeDeclaration methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	aWriter doctype
		root: self root;
		publicID: self publicID;
		systemID: self systemID;
		withInternalSubset: [self writeInnerXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'JAAyer 7/20/2010 10:22'!
addNode: aNode
	^ self nodes addLast: aNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode after: aPriorNode
	^ self nodes add: aNode after: aPriorNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode before: aSubsequentNode
	^ self nodes add: aNode before: aSubsequentNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'JAAyer 7/20/2010 10:22'!
addNodeFirst: aNode
	^ self nodes addFirst: aNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 05:12'!
addNodes: aNodeCollection
	^ self nodes addAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifying' stamp: 'monty 2/2/2014 02:40'!
addedNode: aNode
	aNode parent: self! !

!XMLNodeWithChildren methodsFor: 'notifying' stamp: 'monty 10/31/2014 17:34'!
addingNode: aNode
	aNode hasParent
		ifTrue: [self errorAddingNodeWithParent]! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 3/8/2015 09:17'!
allNodes
	| allNodes |

	allNodes := self newListForSelect.
	self allNodesDo: [:each | allNodes addLast: each].
	^ allNodes.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/7/2014 02:17'!
allNodesCollect: aBlock
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self allNodesDo: [:each | collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:17'!
allNodesDetect: aBlock
	^ self allNodesDetect: aBlock ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/6/2014 21:40'!
allNodesDetect: aBlock ifNone: anExceptionBlock
	self allNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 10/19/2014 08:44'!
allNodesDo: aBlock
	aBlock value: self.
	self hasChildren
		ifTrue: [self nodesDo: [:each | each allNodesDo: aBlock]]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 3/8/2015 09:17'!
allNodesSelect: aBlock
	| selectedNodes |

	selectedNodes := self newListForSelect.
	self allNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedNodes addLast: each]].
	^ selectedNodes.! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'monty 11/23/2014 20:53'!
canHaveChildren
	^ true! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 3/8/2015 09:17'!
descendentNodes
	| descendentNodes |

	descendentNodes := self newListForSelect.
	self descendentNodesDo: [:each | descendentNodes addLast: each].
	^ descendentNodes.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/7/2014 02:18'!
descendentNodesCollect: aBlock
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self descendentNodesDo: [:each | collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/6/2014 21:37'!
descendentNodesDetect: aBlock
	^ self descendentNodesDetect: aBlock ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/6/2014 21:40'!
descendentNodesDetect: aBlock ifNone: anExceptionBlock
	self descendentNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/7/2014 01:55'!
descendentNodesDo: aBlock
	self hasChildren
		ifTrue: [self nodesDo: [:each | each allNodesDo: aBlock]]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 3/8/2015 09:17'!
descendentNodesSelect: aBlock
	| selectedNodes |

	selectedNodes := self newListForSelect.
	self descendentNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedNodes addLast: each]].
	^ selectedNodes.! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 10/31/2014 17:34'!
errorAddingNodeWithParent
	XMLDOMException signal: 'Cannot add child node that already has a parent'! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 10/29/2014 09:52'!
firstNode
	"returns nil when empty"
	^ self nodes first! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'monty 11/22/2014 07:51'!
hasChildren
	"Direct variable access avoids possibly unnecessary lazy initialization of the node list"

	^ nodes notNil and: [nodes size > 0]! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:23'!
includesNode: aNode
	^ self nodes includes: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 2/3/2015 00:10'!
innerXML
	self hasChildren
		ifFalse: [^ ''].
	self
		withNewXMLWriterOn: (String new: self nodes size * 10) writeStream
		do: [:writer |
			self writeInnerXMLOn: writer.
			^ writer contents].! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 10/29/2014 09:52'!
lastNode
	"returns nil when empty"
	^ self nodes last! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:26'!
nodeAfter: aNode
	^ self nodes after: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/17/2014 21:29'!
nodeAt: anIndex
	"returns nil if absent"
	^ self nodes at: anIndex! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/17/2014 16:46'!
nodeAt: anIndex ifAbsent: aBlock
	^ self nodes at: anIndex ifAbsent: aBlock! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/17/2014 16:47'!
nodeAt: anIndex put: aNode
	^ self nodes at: anIndex put: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
nodeBefore: aNode
	^ self nodes before: aNode! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 11/4/2014 19:04'!
nodeListClass
	^ XMLNodeList! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 10/18/2014 01:00'!
nodes
	^ nodes ifNil: [nodes := self nodeListClass observer: self]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:13'!
nodesCollect: aBlock
	^ self nodes collect: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 10/28/2014 19:46'!
nodesDetect: aBlock
	^ self nodes detect: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:14'!
nodesDetect: aBlock ifNone: anExceptionBlock
	^ self nodes detect: aBlock ifNone: anExceptionBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 7/20/2010 10:27'!
nodesDo: aBlock
	self nodes do: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:13'!
nodesSelect: aBlock
	^ self nodes select: aBlock! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 11/1/2014 15:02'!
parserHandlerClass
	^ XMLDOMParser! !

!XMLNodeWithChildren methodsFor: 'copying' stamp: 'monty 10/27/2014 18:29'!
postCopy
	super postCopy.

	nodes
		ifNotNil: [nodes := nodes deepCopyWithObserver: self].! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNode: aNode
	^ self nodes remove: aNode! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'monty 11/19/2014 20:24'!
removeNode: aNode ifAbsent: aBlock
	^ self nodes remove: aNode ifAbsent: aBlock! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes
	self nodes removeAll! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes: aNodeCollection
	^ self nodes removeAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifying' stamp: 'monty 2/2/2014 02:38'!
removedNode: aNode
	aNode parent: nil! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 2/3/2015 00:12'!
replaceNode: aNode with: aReplacementNode
	"replace node in the receiver with another node.
	if the second node is also in the receiver, their
	positions are just swapped"
	| firstIndex secondIndex |

	firstIndex := self nodes indexOf: aNode.
	(secondIndex := self nodes indexOf: aReplacementNode) > 0
		ifTrue: [self nodes swap: firstIndex with: secondIndex]
		ifFalse: [self nodes at: firstIndex put: aReplacementNode].
	^ aReplacementNode.! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeInnerXMLOn: aWriter
	self nodesDo: [:each | each writeXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	self writeInnerXMLOn: aWriter! !

XMLNodeWithChildren subclass: #XMLNodeWithElements
	instanceVariableNames: 'configuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeWithElements commentStamp: 'monty 12/8/2014 15:16' prior: 0!
This class represents a node with elements. You will likely use its API through its subclasses instead.

Instances provide "accessing" messages to retrieve child elements by their name and namespace information. The #elementAt: forms return the first matching element, while the #elementsAt: forms return all matching child elements.

As with node enumeration in the superclass, there are three different modes of enumeration: the #elements* enumerating messages that enumerate child elements only, the #allElements* forms that enumerate all descendent elements, including the receiver if it is an element (but not if it is a document), using depth-first traversal, and the #descendentElement* forms that enumerate descendent elements only and not the receiver.

The #findElementNamed:* forms can be used to search using depth-first traversal for a specific element. Searching will start with the receiver if it is an element.

All element name matching is done using both the qualified and local name of elements, so 'prefix:element-name' will only match 'prefix:element-name' while 'element-name' will match 'element-name', 'prefix:element-name' or 'different-prefix:element-name' and so on.

The inner XML can be accessed as a string using #innerXML and set using #innerXML:.!

XMLNodeWithElements subclass: #XMLDocument
	instanceVariableNames: 'version encoding isStandalone doctypeDeclaration doctypeDefinition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDocument commentStamp: '<historical>' prior: 0!
This class represents a document node, which is often the root of a node tree. It can contain XML version and encoding information, a DOCTYPE declaration with a possible internal subset and a root element child node.!

!XMLDocument class methodsFor: 'instance creation' stamp: 'monty 3/24/2015 18:26'!
root: aRoot
	^ self new root: aRoot! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration
	^ doctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration: aDoctypeDeclaration
	doctypeDeclaration := aDoctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 3/8/2015 10:09'!
doctypeDefinition
	^ doctypeDefinition! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 3/8/2015 10:09'!
doctypeDefinition: aDoctypeDefinition
	doctypeDefinition := aDoctypeDefinition! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 13:34'!
document
	^ self! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 3/24/2015 18:28'!
encoding	
	^ encoding ifNil: [encoding := '']! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 1/17/2002 12:57'!
encoding: aString	
	encoding := aString! !

!XMLDocument methodsFor: 'private' stamp: 'monty 4/6/2015 01:43'!
innerXMLContext
	^ XMLXMLDeclarationContext new! !

!XMLDocument methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:51'!
isDocument
	^ true! !

!XMLDocument methodsFor: 'testing' stamp: 'monty 11/19/2013 05:38'!
isStandalone
	^ isStandalone ifNil: [isStandalone := false]! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 11/19/2013 05:40'!
isStandalone: aBoolean
	isStandalone := aBoolean! !

!XMLDocument methodsFor: 'copying' stamp: 'monty 11/22/2014 08:20'!
postCopy
	super postCopy.

	doctypeDeclaration
		ifNotNil: [doctypeDeclaration := doctypeDeclaration copy].! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 3/24/2015 18:38'!
root
	"returns root element or nil"
	^ self firstElement! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 3/24/2015 18:38'!
root: anElement
	"replaces root element with new element"
	self hasChildren
		ifTrue: [self removeNodes: self elements].
	self addNode: anElement.! !

!XMLDocument methodsFor: 'validating' stamp: 'monty 4/6/2015 23:52'!
validate
	"revalidate against the DTD it was parsed with (if any)"
	self doctypeDefinition
		ifNotNil: [:dtd |
			dtd hasValidators
				ifTrue: [dtd clearIDsAndIDReferences].
			self validateWith: (DTDDocumentValidator doctypeDefinition: dtd)]! !

!XMLDocument methodsFor: 'validating' stamp: 'monty 4/5/2015 21:11'!
validateWith: aValidator
	self root
		ifNotNil: [:rootElement | aValidator validateRootElement: rootElement name].
	super validateWith: aValidator.
	aValidator validateEndDocument.! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
version	
	^ version! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 19:39'!
version: aVersionNumber
	version := aVersionNumber! !

!XMLDocument methodsFor: 'printing' stamp: 'monty 3/24/2015 18:30'!
writeXMLOn: aWriter
	self version
		ifNotNil: [
			aWriter xml
				version: self version;
				encoding: self encoding;
				standalone: self isStandalone].
	self doctypeDeclaration
		ifNotNil: [self doctypeDeclaration writeXMLOn: aWriter].
	
	self writeInnerXMLOn: aWriter.! !

XMLNodeWithElements subclass: #XMLElement
	instanceVariableNames: 'name localName attributes namespaceScope'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLElement commentStamp: 'monty 2/13/2015 21:14' prior: 0!
The class represents an element node. An element consists of a name and optionally attributes, namespace declarations and any number child nodes and elements.

The name can consist of a local name only or a qualified name, which is a namespace prefix and local name separated by a colon. If the name is qualfied and namespace support is enabled (the default), then the prefix must be mapped to a namespace URI in the element or its parent at the time the name is set. The class-side instance creation #name:namespaceURI:* messages and #name:namespaces:* messages allow you set both simulaniously, as does the instance-side #name:namespaceURI: message. If namespace support is disabled, prefixes are ignored (not validated). You can test the name of an element using #isNamed: and #isNamedAny:, which test the qualified and local names of an element for a match.

Namespaces can be declared using #declareNamespace:uri:, using #declareNamespaces: with a collection of associations (where an empty prefix represents the default namespace) or simultaniously while changing the element's name using #name:namespaceURI:. The default namespace can be accessed through #defaultNamespace and #defaultNamespace:. When you print an element, the minimum number of xmlns declarations necessary for it to be valid will be printed. You can query an element's namespace information using #namespaceURI, #prefix, #expandedName. For an element to be namespaced, it must have a qualified name with a valid prefix or have a default namespace declaration in scope for it.

The #attribute- messages provide a Dictionary-like prototcol for manipulating attribute nodes. Attribute value accessors return empty strings when the specified value is not present. The underlying attribute node list can be accessed using #attributeNodes, and an order-preserving dictionary of attribute names/values can be obtained using #attributes.

See the comment for the superclass for how to access child nodes and elements.!

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 10/30/2010 22:09'!
name: aLocalName
	^ self new name: aLocalName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:05'!
name: aLocalName attributes: aDictionaryOrArray
	^ (self named: aLocalName)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
name: aQualifiedName namespaceURI: aURI
	^ self new name: aQualifiedName namespaceURI: aURI! !

!XMLElement class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
name: aQualifiedName namespaceURI: aURI attributes: aDictionaryOrArray
	^ (self name: aQualifiedName namespaceURI: aURI)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray
	^ self new
		declareNamespaces: aNamespaceDictionaryOrArray;
		name: aQualifiedName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray attributes: anAttributeDictionaryOrArray
	^ (self
		name: aQualifiedName
		namespaces: aNamespaceDictionaryOrArray)
			setAttributes: anAttributeDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 3/11/2010 23:10'!
named: aLocalName
	^ self new name: aLocalName! !

!XMLElement methodsFor: 'notifying' stamp: 'monty 10/18/2014 00:39'!
addedAttribute: anAttribute
	anAttribute element: self.
	self validateAttributes.! !

!XMLElement methodsFor: 'notifying' stamp: 'monty 2/2/2014 04:13'!
addedElement: anElement
	super addedElement: anElement.
	(self hasNamespaces
		and: [self namespaceScope ~= anElement namespaceScope])
		ifTrue: [anElement inheritNamespacesFrom: self]! !

!XMLElement methodsFor: 'notifying' stamp: 'monty 10/31/2014 17:35'!
addingAttribute: anAttribute
	anAttribute element
		ifNotNil: [self errorAddingAttributeWithElement]! !

!XMLElement methodsFor: 'enumerating' stamp: 'monty 11/10/2014 21:05'!
allElementsDo: aBlock
	aBlock value: self.
	self hasChildren
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]].! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 11/6/2014 15:43'!
attributeAssociations
	^ self attributeNodes attributeAssociations! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 10/29/2014 09:52'!
attributeAt: aName
	"returns empty string when absent"
	^ self attributeNodes attributeValueAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsentPut: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:34'!
attributeAt: aName put: aValue
	^ self attributeNodes attributeValueAt: aName put: aValue! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/28/2014 18:38'!
attributeListClass
	^ XMLAttributeList! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:08'!
attributeNames
	^ self attributeNodes attributeNames! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:14'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self attributeNodes attributeNamesAndValuesDo: aTwoArgumentBlock! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:09'!
attributeNamesDo: aBlock
	self attributeNodes attributeNamesDo: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName
	^ self attributeNodes attributeNodeAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeNodeAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:36'!
attributeNodeAt: aLocalName namespaceURI: aURI
	^ self attributeNodes attributeNodeAt: aLocalName namespaceURI: aURI! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
attributeNodeAt: aLocalName namespaceURI: aURI ifAbsent: aBlock
	^ self attributeNodes attributeNodeAt: aLocalName namespaceURI: aURI ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 10/27/2014 18:23'!
attributeNodes
	^ attributes ifNil: [attributes := self attributeListClass observer: self]! !

!XMLElement methodsFor: 'enumerating' stamp: 'monty 11/26/2014 19:13'!
attributeNodesDo: aBlock
	self attributeNodes do: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:16'!
attributes
	^ self attributeNodes asDictionary! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 10/10/2014 05:56'!
declareNamespaces: aDictionaryOrArray
	aDictionaryOrArray associationsDo: [:each |
		self
			declarePrefix: each key
			uri: each value]! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 3/21/2015 22:36'!
declarePrefix: aPrefix uri: aURI
	| oldMapping |

	(oldMapping := self namespaceScope prefixMappingAt: aPrefix ifAbsent: [nil]) = aURI
		ifFalse: [
			self
				updateEnclosedScopesWith: [:scope | scope mapPrefix: aPrefix to: aURI]
				whileTrue: [:scope |
					(scope includesPrefix: aPrefix) not
						or: [scope includesPrefix: aPrefix mappedTo: oldMapping]]
				validateAfter: false]! !

!XMLElement methodsFor: 'testing' stamp: 'monty 10/10/2014 06:14'!
declaresDefaultNamespace
	"Did the receiver declare a new default namespace rather than inheriting one?"

	^ self namespaceScope hasDefaultNamespace
		and: [self isRoot or: [self defaultNamespace ~= self parent defaultNamespace]]! !

!XMLElement methodsFor: 'testing' stamp: 'monty 3/21/2015 22:37'!
declaresPrefix: aPrefix uri: aURI
	"Did the receiver declare a new prefix/uri pair rather than inheriting one?"

	^ (self isDeclaredPrefix: aPrefix uri: aURI)
		and: [self isRoot or: [(self parent isDeclaredPrefix: aPrefix uri: aURI) not]]! !

!XMLElement methodsFor: 'namespacing' stamp: 'JAAyer 6/20/2010 23:08'!
defaultNamespace
	^ self namespaceScope defaultNamespace! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 3/21/2015 22:37'!
defaultNamespace: aURI
	| oldNamespace |

	oldNamespace := self defaultNamespace.
	self
		updateEnclosedScopesWith: [:scope | scope defaultNamespace: aURI]
		whileTrue: [:scope | scope defaultNamespace = oldNamespace]
		validateAfter: false.! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/31/2014 17:35'!
errorAddingAttributeWithElement
	XMLDOMException signal: 'Cannot add attribute node from other element'! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 11/16/2014 19:44'!
expandedName
	^ self localName expandWithNamespaceURI: self namespaceURI! !

!XMLElement methodsFor: 'testing' stamp: 'monty 10/14/2014 07:51'!
hasAttributes
	^ attributes notNil and: [attributes notEmpty]! !

!XMLElement methodsFor: 'testing' stamp: 'monty 2/13/2015 22:20'!
hasNamespaceURI
	^ self namespaceURI notEmpty! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 10/16/2010 06:07'!
hasNamespaces
	^ namespaceScope notNil and: [namespaceScope hasNamespaces]! !

!XMLElement methodsFor: 'testing' stamp: 'monty 11/16/2014 17:30'!
hasPrefix
	^ self name size > self localName size! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:23'!
includesAttribute: aName
	^ self attributeNodes includesAttribute: aName.! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/10/2014 23:56'!
inheritNamespacesFrom: anElement
	self allElementsDo: [:each |
		each namespaceScope inheritMappingsFrom:
			anElement namespaceScope]! !

!XMLElement methodsFor: 'private' stamp: 'monty 11/17/2014 02:02'!
innerXMLContext
	^ XMLContentContext new! !

!XMLElement methodsFor: 'testing' stamp: 'monty 2/13/2015 21:10'!
isDeclaredPrefix: aPrefix
	"is namespace prefix in scope for use by receiver and descendants"
	^ self namespaceScope includesPrefix: aPrefix! !

!XMLElement methodsFor: 'testing' stamp: 'monty 3/21/2015 22:37'!
isDeclaredPrefix: aPrefix uri: aURI
	"is namespace prefix mapped to uri in scope for use by receiver and descendants"
	^ self namespaceScope
		includesPrefix: aPrefix
		mappedTo: aURI! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ true! !

!XMLElement methodsFor: 'testing' stamp: 'monty 11/5/2014 11:37'!
isElementNamed: aName
	^ self isNamed: aName! !

!XMLElement methodsFor: 'testing' stamp: 'monty 11/5/2014 11:37'!
isElementNamedAny: aNameCollection
	^ self isNamedAny: aNameCollection! !

!XMLElement methodsFor: 'testing' stamp: 'monty 12/23/2014 17:02'!
isInLanguage: aLanguage
	^ (self
		attributeNodeAt: 'xml:lang'
		ifAbsent: [
			^ self hasParent
				and: [self parent isInLanguage: aLanguage]])
			hasLanguage: aLanguage! !

!XMLElement methodsFor: 'testing' stamp: 'monty 11/16/2014 17:28'!
isNamed: aName
	"Does the receiver's qualified or local name match the argument?'"
	^ aName
		isQualifiedName: self name
		orLocalName: self localName! !

!XMLElement methodsFor: 'testing' stamp: 'monty 11/16/2014 18:09'!
isNamedAny: aNameCollection
	"Does the receiver's qualified or local name match any of the names supplied as the argument?'"
	^ aNameCollection
		includesQualifiedName: self name
		orLocalName: self localName! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 11/26/2010 12:03'!
isRoot
	"Is the receiver the root element?"
	^ (self hasParent and: [self parent isElement]) not! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 11/16/2014 20:21'!
localName
	^ localName ifNil: [localName := self name localNameAfterPrefix]! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 11/16/2014 17:36'!
name
	^ name ifNil: [name := '']! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 11/16/2014 20:21'!
name: aName
	self renameWith: [
		name := aName.
		localName := aName localNameAfterPrefix]! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:36'!
name: aQualifiedName namespaceURI: aURI
	self
		declarePrefix: (aQualifiedName prefixBeforeLocalName) uri: aURI;
		name: aQualifiedName.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 6/20/2010 23:10'!
namespaceScope
	^ namespaceScope ifNil: [namespaceScope := XMLNamespaceScope new]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/14/2010 12:52'!
namespaceURI
	^ self resolvePrefix: self prefix! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 10/29/2014 09:48'!
nextElement
	^ self parent ifNotNil: [:node | node elementAfter: self]! !

!XMLElement methodsFor: 'copying' stamp: 'monty 11/22/2014 08:20'!
postCopy
	super postCopy.

	namespaceScope
		ifNotNil: [namespaceScope := namespaceScope copy].
	attributes
		ifNotNil: [attributes := attributes deepCopyWithObserver: self].! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 11/16/2014 19:41'!
prefix
	^ self name prefixBeforeLocalName: self localName! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 11/16/2014 19:50'!
prefix: aPrefix
	self renameWith: [name := self localName qualifyWithPrefix:  aPrefix]! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 10/29/2014 09:48'!
previousElement
	^ self parent ifNotNil: [:node | node elementBefore: self]! !

!XMLElement methodsFor: 'removing' stamp: 'JAAyer 12/6/2010 21:24'!
removeAttribute: aName
	^ (self attributeNodes removeAttributeNodeAt: aName) value.! !

!XMLElement methodsFor: 'removing' stamp: 'monty 11/23/2014 00:55'!
removeAttribute: aName ifAbsent: aBlock
	^ (self attributeNodes
		removeAttributeNodeAt: aName
		ifAbsent: [^ aBlock value]) value.! !

!XMLElement methodsFor: 'removing' stamp: 'JAAyer 7/22/2010 08:54'!
removeAttributeNode: aNode
	^ self attributeNodes remove: aNode! !

!XMLElement methodsFor: 'removing' stamp: 'monty 11/23/2014 00:55'!
removeAttributeNode: aNode ifAbsent: aBlock
	^ self attributeNodes remove: aNode ifAbsent: aBlock! !

!XMLElement methodsFor: 'removing' stamp: 'monty 11/23/2014 00:29'!
removeAttributes
	self attributeNodes removeAll! !

!XMLElement methodsFor: 'notifying' stamp: 'monty 10/18/2014 00:43'!
removedAttribute: anAttribute
	anAttribute element: nil! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/18/2014 01:35'!
renameWith: aBlock
	| oldName |

	oldName := self name.
	aBlock value.
	self name = oldName
		ifFalse: [
			self validateName.
			self hasParent
				ifTrue: [
					self parent
						renamedElement: self
						from: oldName
						to: self name]].! !

!XMLElement methodsFor: 'notifying' stamp: 'monty 10/18/2014 05:56'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self attributeNodes	
		renamedAttribute: anAttribute
		from: anOldName
		to: aNewName.
	self validateAttributes.! !

!XMLElement methodsFor: 'namespacing' stamp: 'JAAyer 6/20/2010 23:08'!
resolvePrefix: aPrefix
	^ self namespaceScope resolvePrefix: aPrefix! !

!XMLElement methodsFor: 'initialization' stamp: 'monty 4/5/2015 21:30'!
setAttributes: aDictionaryOrArray
	self hasAttributes
		ifTrue: [self attributeNodes removeAll].
	aDictionaryOrArray associationsDo: [:each |
		self attributeNodes
			addName: each key
			value: each value].! !

!XMLElement methodsFor: 'initialization' stamp: 'monty 12/27/2013 03:42'!
setNamespaceScope: aNamespaceScope
	namespaceScope := aNamespaceScope! !

!XMLElement methodsFor: 'initialization' stamp: 'monty 4/3/2015 02:13'!
setQualifiedName: aQualifiedName localName: aLocalName attributes: anAttributeDictionary
	name := aQualifiedName.
	localName := aLocalName.

	anAttributeDictionary isEmpty "avoid ifNotEmpty: closure overhead"
		ifFalse: [
			"The setFrom method is used to avoid revalidating the
			attributes when adding them to the list"
			(attributes := self attributeListClass new: anAttributeDictionary size)
					observer: self;
					setFromDictionary: anAttributeDictionary].! !

!XMLElement methodsFor: 'private' stamp: 'monty 12/2/2014 22:57'!
sortKey
	^ self name! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 11/24/2014 06:54'!
undeclareDefaultNamespace
	self undeclarePrefix: ''! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 11/17/2014 16:45'!
undeclarePrefix: aPrefix
	| currentMapping |

	currentMapping := self namespaceScope prefixMappingAt: aPrefix ifAbsent: [^ self].
	self
		updateEnclosedScopesWith: [:scope | scope unmapPrefix: aPrefix]
		whileTrue: [:scope | scope includesPrefix: aPrefix mappedTo: currentMapping]
		validateAfter: true.! !

!XMLElement methodsFor: 'private' stamp: 'monty 11/4/2014 00:29'!
updateEnclosedScopesWith: aBlock whileTrue: aConditionalBlock validateAfter: aBoolean
	"Updates the receiver's scope and the scopes of all descendants with aBlock
	where aConditional is true. It stops descending down a path of a subtree
	as soon as aConditional is false, but continues down other paths."

	(aConditionalBlock value: self namespaceScope)
		ifFalse: [^ self].

	aBlock value: self namespaceScope.
	aBoolean
		ifTrue: [
			self
				validateName;
				validateAttributes].
	self elementsDo: [:each |
		each
			updateEnclosedScopesWith: aBlock
			whileTrue: aConditionalBlock
			validateAfter: aBoolean]! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/9/2014 21:08'!
validateAttributes
	self usesNamespaces
		ifTrue: [
			self namespaceScope
				validateAttributePrefixes: self attributeNodes attributesByName]! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/9/2014 06:42'!
validateName
	self usesNamespaces
		ifTrue: [
			self namespaceScope
				validateAndResolvePrefix: self prefix
				ofElement: self name]! !

!XMLElement methodsFor: 'validating' stamp: 'monty 4/5/2015 21:29'!
validateWith: aValidator
	| attributeDictionary |

	attributeDictionary := 
		self hasAttributes
			ifTrue: [self attributeNodes asDictionary]
			ifFalse: [Dictionary new].
	aValidator
		validateStartTag: self name
		attributes: attributeDictionary.
	self setAttributes: attributeDictionary.

	super validateWith: aValidator.

	aValidator validateEndTag: self name.! !

!XMLElement methodsFor: 'printing' stamp: 'monty 3/24/2015 19:26'!
writeXMLOn: aWriter
	| tagWriter |

	tagWriter := aWriter tag: self name.
	self hasNamespaces
		ifTrue: [tagWriter setXMLNSDeclarations: self namespaceScope prefixMappings].
	self hasAttributes
		ifTrue: [tagWriter setAttributes: self attributeNodes].

	tagWriter with: [
		self hasChildren
			ifTrue: [self writeInnerXMLOn: aWriter]].! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 11/1/2014 22:58'!
addContent: aStringOrStringNode
	"Adds a string or string node. If the last child node is already a string node,
	the characters being added are concatenated to it."

	(self hasChildren and: [self lastNode acceptsContent])
		ifTrue: [self lastNode addContent: aStringOrStringNode]
		ifFalse: [self addNode:
			(aStringOrStringNode isString
				ifTrue: [self nodeFactory newString: aStringOrStringNode]
				ifFalse: [aStringOrStringNode])]! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 11/1/2014 21:06'!
addElement: anElement
	self deprecated: 'use #addNode: or #addElementNamed: instead'.
	self addNode: anElement.! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 11/6/2014 12:27'!
addElementNamed: aName
	"element is added first before naming so it inherits namespaces from self"
	^ (self addNode: (self nodeFactory classForElement: aName) new)
		name: aName! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 3/21/2015 22:37'!
addElementNamed: aName namespaceURI: aURI
	^ self addNode:
		(self nodeFactory
			newElementNamed: aName
			namespaceURI: aURI)! !

!XMLNodeWithElements methodsFor: 'notifying' stamp: 'monty 10/28/2014 17:59'!
addedElement: anElement
	self addedNode: anElement.
	anElement configuration: self configuration.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 3/8/2015 09:17'!
allElements
	| allElements |

	allElements := self newListForSelect.
	self allElementsDo: [:each | allElements addLast: each].
	^ allElements.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 02:18'!
allElementsCollect: aBlock
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self allElementsDo: [:each | collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:47'!
allElementsDetect: aBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	^ self allElementsDetect: aBlock ifNone: [nil]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:35'!
allElementsDetect: aBlock ifNone: anExceptionBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	self allElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/14/2014 22:36'!
allElementsDo: aBlock
	"Descend depth-first visiting each element with aBlock."

	self hasChildren
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 13:58'!
allElementsNamed: aName
	^ self allElementsSelect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 13:17'!
allElementsNamed: aName do: aBlock
	self allElementsDo: [:each |
		(each isNamed: aName)
			ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
allElementsNamed: aName namespaceURI: aURI
	^ self allElementsSelect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aURI]]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 3/8/2015 09:17'!
allElementsSelect: aBlock
	| selectedElements |

	selectedElements := self newListForSelect.
	self allElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedElements addLast: each]].
	^ selectedElements.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/1/2014 11:42'!
configuration
	^ configuration ifNil: [
		self configuration: self configurationClass new.
		configuration]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/14/2014 07:43'!
configuration: aConfiguration
	configuration := aConfiguration.

	self hasChildren
		ifTrue: [self elementsDo: [:each | each configuration: aConfiguration]]! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/2/2014 18:52'!
configurationClass
	^ XMLDOMConfiguration! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/25/2014 21:08'!
contentNodes
	^ self nodes contentNodes! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:56'!
contentNodesDo: aBlock
	self nodes contentNodesDo: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/4/2014 17:44'!
contentString: aString
	self
		removeNodes;
		addContent: aString! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/25/2014 22:02'!
contentStringAt: aName
	^ self nodes contentStringAt: aName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 3/8/2015 09:17'!
descendentElements
	| descendentElements |

	descendentElements := self newListForSelect.
	self descendentElementsDo: [:each | descendentElements addLast: each].
	^ descendentElements.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 02:18'!
descendentElementsCollect: aBlock
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self descendentElementsDo: [:each | collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:34'!
descendentElementsDetect: aBlock
	^ self descendentElementsDetect: aBlock ifNone: [nil]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:34'!
descendentElementsDetect: aBlock ifNone: anExceptionBlock
	self descendentElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 01:55'!
descendentElementsDo: aBlock
	self hasChildren
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/7/2014 00:35'!
descendentElementsNamed: aName
	^ self descendentElementsSelect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:33'!
descendentElementsNamed: aName do: aBlock
	self descendentElementsDo: [:each |
		(each isNamed: aName)
			ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
descendentElementsNamed: aName namespaceURI: aURI
	^ self descendentElementsSelect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aURI]]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 3/8/2015 09:17'!
descendentElementsSelect: aBlock
	| selectedElements |

	selectedElements := self newListForSelect.
	self descendentElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedElements addLast: each]].
	^ selectedElements.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 2/14/2014 05:31'!
elementAfter: anElement
	^ self nodes elementAfter: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:31'!
elementAt: aName
	^ self nodes elementAt: aName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName ifAbsent: aBlock
	^ self nodes elementAt: aName ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
elementAt: aName namespaceURI: aURI
	^ self nodes elementAt: aName namespaceURI: aURI! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
elementAt: aName namespaceURI: aURI ifAbsent: aBlock
	^ self nodes elementAt: aName namespaceURI: aURI ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 2/14/2014 05:31'!
elementBefore: anElement
	^ self nodes elementBefore: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/26/2014 08:18'!
elementNames
	^ self nodes elementNames! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 5/4/2010 13:18'!
elements
	^ self nodes elements! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementsAt: aName
	^ self nodes elementsAt: aName! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:57'!
elementsAt: aName do: aBlock
	self nodes elementsAt: aName do: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
elementsAt: aName namespaceURI: aURI
	^ self nodes elementsAt: aName namespaceURI: aURI! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsCollect: aBlock
	^ self nodes elementsCollect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock
	^ self nodes elementsDetect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self nodes elementsDetect: aBlock ifNone: anExceptionBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 6/26/2010 00:55'!
elementsDo: aBlock
	self nodes elementsDo: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:36'!
elementsSelect: aBlock
	^ self nodes elementsSelect: aBlock! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 1/5/2015 09:12'!
escapesContentEntitiesWhenWriting
	"avoid initializing config; defualt to true"
	^ configuration isNil
		or: [configuration replacesContentEntityReferences]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 17:24'!
findElementNamed: aName
	^ self allElementsDetect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'monty 3/21/2015 22:37'!
findElementNamed: aName namespaceURI: aURI
	^ self allElementsDetect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aURI]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'monty 3/21/2015 22:37'!
findElementNamed: aName namespaceURI: aURI with: aBlock
	^ self allElementsDetect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aURI
				and: [aBlock value: each]]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 13:48'!
findElementNamed: aName with: aBlock
	^ self allElementsDetect: [:each | (each isNamed: aName) and: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:34'!
firstElement
	"returns nil if no elements"
	^ self nodes firstElement! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 10/25/2014 21:02'!
hasContentNodes
	^ self nodes hasContentNodes! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 11/24/2010 09:27'!
hasElements
	"Faster version used when traversing to avoid initializing the node list
	in leaf elements"
	^ nodes notNil and: [nodes hasElements]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 10/25/2014 21:01'!
hasStringNodes
	^ self nodes hasStringNodes! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 10/25/2014 21:04'!
includesElement: aName
	^ self nodes includesElement: aName! !

!XMLNodeWithElements methodsFor: 'parsing' stamp: 'monty 11/2/2014 19:48'!
innerXML: aStringOrStream
	self
		innerXML: aStringOrStream
		parsedWith: (self parserHandlerClass on: aStringOrStream)! !

!XMLNodeWithElements methodsFor: 'parsing' stamp: 'monty 11/14/2014 20:57'!
innerXML: aStringOrStream parsedWith: aParser
	self removeNodes.

	aParser
		pushInnerXMLRootNode: self;
		parseDocument.! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/2/2014 19:48'!
innerXMLContext
	self subclassResponsibility! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:15'!
isContentNode
	^ true! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:34'!
lastElement
	"returns nil if no elements"
	^ self nodes lastElement! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 8/7/2010 20:11'!
namespaceScope
	^ XMLNamespaceScope new! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:54'!
newCData: aString
	^ self nodeFactory newCData: aString! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:57'!
newComment: aString
	^ self nodeFactory newComment: aString! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/3/2014 19:37'!
newDocument
	^ self nodeFactory newDocument
		configuration: self configuration! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/3/2014 19:37'!
newElement
	^ self nodeFactory newElement
		configuration: self configuration! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/3/2014 19:37'!
newElementNamed: aName
	^ (self nodeFactory newElementNamed: aName)
		configuration: self configuration! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
newElementNamed: aName namespaceURI: aURI
	^ (self nodeFactory
		newElementNamed: aName
		namespaceURI: aURI)
		configuration: self configuration! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/1/2014 23:00'!
newPI
	^ self nodeFactory newPI! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:58'!
newString: aString
	^ self nodeFactory newString: aString! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/1/2014 11:02'!
nodeFactory
	^ self configuration nodeFactory! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/1/2014 11:37'!
nodeFactory: aNodeFactory
	self configuration nodeFactory: aNodeFactory! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/26/2013 22:23'!
nodeListClass
	^ self configuration nodeListClass! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/1/2014 15:03'!
parserHandlerClass
	^ self configuration parserHandlerClass! !

!XMLNodeWithElements methodsFor: 'printing' stamp: 'monty 11/19/2014 23:44'!
printRawContentOn: aStream
	self contentNodesDo: [:each | each printRawContentOn: aStream]! !

!XMLNodeWithElements methodsFor: 'notifying' stamp: 'monty 10/28/2014 17:59'!
removedElement: anElement
	self removedNode: anElement! !

!XMLNodeWithElements methodsFor: 'notifying' stamp: 'monty 10/18/2014 01:08'!
renamedElement: anElement from: anOldName to: aNewName
	self nodes
		renamedElement: anElement
		from: anOldName
		to: aNewName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/25/2014 21:00'!
stringNodes
	^ self nodes stringNodes! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:00'!
stringNodesDo: aBlock
	self nodes stringNodesDo: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/25/2014 21:00'!
strings
	^ self nodes strings! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:00'!
stringsDo: aBlock
	self nodes stringsDo: aBlock! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 14:48'!
usesNamespaces
	"Uses direct access and isNil test for performance"
	^ configuration isNil or: [configuration usesNamespaces]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 12:59'!
usesNamespaces: aBoolean
	self configuration usesNamespaces: aBoolean! !

!XMLNodeWithElements methodsFor: 'validating' stamp: 'monty 4/5/2015 21:06'!
validateWith: aValidator
	self hasChildren
		ifTrue: [self nodesDo: [:each | each validateWith: aValidator]]! !

XMLNode subclass: #XMLPI
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPI commentStamp: 'JAAyer 6/25/2010 14:06' prior: 0!
This class represents a processing instruction.
!

!XMLPI class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:52'!
target: aTargetString data: aDataString
	^ self new
		target: aTargetString;
		data: aDataString! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
data
	^ data ifNil: [data := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
data: aString
	data := aString! !

!XMLPI methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ true! !

!XMLPI methodsFor: 'private' stamp: 'monty 12/3/2014 01:08'!
sortKey
	^ self target! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/21/2010 12:42'!
target
	^ target ifNil: [target := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
target: aString
	target := aString! !

!XMLPI methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter pi
		target: self target;
		data: self data! !

XMLNode subclass: #XMLString
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLString commentStamp: '<historical>' prior: 0!
This class represents string nodes, or nodes that contain no markup. String nodes can be created directly using the class-side #string: message or by sending a String object #asXMLString. When printing, they escape special characters in their contents.!

XMLString subclass: #XMLCData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLCData commentStamp: 'monty 11/20/2014 04:08' prior: 0!
This class represents a preserved CDATA section from a parsed document. It behaves like its superclass, except that its printed form differs in that it contains a "<!![CDATA[" prefix and a "]]>" suffix and its contents are NOT escaped. By default, CDATA sections are converted to regular string nodes during parsing.

To preserve them as CDATA nodes, use #preservesCDataNodes: with the DOM parser before parsing.!

!XMLCData methodsFor: 'private' stamp: 'monty 2/11/2014 00:14'!
acceptsContent
	^ false! !

!XMLCData methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ true! !

!XMLCData methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter cdata: self string! !

!XMLString class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
string: aString
	^ self new string: aString! !

!XMLString methodsFor: 'private' stamp: 'monty 2/11/2014 00:14'!
acceptsContent
	^ true! !

!XMLString methodsFor: 'adding' stamp: 'monty 10/26/2014 05:53'!
addContent: aStringOrStringNode
	string :=
		string
			ifNil: [aStringOrStringNode asString]
			ifNotNil: [string, aStringOrStringNode asString]! !

!XMLString methodsFor: 'adding' stamp: 'monty 11/3/2014 22:12'!
append: aStringOrStringNode
	self addContent: aStringOrStringNode! !

!XMLString methodsFor: 'converting' stamp: 'monty 1/1/2014 07:14'!
asString
	^ self string! !

!XMLString methodsFor: 'converting' stamp: 'monty 11/4/2014 14:51'!
asXMLEscapedString
	^ self string asXMLEscapedString! !

!XMLString methodsFor: 'converting' stamp: 'JAAyer 7/22/2010 05:19'!
asXMLString
	^ self! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/3/2014 21:43'!
beginsWith: aStringOrStringNode
	^ self string beginsWith: aStringOrStringNode asString! !

!XMLString methodsFor: 'accessing' stamp: 'monty 11/4/2014 17:44'!
contentString: aString
	self string: aString! !

!XMLString methodsFor: 'replacing' stamp: 'monty 11/4/2014 12:38'!
deleteFrom: aStart to: anEnd
	self replaceFrom: aStart to: anEnd with: ''! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/3/2014 21:43'!
endsWith: aStringOrStringNode
	^ self string endsWith: aStringOrStringNode asString! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/6/2014 20:11'!
includesSubstring: aStringOrStringNode
	^ (self indexOfSubstring: aStringOrStringNode) > 0! !

!XMLString methodsFor: 'accessing' stamp: 'monty 2/2/2015 17:29'!
indexOfSubstring: aStringOrStringNode
	^ self string
		findString: aStringOrStringNode asString
		startingAt: 1! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:26'!
isContentNode
	^ true! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/3/2014 22:28'!
isEmpty
	^ self string isEmpty! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/5/2014 12:05'!
isString: aStringOrStringNode
	^ self string = aStringOrStringNode asString! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:35'!
isStringNode
	^ true! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/5/2014 12:09'!
isStringNode: aStringOrStringNode
	^ self isString: aStringOrStringNode! !

!XMLString methodsFor: 'testing' stamp: 'monty 4/4/2015 23:22'!
isWhitespace
	^ self isEmpty not
		and: [self string allSatisfy: [:each | each xmlParserIsWhitespace]]! !

!XMLString methodsFor: 'splitting/merging' stamp: 'monty 2/2/2015 17:30'!
mergeAdjacent
	"merges a string node with adjacent string nodes in parent"
	| siblingNodes selfIndex adjacentNode |

	self hasParent
		ifFalse: [^ self].

	siblingNodes := self parent nodes.
	selfIndex := siblingNodes indexOf: self.
	[(adjacentNode := siblingNodes at: selfIndex + 1) notNil
		and: [adjacentNode isStringNode]]
		whileTrue: [
			self append: adjacentNode.
			siblingNodes removeAt: selfIndex + 1].

	[(adjacentNode := siblingNodes at: selfIndex - 1) notNil
		and: [adjacentNode isStringNode]]
		whileTrue: [
			self prepend: adjacentNode.
			siblingNodes removeAt: selfIndex - 1.
			selfIndex := selfIndex - 1].! !

!XMLString methodsFor: 'adding' stamp: 'monty 11/3/2014 21:43'!
prepend: aStringOrStringNode
	self string: aStringOrStringNode asString, self string! !

!XMLString methodsFor: 'printing' stamp: 'monty 11/19/2014 23:44'!
printRawContentOn: aStream
	aStream nextPutAll: self string! !

!XMLString methodsFor: 'replacing' stamp: 'monty 11/4/2014 18:51'!
replaceAll: aSource with: aReplacement
	"works like copyReplaceAll:with:"
	self string:
		(self string
			copyReplaceAll: aSource asString
			with: aReplacement asString)! !

!XMLString methodsFor: 'replacing' stamp: 'monty 11/4/2014 18:54'!
replaceFrom: aStart to: aStop with: aReplacement
	"works like copyReplaceFrom:to:with: and
	can be used to insert, delete, and replace"
	| start stop |

	start := (aStart max: 1) min: self size + 1.
	stop := (aStop max: start - 1) min: self size.
	self string:
		(self string
			copyReplaceFrom: start
			to: stop
			with: aReplacement asString)! !

!XMLString methodsFor: 'accessing' stamp: 'monty 11/3/2014 23:41'!
size
	^ self string size! !

!XMLString methodsFor: 'private' stamp: 'monty 12/2/2014 22:56'!
sortKey
	^ self string! !

!XMLString methodsFor: 'splitting/merging' stamp: 'monty 11/3/2014 22:02'!
splitAt: anOffset
	| newNode |

	newNode := self copy.
	self replaceFrom: anOffset to: self size with: ''.
	newNode replaceFrom: 1 to: anOffset - 1 with: ''.

	self hasParent
		ifTrue: [self parent addNode: newNode after: self].

	^ newNode.! !

!XMLString methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:19'!
string
	^ string ifNil: [string := '']! !

!XMLString methodsFor: 'accessing' stamp: 'monty 1/1/2014 07:23'!
string: aString
	string := aString asString! !

!XMLString methodsFor: 'validating' stamp: 'monty 4/5/2015 21:04'!
validateWith: aValidator
	aValidator validatePCData: self string! !

!XMLString methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter string: self string! !

Object subclass: #XMLNodeContentWriteStreamAdapter
	instanceVariableNames: 'stream lastChar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLNodeContentWriteStreamAdapter commentStamp: 'monty 11/20/2014 03:56' prior: 0!
A stream for writing node content with whitespace insertion if needed between writes.!

!XMLNodeContentWriteStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 12/8/2014 08:55'!
on: aStream
	^ self new on: aStream! !

!XMLNodeContentWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 11/19/2014 23:14'!
contents
	^ stream contents! !

!XMLNodeContentWriteStreamAdapter methodsFor: 'private' stamp: 'monty 4/4/2015 23:21'!
insertSpaceBefore: aCharacter
	(lastChar == nil
		or: [lastChar xmlParserIsWhitespace
			or: [aCharacter xmlParserIsWhitespace]])
		ifFalse: [stream space]! !

!XMLNodeContentWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 12/11/2014 10:13'!
nextPut: aCharacter
	self insertSpaceBefore: aCharacter.
	stream nextPut: aCharacter.
	^ lastChar := aCharacter.! !

!XMLNodeContentWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 12/11/2014 10:13'!
nextPutAll: aString
	aString size > 0
		ifTrue: [
			self insertSpaceBefore: aString first.
			stream nextPutAll: aString.
			lastChar := aString last].
	^ aString.! !

!XMLNodeContentWriteStreamAdapter methodsFor: 'initialization' stamp: 'monty 11/19/2014 23:14'!
on: aStream
	stream := aStream! !

Object subclass: #XMLNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeFactory commentStamp: '<historical>' prior: 0!
This class is the default node factory. Its accessor messages return the class objects used by the DOM parser to build a node tree. You can override these messages to return different class objects and inject an instance of your factory using the DOM parser's #nodeFactory: accessor before parsing.!

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:58'!
attributeDeclarationClass
	^ XMLAttributeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
cdataClass
	^ XMLCData! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 13:23'!
classForElement: aQualifiedName
	^ self
		classForElement: aQualifiedName
		namespaceURI: ''! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
classForElement: aQualifiedName namespaceURI: aURI
	| prefix |

	^ self
		classForElement: aQualifiedName
		prefix: (prefix := aQualifiedName prefixBeforeLocalName)
		uri: aURI
		localName: (aQualifiedName localNameAfterPrefix: prefix)! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
classForElement: aQualifiedName prefix: aPrefix uri: aNamespaceURI localName: aLocalName
	^ self elementClass! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
commentClass
	^ XMLComment! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
doctypeDeclarationClass
	^ XMLDoctypeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
documentClass
	^ XMLDocument! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
elementClass
	^ XMLElement! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
elementDeclarationClass
	^ XMLElementDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
generalEntityDeclarationClass
	^ XMLGeneralEntityDeclaration! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:57'!
newCData: aString
	^ self cdataClass string: aString! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:57'!
newComment: aString
	^ self commentClass string: aString! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/6/2014 22:00'!
newDocument
	^ self documentClass new! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/6/2014 22:00'!
newElement
	^ self elementClass new! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/6/2014 22:01'!
newElementNamed: aName
	^ (self classForElement: aName)
		named: aName! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
newElementNamed: aName namespaceURI: aURI
	^ (self classForElement: aName namespaceURI: aURI)
		name: aName
		namespaceURI: aURI! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:55'!
newPI
	^ self piClass new! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:58'!
newString: aString
	^ self stringClass string: aString! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
notationDeclarationClass
	^ XMLNotationDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
parameterEntityDeclarationClass
	^ XMLParameterEntityDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
piClass
	^ XMLPI! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
stringClass
	^ XMLString! !

XMLNodeFactory subclass: #XMLPluggableNodeFactory
	instanceVariableNames: 'documentClass elementClass stringClass cdataClass commentClass piClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPluggableNodeFactory commentStamp: 'monty 11/1/2014 11:23' prior: 0!
This is a pluggable factory that can be used to change which classes the DOM parser uses to build the DOM tree. Here is an example:

	(XMLDOMParser on: someXML)
		nodeFactory:
			(XMLPluggableFactory new
				documentClass: MyDocumentClass;
				elementClass: MyElementClass;
				stringClass: MyStringClass)
		parseDocument.!

XMLPluggableNodeFactory subclass: #XMLPluggableElementFactory
	instanceVariableNames: 'elementClassesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPluggableElementFactory commentStamp: '<historical>' prior: 0!
This class is a pluggable factory that can map elements to different XMLElement subclasses based on the name and namespace information of those elements. You have to create an instance, configure it to handle certain elements with certain classes, and then inject your instance into a DOM parser using #nodeFactory: before parsing. Here is an example of its use:

	doc := (XMLDOMParser on: someXML)
		nodeFactory:
			(XMLPluggableElementFactory new
				elementClass: GenericElement;
				handleElement: 'user' withClass: UserElement;
				handleElement: 'report' withClass: ReportElement;
				handleElement: 'report' namespaceURI: 'urn:specialreprot' withClass: SpecialReportElement)
		parseDocument.
		
The #handleElementsMatchingClass*: forms try to match elements to the specified classes based on the names of those classes when stripped of any prefix (like XML) and "Element" suffix and converted to both camel case and a hyphenated form. So this:
	XMLPluggableElementFactory new
		handleElementsMatchingClass: MYRootUserElement
		....
will handle 'rootUser' and 'root-user' elements with the MYRootUserElement class.!

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
classForElement: aName prefix: aPrefix uri: aURI localName: aLocalName
	^ (self classForElement: aName uri: aURI)
		ifNil: [
			aPrefix isEmpty
				ifTrue: [self elementClass]
				ifFalse: [
					(self classForElement: aLocalName uri: aURI)
						ifNil: [self elementClass]]]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'monty 3/21/2015 22:37'!
classForElement: aName uri: aURI
	| classesForElement |

	classesForElement :=
		self elementClassesByName
			at: aName
			ifAbsent: [^ nil].

	^ classesForElement
		at: aURI
		ifAbsent: [
			classesForElement
				at: ''
				ifAbsent: [nil]].! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/13/2010 08:36'!
elementClassesByName
	^ elementClassesByName ifNil: [elementClassesByName := Dictionary new]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'monty 3/21/2015 22:37'!
handleElement: anElementName namespaceURI: aURI withClass: aClass
	(self elementClassesByName
		at: anElementName
		ifAbsentPut: [Dictionary new])
			at: aURI put: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/13/2010 19:37'!
handleElement: anElementName withClass: aClass
	self
		handleElement: anElementName
		namespaceURI: ''
		withClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:39'!
handleElements: anElementNameCollection namespaceURI: aNamespaceURI withClass: aClass
	anElementNameCollection do: [:each |
		self
			handleElement: each
			namespaceURI: aNamespaceURI
			withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:36'!
handleElements: anElementNameCollection withClass: aClass
	anElementNameCollection do: [:each | self handleElement: each withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/22/2010 11:10'!
handleElementsMatchingClass: aClass
	| className |

	(className := self toNameWithoutPrefixAndElementSuffix: aClass name asString)
		ifEmpty: [^ self].

	self
		handleElement: (self toCamelCasedName: className)
		withClass: aClass.
	self
		handleElement: (self toHyphenatedName: className)
		withClass: aClass.! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:26'!
handleElementsMatchingClasses: aClassCollection
	aClassCollection do: [:each | self handleElementsMatchingClass: each]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:29'!
toCamelCasedName: aString
	^ aString allButFirst copyWithFirst: aString first asLowercase! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:34'!
toHyphenatedName: aString
	| nameStream hyphenatedStream previousChar nextChar |

	nameStream := aString readStream.
	hyphenatedStream := (String new: nameStream size) writeStream.
	[nameStream atEnd]
		whileFalse: [
			((nextChar := nameStream next) isUppercase
				and: [previousChar notNil and: [previousChar isLowercase]])
				ifTrue: [hyphenatedStream nextPut: $-].
			hyphenatedStream nextPut: nextChar asLowercase.
			previousChar := nextChar].
	^ hyphenatedStream contents.! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'monty 12/14/2014 14:41'!
toNameWithoutPrefixAndElementSuffix: aString
	| nameStream |

	nameStream := aString readStream.
	[nameStream atEnd not and: [nameStream peek isUppercase]]
		whileTrue: [nameStream next].
	nameStream atEnd
		ifTrue: [^ ''].
	nameStream position: nameStream position - 1.

	^ nameStream upToEnd copyReplaceAll: 'Element' with: ''.! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:44'!
cdataClass
	^ cdataClass ifNil: [cdataClass := super cdataClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:44'!
cdataClass: aClass
	cdataClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:45'!
commentClass
	^ commentClass ifNil: [commentClass := super commentClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:45'!
commentClass: aClass
	commentClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:42'!
documentClass
	^ documentClass ifNil: [documentClass := super documentClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:42'!
documentClass: aClass
	documentClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:41'!
elementClass
	^ elementClass ifNil: [elementClass := super elementClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:42'!
elementClass: aClass
	elementClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:44'!
piClass
	^ piClass ifNil: [piClass := super piClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:44'!
piClass: aClass
	piClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:43'!
stringClass
	^ stringClass ifNil: [stringClass := super stringClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:43'!
stringClass: aClass
	stringClass := aClass! !

Object subclass: #XMLParser
	instanceVariableNames: 'tokenizer driver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParser commentStamp: '<historical>' prior: 0!
This class is a facade that parses XML using a driver and tokenizer. See SAXHandler and XMLDOMParser for ready-made front ends to it.!

!XMLParser class methodsFor: 'instance creation' stamp: 'JAAyer 11/28/2010 07:09'!
driver: aDriver on: aStringOrStream
	^ self new setDriver: aDriver tokenizer: (aDriver tokenizerOn: aStringOrStream)! !

!XMLParser methodsFor: 'testing' stamp: 'JAAyer 11/27/2010 07:17'!
atEnd
	^ self tokenizer atEnd! !

!XMLParser methodsFor: 'accessing' stamp: 'monty 11/14/2013 23:34'!
currentLineNumber
	^ self tokenizer currentLineNumber! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 07:07'!
driver
	^ driver! !

!XMLParser methodsFor: 'private' stamp: 'monty 11/26/2014 07:46'!
handleParserError: anError
	self tokenizer closeStreams.

	(anError isKindOf: XMLParserException)
		ifTrue: [
			anError appendLineNumber: self currentLineNumber.
			anError shouldPassOnWhenCaught
				ifTrue: [anError pass]]
		ifFalse: [anError pass].! !

!XMLParser methodsFor: 'accessing' stamp: 'monty 11/9/2014 16:01'!
initialContext: aContext
	self tokenizer tokenContext: aContext! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/24/2014 19:31'!
parseDocument
	[[self tokenizer atEnd]
		whileFalse: [self tokenizer nextToken]]
		on: Error
		do: [:error | self handleParserError: error]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/26/2014 21:27'!
parseDocumentUntil: aBlock
	[[self tokenizer atEnd or: [aBlock value]]
		whileFalse: [self tokenizer nextToken]]
		on: Error
		do: [:error | self handleParserError: error]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/24/2014 19:31'!
parseDocumentWhile: aBlock
	[[self tokenizer atEnd not and: [aBlock value]]
		whileTrue: [self tokenizer nextToken]]
		on: Error
		do: [:error | self handleParserError: error]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/24/2014 19:31'!
parseToken
	^ [self tokenizer nextToken]
		on: Error
		do: [:error | self handleParserError: error]! !

!XMLParser methodsFor: 'initialization' stamp: 'JAAyer 11/28/2010 07:09'!
setDriver: aDriver tokenizer: aTokenizer
	driver := aDriver.
	tokenizer := aTokenizer.! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 07:16'!
tokenizer
	^ tokenizer! !

Object subclass: #XMLParserDriver
	instanceVariableNames: 'uris'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParserDriver commentStamp: '<historical>' prior: 0!
This is a base class for parser drivers. Drivers are sent messages by a tokenizer to handle various XML tokens as they are read. To create a driver, subclass this class and override the handler messages and testing messages that control tokenization.!

XMLParserDriver subclass: #DTDExternalSubsetParserDriver
	instanceVariableNames: 'saxDriver doctypeDefinition documentURI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!DTDExternalSubsetParserDriver commentStamp: '<historical>' prior: 0!
This class is a driver for parsing the external subset of a DTD.!

!DTDExternalSubsetParserDriver class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 23:54'!
saxDriver: aSaxDriver documentURI: anExternalSubsetURI
	^ self new
		setSAXDriver: aSaxDriver
		documentURI: anExternalSubsetURI! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:14'!
doctypeDefinition
	^ doctypeDefinition! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 3/21/2015 23:23'!
documentURI
	^ documentURI! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:04'!
externalEntityResolver
	^ self saxDriver externalEntityResolver! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleAttributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self isValidating
		ifTrue: [
			self doctypeDefinition
				addValidatorForAttributeIn: anElement
				named: aName
				type: aType
				defaultPragma: aDefaultPragma
				defaultValue: aDefaultValue]! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleElementDeclaration: aName contentModel: aContentModel
	self isValidating
		ifTrue: [
			self doctypeDefinition
				addValidatorForElement: aName
				contentModel: aContentModel]! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 4/5/2015 07:54'!
handleGeneralEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotation
	aNotation isEmpty
		ifTrue: [	
			self doctypeDefinition
				addExternalGeneralEntity: aName
				resolver: self externalEntityResolver
				uri: aSystemID
				from: self currentURI]
		ifFalse: [
			self isValidating
				ifTrue: [
					self doctypeDefinition unparsedEntityValidator
						validateUnparsedEntity: aName
						withNotation: aNotation].
			self doctypeDefinition
				addUnparsedEntity: aName
				ndata: aNotation]! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleGeneralEntityDeclaration: aName replacement: aReplacement
	self doctypeDefinition
		addGeneralEntity: aName
		replacement: aReplacement
		from: self currentURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:36'!
handleNotationDeclaration: aName publicID: aPublicID systemID: aSystemID
	self doctypeDefinition addNotation: aName! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:36'!
handleParameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID
	self doctypeDefinition
		addExternalParameterEntity: aName
		resolver: self externalEntityResolver
		uri: aSystemID
		from: self currentURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleParameterEntityDeclaration: aName replacement: aReplacement
	self doctypeDefinition
		addParameterEntity: aName
		replacement:aReplacement
		from: self currentURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 1/4/2015 00:11'!
handleParameterEntityReference: aName
	^ self saxDriver handleParameterEntityReference: aName! !

!DTDExternalSubsetParserDriver methodsFor: 'initialization' stamp: 'monty 11/17/2014 02:02'!
initialContext
	^ XMLExternalSubsetContext new textDeclarationContext! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 2/7/2014 23:33'!
isParsingExternalSubset
	^ true! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:58'!
isValidating
	^ self saxDriver isValidating! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 1/16/2014 19:10'!
maxEntityReplacementDepth
	^ self saxDriver maxEntityReplacementDepth! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 1/6/2015 04:12'!
preservesUndeclaredEntityReferences
	^ self saxDriver preservesUndeclaredEntityReferences! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 1/27/2015 22:51'!
readLimit
	^ self externalEntityResolver externalSubsetReadLimit! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 1/4/2015 00:15'!
replacesContentEntityReferences
	^ self saxDriver replacesContentEntityReferences! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 1/4/2015 00:17'!
resolvesExternalEntities
	^ self saxDriver resolvesExternalEntities! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:04'!
saxDriver
	^ saxDriver! !

!DTDExternalSubsetParserDriver methodsFor: 'initialization' stamp: 'monty 3/21/2015 23:23'!
setSAXDriver: aSaxdriver documentURI: aURI
	saxDriver := aSaxdriver.
	doctypeDefinition := aSaxdriver doctypeDefinition.
	documentURI := aURI.! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 11/30/2013 21:55'!
usesNamespaces
	^ self saxDriver usesNamespaces! !

XMLParserDriver subclass: #SAXParserDriver
	instanceVariableNames: 'saxHandler elementNester scopeAllocator doctypeDefinition validator externalSubsetURI isStandalone hasUnhandledReservedAttributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXParserDriver commentStamp: 'monty 2/9/2014 02:43' prior: 0!
SAXParserDriver together with SAXHandler provides a SAX-like API for XML parsing. The driver processes XML tokens and signals SAX events to a SAXHandler. To use it, create a subclass of SAXHandler and override messages to handle SAX events of interest. See "instance creation" on the class-side of SAXHandler for ways to instantiate such a parser.!

!SAXParserDriver class methodsFor: 'instance creation' stamp: 'monty 10/27/2014 09:12'!
saxHandler: aHandler
	^ self new setSAXHandler: aHandler! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/3/2014 19:19'!
checkForReservedAttributesIn: anAttributeDictionary after: anIndex
	"validation may add attributes"
	self hasUnhandledReservedAttributes
		ifFalse: [
			anIndex + 1
				to: anAttributeDictionary size
				do: [:i |
					(anAttributeDictionary keyAtIndex: i) isReservedAttributeName
						ifTrue: [^ self haveUnhandledReservedAttributes]]]! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 04:13'!
currentElement
	^ self elementNester currentElement! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 1/5/2015 21:05'!
declareNamespaces: aDictionary inScope: aScope
	aScope mapPrefixesAndURIs: aDictionary.
	aDictionary keysAndValuesDo: [:prefix :uri |
		self saxHandler startPrefixMapping: prefix uri: uri].! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:16'!
doctypeDefinition
	^ doctypeDefinition ifNil: [doctypeDefinition := DTDDoctypeDefinition new]! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:44'!
documentReadLimit
	^ self saxHandler documentReadLimit! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 11/16/2013 04:35'!
documentURI
	^ self saxHandler documentURI! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 10/27/2014 09:11'!
elementNester
	^ elementNester! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 12/27/2013 04:08'!
elementNesterClass
	^ XMLWellFormedElementNester! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 2/11/2014 01:53'!
endNestingOfElement: anElementName withScopedAttributes: anAttributeDictionary
	| base |

	base :=
		anAttributeDictionary
			at: 'xml:base'
			ifAbsent: [
				self elementNester endElement: anElementName.
				^ self].
	(self elementNester
		endElement: anElementName;
		scopedAttributeAt: 'xml:base') = base
			ifFalse: [self removeURI: base].! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/2/2014 18:39'!
errorCharacterDataOutsideElement
	XMLWellFormednessException signal: 'Character data outside of element'! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 17:23'!
errorDocumentConstraintsRequired
	XMLValidationException signal: 'Document validation constraints required'! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 21:41'!
errorUndeclaredGeneralEntity: aName
	XMLValidationException
		formatSignal: 'Undeclared general entity reference &{1};'
		with: aName! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 21:14'!
errorUndeclaredParameterEntity: aName
	XMLValidationException
		formatSignal: 'Undeclared parameter entity reference %{1};'
		with: aName! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 21:41'!
errorUndeclaredUnparsedEntity: aName
	XMLValidationException
		formatSignal: 'Undeclared unparsed entity reference &{1}; as attribute value'
		with: aName! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 2/3/2014 17:42'!
externalEntityResolver
	^ self saxHandler externalEntityResolver! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 3/21/2015 22:36'!
externalSubsetParserOnURI: aURI
	^ self saxHandler parserClass
		driver:
			(DTDExternalSubsetParserDriver
				saxDriver: self
				documentURI: aURI asString)
		on: (self externalEntityResolver resolveEntityURI: aURI)
	! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 3/21/2015 23:24'!
externalSubsetURI
	^ externalSubsetURI! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleAttributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self isValidating
		ifTrue: [
			self doctypeDefinition
				addValidatorForAttributeIn: anElement
				named: aName
				type: aType
				defaultPragma: aDefaultPragma
				defaultValue: aDefaultValue].
	self saxHandler
		attributeDeclaration: anElement
		name: aName
		type: aType
		defaultPragma: aDefaultPragma
		defaultValue: aDefaultValue! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 11/3/2014 13:14'!
handleCData: aString
	self elementNester isInElement
		ifFalse: [self errorCharacterDataOutsideElement].
	self isValidating
		ifTrue: [self validator validatePCData: aString].

	self saxHandler
		startCData;
		startCDataSection;
		characters: aString;
		endCDataSection;
		endCData.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 12/11/2013 22:26'!
handleComment: aString
	self saxHandler comment: aString! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleElementDeclaration: aName contentModel: aContentModel
	self isValidating
		ifTrue: [
			self doctypeDefinition
				addValidatorForElement: aName
				contentModel: aContentModel].

	self saxHandler
		elementDeclaration: aName
		contentModel: aContentModel! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 11/16/2013 05:10'!
handleEndContentEntityReplacement: aName
	self saxHandler endEntity: aName! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/24/2015 20:26'!
handleEndDTD
	(self resolvesExternalEntities
		and: [self externalSubsetURI notEmpty
			and: [self isValidating or: [self isStandalone not]]])
		ifTrue: [self parseExternalSubset: self externalSubsetURI].

	self saxHandler
		doctypeDefinition: self doctypeDefinition;
		endDTD.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 2/2/2014 02:44'!
handleEndDocument
	self elementNester endNesting.
	self isValidating
		ifTrue: [self validator validateEndDocument].

	self saxHandler endDocument.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 11/16/2014 20:23'!
handleEndTag: anElementName
	"performance critical method"
	| prefix currentScope |

	self elementNester hasScopedAttributes
		ifTrue: [
			self
				endNestingOfElement: anElementName
				withScopedAttributes: self elementNester currentScopedAttributes]
		ifFalse: [self elementNester endElement: anElementName].

	prefix := anElementName prefixBeforeLocalName.
	self saxHandler 
		endElement: anElementName
		prefix: prefix
		uri:
			(self usesNamespaces
				ifTrue: [
					currentScope := self scopeAllocator exitScope.
					currentScope resolvePrefix: prefix]
				ifFalse: [''])
		localName: (anElementName localNameAfterPrefix: prefix).

	currentScope ifNotNil: [
		currentScope hasNamespaces
			ifTrue: [self undeclareNamespacesInScope: currentScope]].
	self isValidating
		ifTrue: [self validator validateEndTag: anElementName].! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 4/5/2015 07:53'!
handleGeneralEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotation
	aNotation isEmpty
		ifTrue: [
			self resolvesExternalEntities
				ifTrue: [
					self doctypeDefinition
						addExternalGeneralEntity: aName
						resolver: self externalEntityResolver
						uri: aSystemID
						from: self currentURI]]
		ifFalse: [
			self isValidating
				ifTrue: [
					self doctypeDefinition unparsedEntityValidator
						validateUnparsedEntity: aName
						withNotation: aNotation].
			self doctypeDefinition
				addUnparsedEntity: aName
				ndata: aNotation].

	self saxHandler
		generalEntityDeclaration: aName
		publicID: aPublicID
		systemID: aSystemID
		ndata: aNotation.! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleGeneralEntityDeclaration: aName replacement: aReplacement
	self doctypeDefinition
		addGeneralEntity: aName
		replacement: aReplacement
		from: self currentURI.

	self saxHandler
		generalEntityDeclaration: aName
		replacement: aReplacement.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 3/2/2015 11:14'!
handleGeneralEntityReference: aName
	^ self doctypeDefinition
		generalEntityAt: aName
		ifAbsent: [
			self isValidating
				ifTrue: [self errorUndeclaredGeneralEntity: aName].
			nil]! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:36'!
handleNotationDeclaration: aName publicID: aPublicID systemID: aSystemID
	self isValidating
		ifTrue: [self doctypeDefinition addNotation: aName].

	self saxHandler
		notationDeclaration: aName
		publicID: aPublicID
		systemID: aSystemID.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 11/3/2014 13:09'!
handlePCData: aString
	self elementNester isInElement
		ifFalse: [self errorCharacterDataOutsideElement].
	self isValidating
		ifTrue: [self validator validatePCData: aString].

	self saxHandler characters: aString.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 3/21/2015 23:28'!
handlePI: aTargetString data: aDataString
	self saxHandler 
		processingInstruction: aTargetString
		data: aDataString! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:36'!
handleParameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID
	self resolvesExternalEntities
		ifTrue: [
			self doctypeDefinition
				addExternalParameterEntity: aName
				resolver: self externalEntityResolver
				uri: aSystemID
				from: self currentURI].

	self saxHandler
		parameterEntityDeclaration: aName
		publicID: aPublicID
		systemID: aSystemID.! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleParameterEntityDeclaration: aName replacement: aReplacement
	self doctypeDefinition
		addParameterEntity: aName
		replacement:aReplacement
		from: self currentURI.

	self saxHandler
		parameterEntityDeclaration: aName
		replacement: aReplacement.! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleParameterEntityReference: aName
	^ self doctypeDefinition
		parameterEntityAt: aName
		ifAbsent: [
			self isValidating
				ifTrue: [self errorUndeclaredParameterEntity: aName].
			nil]! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/6/2014 02:28'!
handleReservedAttributes: anAttributeDictionary
	anAttributeDictionary
		at: 'xml:base'
		ifPresent: [:uri | self currentURI: uri].
	self isValidating
		ifTrue: [self validator validateXMLIDAttributeIn: anAttributeDictionary].

	self elementNester
		setScopedAttributesNamed: self scopedAttributeNames
		from: anAttributeDictionary.

	hasUnhandledReservedAttributes := false.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 3/24/2015 20:28'!
handleStartContent
	self saxHandler startContent! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 12/18/2013 22:08'!
handleStartContentEntityReplacement: aName
	self saxHandler startEntity: aName! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/24/2015 20:25'!
handleStartDTD: aRoot publicID: aPublicID systemID: aSystemID
	externalSubsetURI := aSystemID.

	self doctypeDefinition root: aRoot.
	self saxHandler
		startDTD: aRoot
		publicID: aPublicID
		systemID: aSystemID.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'mir 8/14/2000 18:29'!
handleStartDocument
	self saxHandler startDocument! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 11/16/2014 20:23'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary
	"performance critical method"
	| prefix totalAttributes namespaceUri currentScope |

	self elementNester startElement: anElementName.

	prefix := anElementName prefixBeforeLocalName.
	totalAttributes := anAttributeDictionary size.
	self usesNamespaces
		ifTrue: [
			currentScope := self scopeAllocator enterScope.
			aNamespaceDictionary
				ifNotNil: [self declareNamespaces: aNamespaceDictionary inScope: currentScope].
			namespaceUri := currentScope validateAndResolvePrefix: prefix ofElement: anElementName.
			totalAttributes > 0
				ifTrue: [currentScope validateAttributePrefixes: anAttributeDictionary]].
	self isValidating
		ifTrue: [
			self elementNester isCurrentElementRoot
				ifTrue: [self validateRootElement: anElementName].
			self validator
				validateStartTag: anElementName
				attributes: anAttributeDictionary.
			anAttributeDictionary size > totalAttributes
				ifTrue: [self checkForReservedAttributesIn: anAttributeDictionary after: totalAttributes]].
	self hasUnhandledReservedAttributes
		ifTrue: [self handleReservedAttributes: anAttributeDictionary].

	self saxHandler
		startElement: anElementName
		prefix: prefix
		uri: (namespaceUri ifNil: [''])
		localName: (anElementName localNameAfterPrefix: prefix)
		attributes: anAttributeDictionary.
	currentScope
		ifNotNil: [self saxHandler currentScope: currentScope].! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 1/4/2015 00:01'!
handleUnreplacedGeneralEntityReference: aName
	self saxHandler skippedEntity: aName! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 11/2/2014 19:58'!
handleWhitespace: aString
	(self elementNester hasScopedAttributes
		and: [(self elementNester scopedAttributeAt: 'xml:space') = 'preserve'])
		ifTrue: [self handlePCData: aString]
		ifFalse: [self saxHandler ignorableWhitespace: aString]! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 3/9/2015 13:59'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo
	isStandalone := aYesOrNo = 'yes'.

	self saxHandler
		xmlVersion: (Float readFrom: aVersion readStream)
		encoding: anEncoding
		standalone: isStandalone.! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/3/2014 11:19'!
hasUnhandledReservedAttributes
	^ hasUnhandledReservedAttributes! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/3/2014 16:16'!
haveUnhandledReservedAttributes
	hasUnhandledReservedAttributes := true! !

!SAXParserDriver methodsFor: 'initialization' stamp: 'monty 4/6/2015 01:44'!
initialContext
	^ XMLXMLDeclarationContext new! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 3/12/2015 14:06'!
isStandalone
	^ isStandalone! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:43'!
isValidating
	^ self saxHandler isValidating! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:45'!
maxEntityReplacementDepth
	^ self saxHandler maxEntityReplacementDepth! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 3/21/2015 22:36'!
parseExternalSubset: aURI
	| parser |

	parser :=
		self externalSubsetParserOnURI:
			(self externalEntityResolver
				combineURI: self documentURI
				with: aURI).
	parser parseDocument.! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 1/6/2015 04:12'!
preservesUndeclaredEntityReferences
	^ self saxHandler preservesUndeclaredEntityReferences! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:45'!
readLimit
	^ self documentReadLimit! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 1/4/2015 00:14'!
replacesContentEntityReferences
	^ self saxHandler replacesContentEntityReferences! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/9/2014 02:28'!
resolvesExternalEntities
	^ self saxHandler resolvesExternalEntities! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 08:00'!
saxHandler
	^ saxHandler! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 03:40'!
scopeAllocator
	^ scopeAllocator ifNil: [scopeAllocator := XMLNamespaceScopeAllocator new]! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/2/2014 20:45'!
scopedAttributeNames
	^ #('xml:space' 'xml:base' 'xml:lang')! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 10/27/2014 15:01'!
setRootElement: anElementName currentScope: aNamespaceScope
	self elementNester setExternalRootElement: anElementName.
	self usesNamespaces
		ifTrue: [self scopeAllocator currentScope: aNamespaceScope].! !

!SAXParserDriver methodsFor: 'initialization' stamp: 'monty 3/21/2015 23:24'!
setSAXHandler: aHandler
	saxHandler := aHandler.
	elementNester := self elementNesterClass new.
	externalSubsetURI := ''.
	isStandalone := false.
	hasUnhandledReservedAttributes := false.! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/10/2014 05:53'!
undeclareNamespacesInScope: aScope
	| currentScope |

	currentScope := self scopeAllocator currentScope.
	aScope prefixesAndURIsDo: [:prefix :uri |
		(currentScope includesPrefix: prefix mappedTo: uri)
			ifFalse: [self saxHandler endPrefixMapping: prefix]].! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:43'!
usesNamespaces
	^ self saxHandler usesNamespaces! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 17:23'!
validateRootElement: aStartTag
	(self saxHandler requiresDocumentConstraints
		and: [self validator hasElementValidators not])
		ifTrue: [self errorDocumentConstraintsRequired].

	self validator validateRootElement: aStartTag.! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:14'!
validator
	^ validator ifNil: [validator := DTDDocumentValidator doctypeDefinition: self doctypeDefinition]! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 4/5/2015 07:48'!
currentURI
	^ self uris lastOrNil ifNil: [self documentURI]! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
currentURI: aURI
	self uris addLast: aURI! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:14'!
doctypeDefinition
	^ nil! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 11/16/2013 04:35'!
documentURI
	^ nil! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 12/30/2013 23:46'!
handleAttributeDeclaration: anElement name: anName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleCData: aString! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleComment: aString! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 11/14/2013 03:46'!
handleElementDeclaration: aName contentModel: aContentModel! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 11/16/2013 05:10'!
handleEndContentEntityReplacement: aName
! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 11/12/2013 17:36'!
handleEndDTD! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleEndDocument! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleEndTag: anElementName! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:37'!
handleGeneralEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotation! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 11/14/2013 03:10'!
handleGeneralEntityDeclaration: aName replacement: aReplacement! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 11/14/2013 20:44'!
handleGeneralEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:37'!
handleNotationDeclaration: aName publicID: aPublicID systemID: aSystemID! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handlePCData: aString! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 3/21/2015 23:27'!
handlePI: aTargetString data: aDataString! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:37'!
handleParameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 11/14/2013 03:10'!
handleParameterEntityDeclaration: aName replacement: aReplacement! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 11/14/2013 20:44'!
handleParameterEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 1/4/2014 07:42'!
handleParseError: anErrorString
	XMLWellFormednessException signal: anErrorString! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 3/24/2015 20:27'!
handleStartContent! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 11/16/2013 05:10'!
handleStartContentEntityReplacement: aName! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:37'!
handleStartDTD: aRoot publicID: aPublicID systemID: aSystemID! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleStartDocument! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 1/4/2015 00:01'!
handleUnreplacedGeneralEntityReference: aName! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleWhitespace: aString! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo! !

!XMLParserDriver methodsFor: 'initialization' stamp: 'monty 11/2/2014 19:09'!
initialContext
	^ nil! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 2/8/2014 02:03'!
isParsingExternalSubset
	^ false! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 2/9/2014 02:41'!
isValidating
	^ false! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 1/7/2015 03:46'!
preservesUndeclaredEntityReferences
	^ false! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 1/16/2014 19:45'!
readLimit
	^ nil! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
removeURI: aURI
	"remove: removes the first match"
	self uris removeAt:
		(self uris
			lastIndexOf: aURI
			ifAbsent: [^ self])! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 1/4/2015 00:14'!
replacesContentEntityReferences
	^ true! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 1/4/2015 00:14'!
resolvesExternalEntities
	^ false! !

!XMLParserDriver methodsFor: 'private' stamp: 'monty 2/10/2014 02:43'!
tokenizerClass
	^ XMLWellFormedParserTokenizer! !

!XMLParserDriver methodsFor: 'initialization' stamp: 'monty 1/17/2014 01:28'!
tokenizerOn: aStringOrStream
	^ self tokenizerClass
		driver: self
		on: aStringOrStream
		readLimit: self readLimit! !

!XMLParserDriver methodsFor: 'private' stamp: 'monty 1/20/2014 02:25'!
uris
	^ uris ifNil: [uris := OrderedCollection new]! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 2/9/2014 02:41'!
usesNamespaces
	^ true! !

Object subclass: #XMLParserLimits
	instanceVariableNames: 'documentReadLimit maxEntityReplacementDepth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

XMLParserLimits class
	instanceVariableNames: 'defaultReadLimit'!

!XMLParserLimits commentStamp: 'monty 1/25/2014 04:00' prior: 0!
This class places security limits on parser input. You can set these using the configuration messages in SAXHandler or remove them using removeLimits.!

XMLParserLimits class
	instanceVariableNames: 'defaultReadLimit'!

!XMLParserLimits class methodsFor: 'accessing' stamp: 'monty 12/9/2014 07:41'!
defaultDocumentReadLimit
	^ self defaultReadLimit! !

!XMLParserLimits class methodsFor: 'accessing' stamp: 'monty 12/9/2014 07:52'!
defaultReadLimit
	^ defaultReadLimit! !

!XMLParserLimits class methodsFor: 'accessing' stamp: 'monty 12/9/2014 07:41'!
defaultReadLimit: aReadLimit
	defaultReadLimit := aReadLimit! !

!XMLParserLimits class methodsFor: 'class initialization' stamp: 'monty 12/9/2014 07:41'!
initialize
	"self initialize"

	defaultReadLimit := 10000000! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:43'!
documentReadLimit
	^ documentReadLimit! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:44'!
documentReadLimit: aLimit
	documentReadLimit := aLimit! !

!XMLParserLimits methodsFor: 'initialization' stamp: 'monty 1/27/2015 22:46'!
initialize
	super initialize.

	documentReadLimit := self class defaultDocumentReadLimit.
	maxEntityReplacementDepth := 8.! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:45'!
maxEntityReplacementDepth
	^ maxEntityReplacementDepth! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:45'!
maxEntityReplacementDepth: aLimit
	maxEntityReplacementDepth := aLimit! !

!XMLParserLimits methodsFor: 'removing' stamp: 'monty 1/27/2015 22:46'!
removeAll
	documentReadLimit := nil.
	maxEntityReplacementDepth := nil.! !

Object subclass: #XMLParserTokenizer
	instanceVariableNames: 'context driver nameStream streamReader writeStream'
	classVariableNames: 'PredefinedEntities'
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParserTokenizer commentStamp: 'monty 2/9/2014 02:44' prior: 0!
This is a base class for tokenizers. It reads tokens from a stream using a NestedStreamReader. Sending it #nextToken causes a token to be read and one or more handler messages to be dispatched to a driver.

Be careful changing the code in this class or subclass because it is optimized.!

!XMLParserTokenizer class methodsFor: 'instance creation' stamp: 'monty 1/17/2014 01:27'!
driver: aDriver on: aStringOrStream
	^self
		driver: aDriver
		on: aStringOrStream
		readLimit: nil! !

!XMLParserTokenizer class methodsFor: 'instance creation' stamp: 'monty 1/17/2014 01:26'!
driver: aDriver on: aStringOrStream readLimit: aReadLimit
	^self new
		setDriver: aDriver
		stream:
			(aStringOrStream isStream
				ifTrue: [aStringOrStream]
				ifFalse: [aStringOrStream readStream])
		readLimit: aReadLimit! !

!XMLParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 07:11'!
initialize
	"self initialize"

	(PredefinedEntities := Dictionary new)
		at: 'lt' put: $<;
		at: 'gt' put: $>;
		at: 'amp' put: $&;
		at: 'apos' put: $';
		at: 'quot' put: $"! !

!XMLParserTokenizer class methodsFor: 'accessing' stamp: 'monty 11/18/2014 07:11'!
predefinedEntities
	^ PredefinedEntities! !

!XMLParserTokenizer methodsFor: 'testing' stamp: 'monty 11/17/2014 02:03'!
atEnd
	^ context isTerminatedContext! !

!XMLParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 1/4/2015 23:02'!
characterFromCodePoint: anInteger
	(anInteger notNil and: [anInteger > 0])
		ifFalse: [^ nil].
	^ [anInteger asCharacter]
		on: Error
		do: [:error | nil].! !

!XMLParserTokenizer methodsFor: 'file open/close' stamp: 'monty 11/24/2014 19:27'!
closeStreams
	streamReader closeStreams! !

!XMLParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 11/10/2014 00:46'!
constructReplacementForGeneralEntity: anEntity! !

!XMLParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 11/10/2014 00:37'!
constructReplacementForParameterEntity: anEntity! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
currentLineNumber
	^ streamReader currentLineNumber! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
driver
	^ driver! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:35'!
expectNext: aCharacter
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:37'!
expectNextAll: aString
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/3/2015 17:50'!
expectSeparators
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/3/2015 01:32'!
expectTerminator
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:38'!
expectUpToAll: aString
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/9/2013 01:15'!
nextCDataSection
	self expectNextAll: '[CDATA['.

	driver handleCData: (self expectUpToAll: ']]>').
! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/12/2014 06:52'!
nextContentMarkupToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/12/2014 06:48'!
nextContentToken
	streamReader peek == $<
		ifTrue: [
			streamReader next.
			^ self nextContentMarkupToken].
	self nextPCDataToken.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2015 04:31'!
nextDoctypeDeclaration
	| root publicID systemID |

	context := context doctypeDeclarationContext.
	self
		expectNextAll: 'DOCTYPE';
		expectSeparators.

	root := self nextName.
	(streamReader peek == $[
		or: [streamReader peek == $>])
		ifFalse: [
			self expectSeparators.
			streamReader peek == $P
				ifTrue: [
					publicID := self nextPublicID.
					self expectSeparators.
					systemID := self nextSystemIDLiteral]
				ifFalse: [
					streamReader peek == $S
						ifTrue: [systemID := self nextSystemID]]].
	driver
		handleStartDTD: root
		publicID: (publicID ifNil: [''])
		systemID: (systemID ifNil: ['']).

	streamReader skipSeparators.
	streamReader peek == $[
		ifTrue: [self nextInternalSubsetStart]
		ifFalse: [self nextEndDoctypeDeclaration].! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2015 04:43'!
nextEndDoctypeDeclaration
	self expectTerminator.
	context := context postDoctypeDeclarationContext.

	driver handleEndDTD.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/24/2014 21:11'!
nextEndDocument
	context isTerminatedContext
		ifFalse: [
			self closeStreams.
			context := context terminatedContext.
			driver handleEndDocument]! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 03:43'!
nextEndInternalSubset
	"skip ]"
	streamReader
		next;
		skipSeparators.
	self nextEndDoctypeDeclaration.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/9/2014 16:35'!
nextInternalSubsetStart
	"skip ["
	streamReader next.
	context := context internalSubsetContext.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 01:14'!
nextName
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/12/2014 06:49'!
nextPCDataToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/6/2015 07:16'!
nextPI
	"skip ?"
	streamReader next.
	self nextPIWithTarget: self nextPITarget.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/6/2015 02:09'!
nextPIOrXMLDeclaration
	| target |

	streamReader next.
	(target := self nextPITarget) = 'xml'
		ifTrue: [^ self nextXMLDeclaration].

	context := context prologContext.
	self nextPIWithTarget: target.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:16'!
nextPITarget
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/6/2015 01:54'!
nextPIWithTarget: aTarget
	streamReader peek == $?
		ifTrue: [
			streamReader next.
			self expectTerminator.
			driver
				handlePI: aTarget
				data: '']
		ifFalse: [
			self expectSeparators.
			driver
				handlePI: aTarget
				data: (self expectUpToAll: '?>')].! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/7/2015 02:28'!
nextParameterEntityReference
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:19'!
nextPrologToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 03:42'!
nextPublicID
	^ self
		expectNextAll: 'PUBLIC';
		expectSeparators;
		nextPublicIDLiteral! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 03:42'!
nextPublicIDLiteral
	self subclassResponsiblity! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/24/2015 20:38'!
nextStartContent
	context := context contentContext.
	driver handleStartContent.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/6/2015 01:43'!
nextStartDocument
	context := driver initialContext ifNil: [XMLXMLDeclarationContext new].
	context isPrologContext
		ifTrue: [streamReader detectCurrentEncoding].
	driver handleStartDocument.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:21'!
nextSubsetMarkupToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/3/2015 01:46'!
nextSubsetToken
	streamReader skipSeparators.
	streamReader atEnd
		ifTrue: [^ self].

	streamReader peek == $%
		ifTrue: [
			streamReader next.
			^ self nextParameterEntityReference].
	(context isInternalSubsetContext
		and: [streamReader peek == $]])
		ifTrue: [^ self nextEndInternalSubset].

	self nextSubsetMarkupToken.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 03:41'!
nextSystemID
	^ self
		expectNextAll: 'SYSTEM';
		expectSeparators;
		nextSystemIDLiteral! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 03:41'!
nextSystemIDLiteral
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 18:25'!
nextToken
	(context isInitializedContext
		and: [streamReader atEnd])
		ifTrue: [^ self nextEndDocument].
	context nextTokenFrom: self.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:33'!
nextXMLAttributeName: aName
	self expectNextAll: aName.
	streamReader skipSeparators.
	self expectNext: $=.
	streamReader skipSeparators.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/6/2015 01:39'!
nextXMLDeclaration
	| version encoding |

	self expectSeparators.
	version := self nextXMLVersionAttributeRequired: true.
	streamReader peek == $?
		ifFalse: [self expectSeparators].
	(encoding := self nextXMLEncodingAttributeRequired: false) isEmpty
		ifFalse: [
			streamReader convertFromEncoding: encoding.
			streamReader peek == $?
				ifFalse: [self expectSeparators]].
	
	driver
		handleXMLVersion: version
		encoding: encoding
		standalone: self nextXMLStandaloneAttribute.
	context := context prologContext.

	streamReader skipSeparators.
	self
		expectNext: $?;
		expectTerminator.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/3/2015 18:12'!
nextXMLEncodingAttributeRequired: aBoolean
	(aBoolean or: [streamReader peek == $e])
		ifTrue: [	
			^ self
				nextXMLAttributeName: 'encoding';
				nextXMLEncodingAttributeValue]
		ifFalse: [^ '']! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:47'!
nextXMLEncodingAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/3/2015 18:13'!
nextXMLStandaloneAttribute
	"always optional"
	(streamReader peek == $s)
		ifTrue: [
			^ self
				nextXMLAttributeName: 'standalone';
				nextXMLStandaloneAttributeValue]
		ifFalse: [^ '']! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:44'!
nextXMLStandaloneAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/5/2015 07:50'!
nextXMLTextDeclaration
	(streamReader nextMatchAll: '<?xml')
		ifTrue: [| encoding |
			self expectSeparators.
			(self nextXMLVersionAttributeRequired: false) isEmpty
				ifFalse: [self expectSeparators].
			(encoding := self nextXMLEncodingAttributeRequired: true) isEmpty
				ifFalse: [streamReader convertFromEncoding: encoding].

			streamReader skipSeparators.
			self
				expectNext: $?;
				expectTerminator].
	context := context previousContext.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/3/2015 18:13'!
nextXMLVersionAttributeRequired: aBoolean
	(aBoolean or: [streamReader peek == $v])
		ifTrue: [
			^ self
				nextXMLAttributeName: 'version';
				nextXMLVersionAttributeValue]
		ifFalse: [^ '']! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:34'!
nextXMLVersionAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 1/16/2014 09:08'!
normalizedLineEndingChar
	^ streamReader normalizedLineEndingChar! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/18/2013 17:26'!
normalizedLineEndingChar: aCharacter
	streamReader normalizedLineEndingChar: aCharacter! !

!XMLParserTokenizer methodsFor: 'initialization' stamp: 'monty 12/13/2014 20:17'!
setDriver: aDriver stream: aStream readLimit: aReadLimit
	driver := aDriver.
	streamReader :=
		(XMLNestedStreamReader
			on: aStream
			readLimit: aReadLimit).
	writeStream := (String new: 128) writeStream.
	nameStream := (String new: 16) writeStream.
	context := XMLUnitializedContext new.! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
streamReader
	^ streamReader! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
tokenContext
	^ context! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 11/9/2014 15:59'!
tokenContext: aTokenContext
	context := aTokenContext! !

XMLParserTokenizer subclass: #XMLWellFormedParserTokenizer
	instanceVariableNames: 'activeEntities'
	classVariableNames: 'AdditionalNameChars EncodingNameChars NameChars NormalizedSpace PubidChars UnaryOperatorChars'
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLWellFormedParserTokenizer commentStamp: 'monty 1/17/2014 01:50' prior: 0!
This is tokenizer for well-formed XML that supports DTD subsets and entity replacement.!

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/10/2014 07:41'!
initialize
	"self initialize"

	self
		initializeNormalizedSpace;
		initializeAdditionalNameChars;
		initializeNameChars;
		initializePubidChars;
		initializeEncodingNameChars;
		initializeUnaryOperatorChars! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'JAA 3/22/2011 12:43'!
initializeAdditionalNameChars
	"This creates a character set that contains all those defined in the XML spec for the
	NameChar production rule minus those also defined for NameStartChar
	(NameChar - NameStartChar)"

	(AdditionalNameChars := BitmapCharacterSet new: 16r2041)
		add: $-;
		add: $.;
		addAll: ($0 to: $9);
		add: (Character value: 16rB7).
	16r0300 to: 16r036F do: [:each | AdditionalNameChars add: (Character value: each)].
	16r203F to: 16r2040 do: [:each | AdditionalNameChars add: (Character value: each)].! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 23:45'!
initializeEncodingNameChars
	(EncodingNameChars := BitmapCharacterSet new)
		addAll: ($a to: $z);
		addAll: ($A to: $Z);
		addAll: ($0 to: $9);
		add: $-;
		add: $_;
		add: $.! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 2/1/2014 02:50'!
initializeNameChars
	"This creates a character set that coveres the entire range of characters speicified for
	the NameChar production rule in the XML spec."

	(NameChars := BitmapCharacterSet new: 16rF0000)
		add: $:;
		addAll: ($A to: $Z);
		add: $_;
		addAll: ($a to: $z).
	16rC0 to: 16rD6 do: [:each | NameChars add: (Character value: each)].
	16rD8 to: 16rF6 do: [:each | NameChars add: (Character value: each)].
	16rF8 to: 16r2FF do: [:each | NameChars add: (Character value: each)].
	16r370 to: 16r37D do: [:each | NameChars add: (Character value: each)].
	16r37F to: 16r1FFF do: [:each | NameChars add: (Character value: each)].
	16r200C to: 16r200D do: [:each | NameChars add: (Character value: each)].
	16r2070 to: 16r218F do: [:each | NameChars add: (Character value: each)].
	16r2C00 to: 16r2FEF do: [:each | NameChars add: (Character value: each)].
	16r3001 to: 16rD7FF do: [:each | NameChars add: (Character value: each)].
	16rF900 to: 16rFDCF do: [:each | NameChars add: (Character value: each)].
	16rFDF0 to: 16rFFFD do: [:each | NameChars add: (Character value: each)].
	16r10000 to: 16rEFFFF do: [:each | NameChars add: (Character value: each)].

	AdditionalNameChars do: [:each | NameChars add: each].! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 2/10/2014 01:18'!
initializeNormalizedSpace
	NormalizedSpace := Character space! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 11/12/2013 01:44'!
initializePubidChars
	(PubidChars := BitmapCharacterSet new)
		add: Character space;
		add: Character cr;
		add: Character lf;
		addAll: ($a to: $z);
		addAll: ($A to: $Z);
		addAll: ($0 to: $9);
		addAll: '-''()+,./:=?;!!*#@$_%'! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 23:45'!
initializeUnaryOperatorChars
	UnaryOperatorChars := BitmapCharacterSet with: $? with: $* with: $+! !

!XMLWellFormedParserTokenizer class methodsFor: 'testing' stamp: 'monty 12/23/2014 18:03'!
isNCName: aString
	"optimized"
	1 to: aString size do: [:i | | nextChar |
		(nextChar := aString at: i) == $:
			ifTrue: [^ false].
		((NameChars includes: nextChar)
			and: [i > 1
				or: [(AdditionalNameChars includes: nextChar) not]])
			ifFalse: [^ false]].
	^ aString size > 0.! !

!XMLWellFormedParserTokenizer class methodsFor: 'testing' stamp: 'monty 12/11/2014 05:17'!
isName: aString
	"optimized"
	1 to: aString size do: [:i | 
		((NameChars includes: (aString at: i))
			and: [i > 1
				or: [(AdditionalNameChars includes: (aString at: i)) not]])
			ifFalse: [^ false]].
	^ aString size > 0.! !

!XMLWellFormedParserTokenizer class methodsFor: 'testing' stamp: 'monty 12/11/2014 05:17'!
isNmtoken: aString
	"optimized"
	1 to: aString size do: [:i |
		(NameChars includes: (aString at: i))
			ifFalse: [^ false]].
	^ aString size > 0.! !

!XMLWellFormedParserTokenizer class methodsFor: 'streaming' stamp: 'monty 12/29/2014 20:57'!
nextNCNameFrom: aReadStream on: aWriteStream
	| nextChar |

	(aReadStream atEnd
		or: [(NameChars includes: (nextChar := aReadStream peek)) not
			or: [nextChar == $:
				or: [AdditionalNameChars includes: nextChar]]])
		ifTrue: [^ false].
	aWriteStream nextPut: aReadStream next.

	[aReadStream atEnd
		or: [(NameChars includes: (nextChar := aReadStream peek)) not
			or: [nextChar == $:]]]
		whileFalse: [aWriteStream nextPut: aReadStream next].
	^ true.! !

!XMLWellFormedParserTokenizer class methodsFor: 'streaming' stamp: 'monty 12/23/2014 16:45'!
nextNameFrom: aReadStream on: aWriteStream
	(aReadStream atEnd
		or: [(NameChars includes: aReadStream peek) not
			or: [AdditionalNameChars includes: aReadStream peek]])
		ifTrue: [^ false].
	aWriteStream nextPut: aReadStream next.

	[aReadStream atEnd not
		and: [NameChars includes: aReadStream peek]]
		whileTrue: [aWriteStream nextPut: aReadStream next].
	^ true.! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 1/17/2014 05:28'!
activeEntities
	^ activeEntities ifNil: [activeEntities := OrderedCollection new]! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 10/31/2014 17:45'!
addActiveEntity: anEntity
	(self hasActiveEntityNamed: anEntity name)
		ifTrue: [self errorSelfReferentialEntity: anEntity].
	self maxEntityReplacementDepth
		ifNotNil: [:maxDepth |
			(self activeEntities size + 1> maxDepth)
				ifTrue: [self errorEntityDepthLimitExceeded]].
	
	self activeEntities addLast: anEntity.
	anEntity isExternal
		ifTrue: [driver currentURI: anEntity uri].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 11/28/2014 09:31'!
constructReplacementForGeneralEntity: anEntity
	context isLiteralValueContext
		ifTrue: [
			anEntity isExternal
				ifTrue: [self errorLiteralContextExternalEntity: anEntity].
			^ self constructReplacementForLiteralContextEntity: anEntity].

	driver handleStartContentEntityReplacement: anEntity name.
	self
		pushBackEntity: anEntity
		onClose: [driver handleEndContentEntityReplacement: anEntity name].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 3/1/2015 17:16'!
constructReplacementForLiteralContextEntity: anEntity
	| wasEscapingQuotes |

	wasEscapingQuotes := streamReader isEscapingQuotes.
	self
		pushBackEntity: anEntity
		onClose: [streamReader isEscapingQuotes: wasEscapingQuotes].

	"wait till after the text declaration was read to start escaping"
	(self currentActiveEntity == anEntity
		and: [wasEscapingQuotes not])
		ifTrue: [streamReader isEscapingQuotes: true].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 11/28/2014 09:30'!
constructReplacementForParameterEntity: anEntity
	context isLiteralValueContext
		ifTrue: [^ self constructReplacementForLiteralContextEntity: anEntity].

	streamReader pushBackCharacter: NormalizedSpace.
	self
		pushBackEntity: anEntity
		onClose: nil.
	streamReader pushBackCharacter: NormalizedSpace.! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 11/16/2014 05:53'!
currentActiveEntity
	^ self activeEntities lastOrNil! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/1/2014 15:40'!
errorEntityDepthLimitExceeded
	XMLLimitException signal: 
		'Cannot further nest entity replacements; ',
		'use #maxEntityReplacementDepth: to change or disable'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 12/10/2013 07:28'!
errorExpected: anExpectedString
	self parseError: 'Expected ', anExpectedString! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 12/31/2014 23:12'!
errorExpected: anExpectedCharacterOrString butGot: aReceivedCharacterOrString
	self errorExpected:
		('{1} but got "{2}"' format:
			(Array
				with: anExpectedCharacterOrString
				with: (aReceivedCharacterOrString ifNil: [''])))! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/13/2015 13:06'!
errorExpectedAttributeNameValueSeparator
	self errorExpected: 'attribute name-value "=" separator'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/12/2015 13:32'!
errorExpectedAttributeValueEndButGot: aCharacter
	aCharacter == $<
		ifTrue: [self parseError: 'Attribute value contains "<" character']
		ifFalse: [self errorExpected: 'end of quoted attribute value']! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 17:41'!
errorExpectedAttributeValueStart
	self errorExpected: 'start of quoted attribute value'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/6/2014 12:38'!
errorExpectedLiteral: aCharacterOrString
	self
		errorExpectedLiteral: aCharacterOrString
		butGot: nil! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/1/2014 15:26'!
errorExpectedLiteral: anExpectedLiteral butGot: aCharacter
	self
		errorExpectedLiteral: anExpectedLiteral
		butGot: aCharacter
		at: 1! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/1/2014 15:46'!
errorExpectedLiteral: anExpectedLiteral butGot: aCharacter at: aPosition
	| literal replacement |

	literal := anExpectedLiteral asString.
	replacement := (aCharacter ifNil: ['']) asString.
	self errorExpected:
		('"{1}" but got "{2}"' format:
			(Array
				with: literal
				with:
					(literal
						copyReplaceFrom: aPosition
						to: literal size
						with: replacement)))! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 15:10'!
errorExpectedName
	self errorExpected: 'a name'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/6/2014 12:38'!
errorExpectedSectionTerminator
	self errorExpectedLiteral: ']]>'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/3/2015 17:50'!
errorExpectedSeparators
	self errorExpected: 'one or more whitespace characters'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/3/2015 02:15'!
errorExpectedTerminatorButGot: aReceivedCharacer
	self
		errorExpected: '">" terminator'
		butGot: aReceivedCharacer! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/6/2015 04:02'!
errorInternalDeclarationParameterEntityReference: aName
	self parseError: 'Parameter entity reference %', aName, '; in internal subset declaration'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/1/2014 12:55'!
errorInternalSubsetIncludeOrIgnoreSection
	self parseError: 'Cannot have <!![INCLUDE[...]> or <!![IGNORE[...]> sections in internal subset'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 20:36'!
errorInvalidCodePoint: aCodePoint
	self parseError: 'Invalid character code point ', aCodePoint printString! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 20:39'!
errorLiteralContextExternalEntity: anEntity
	self parseError: 'External general entity ', anEntity asReference, ' in literal context'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/6/2015 04:38'!
errorMultipleDoctypeDeclarations
	self parseError: 'Multiple <!!DOCTYPE ...> declarations'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 20:43'!
errorParameterEntityTerminatingSubset
	self parseError: 'Parameter entity replacement cannot terminate internal subset'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 17:45'!
errorSelfReferentialEntity: anEntity
	self parseError: 'Illegal self-referential entity ', anEntity asReference! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/6/2015 02:11'!
errorXMLDeclarationNotAtBeginning
	self parseError: 'XML declaration must only occur at beginning of document'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/6/2015 05:17'!
errorXMLPITargetReserved
	self parseError: 'Processing instruction target "XML" is reserved'! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 1/4/2014 06:50'!
expectDigit
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [nextChar isDigit])
		ifFalse: [
			self
				errorExpected: 'digit'
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 1/4/2014 06:52'!
expectLetter
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [nextChar isLetter])
		ifFalse: [
			self
				errorExpected: 'letter'
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 11/1/2014 15:27'!
expectNext: aCharacter
	| nextChar |

	(nextChar := streamReader next) == aCharacter
		ifFalse: [
			self
				errorExpectedLiteral: aCharacter
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 11/2/2014 15:02'!
expectNextAll: anExpectedLiteral
	1 to: anExpectedLiteral size do: [:i | | nextChar |
		(nextChar := streamReader next) == (anExpectedLiteral at: i)
			ifFalse: [
				self
					errorExpectedLiteral: anExpectedLiteral
					butGot: nextChar
					at: i]].
	^ anExpectedLiteral.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/9/2013 06:51'!
expectQuote
	| nextChar |

	(((nextChar := streamReader next) == $")
		or: [nextChar == $'])
		ifFalse: [
			self
				errorExpected: 'quote character delimiter'
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/5/2015 06:37'!
expectSeparators
	"nil understands xmlParserIsSeparator so no nil test is needed"
	streamReader peek xmlParserIsSeparator
		ifTrue: [
			streamReader
				next;
				skipSeparators]
		ifFalse: [self errorExpectedSeparators]! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/4/2015 21:26'!
expectSeparatorsInDeclaration
	"this can be used to replace references within declarations"

	[streamReader peek == $%]
		whileTrue: [
			streamReader next.
			self nextParameterEntityReferenceInDeclaration].
	self expectSeparators.
	[streamReader peek == $%]
		whileTrue: [
			streamReader next.
			self nextParameterEntityReferenceInDeclaration.
			streamReader skipSeparators].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/3/2015 02:18'!
expectTerminator
	| nextChar |

	(nextChar := streamReader next) == $>
		ifFalse: [self errorExpectedTerminatorButGot: nextChar]! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 11/28/2014 05:19'!
expectUpToAll: aString
	writeStream reset.
	[streamReader atEnd]
		whileFalse: [
			(streamReader nextMatchAll: aString)
				ifTrue: [^ writeStream contents].
			writeStream nextPut: streamReader next].
	self errorExpectedLiteral: aString.! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:55'!
hasActiveEntities
	^ activeEntities notNil and: [activeEntities notEmpty]! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:58'!
hasActiveEntityNamed: aName
	^ self activeEntities anySatisfy: [:each | each name = aName]! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:59'!
hasActiveExternalEntity
	^ self activeEntities anySatisfy: [:each | each isExternal]! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 1/17/2014 05:28'!
maxEntityReplacementDepth
	^ driver maxEntityReplacementDepth! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextAttlistDeclaration
	| element defaultPragma |

	self
		expectNextAll: 'ATTLIST';
		expectSeparatorsInDeclaration.

	element := self nextName.
	[streamReader peek == $>
		or: [
			self expectSeparatorsInDeclaration.
			streamReader peek == $>]]
		whileFalse: [
			driver
				handleAttributeDeclaration: element
				name: self nextName
				type: self nextAttributeType
				defaultPragma: (defaultPragma := self nextAttributeDefaultPragma)
				defaultValue: (self nextAttributeDefaultValueForPragma: defaultPragma)].
	self expectTerminator.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextAttributeDefaultPragma
	self expectSeparatorsInDeclaration.
	streamReader peek == $#
		ifFalse: [^ ''].
	streamReader next.

	streamReader peek == $R
		ifTrue: [^ self expectNextAll: 'REQUIRED'].
	streamReader peek == $I
		ifTrue: [^ self expectNextAll: 'IMPLIED'].
	^ self expectNextAll: 'FIXED'.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextAttributeDefaultValueForPragma: aPragma
	| previousContext attributeValue |

	aPragma isEmpty
		ifFalse: [
			(aPragma first == $F) "FIXED"
				ifFalse: [^ ''].
			self expectSeparatorsInDeclaration].
	previousContext := context.
	context := context contentContext.
	attributeValue := self nextAttributeValue.
	context := previousContext.
	^ attributeValue.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2014 08:21'!
nextAttributeEntityType
	streamReader next.
	self expectNextAll: 'NTIT'.
	streamReader peek == $Y
		ifTrue: [
			streamReader next.
			^ 'ENTITY'].
	self expectNextAll: 'IES'.
	^ 'ENTITIES'.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextAttributeEnumerationType
	writeStream
		reset;
		nextPut: (self expectNext: $().
	self
		skipSeparatorsInDeclaration;
		nextNmtokenOnWriteStream;
		skipSeparatorsInDeclaration.
	[streamReader peek == $)]
		whileFalse: [
			writeStream nextPut: (self expectNext: $|).
			self
				skipSeparatorsInDeclaration;
				nextNmtokenOnWriteStream;
				skipSeparatorsInDeclaration].
	writeStream nextPut: streamReader next.

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2014 09:56'!
nextAttributeIDOrIDRefType
	streamReader next.
	self expectNext: $D.
	streamReader peek == $R
		ifFalse: [^ 'ID'].	
	self expectNextAll: 'REF'.
	streamReader peek == $S
		ifTrue: [
			streamReader next.
			^ 'IDREFS'].
	^ 'IDREF'.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2014 08:25'!
nextAttributeNotationEnumerationOrNmtokenType
	streamReader next.
	streamReader peek == $O
		ifTrue: [
			self expectNextAll: 'OTATION'.
			^ self nextAttributeNotationEnumerationType].
	self expectNextAll: 'MTOKEN'.
	streamReader peek == $S
		ifTrue: [
			streamReader next.
			^ 'NMTOKENS'].
	^ 'NMTOKEN'.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextAttributeNotationEnumerationType
	writeStream
		reset;
		nextPutAll: 'NOTATION '.
	self expectSeparatorsInDeclaration.
	writeStream nextPut: (self expectNext: $().

	self
		skipSeparatorsInDeclaration;
		nextNameOnWriteStream;
		skipSeparatorsInDeclaration;
		nextNameListValuesOnWriteStream.

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextAttributeType
	| nextChar |

	self expectSeparatorsInDeclaration.
	(nextChar := streamReader peek) == $C
		ifTrue: [^ self expectNextAll: 'CDATA'].
	nextChar == $I
		ifTrue: [^ self nextAttributeIDOrIDRefType].
	nextChar == $E
		ifTrue: [^ self nextAttributeEntityType].
	nextChar == $N
		ifTrue: [^ self nextAttributeNotationEnumerationOrNmtokenType].
	^ self nextAttributeEnumerationType.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/4/2015 23:22'!
nextAttributeValue
	| quote nextChar |

	((quote := streamReader next) == $"
		or: [quote == $'])
		ifFalse: [self errorExpectedAttributeValueStart].
	
	context enterLiteralValue.
	writeStream reset.
	[(nextChar := streamReader next) == quote
		or: [nextChar == $<
			or: [nextChar == nil]]]
		whileFalse: [
			nextChar == $&
				ifTrue: [
					"char refs should be ignored by whitespace normalization"
					streamReader peek == $#
						ifTrue: [self nextCharacterReferenceOnWriteStream]
						ifFalse: [self nextGeneralEntityReferenceOnWriteStream]]
				ifFalse: [
					nextChar xmlParserIsSeparator
						ifTrue: [nextChar := NormalizedSpace].
					writeStream nextPut: nextChar]].
	context exitLiteralValue.

	nextChar == quote
		ifFalse: [self errorExpectedAttributeValueEndButGot: nextChar].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/6/2015 04:14'!
nextCharacterReferenceOnWriteStream
	"returns true if writeStream is written to, false otherwise"
	| codePoint character |

	"skip #"
	streamReader next.
	codePoint :=
		(streamReader nextInteger
			ifNil: [self errorExpected: 'integer']).
	self expectNext: $;.

	character :=
		(self characterFromCodePoint: codePoint)
			ifNil: [self errorInvalidCodePoint: codePoint].
	(context isContentContext
		and: [driver replacesContentEntityReferences not])
		ifTrue: [
			writeStream
				nextPutAll: '&#';
				nextPutAll: codePoint asString;
				nextPut: $;.
			^ true].
	writeStream nextPut: character.
	^ true.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/3/2015 01:32'!
nextComment
	| nextChar |

	"Skip first -"
	streamReader next.
	self expectNext: $-.

	"xml comments cannot contain -- sequences not followed by terminating >"
	writeStream reset.
	[(nextChar := streamReader next) == nil
		or: [nextChar == $-
			and: [(streamReader peek) == $-]]]
		whileFalse: [writeStream nextPut: nextChar].
	self
		expectNext: $-;
		expectTerminator.

	driver handleComment: writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/4/2015 22:29'!
nextConditionalSection
	"skip ["
	streamReader next.

	writeStream reset.
	self nextConditionalSectionOnWriteStream.
	streamReader pushBack: writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextConditionalSectionOnWriteStream
	self
		skipSeparatorsInDeclaration;
		expectNext: $I.

	streamReader peek == $N
		ifTrue: [self nextIncludeSectionOnWriteStream]
		ifFalse: [self nextIgnoreSection].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/6/2015 07:16'!
nextContentMarkupToken
	streamReader peek == $?
		ifTrue: [^ self nextPI].
	streamReader peek == $!!
		ifTrue: [
			streamReader next.
			streamReader peek == $-
				ifTrue: [^ self nextComment].
			^ self nextCDataSection].
	self nextTag.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextContentModel
	self expectSeparatorsInDeclaration.
	streamReader peek == $A
		ifTrue: [^ self expectNextAll: 'ANY'].
	streamReader peek == $E
		ifTrue: [^ self expectNextAll: 'EMPTY'].
	
	writeStream
		reset;
		nextPut: (self expectNext: $().
	self skipSeparatorsInDeclaration.
	streamReader peek == $#
		ifTrue: [self nextMixedContentListValuesOnWriteStream]
		ifFalse: [self nextContentModelChoiceOrSequenceValuesOnWriteStream].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextContentModelChoiceOrSequenceValuesOnWriteStream
	self
		nextContentModelParticleOnWriteStream;
		skipSeparatorsInDeclaration.
	streamReader peek == $|
		ifTrue: [self nextContentModelChoiceValuesOnWriteStream]
		ifFalse: [self nextContentModelSequenceValuesOnWriteStream].
	self nextContentModelUnaryOperatorOnWriteStream.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextContentModelChoiceValuesOnWriteStream
	[streamReader peek == $)]
		whileFalse: [
			writeStream nextPut: (self expectNext: $|).
			self
				nextContentModelParticleOnWriteStream;
				skipSeparatorsInDeclaration].
	writeStream nextPut: streamReader next.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextContentModelParticleOnWriteStream
	self skipSeparatorsInDeclaration.
	streamReader peek == $(
		ifTrue: [
			writeStream nextPut: streamReader next.
			self nextContentModelChoiceOrSequenceValuesOnWriteStream]
		ifFalse: [
			self
				nextNameOnWriteStream;
				nextContentModelUnaryOperatorOnWriteStream]! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextContentModelSequenceValuesOnWriteStream
	[streamReader peek == $)]
		whileFalse: [
			writeStream nextPut: (self expectNext: $,).
			self
				nextContentModelParticleOnWriteStream;
				skipSeparatorsInDeclaration].
	writeStream nextPut: streamReader next.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/4/2015 22:33'!
nextContentModelUnaryOperatorOnWriteStream
	| nextChar |

	((nextChar := streamReader peek) notNil
		and: [UnaryOperatorChars includes: nextChar])
		ifTrue: [writeStream nextPut: streamReader next]! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/6/2015 04:45'!
nextDoctypeDeclarationOrComment
	"skip !!"
	streamReader next.
	streamReader peek == $-
		ifTrue: [
			context isXMLDeclarationContext
				ifTrue: [context := context prologContext].
			^ self nextComment].
	context isPostDoctypeDeclarationContext
		ifTrue: [self errorMultipleDoctypeDeclarations].
	self nextDoctypeDeclaration.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextElementDeclaration
	self
		expectNextAll: 'LEMENT';
		expectSeparatorsInDeclaration.
	driver
		handleElementDeclaration: self nextName
		contentModel: self nextContentModel.
	self
		skipSeparatorsInDeclaration;
		expectTerminator.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 16:14'!
nextEndDocument
	context isInternalSubsetContext
		ifTrue: [self errorExpected: 'end of internal subset'].
	super nextEndDocument.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 10/31/2014 20:42'!
nextEndInternalSubset
	self hasActiveEntities
		ifTrue: [self errorParameterEntityTerminatingSubset].

	super nextEndInternalSubset.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/3/2015 01:33'!
nextEndTag
	"Skip /"
	streamReader next.
	driver handleEndTag: self nextName.
	streamReader skipSeparators.
	self expectTerminator.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/5/2015 06:37'!
nextEntityDeclaration
	| isParameterEntity name |

	self expectNextAll: 'NTITY'.

	context isExternalSubsetContext
		ifTrue: [
			"this is to distinguish a parameter reference from declaration"
			[isParameterEntity isNil]
				whileTrue: [
					streamReader skipSeparators.
					streamReader peek == $%
						ifTrue: [
							streamReader next.
							"nil understands xmlParserIsSeparator so no nil test is needed"
							streamReader peek xmlParserIsSeparator
								ifTrue: [
									isParameterEntity := true.
									self expectSeparatorsInDeclaration]
								ifFalse: [self nextParameterEntityReference]]
						ifFalse: [isParameterEntity := false]]]
		ifFalse: [
			self expectSeparators.
			(isParameterEntity := (streamReader peek == $%))
				ifTrue: [
					streamReader next.
					self expectSeparators]].

	name := self nextName.
	self expectSeparatorsInDeclaration.
	streamReader atQuote
		ifTrue: [
			self
				nextInternalEntityDeclaration: name
				isParameterEntity: isParameterEntity]
		ifFalse: [
			self
				nextExternalEntityDeclaration: name
				isParameterEntity: isParameterEntity].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 04:54'!
nextEntityValue
	| quote nextChar |

	quote := streamReader next.
	context enterLiteralValue.
	writeStream reset.
	[(nextChar := streamReader peek) == quote
		or: [nextChar == nil]]
		whileFalse: [
			streamReader next == $&
				ifTrue: [
					streamReader peek == $#
						ifTrue: [self nextCharacterReferenceOnWriteStream]
						ifFalse: [writeStream nextPut: $&]]
				ifFalse: [
					nextChar == $%
						ifTrue: [self nextParameterEntityReferenceOnWriteStream]
						ifFalse: [writeStream nextPut: nextChar]]].
	context exitLiteralValue.
	self expectNext: quote.

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 22:15'!
nextExternalEntityDeclaration: aName isParameterEntity: aBoolean
	| publicID systemID notation |

	streamReader peek == $P
		ifTrue: [
			publicID := self nextPublicIDInDeclaration.
			self expectSeparatorsInDeclaration.
			systemID := self nextSystemIDLiteral]
		ifFalse: [
			publicID := ''.
			systemID := self nextSystemIDInDeclaration].
	aBoolean
		ifTrue: [
			self
				skipSeparatorsInDeclaration;
				expectTerminator.
			driver
				handleParameterEntityDeclaration: aName
				publicID: publicID
				systemID: systemID]
		ifFalse: [
			(streamReader peek == $>
				or: [
					self expectSeparatorsInDeclaration.
					streamReader peek == $>])
				ifTrue: [	
					streamReader next.
					notation := '']
				ifFalse: [
					self
						expectNextAll: 'NDATA';
						expectSeparatorsInDeclaration.
					notation := self nextName.
					self
						skipSeparatorsInDeclaration;
						expectTerminator].
			driver
				handleGeneralEntityDeclaration: aName
				publicID: publicID
				systemID: systemID
				ndata: notation].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/6/2015 05:14'!
nextGeneralEntityReferenceOnWriteStream
	"returns true if writeStream is written to, false otherwise"
	| name entity |

	name := self nextName.
	self expectNext: $;.
	PredefinedEntities
		at: name
		ifPresent: [:character |
			driver replacesContentEntityReferences
				ifFalse: [^ self nextUnreplacedGeneralEntityOnWriteStreamNamed: name].
			writeStream nextPut: character.
			^ true].

	(entity := driver handleGeneralEntityReference: name)
		ifNil: [
			(driver replacesContentEntityReferences not
				or: [driver preservesUndeclaredEntityReferences])
				ifTrue: [^ self nextUnreplacedGeneralEntityOnWriteStreamNamed: name].
			driver handleUnreplacedGeneralEntityReference: name.
			^ false].
	driver replacesContentEntityReferences
		ifFalse: [^ self nextUnreplacedGeneralEntityOnWriteStreamNamed: name].
	entity constructReplacementOn: self.
	^ false.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/6/2014 18:34'!
nextIgnoreSection
	| openSections |

	self expectNextAll: 'GNORE'.
	streamReader skipSeparators.
	self expectNext: $[.

	openSections := 1.
	[openSections > 0 and: [streamReader atEnd not]]
		whileTrue: [
			(streamReader nextMatchAll: ']]>')
				ifTrue: [openSections := openSections - 1]
				ifFalse: [
					(streamReader nextMatchAll: '<!![')
						ifTrue: [openSections := openSections + 1]
						ifFalse: [streamReader next]]].

	openSections > 0
		ifTrue: [self errorExpectedSectionTerminator].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/4/2015 22:29'!
nextIncludeSectionOnWriteStream
	| isOpen  |

	self expectNextAll: 'NCLUDE'.
	streamReader skipSeparators.
	self expectNext: $[.

	isOpen := true.
	[isOpen and: [streamReader atEnd not]]
		whileTrue: [
			(streamReader nextMatchAll: '<!![')
				ifTrue: [self nextConditionalSectionOnWriteStream]
				ifFalse: [
					(streamReader nextMatchAll: ']]>')
						ifTrue: [isOpen := false]
						ifFalse: [writeStream nextPut: streamReader next]]].
	isOpen
		ifTrue: [self errorExpectedSectionTerminator].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextInternalEntityDeclaration: aName isParameterEntity: aBoolean
	| value |

	value := self nextEntityValue.
	self
		skipSeparatorsInDeclaration;
		expectTerminator.

	aBoolean
		ifTrue: [driver handleParameterEntityDeclaration: aName replacement: value]
		ifFalse: [driver handleGeneralEntityDeclaration: aName replacement: value].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextMixedContentListValuesOnWriteStream
	| hasElements |

	writeStream nextPutAll: (self expectNextAll: '#PCDATA').

	self skipSeparatorsInDeclaration.
	hasElements := streamReader peek == $|.

	self nextNameListValuesOnWriteStream.
	streamReader peek == $*
		ifTrue: [writeStream nextPut: streamReader next]
		ifFalse: [
			hasElements
				ifTrue: [self errorExpectedLiteral: $*]].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2015 22:43'!
nextName
	| nextChar |

	nameStream reset.
	((nextChar := streamReader next) notNil
		and: [(NameChars includes: nextChar)
			and: [(AdditionalNameChars includes: nextChar) not]])
		ifFalse: [self errorExpectedName].
	nameStream nextPut: nextChar.

	[(nextChar := streamReader peek) notNil
		and: [NameChars includes: nextChar]]
		whileTrue: [nameStream nextPut: streamReader next].
	^ nameStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextNameListValuesOnWriteStream
	[streamReader peek == $)]
		whileFalse: [
			writeStream nextPut: (self expectNext: $|).
			self
				skipSeparatorsInDeclaration;
				nextNameOnWriteStream;
				skipSeparatorsInDeclaration].
	writeStream nextPut: streamReader next.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/4/2015 22:37'!
nextNameOnWriteStream
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [(NameChars includes: nextChar)
			and: [(AdditionalNameChars includes: nextChar) not]])
		ifFalse: [self errorExpectedName].
	writeStream nextPut: nextChar.

	[(nextChar := streamReader peek) notNil
		and: [NameChars includes: nextChar]]
		whileTrue: [writeStream nextPut: streamReader next].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/4/2015 22:23'!
nextNmtokenOnWriteStream
	"matches the nmtoken production"
	| nextChar |

	((nextChar := streamReader peek) notNil
		and: [(NameChars includes: nextChar)])
		ifFalse: [self errorExpected: 'name token'].
	writeStream nextPut: streamReader next.
		
	[(nextChar := streamReader peek) notNil
		and: [NameChars includes: nextChar]]
		whileTrue: [writeStream nextPut: streamReader next].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 22:15'!
nextNotationDeclaration
	| name publicID systemID |

	self
		expectNextAll: 'NOTATION';
		expectSeparatorsInDeclaration.

	name := self nextName.
	self expectSeparatorsInDeclaration.
	streamReader peek == $P
		ifTrue: [
			publicID := self nextPublicIDInDeclaration.
			(streamReader peek == $>
				or: [
					self expectSeparatorsInDeclaration.
					streamReader peek == $>])
				ifFalse: [systemID := self nextSystemIDLiteral]]
		ifFalse: [systemID := self nextSystemIDInDeclaration].
	self
		skipSeparatorsInDeclaration;
		expectTerminator.

	driver
		handleNotationDeclaration: name
		publicID: (publicID ifNil: [''])
		systemID: (systemID ifNil: ['']).! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/5/2015 06:37'!
nextPCDataToken
	| isIgnorableWhitespace nextChar |

	isIgnorableWhitespace := true.
	writeStream reset.
	[(nextChar := streamReader peek) == $<
		or: [nextChar == nil]]
		whileFalse: [
			streamReader next == $&
				ifTrue: [
					(streamReader peek == $#
						ifTrue: [self nextCharacterReferenceOnWriteStream]
						ifFalse: [self nextGeneralEntityReferenceOnWriteStream])
							ifTrue: [isIgnorableWhitespace := false]]
				ifFalse: [
					"we do not check for illegal ]]> CDATA terminators in PCDATA
					for performance"
					isIgnorableWhitespace
						ifTrue: [isIgnorableWhitespace := nextChar xmlParserIsSeparator].
					writeStream nextPut: nextChar]].
	isIgnorableWhitespace
		ifTrue: [
			writeStream xmlParserAtBeginning
				ifFalse: [driver handleWhitespace: writeStream contents]]
		ifFalse: [driver handlePCData: writeStream contents].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/6/2015 05:16'!
nextPITarget
	| target |

	target := self nextName.
	context isXMLDeclarationContext
		ifFalse: [
			target = 'xml'
				ifTrue: [self errorXMLDeclarationNotAtBeginning]].
	target = 'XML'
		ifTrue: [self errorXMLPITargetReserved].

	^ target.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/7/2015 02:49'!
nextParameterEntityReference
	| name |

	name := self nextName.
	self expectNext: $;.
	(driver handleParameterEntityReference: name)
		ifNotNil: [:entity | entity constructReplacementOn: self].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 20:58'!
nextParameterEntityReferenceInDeclaration
	| name |

	name := self nextName.
	context isInternalSubsetContext
		ifTrue: [self errorInternalDeclarationParameterEntityReference: name].
	self expectNext: $;.
	(driver handleParameterEntityReference: name)
		ifNotNil: [:entity | entity constructReplacementOn: self].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/7/2015 02:50'!
nextParameterEntityReferenceOnWriteStream
	| name entity |

	name := self nextName.
	self expectNext: $;.
	(entity := driver handleParameterEntityReference: name)
		ifNil: [
			driver preservesUndeclaredEntityReferences
				ifTrue: [
					writeStream
						nextPut: $%;
						nextPutAll: name;
						nextPut: $;].
			^ true].
	entity constructReplacementOn: self.
	^ false.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/6/2015 04:44'!
nextPrologToken
	streamReader skipSeparators.
	streamReader atEnd
		ifTrue: [^ self].

	self expectNext: $<.
	streamReader peek == $?
		ifTrue: [^ self nextPIOrXMLDeclaration].
	streamReader peek == $!!
		ifTrue: [^ self nextDoctypeDeclarationOrComment].
	self
		nextStartContent;
		nextTag.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 22:14'!
nextPublicIDInDeclaration
	^ self
		expectNextAll: 'PUBLIC';
		expectSeparatorsInDeclaration;
		nextPublicIDLiteral! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 03:42'!
nextPublicIDLiteral
	| quote nextChar |

	quote := self expectQuote.
	writeStream reset.
	[(nextChar := streamReader peek) == nil
		or: [nextChar == quote
			or: [(PubidChars includes: nextChar) not]]]
		whileFalse: [writeStream nextPut: streamReader next].
	self expectNext: quote.

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 03:56'!
nextSubsetDeclaration
	streamReader peek == $E
		ifTrue: [
			streamReader next.
			streamReader peek == $N
				ifTrue: [^ self nextEntityDeclaration]
				ifFalse: [^ self nextElementDeclaration]].
	streamReader peek == $A
		ifTrue: [^ self nextAttlistDeclaration].
	streamReader peek == $N
		ifTrue: [^ self nextNotationDeclaration].

	self errorExpected: 'DTD subset declaration'.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2015 07:16'!
nextSubsetMarkupToken
	self expectNext: $<.
	streamReader peek == $?
		ifTrue: [^ self nextPI].

	self expectNext: $!!.
	streamReader peek == $-
		ifTrue: [^ self nextComment].

	(streamReader peek == $[)
		ifTrue: [
			(context isExternalSubsetContext
				or: [self hasActiveExternalEntity])
				ifTrue: [^ self nextConditionalSection].
			self errorInternalSubsetIncludeOrIgnoreSection].

	self nextSubsetDeclaration.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 22:14'!
nextSystemIDInDeclaration
	^ self
		expectNextAll: 'SYSTEM';
		expectSeparatorsInDeclaration;
		nextSystemIDLiteral! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 03:41'!
nextSystemIDLiteral
	| quote nextChar |
	
	quote := self expectQuote.
	writeStream reset.
	[(nextChar := streamReader peek) == nil
		or: [nextChar == quote]]
		whileFalse: [writeStream nextPut: streamReader next].
	self expectNext: quote.

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/5/2015 02:29'!
nextTag
	"performance critical method"
	| tagName attributes nextChar namespaces |

	(streamReader peek == $/)
		ifTrue: [^ self nextEndTag].
	tagName := self nextName.

	attributes := OrderPreservingStringDictionary new: 10.
	"nil understands xmlParserIsSeparator so is not tested for"
	[(nextChar := streamReader peek) xmlParserIsSeparator
		and: [
			streamReader
				next;
				skipSeparators.
			((nextChar := streamReader peek) == $>
				or: [nextChar == $/]) not]]
		whileTrue: [ | attributeName attributeValue |
			attributeName := self nextName.
			streamReader skipSeparators.
			streamReader next == $=
				ifFalse: [self errorExpectedAttributeNameValueSeparator].
			streamReader skipSeparators.
			attributeValue := self nextAttributeValue.
	
			(attributeName at: 1) == $x
				ifTrue: [
					(attributeName isNamespaceAttributeName
						and: [driver usesNamespaces])
						ifTrue: [
							"initialize only if needed"
							(namespaces
								ifNil: [namespaces := OrderPreservingStringDictionary new: 5])
								at:
									((attributeName size > 6)
										ifTrue: [attributeName copyFrom: 7 to: attributeName size]
										ifFalse: [''])
								put: attributeValue]
						ifFalse: [							
							attributeName isReservedAttributeName
								ifTrue: [driver haveUnhandledReservedAttributes].
							attributes at: attributeName put: attributeValue]]
				ifFalse: [attributes at: attributeName put: attributeValue]].

	driver
		handleStartTag: tagName
		attributes: attributes
		namespaces: namespaces.

	nextChar == $/
		ifTrue: [
			streamReader next.
			driver handleEndTag: tagName].
	self expectTerminator.! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 1/6/2015 04:22'!
nextUnreplacedGeneralEntityOnWriteStreamNamed: aName
	driver handleUnreplacedGeneralEntityReference: aName.
	writeStream
		nextPut: $&;
		nextPutAll: aName;
		nextPut: $;.
	^ true.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/28/2014 01:39'!
nextXMLEncodingAttributeValue
	| quote |

	quote := self expectQuote.

	writeStream
		reset;
		nextPut: self expectLetter.
	[streamReader atEnd not
		and: [EncodingNameChars includes: streamReader peek]]
		whileTrue: [writeStream nextPut: streamReader next].

	self expectNext: quote.
	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:44'!
nextXMLStandaloneAttributeValue
	| quote yesOrNo |

	quote := self expectQuote.
	yesOrNo := 
		streamReader peek == $y
			ifTrue: [self expectNextAll: 'yes']
			ifFalse: [self expectNextAll: 'no'].
	self expectNext: quote.

	^ yesOrNo.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/18/2014 19:37'!
nextXMLVersionAttributeValue
	| quote |

	quote := self expectQuote.

	writeStream
		reset;
		nextPut: self expectDigit;
		nextPut: (self expectNext: $.);
		nextPut: self expectDigit.

	[streamReader atEnd
		or: [streamReader peek == quote]]
		whileFalse: [writeStream nextPut: self expectDigit].

	self expectNext: quote.
	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 12/7/2013 01:15'!
parseError: anErrorString
	driver handleParseError: anErrorString! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 4/5/2015 07:56'!
pushBackEntity: anEntity onClose: aBlock
	anEntity hasReplacement
		ifFalse: [
			aBlock ifNotNil: [aBlock value].
			^ self].

	self addActiveEntity: anEntity.
	streamReader
		pushBackStream: anEntity replacement readStream
		onClose: [
			self removeActiveEntity: anEntity.
			aBlock ifNotNil: [aBlock value]].

	anEntity isExternal
		ifTrue: [
			streamReader detectCurrentEncoding.
			(context := context textDeclarationContext)
				nextTokenFrom: self].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 1/20/2014 22:14'!
removeActiveEntity: anEntity
	self activeEntities remove: anEntity.
	anEntity isExternal
		ifTrue: [driver removeURI: anEntity uri].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:26'!
skipSeparatorsInDeclaration
	"this can be used to replace references within declarations"

	streamReader skipSeparators.
	[streamReader peek == $%]
		whileTrue: [
			streamReader next.
			self nextParameterEntityReferenceInDeclaration.
			streamReader skipSeparators]! !

Object subclass: #XMLReadStreamAdapter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLReadStreamAdapter commentStamp: 'monty 12/8/2014 06:00' prior: 0!
An abstract class for read stream adapters for basic read streams that support peek, next, and atEnd.!

XMLReadStreamAdapter subclass: #XMLBinaryReadStreamAdapter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLBinaryReadStreamAdapter commentStamp: 'monty 1/29/2015 01:11' prior: 0!
This adapter converts the byte contents of a binary stream to character objects. It is needed because most Pharo/Squeak streams don't support binary to text conversion.!

!XMLBinaryReadStreamAdapter methodsFor: 'testing' stamp: 'monty 12/8/2014 05:46'!
atEnd
	^ stream atEnd! !

!XMLBinaryReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 12/8/2014 05:46'!
basicNext
	^ stream next asCharacter! !

!XMLBinaryReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 12/8/2014 05:46'!
next
	^ stream next asCharacter! !

!XMLBinaryReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 12/8/2014 05:46'!
peek
	^ stream peek asCharacter! !

XMLReadStreamAdapter subclass: #XMLDecodingReadStreamAdapter
	instanceVariableNames: 'streamDecoder peekChar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLDecodingReadStreamAdapter commentStamp: 'monty 12/8/2014 15:40' prior: 0!
This stream adapter decodes stream contents using a stream decoder.!

!XMLDecodingReadStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 11/20/2014 00:18'!
on: aStream
	^ self
		on: aStream
		streamDecoder: XMLStreamDecoder defaultStreamDecoder new! !

!XMLDecodingReadStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 12/8/2014 05:45'!
on: aStream streamDecoder: aDecoder
	^ (self new on: aStream)
		streamDecoder: aDecoder! !

!XMLDecodingReadStreamAdapter methodsFor: 'testing' stamp: 'monty 11/12/2014 23:06'!
atEnd
	^ peekChar == nil and: [stream atEnd]! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 11/11/2014 18:40'!
basicNext
	stream atEnd
		ifTrue: [^ nil]
		ifFalse: [^ stream next].! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 11/17/2014 21:14'!
next
	peekChar
		ifNil: [^ streamDecoder nextFromStream: self]
		ifNotNil: [:nextChar |
			peekChar := nil.
			^ nextChar]! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 11/17/2014 21:14'!
peek
	^ peekChar ifNil: [peekChar := streamDecoder nextFromStream: self]! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 11/11/2014 15:48'!
streamDecoder
	^ streamDecoder! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 11/11/2014 15:49'!
streamDecoder: aStreamDecoder
	streamDecoder := aStreamDecoder! !

!XMLReadStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 12/8/2014 05:41'!
on: aStream
	^ self new on: aStream! !

!XMLReadStreamAdapter methodsFor: 'testing' stamp: 'monty 12/8/2014 05:42'!
atEnd
	self subclassResponsibility! !

!XMLReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 12/8/2014 08:42'!
basicNext
	self subclassResponsibility! !

!XMLReadStreamAdapter methodsFor: 'file open/close' stamp: 'monty 12/8/2014 05:41'!
close
	(stream respondsTo: #close)
		ifTrue: [stream close]! !

!XMLReadStreamAdapter methodsFor: 'testing' stamp: 'monty 12/8/2014 05:41'!
isBinary
	^ false! !

!XMLReadStreamAdapter methodsFor: 'testing' stamp: 'monty 12/8/2014 05:41'!
isStream
	^ true! !

!XMLReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 12/8/2014 05:42'!
next
	self subclassResponsibility! !

!XMLReadStreamAdapter methodsFor: 'instance creation' stamp: 'monty 12/8/2014 05:41'!
on: aStream
	stream := aStream! !

!XMLReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 12/8/2014 05:42'!
peek
	self subclassResponsibility! !

Object subclass: #XMLSingleCharacterReadStream
	instanceVariableNames: 'character'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLSingleCharacterReadStream commentStamp: 'monty 1/27/2015 23:39' prior: 0!
A read stream for single characters. (faster and less memory than converting a char to a string and creating a read stream on it)!

!XMLSingleCharacterReadStream class methodsFor: 'instance creation' stamp: 'monty 11/16/2014 04:43'!
on: aCharacter
	^ self new on: aCharacter! !

!XMLSingleCharacterReadStream methodsFor: 'testing' stamp: 'monty 11/16/2014 04:44'!
atEnd
	^ character == nil! !

!XMLSingleCharacterReadStream methodsFor: 'accessing' stamp: 'monty 11/20/2014 04:18'!
basicNext
	^ self next! !

!XMLSingleCharacterReadStream methodsFor: 'file open/close' stamp: 'monty 11/20/2014 01:56'!
close! !

!XMLSingleCharacterReadStream methodsFor: 'testing' stamp: 'monty 11/20/2014 02:02'!
isBinary
	^ false! !

!XMLSingleCharacterReadStream methodsFor: 'testing' stamp: 'monty 11/20/2014 01:56'!
isStream
	^ true! !

!XMLSingleCharacterReadStream methodsFor: 'accessing' stamp: 'monty 11/16/2014 04:44'!
next
	^ character ifNotNil: [:nextChar |
		character := nil.
		nextChar]! !

!XMLSingleCharacterReadStream methodsFor: 'initialization' stamp: 'monty 11/16/2014 04:50'!
on: aCharacter
	character := aCharacter! !

!XMLSingleCharacterReadStream methodsFor: 'accessing' stamp: 'monty 11/16/2014 04:44'!
peek
	^ character! !

Object subclass: #XMLStreamDecoder
	instanceVariableNames: 'encodingName'
	classVariableNames: 'PreferredStreamDecoders StreamDecoderCache'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

XMLStreamDecoder class
	instanceVariableNames: 'encodingNames'!

!XMLStreamDecoder commentStamp: 'monty 11/11/2014 21:31' prior: 0!
An abstract class for stream decoders.!

XMLStreamDecoder class
	instanceVariableNames: 'encodingNames'!

XMLStreamDecoder subclass: #XMLByteStreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLByteStreamDecoder commentStamp: 'monty 12/18/2014 19:04' prior: 0!
A stream decoder for ISO Latin-1  or other byte character streams.!

!XMLByteStreamDecoder class methodsFor: 'private' stamp: 'monty 12/18/2014 19:04'!
basicEncodingNames
	^ #('' 'ASCII' 'ISO-8859-1' 'iso-ir-100' 'csISOLatin1' 'Latin1' 'Latin-1' 'l1' 'IBM819' 'CP819')! !

!XMLByteStreamDecoder methodsFor: 'decoding' stamp: 'monty 12/14/2014 14:48'!
nextFromStream: aStream
	^ aStream basicNext! !

!XMLStreamDecoder class methodsFor: 'private' stamp: 'monty 12/14/2014 14:46'!
basicEncodingNames
	^ #()! !

!XMLStreamDecoder class methodsFor: 'testing' stamp: 'monty 12/14/2014 14:46'!
canDecodeEncoding: anEncoding
	^ self encodingNames includes: anEncoding! !

!XMLStreamDecoder class methodsFor: 'accessing' stamp: 'monty 11/13/2014 05:34'!
defaultStreamDecoder
	^ XMLUTF8StreamDecoder! !

!XMLStreamDecoder class methodsFor: 'accessing' stamp: 'monty 12/14/2014 14:46'!
encodingNames
	^ encodingNames ifNil: [
		encodingNames := self basicEncodingNames asSet.
		self basicEncodingNames do: [:each |
			encodingNames
				add: each asLowercase;
				add: each asUppercase].
		encodingNames]! !

!XMLStreamDecoder class methodsFor: 'private' stamp: 'monty 11/12/2014 04:59'!
findStreamDecoderForEncoding: anEncoding
	self preferredStreamDecoders do: [:each |
		(each canDecodeEncoding: anEncoding)
			ifTrue: [^ each]].

	self allSubclassesDo: [:each |
		((self preferredStreamDecoders includes: each) not
			and: [each canDecodeEncoding: anEncoding])
			ifTrue: [^ each]].

	^ nil.! !

!XMLStreamDecoder class methodsFor: 'class initialization' stamp: 'monty 1/12/2015 00:36'!
initialize
	"self initialize"

	StreamDecoderCache := XMLKeyValueCache maxSize: 128.
	(PreferredStreamDecoders := IdentitySet new)
		addAll: XMLUTFStreamDecoder allSubclasses;
		add: XMLByteStreamDecoder.! !

!XMLStreamDecoder class methodsFor: 'testing' stamp: 'monty 11/13/2014 05:35'!
isDefaultStreamDecoder
	^ self == self defaultStreamDecoder! !

!XMLStreamDecoder class methodsFor: 'instance creation' stamp: 'monty 12/14/2014 22:38'!
newForEncoding: anEncoding
	^ (StreamDecoderCache
		at: anEncoding
		ifAbsentPut: [self findStreamDecoderForEncoding: anEncoding])
			ifNotNil: [:decoderClass |
				decoderClass new
					setEncodingName: anEncoding]! !

!XMLStreamDecoder class methodsFor: 'accessing' stamp: 'monty 11/12/2014 04:56'!
preferredStreamDecoders
	^ PreferredStreamDecoders! !

!XMLStreamDecoder methodsFor: 'accessing' stamp: 'monty 12/14/2014 15:02'!
encodingName
	^ encodingName
		ifNil: [encodingName := self class encodingName anyOne]! !

!XMLStreamDecoder methodsFor: 'testing' stamp: 'monty 11/14/2014 21:13'!
isDefaultStreamDecoder
	^ self class isDefaultStreamDecoder! !

!XMLStreamDecoder methodsFor: 'decoding' stamp: 'monty 11/17/2014 21:14'!
nextFromStream: aStream
	self subclassResponsibility! !

!XMLStreamDecoder methodsFor: 'initialization' stamp: 'monty 12/14/2014 15:00'!
setEncodingName: anEncodingName
	encodingName := anEncodingName! !

XMLStreamDecoder subclass: #XMLTextConverterStreamDecoder
	instanceVariableNames: 'textConverter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

XMLTextConverterStreamDecoder class
	instanceVariableNames: 'textConverterCache'!

!XMLTextConverterStreamDecoder commentStamp: 'monty 11/12/2014 04:34' prior: 0!
A stream decoder that uses Pharo TextConverter classes.!

XMLTextConverterStreamDecoder class
	instanceVariableNames: 'textConverterCache'!

!XMLTextConverterStreamDecoder class methodsFor: 'testing' stamp: 'monty 11/12/2014 04:36'!
canDecodeEncoding: anEncoding
	^ (Smalltalk includesKey: #TextConverter)
		and: [(self textConverterForEncoding: anEncoding) notNil]! !

!XMLTextConverterStreamDecoder class methodsFor: 'private' stamp: 'monty 1/12/2015 00:37'!
textConverterCache
	^ textConverterCache
		ifNil: [textConverterCache := XMLKeyValueCache maxSize: 32]! !

!XMLTextConverterStreamDecoder class methodsFor: 'private' stamp: 'monty 1/12/2015 00:34'!
textConverterForEncoding: anEncoding
	^ self textConverterCache
		at: anEncoding
		ifAbsentPut: [
			(Smalltalk at: #TextConverter)
				defaultConverterClassForEncoding: anEncoding asLowercase]! !

!XMLTextConverterStreamDecoder methodsFor: 'decoding' stamp: 'monty 11/17/2014 21:14'!
nextFromStream: aStream
	^ textConverter nextFromStream: aStream! !

!XMLTextConverterStreamDecoder methodsFor: 'initialization' stamp: 'monty 12/14/2014 15:05'!
setEncodingName: anEncodingName
	super setEncodingName: anEncodingName.
	self textConverter:
		(self class textConverterForEncoding: anEncodingName) new.! !

!XMLTextConverterStreamDecoder methodsFor: 'accessing' stamp: 'monty 11/12/2014 01:55'!
textConverter
	^ textConverter! !

!XMLTextConverterStreamDecoder methodsFor: 'accessing' stamp: 'monty 11/12/2014 01:54'!
textConverter: aTextConverter
	textConverter := aTextConverter! !

XMLStreamDecoder subclass: #XMLUTFStreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTFStreamDecoder commentStamp: 'monty 11/20/2014 04:00' prior: 0!
An abstract class for Unicode stream decoders.!

XMLUTFStreamDecoder subclass: #XMLUTF16StreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF16StreamDecoder commentStamp: 'monty 11/20/2014 03:58' prior: 0!
An abstract class for UTF-16 stream decoders.!

XMLUTF16StreamDecoder subclass: #XMLUTF16BigEndianStreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF16BigEndianStreamDecoder commentStamp: 'monty 11/20/2014 03:59' prior: 0!
A stream decoder for big endian UTF-16 streams.!

!XMLUTF16BigEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/12/2014 01:23'!
basicEncodingNames
	^ #('utf16' 'utf-16' 'utf16be' 'utf-16be' 'utf-16-be' 'utf16-be')! !

!XMLUTF16BigEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rFE 16rFF)! !

!XMLUTF16BigEndianStreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 14:28'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte
	^ (aFirstByte bitShift: 8) + aSecondByte! !

!XMLUTF16BigEndianStreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:47'!
isBigEndian
	^ true! !

XMLUTF16StreamDecoder subclass: #XMLUTF16LittleEndianStreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF16LittleEndianStreamDecoder commentStamp: 'monty 11/20/2014 03:59' prior: 0!
A stream decoder for little endian UTF-16 streams.!

!XMLUTF16LittleEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 22:05'!
basicEncodingNames
	^ #('utf16le' 'utf-16le' 'utf-16-le' 'utf16-le')! !

!XMLUTF16LittleEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rFF 16rFE)! !

!XMLUTF16LittleEndianStreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 14:28'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte
	^ (aSecondByte bitShift: 8) + aFirstByte! !

!XMLUTF16LittleEndianStreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:47'!
isBigEndian
	^ false! !

!XMLUTF16StreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 14:44'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte
	self subclassResponsibility! !

!XMLUTF16StreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 15:54'!
errorInvalidCodePoint
	XMLEncodingException signal: 'Invalid UTF-16 code point'! !

!XMLUTF16StreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:46'!
isBigEndian
	self subclassResponsibility! !

!XMLUTF16StreamDecoder methodsFor: 'decoding' stamp: 'monty 11/17/2014 21:14'!
nextFromStream: aStream
	| codePoint |

	codePoint :=
		self
			codePointFromFirstByte:
				(aStream basicNext ifNil: [^ nil]) asciiValue
			secondByte:
				(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue.

	(codePoint < 16rD800 or: [codePoint > 16rDBFF])
		ifTrue: [^ self characterWithValue: codePoint]
		ifFalse: [
			^ self
				surrogatePairFromStream: aStream
				withHighSurrogate: codePoint]! !

!XMLUTF16StreamDecoder methodsFor: 'private' stamp: 'monty 11/12/2014 20:21'!
surrogatePairFromStream: aStream withHighSurrogate: aHighSurrogate
	| lowSurrogate |

	lowSurrogate :=
		self
			codePointFromFirstByte:
				(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue
			secondByte:
				(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue.

	^ self characterWithValue:
		((aHighSurrogate bitAnd: 16r3FF) bitShift: 10) +
		(lowSurrogate bitAnd: 16r3FF) +
		16r10000.! !

XMLUTFStreamDecoder subclass: #XMLUTF32StreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF32StreamDecoder commentStamp: 'monty 11/20/2014 03:58' prior: 0!
An abstract class for UTF-32 stream decoders.!

XMLUTF32StreamDecoder subclass: #XMLUTF32BigEndianStreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF32BigEndianStreamDecoder commentStamp: 'monty 11/20/2014 03:59' prior: 0!
A stream decoder for big endian UTF-32 streams.!

!XMLUTF32BigEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/12/2014 01:24'!
basicEncodingNames
	^ #('utf32' 'utf-32' 'utf32be' 'utf-32be' 'utf-32-be' 'utf32-be')! !

!XMLUTF32BigEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16r00 16r00 16rFE 16rFF)! !

!XMLUTF32BigEndianStreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 18:57'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte
	^ self characterWithValue:
		(aFirstByte bitShift: 24) +
		(aSecondByte bitShift: 16) +
		(aThirdByte bitShift: 8) +
		aFourthByte! !

!XMLUTF32BigEndianStreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:49'!
isBigEndian
	^ true! !

XMLUTF32StreamDecoder subclass: #XMLUTF32LittleEndianStreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF32LittleEndianStreamDecoder commentStamp: 'monty 11/20/2014 03:59' prior: 0!
A stream decoder for little endian UTF-32 streams.!

!XMLUTF32LittleEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:59'!
basicEncodingNames
	^ #('utf32le' 'utf-32le' 'utf-32-le' 'utf32-le')! !

!XMLUTF32LittleEndianStreamDecoder class methodsFor: 'accessing' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rFF 16rFE 16r00 16r00)! !

!XMLUTF32LittleEndianStreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 18:57'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte
	^ self characterWithValue:
		(aFourthByte bitShift: 24) +
		(aThirdByte bitShift: 16) +
		(aSecondByte bitShift: 8) +
		aFirstByte! !

!XMLUTF32LittleEndianStreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:50'!
isBigEndian
	^ false! !

!XMLUTF32StreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 14:58'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte
	self subclassResponsibility! !

!XMLUTF32StreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 15:54'!
errorInvalidCodePoint
	XMLEncodingException signal: 'Invalid UTF-32 code point'! !

!XMLUTF32StreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:49'!
isBigEndian
	self subclassResponsibility! !

!XMLUTF32StreamDecoder methodsFor: 'decoding' stamp: 'monty 11/17/2014 21:14'!
nextFromStream: aStream
	^ self
		codePointFromFirstByte:
			(aStream basicNext ifNil: [^ nil]) asciiValue
		secondByte:
			(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue
		thirdByte:
			(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue
		fourthByte: 
			(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue! !

XMLUTFStreamDecoder subclass: #XMLUTF8StreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF8StreamDecoder commentStamp: 'monty 11/28/2014 23:18' prior: 0!
A stream decoder for UTF-8 streams. (accepts overlong encodings)!

!XMLUTF8StreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:57'!
basicEncodingNames
	^ #('utf8' 'utf-8')! !

!XMLUTF8StreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rEF 16rBB 16rBF)! !

!XMLUTF8StreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 04:39'!
errorInvalidCodePoint
	XMLEncodingException signal: 'Invalid UTF-8 code point'! !

!XMLUTF8StreamDecoder methodsFor: 'decoding' stamp: 'monty 12/22/2014 02:17'!
nextFromStream: aStream
	| firstChar firstByte secondByte thirdByte |

	((firstChar := aStream basicNext) == nil
		or: [(firstByte := firstChar asciiValue) < 16r80]) "single byte character"
		ifTrue: [^ firstChar].

	firstByte < 16rC0 "second bit must be set"
		ifTrue: [self errorInvalidCodePoint].

	secondByte := (aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue.
	(firstByte bitAnd: 16rF0) < 16rE0 "two bytes"
		ifTrue: [
			^ self characterWithValue: 
				((firstByte bitAnd: 16r1F) bitShift: 6) +
				(secondByte bitAnd: 16r3F)].

	thirdByte := (aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue.
	(firstByte bitAnd: 16rF0) = 16rE0 "three bytes"
		ifTrue: [
			^ self characterWithValue: 
				((firstByte bitAnd: 16r0F) bitShift: 12) +
				((secondByte bitAnd: 16r3F) bitShift: 6) +
				(thirdByte bitAnd: 16r3F)].

	^ self characterWithValue: "four"
		((firstByte bitAnd: 16r07) bitShift: 18) +
		((secondByte bitAnd: 16r3F) bitShift: 12) +
		((thirdByte bitAnd: 16r3F) bitShift: 6) +
		((aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue bitAnd: 16r3F).! !

!XMLUTFStreamDecoder class methodsFor: 'accessing' stamp: 'monty 12/14/2014 07:04'!
byteOrderMark
	"avoid asString for portability"
	^ String withAll: (self byteOrderMarkBytes collect: [:each | each asCharacter])! !

!XMLUTFStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 22:01'!
byteOrderMarkBytes
	^ #()! !

!XMLUTFStreamDecoder methodsFor: 'converting' stamp: 'monty 11/11/2014 18:55'!
characterWithValue: aCodePoint
	aCodePoint > 16r10FFFF
		ifTrue: [self errorInvalidCodePoint].

	^ Character value: aCodePoint.! !

!XMLUTFStreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 18:50'!
errorInvalidCodePoint
	self subclassResponsibility! !

Object subclass: #XMLStreamEncodingDetector
	instanceVariableNames: 'stream streamReader'
	classVariableNames: 'EncodingHandlerTable NullCharacter UTF16BigEndianBOM UTF16LittleEndianBOM UTF32BigEndianBOM UTF32LittleEndianBOM UTF8BOM'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLStreamEncodingDetector commentStamp: 'monty 3/2/2015 04:12' prior: 0!
An automatic encoding detector for stream reader streams that infers encodings from byte order marks (BOM) or implicitly from null bytes before or after the first non-null byte.!

!XMLStreamEncodingDetector class methodsFor: 'instance creation' stamp: 'monty 11/28/2014 10:16'!
for: aStreamReader
	^ self new streamReader: aStreamReader! !

!XMLStreamEncodingDetector class methodsFor: 'private' stamp: 'monty 1/27/2015 23:28'!
handlerForLeadByte: aByte
	aByte = 0
		ifTrue: [^ #detectBigEndianUTF32BOMOrImplicitEncoding].
	aByte = XMLUTF16BigEndianStreamDecoder byteOrderMarkBytes first
		ifTrue: [^ #detectBigEndianUTF16BOM].
	aByte = XMLUTF8StreamDecoder byteOrderMarkBytes first
		ifTrue: [^ #detectUTF8BOM].
	aByte = XMLUTF16LittleEndianStreamDecoder byteOrderMarkBytes first
		ifTrue: [^ #detectLittleEndianUTF16BOMOrUTF32BOM].
	aByte = $< asciiValue
		ifTrue: [^ #detectLittleEndianImplicitEncoding].
	^ nil.! !

!XMLStreamEncodingDetector class methodsFor: 'class initialization' stamp: 'monty 1/27/2015 23:36'!
initialize
	"self initialize"
	self
		initializeNullCharacter;
		initializeByteOrderMarks;
		initializeEncodingHandlerTable.! !

!XMLStreamEncodingDetector class methodsFor: 'class initialization' stamp: 'monty 1/27/2015 23:31'!
initializeByteOrderMarks
	UTF8BOM := XMLUTF8StreamDecoder byteOrderMark.
	UTF16BigEndianBOM := XMLUTF16BigEndianStreamDecoder byteOrderMark.
	UTF16LittleEndianBOM := XMLUTF16LittleEndianStreamDecoder byteOrderMark.
	UTF32BigEndianBOM := XMLUTF32BigEndianStreamDecoder byteOrderMark.
	UTF32LittleEndianBOM := XMLUTF32LittleEndianStreamDecoder byteOrderMark.! !

!XMLStreamEncodingDetector class methodsFor: 'class initialization' stamp: 'monty 11/28/2014 03:00'!
initializeEncodingHandlerTable
	EncodingHandlerTable := Array new: 256.

	0 to: 255 do: [:each |
		EncodingHandlerTable
			at: each + 1
			put: (self handlerForLeadByte: each)].! !

!XMLStreamEncodingDetector class methodsFor: 'class initialization' stamp: 'monty 1/27/2015 23:36'!
initializeNullCharacter
	NullCharacter := Character value: 0! !

!XMLStreamEncodingDetector methodsFor: 'private' stamp: 'monty 1/27/2015 23:32'!
detectBOM: aByteOrderMark forDecoderClass: aClass
	stream next.
	2 to: aByteOrderMark size do: [:i |
		(stream atEnd not
			and: [stream peek == (aByteOrderMark at: i)])
			ifFalse: [^ aByteOrderMark copyFrom: 1 to: i - 1].
		stream next].
	streamReader decodeStreamWithDecoder: aClass new.
	^ nil.! !

!XMLStreamEncodingDetector methodsFor: 'private' stamp: 'monty 1/27/2015 23:32'!
detectBigEndianUTF16BOM
	(self
		detectBOM: UTF16BigEndianBOM
		forDecoderClass: XMLUTF16BigEndianStreamDecoder)
		ifNotNil: [:remainder | streamReader pushBack: remainder]! !

!XMLStreamEncodingDetector methodsFor: 'private' stamp: 'monty 1/27/2015 23:32'!
detectBigEndianUTF32BOMOrImplicitEncoding
	| remainder |

	(remainder :=
		self
			detectBOM: UTF32BigEndianBOM
			forDecoderClass: XMLUTF32BigEndianStreamDecoder)
		ifNil: [^ self].
	(remainder allSatisfy: [:each | each == NullCharacter])
		ifFalse: [
			streamReader pushBack: remainder.
			^ self].

	(remainder size = 1
		and: [stream atEnd not
			and: [stream peek == $<]])
		ifTrue: [
			stream next.
			streamReader
				decodeStreamWithDecoder: XMLUTF16BigEndianStreamDecoder new;
				pushBackCharacter: $<.
			^ self].
	(remainder size = 2
		and: [stream atEnd not
			and: [stream peek == NullCharacter]])
		ifTrue: [
			stream next.
			(stream atEnd not
				and: [stream peek == $<])
				ifTrue: [ 
					stream next.
					streamReader
						decodeStreamWithDecoder: XMLUTF32BigEndianStreamDecoder new;
						pushBackCharacter: $<.
					^ self].
			remainder := remainder copyWith: NullCharacter].

	streamReader pushBack: remainder.! !

!XMLStreamEncodingDetector methodsFor: 'detecting' stamp: 'monty 11/28/2014 10:17'!
detectCurrentEncoding
	| characterCode handlerSelector |

	(stream := streamReader stream) atEnd
		ifTrue: [^ self].
	((characterCode := stream peek asciiValue) < 256
		and: [(handlerSelector := EncodingHandlerTable at: characterCode + 1) notNil])
		ifTrue: [self perform: handlerSelector].! !

!XMLStreamEncodingDetector methodsFor: 'private' stamp: 'monty 12/26/2014 22:53'!
detectLittleEndianImplicitEncoding
	"skip <"
	stream next.
	(stream atEnd
		or: [stream peek asciiValue > 0])
		ifTrue: [
			streamReader pushBackCharacter: $<.
			^ self].
	stream next.
	(stream atEnd not
		and: [stream peek == NullCharacter])
		ifTrue: [
			stream next.
			stream atEnd
				ifFalse: [stream next].
			streamReader decodeStreamWithDecoder: XMLUTF32LittleEndianStreamDecoder new]
		ifFalse: [streamReader decodeStreamWithDecoder: XMLUTF16LittleEndianStreamDecoder new].
	
	streamReader pushBackCharacter: $<.! !

!XMLStreamEncodingDetector methodsFor: 'private' stamp: 'monty 1/27/2015 23:32'!
detectLittleEndianUTF16BOMOrUTF32BOM
	(self
		detectBOM: UTF32LittleEndianBOM
		forDecoderClass: XMLUTF32LittleEndianStreamDecoder)
		ifNotNil: [:remainder |
			remainder = UTF16LittleEndianBOM
				ifTrue: [
					streamReader decodeStreamWithDecoder:
						XMLUTF16LittleEndianStreamDecoder new]
				ifFalse: [streamReader pushBack: remainder]]! !

!XMLStreamEncodingDetector methodsFor: 'private' stamp: 'monty 1/27/2015 23:32'!
detectUTF8BOM
	(self
		detectBOM: UTF8BOM
		forDecoderClass: XMLUTF8StreamDecoder)
		ifNotNil: [:remainder | streamReader pushBack: remainder]! !

!XMLStreamEncodingDetector methodsFor: 'accessing' stamp: 'monty 11/28/2014 10:16'!
streamReader: aStreamReader
	streamReader := aStreamReader! !

Object subclass: #XMLTokenContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLTokenContext commentStamp: 'monty 4/6/2015 09:21' prior: 0!
Abstract class for token parsing contexts.!

XMLTokenContext subclass: #XMLContentContext
	instanceVariableNames: 'isLiteralValueContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLContentContext commentStamp: 'monty 11/9/2014 16:20' prior: 0!
A context for content tokens.!

!XMLContentContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
enterLiteralValue
	isLiteralValueContext := true! !

!XMLContentContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
exitLiteralValue
	isLiteralValueContext := false! !

!XMLContentContext methodsFor: 'testing' stamp: 'monty 1/4/2015 04:32'!
isContentContext
	^ true! !

!XMLContentContext methodsFor: 'testing' stamp: 'monty 4/6/2015 02:51'!
isLiteralValueContext
	^ isLiteralValueContext ifNil: [isLiteralValueContext := false]! !

!XMLContentContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 15:41'!
nextTokenFrom: aTokenizer
	aTokenizer nextContentToken! !

XMLTokenContext subclass: #XMLExternalSubsetContext
	instanceVariableNames: 'isLiteralValueContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLExternalSubsetContext commentStamp: 'monty 11/9/2014 16:23' prior: 0!
A context for external subset tokens.!

!XMLExternalSubsetContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
enterLiteralValue
	isLiteralValueContext := true! !

!XMLExternalSubsetContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
exitLiteralValue
	isLiteralValueContext := false! !

!XMLExternalSubsetContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isExternalSubsetContext
	^ true! !

!XMLExternalSubsetContext methodsFor: 'testing' stamp: 'monty 4/6/2015 02:53'!
isLiteralValueContext
	^ isLiteralValueContext ifNil: [isLiteralValueContext := false]! !

!XMLExternalSubsetContext methodsFor: 'tokenizing' stamp: 'monty 12/3/2014 07:24'!
nextTokenFrom: aTokenizer
	aTokenizer nextSubsetToken! !

XMLTokenContext subclass: #XMLPrologContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLPrologContext commentStamp: 'monty 11/9/2014 16:23' prior: 0!
A context for prolog tokens.!

XMLPrologContext subclass: #XMLDoctypeDeclarationContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDoctypeDeclarationContext commentStamp: 'monty 4/6/2015 09:20' prior: 0!
A context for <!!DOCTYPE ...> and internal subset tokens.!

!XMLDoctypeDeclarationContext methodsFor: 'testing' stamp: 'monty 4/6/2015 04:31'!
isDoctypeDeclarationContext
	^ true! !

XMLDoctypeDeclarationContext subclass: #XMLInternalSubsetContext
	instanceVariableNames: 'isLiteralValueContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLInternalSubsetContext commentStamp: 'monty 11/9/2014 16:23' prior: 0!
A context for internal subset tokens.!

!XMLInternalSubsetContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
enterLiteralValue
	isLiteralValueContext := true! !

!XMLInternalSubsetContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
exitLiteralValue
	isLiteralValueContext := false! !

!XMLInternalSubsetContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isInternalSubsetContext
	^ true! !

!XMLInternalSubsetContext methodsFor: 'testing' stamp: 'monty 4/6/2015 02:53'!
isLiteralValueContext
	^ isLiteralValueContext ifNil: [isLiteralValueContext := false]! !

!XMLInternalSubsetContext methodsFor: 'tokenizing' stamp: 'monty 12/3/2014 07:24'!
nextTokenFrom: aTokenizer
	aTokenizer nextSubsetToken! !

XMLPrologContext subclass: #XMLPostDoctypeDeclarationContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLPostDoctypeDeclarationContext commentStamp: 'monty 4/6/2015 09:20' prior: 0!
A context for parsing prolog tokens after <!!DOCTYPE ...> declarations.!

!XMLPostDoctypeDeclarationContext methodsFor: 'testing' stamp: 'monty 4/6/2015 04:36'!
isPostDoctypeDeclarationContext
	^ true! !

!XMLPrologContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:12'!
isPrologContext
	^ true! !

!XMLPrologContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 15:41'!
nextTokenFrom: aTokenizer
	aTokenizer nextPrologToken! !

XMLPrologContext subclass: #XMLXMLDeclarationContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLXMLDeclarationContext commentStamp: 'monty 4/6/2015 09:19' prior: 0!
A context for parsing the <?xml ...?> token.!

!XMLXMLDeclarationContext methodsFor: 'testing' stamp: 'monty 4/6/2015 01:38'!
isXMLDeclarationContext
	^ true! !

XMLTokenContext subclass: #XMLTerminatedContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLTerminatedContext commentStamp: 'monty 11/9/2014 16:28' prior: 0!
A context for after parsing.!

!XMLTerminatedContext methodsFor: 'testing' stamp: 'monty 11/17/2014 02:03'!
isTerminatedContext
	^ true! !

!XMLTerminatedContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 16:27'!
nextTokenFrom: aTokenizer
	aTokenizer nextEndDocument! !

XMLTokenContext subclass: #XMLTextDeclarationContext
	instanceVariableNames: 'previousContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLTextDeclarationContext commentStamp: 'monty 4/6/2015 09:20' prior: 0!
A context for <?xml ...?> text declaration tokens at the start of external entities.!

!XMLTextDeclarationContext class methodsFor: 'instance creation' stamp: 'monty 11/9/2014 17:13'!
previousContext: aTokenContex
	^ self new previousContext: aTokenContex! !

!XMLTextDeclarationContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 17:15'!
nextTokenFrom: aTokenizer
	aTokenizer nextXMLTextDeclaration! !

!XMLTextDeclarationContext methodsFor: 'accessing' stamp: 'monty 11/9/2014 18:27'!
previousContext
	^ previousContext! !

!XMLTextDeclarationContext methodsFor: 'accessing' stamp: 'monty 11/9/2014 17:13'!
previousContext: aTokenContex
	previousContext := aTokenContex! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:02'!
contentContext
	^ XMLContentContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 4/6/2015 04:31'!
doctypeDeclarationContext
	^ XMLDoctypeDeclarationContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:02'!
externalSubsetContext
	^ XMLExternalSubsetContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 3/1/2015 17:20'!
internalSubsetContext
	^ XMLInternalSubsetContext new! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 1/4/2015 04:32'!
isContentContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 4/6/2015 04:31'!
isDoctypeDeclarationContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isExternalSubsetContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:15'!
isInitializedContext
	^ true! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isInternalSubsetContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 4/6/2015 02:27'!
isLiteralValueContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 4/6/2015 04:37'!
isPostDoctypeDeclarationContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:12'!
isPrologContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/17/2014 02:03'!
isTerminatedContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 4/6/2015 01:38'!
isXMLDeclarationContext
	^ false! !

!XMLTokenContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 16:40'!
nextTokenFrom: aTokenizer
	self subclassResponsibility! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 4/6/2015 04:37'!
postDoctypeDeclarationContext
	^ XMLPostDoctypeDeclarationContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:02'!
prologContext
	^ XMLPrologContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:03'!
terminatedContext
	^ XMLTerminatedContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:01'!
textDeclarationContext
	^ XMLTextDeclarationContext previousContext: self! !

XMLTokenContext subclass: #XMLUnitializedContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLUnitializedContext commentStamp: 'monty 11/9/2014 16:28' prior: 0!
A context for before parsing.!

!XMLUnitializedContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:15'!
isInitializedContext
	^ false! !

!XMLUnitializedContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 16:27'!
nextTokenFrom: aTokenizer
	aTokenizer nextStartDocument! !

Object subclass: #XMLURI
	instanceVariableNames: 'scheme hasAuthorityPrefix userInfo host port pathSegments queryString fragment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-URI'!

!XMLURI commentStamp: 'monty 1/27/2015 23:00' prior: 0!
A class for XML URIs. It uses the XMLURIParser to parse URIs and can combine URIs using /, can decode URI paths, and can retrieve contents of http:// and file:// URLs using XMLHTTPRequest and XMLFileHandle.!

!XMLURI class methodsFor: 'instance creation' stamp: 'monty 2/9/2015 06:40'!
fromString: aString
	^ self basicNew initialize fromString: aString! !

!XMLURI class methodsFor: 'instance creation' stamp: 'monty 2/9/2015 06:40'!
new
	^ self fromString: ''! !

!XMLURI methodsFor: 'combining' stamp: 'monty 3/22/2015 03:52'!
/ aURIOrString
	| uriToCombineWith |

	(uriToCombineWith := aURIOrString asXMLURI) isEmpty
		ifFalse: [
			(uriToCombineWith isRelative)
				ifTrue: [^ self copy combineWithRelativeURI: uriToCombineWith]
				ifFalse: [^ uriToCombineWith]]! !

!XMLURI methodsFor: 'comparing' stamp: 'monty 2/9/2015 06:43'!
= anObject
	self == anObject
		ifTrue: [^ true].

	self class == anObject class
		ifFalse: [^ false].

	^ scheme = anObject scheme
		and: [hasAuthorityPrefix = anObject hasAuthorityPrefix
			and: [userInfo = anObject userInfo
				and: [host = anObject host
					and: [port = anObject port
						and: [pathSegments = anObject pathSegments
							and: [queryString = anObject queryString
								and: [fragment = anObject fragment]]]]]]].! !

!XMLURI methodsFor: 'private' stamp: 'monty 2/7/2015 05:41'!
addDotPathSegment
	self hasNonDirectoryPath
		ifTrue: [self haveDirectoryPath]! !

!XMLURI methodsFor: 'private' stamp: 'monty 2/9/2015 00:01'!
addDoubleDotPathSegment
	self hasPath
		ifFalse: [^ self].

	self hasRootPath
		ifTrue: [pathSegments removeAll]
		ifFalse: [| hadAbsolutePath |
			hadAbsolutePath := self hasAbsolutePath.
			self hasDirectoryPath
				ifTrue: [pathSegments removeLast].
			self hasPath
				ifTrue: [
					pathSegments removeLast.
					(self hasPath
						or: [hadAbsolutePath])
						ifTrue: [self haveDirectoryPath]]].! !

!XMLURI methodsFor: 'private' stamp: 'monty 2/7/2015 05:49'!
addPathSegment: aSegment
	aSegment = '.'
		ifTrue: [^ self addDotPathSegment].
	aSegment = '..'
		ifTrue: [^ self addDoubleDotPathSegment].

	self hasPath
		ifTrue: [
			(self hasDirectoryPath
				and: [pathSegments size > 1])
				ifTrue: [pathSegments removeLast]]
		ifFalse: [
			self hasHost
				ifTrue: [pathSegments addLast: '']].

	pathSegments addLast: aSegment.! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/21/2015 22:37'!
addPathSegmentsFrom: aURI
	aURI hasPath
		ifFalse: [^ self].

	aURI hasAbsolutePath
		ifTrue: [pathSegments removeAll]
		ifFalse: [
			self hasNonDirectoryPath
				ifTrue: [
					pathSegments removeLast.
					self hasPath
						ifTrue: [self haveDirectoryPath]]].

	aURI pathSegments do: [:each | self addPathSegment: each].! !

!XMLURI methodsFor: 'converting' stamp: 'monty 1/21/2014 23:37'!
asRetrievableResource
	^ self isLocalURI
		ifTrue: [XMLFileHandle path: self unescapedPath]
		ifFalse: [XMLHTTPRequest url: self asString]! !

!XMLURI methodsFor: 'converting' stamp: 'monty 4/6/2015 08:05'!
asString
	"for compatibility with GS where Object>>asString isn't defined"
	^ self printString! !

!XMLURI methodsFor: 'converting' stamp: 'monty 1/10/2014 00:56'!
asXMLURI
	^ self! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/21/2015 23:05'!
combineWithRelativeURI: aRelativeURI
	(aRelativeURI hasHost
		or: [aRelativeURI hasPath])
		ifTrue: [
			queryString := ''.
			fragment := ''].
	aRelativeURI hasHost
		ifTrue: [
			host := aRelativeURI host.
			pathSegments removeAll].
	aRelativeURI hasPort
		ifTrue: [port := aRelativeURI port].
	aRelativeURI hasPath
		ifTrue: [self addPathSegmentsFrom: aRelativeURI].
	aRelativeURI hasQueryString
		ifTrue: [queryString := aRelativeURI queryString].
	aRelativeURI hasFragment
		ifTrue: [fragment := aRelativeURI fragment].! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 02:02'!
fragment
	^ fragment! !

!XMLURI methodsFor: 'initialization' stamp: 'monty 2/9/2015 06:55'!
fromString: aString
	| parser  |

	parser := XMLURIParser on: aString.

	scheme := parser parseScheme.
	hasAuthorityPrefix := parser parseAuthorityPrefix.
	(hasAuthorityPrefix
		or: [scheme notEmpty])
		ifTrue: [
			userInfo := parser parseUserInfo.
			host := parser parseHost.
			port := parser parsePort]
		ifFalse: [
			userInfo := ''.
			host := ''].
	self setPathSegmentsFromString: parser parsePath.
	queryString := parser parseQueryString.
	fragment := parser parseFragment.! !

!XMLURI methodsFor: 'retrieving' stamp: 'monty 1/10/2014 00:13'!
get
	^ self getUpToLimit: nil! !

!XMLURI methodsFor: 'retrieving' stamp: 'monty 1/10/2014 00:13'!
getUpToLimit: aMaxSize
	^ self asRetrievableResource getUpToLimit: aMaxSize! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/10/2014 06:51'!
hasAbsolutePath
	^ pathSegments notEmpty
		and: [pathSegments first isEmpty]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 2/9/2015 01:43'!
hasAuthority
	^ self hasUserInfo
		or: [self hasHost
			or: [self hasPort]]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 2/9/2015 06:43'!
hasAuthorityPrefix
	^ hasAuthorityPrefix! !

!XMLURI methodsFor: 'testing' stamp: 'monty 2/6/2015 23:24'!
hasDirectoryPath
	^ pathSegments notEmpty
		and: [pathSegments last isEmpty]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 10/22/2014 07:32'!
hasFragment
	^ fragment notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/16/2014 08:56'!
hasHost
	^ host notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 2/7/2015 00:00'!
hasNonDirectoryPath
	^ self hasPath and: [self hasDirectoryPath not]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 2/6/2015 23:05'!
hasPath
	^ pathSegments notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 10/22/2014 07:27'!
hasPort
	^ port notNil! !

!XMLURI methodsFor: 'testing' stamp: 'monty 10/22/2014 07:31'!
hasQueryString
	^ queryString notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 2/9/2015 00:02'!
hasRootPath
	^ pathSegments size = 2
		and: [pathSegments allSatisfy: [:each | each isEmpty]]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 10/22/2014 07:30'!
hasScheme
	^ scheme notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 2/7/2015 21:11'!
hasUserInfo
	^ userInfo notEmpty! !

!XMLURI methodsFor: 'comparing' stamp: 'monty 1/13/2014 05:05'!
hash
	^ host hash bitXor: pathSegments hash! !

!XMLURI methodsFor: 'private' stamp: 'monty 2/7/2015 22:07'!
haveDirectoryPath
	self hasPath
		ifFalse: [pathSegments addLast: ''].
	pathSegments addLast: ''.! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 01:50'!
host
	^ host! !

!XMLURI methodsFor: 'testing' stamp: 'monty 2/9/2015 01:46'!
isEmpty
	^ (self hasScheme
		or: [self hasAuthority
			or: [self hasPath
				or: [self hasQueryString
					or: [self hasFragment]]]]) not! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/7/2014 01:54'!
isFileURI
	^ scheme = 'file'! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/10/2014 00:47'!
isLocalURI
	^ self isFileURI or: [self isRelative]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/10/2014 06:54'!
isRelative
	^ scheme isEmpty
		or: [host isEmpty
			and: [self isFileURI not]]! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/30/2015 20:34'!
path
	^ self pathPrintedUnescaped: false! !

!XMLURI methodsFor: 'printing' stamp: 'monty 2/6/2015 23:05'!
pathPrintedUnescaped: aBoolean
	| writeStream |

	self hasPath
		ifFalse: [^ ''].

	writeStream := (String new: pathSegments size * 10) writeStream.
	self
		printPathSegmentsOn: writeStream
		unescaped: aBoolean.
	^ writeStream contents.! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/10/2014 00:07'!
pathSegments
	^ pathSegments! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 10/22/2014 06:42'!
port
	^ port! !

!XMLURI methodsFor: 'copying' stamp: 'monty 1/10/2014 00:13'!
postCopy
	pathSegments := pathSegments copy! !

!XMLURI methodsFor: 'printing' stamp: 'monty 2/9/2015 06:54'!
printAuthorityOn: aStream
	self hasAuthority
		ifTrue: [
			(self hasScheme not
				and: [self hasAuthorityPrefix])
				ifTrue: [aStream nextPutAll: '//'].
			self
				printUserInfoOn: aStream;
				printHostOn: aStream;
				printPortOn: aStream]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 10/22/2014 07:32'!
printFragmentOn: aStream
	self hasFragment
		ifTrue: [
			aStream
				nextPut: $#;
				nextPutAll: fragment]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 2/7/2015 21:12'!
printHostOn: aStream
	self hasHost
		ifTrue: [aStream nextPutAll: host]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 2/7/2015 21:13'!
printOn: aStream
	self
		printSchemeOn: aStream;
		printAuthorityOn: aStream;
		printPathSegmentsOn: aStream unescaped: false;
		printQueryStringOn: aStream;
		printFragmentOn: aStream
		! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/29/2015 16:20'!
printPathSegmentsOn: aStream unescaped: aBoolean
	pathSegments
		do: [:each |
			aBoolean
				ifTrue: [self printUnescapedPathSegment: each on: aStream]
				ifFalse: [aStream nextPutAll: each]]
		separatedBy: [aStream nextPut: $/]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 10/22/2014 07:29'!
printPortOn: aStream
	self hasPort
		ifTrue: [
			aStream
				nextPut: $:;
				nextPutAll: port asString]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 10/22/2014 07:32'!
printQueryStringOn: aStream
	self hasQueryString
		ifTrue: [
			aStream
				nextPut: $?;
				nextPutAll: queryString]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 2/9/2015 06:54'!
printSchemeOn: aStream
	self hasScheme
		ifTrue: [
			aStream 
				nextPutAll: scheme;
				nextPut: $:.
			self hasAuthorityPrefix
				ifTrue: [aStream nextPutAll: '//']]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/29/2015 16:20'!
printUnescapedPathSegment: aString on: aStream
	| readStream nextChar charCode |

	readStream := aString readStream.
	[readStream atEnd]
		whileFalse: [
			(nextChar := readStream next) == $%
				ifTrue: [
					charCode := 
						XMLIntegerReader
							readFrom: readStream
							withBase: 16
							upTo: 2.
					aStream nextPut: (charCode ifNil: [$%]) asCharacter]
				ifFalse: [aStream nextPut: nextChar]].! !

!XMLURI methodsFor: 'printing' stamp: 'monty 2/7/2015 21:12'!
printUserInfoOn: aStream
	self hasUserInfo
		ifTrue: [
			aStream
				nextPutAll: userInfo;
				nextPut: $@]! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 03:38'!
queryString
	^ queryString! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 01:50'!
scheme
	^ scheme! !

!XMLURI methodsFor: 'private' stamp: 'monty 4/5/2015 08:01'!
setPathSegmentsFromString: aString
	pathSegments := OrderedCollection new.

	"pathSegments must remain empty if the path string is empty, but splitting
	an empty string would add a single empty string to it"
	aString isEmpty
		ifFalse: [
			$/
				xmlParserSplit: aString
				do: [:segment | pathSegments addLast: segment]].! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/30/2015 20:34'!
unescapedPath
	^ self pathPrintedUnescaped: true! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 2/7/2015 21:09'!
userInfo
	^ userInfo! !

Object subclass: #XMLURIParser
	instanceVariableNames: 'writeStream readStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-URI'!

!XMLURIParser commentStamp: 'monty 1/7/2014 04:42' prior: 0!
This is a basic URI parser that can split a URI into scheme, host (with port), path, query string, and fragment.!

!XMLURIParser class methodsFor: 'instance creation' stamp: 'monty 2/7/2015 22:22'!
on: aString
	^ self new on: aString! !

!XMLURIParser methodsFor: 'initialization' stamp: 'monty 2/8/2015 02:26'!
on: aString
	readStream := aString readStream.
	writeStream := (String new: aString size) writeStream.! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 2/9/2015 06:55'!
parseAuthorityPrefix
	| oldPosition |

	oldPosition := readStream position.
	((self parseNext: $/)
		and: [self parseNext: $/])
		ifTrue: [^ true]
		ifFalse: [
			readStream position: oldPosition.
			^ false].! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 2/7/2015 21:01'!
parseFragment
	^ self parseUpToEndAfterNext: $#! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 2/7/2015 20:45'!
parseHost
	^ (self parseUpToAnyNotSkipping: ':/?#') asLowercase! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 2/9/2015 06:59'!
parseNext: aCharacter
	(readStream atEnd not
		and: [readStream peek == aCharacter])
		ifTrue: [
			readStream next.
			^ true]
		ifFalse: [^ false]! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 1/7/2014 01:34'!
parsePath
	^ self parseUpToAnyNotSkipping: '?#'! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 2/9/2015 06:59'!
parsePort
	(self parseNext: $:)
		ifTrue: [
			^ XMLIntegerReader
				readFrom: (self parseUpToAnyNotSkipping: '/?#') readStream
				withBase: 10]
		ifFalse: [^ nil]! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 2/7/2015 20:52'!
parseQueryString
	^ self
		parseUpToAnyNotSkipping: '#'
		afterNext: $?! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 2/8/2015 02:26'!
parseScheme
	| scheme |

	scheme := self parseUpToAnyNotSkipping: ':/@?#'.
	(self parseNext: $:)
		ifTrue: [^ scheme asLowercase]
		ifFalse: [
			readStream reset.
			^ ''].! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 2/7/2015 20:52'!
parseUpToAnyNotSkipping: aCollection
	^ self
		parseUpToAnyNotSkipping: aCollection
		afterNext: nil! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 2/8/2015 02:26'!
parseUpToAnyNotSkipping: aCollection afterNext: aCharacter
	aCharacter
		ifNotNil: [
			(self parseNext: aCharacter)
				ifFalse: [^ '']].

	writeStream reset.
	[readStream atEnd
		or: [aCollection includes: readStream peek]]
		whileFalse: [writeStream nextPut: readStream next].
	^ writeStream contents.! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 2/8/2015 02:26'!
parseUpToEndAfterNext: aCharacter
	(self parseNext: aCharacter)
		ifTrue: [^ readStream upToEnd]
		ifFalse: [^ '']! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 2/8/2015 02:26'!
parseUserInfo
	| userInfo oldPosition |

	oldPosition := readStream position.
	userInfo := self parseUpToAnyNotSkipping: ':/@?#'.
	(self parseNext: $@)
		ifTrue: [^ userInfo]
		ifFalse: [
			readStream position: oldPosition.
			^ ''].! !

Object subclass: #XMLUnparsedEntityValidator
	instanceVariableNames: 'elementsAndNotationAttributes unparsedEntities notations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLUnparsedEntityValidator commentStamp: 'monty 1/1/2015 21:00' prior: 0!
Used by attribute validators to validate attribute references to unparsed entities and notations.!

!XMLUnparsedEntityValidator class methodsFor: 'instance creation' stamp: 'monty 1/1/2015 21:12'!
unparsedEntities: anUnparsedEntityDictionary notations: aNotationSet
	^ self new
		setUnparsedEntities: anUnparsedEntityDictionary
		notations: aNotationSet! !

!XMLUnparsedEntityValidator methodsFor: 'accessing' stamp: 'monty 3/2/2015 12:55'!
elementsAndNotationAttributes
	^ elementsAndNotationAttributes ifNil: [elementsAndNotationAttributes := Dictionary new]! !

!XMLUnparsedEntityValidator methodsFor: 'private' stamp: 'monty 1/1/2015 20:45'!
errorMultipleNotationAttributesForElement: anElement
	XMLValidationException
		formatSignal: 'Element <{1}> has multiple NOTATION list attributes'
		with: anElement! !

!XMLUnparsedEntityValidator methodsFor: 'private' stamp: 'monty 1/3/2015 02:56'!
errorUndeclaredNotation: aNotation inAttribute: anAttribute
	XMLValidationException
		formatSignal: 'Undeclared NOTATION "{1}" in "{2}" attribute'
		with: aNotation
		with: anAttribute! !

!XMLUnparsedEntityValidator methodsFor: 'private' stamp: 'monty 1/7/2015 03:47'!
errorUndeclaredNotation: aNotation inEntity: anEntity
	XMLValidationException
		formatSignal: 'Undeclared notation "{1}" referenced by &{2}; entity'
		with: aNotation
		with: anEntity! !

!XMLUnparsedEntityValidator methodsFor: 'private' stamp: 'monty 1/7/2015 02:19'!
errorUndeclaredUnparsedEntityReference: aName inAttribute: anAttriubute
	XMLValidationException
		formatSignal: 'Undeclared unparsed entity "{1}" in "{2}" attribute'
		with: aName
		with: anAttriubute! !

!XMLUnparsedEntityValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 20:45'!
notations
	^ notations! !

!XMLUnparsedEntityValidator methodsFor: 'initialization' stamp: 'monty 1/1/2015 21:12'!
setUnparsedEntities: anUnparsedEntityDictionary notations: aNotationSet
	unparsedEntities := anUnparsedEntityDictionary.
	notations := aNotationSet.! !

!XMLUnparsedEntityValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 20:34'!
unparsedEntities
	^ unparsedEntities! !

!XMLUnparsedEntityValidator methodsFor: 'validating' stamp: 'monty 1/7/2015 02:19'!
validateEntityReference: aName inAttribute: anAttribute
	(self unparsedEntities includesKey: aName)
		ifFalse: [
			self
				errorUndeclaredUnparsedEntityReference: aName
				inAttribute: anAttribute]! !

!XMLUnparsedEntityValidator methodsFor: 'validating' stamp: 'monty 3/2/2015 12:55'!
validateNotationAttribute: anAttribute forElement: anElement
	| oldSize |

	"checking size after is faster than includes"
	oldSize := self elementsAndNotationAttributes size.
	(self elementsAndNotationAttributes
		at: anElement
		put: anAttribute;
		size) > oldSize
		ifFalse: [self errorMultipleNotationAttributesForElement: anElement].! !

!XMLUnparsedEntityValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:31'!
validateNotationReference: aNotation inAttribute: anAttribute
	(self notations includes: aNotation)
		ifFalse: [
			self
				errorUndeclaredNotation: aNotation
				inAttribute: anAttribute]! !

!XMLUnparsedEntityValidator methodsFor: 'validating' stamp: 'monty 1/7/2015 03:47'!
validateUnparsedEntity: anEntity withNotation: aNotation
	(self notations includes: aNotation)
		ifFalse: [
			self
				errorUndeclaredNotation: aNotation
				inEntity: anEntity]! !
XMLIntegerReader initialize!
XMLParserTokenizer initialize!
XMLParserLimits initialize!
XMLAttributeValidator initialize!
XMLStreamEncodingDetector initialize!
XMLNestedStreamReader initialize!
XMLWellFormedParserTokenizer initialize!
XMLStreamDecoder initialize!
DTDAttributeDefinitionParser initialize!
DTDDFAContentModelParser initialize!
